package org.ebmwebsourcing.b2bgw.edifact.lexers_parsers.edifact_standard.composite_data_element;
   
import java_cup.runtime.*;
      
%%////////////////////////////////////////////////////////////////////////

%class LexerCDE
%public

%line
%column
%16bit

%cup
   
%{   
    private Symbol symbol(int type) {
        return new Symbol(type, yyline, yycolumn);
    }

    private Symbol  symbol(int type, Object value) {
        return new Symbol(type, yyline, yycolumn, value);
    }  
    
    private boolean codeline = false;
    
    private boolean begtag = false;

    
%}

newline = \r|\n|\r\n
   
anything = [A-Za-z_][A-Za-z_0-9]* | 
					[A-Za-Z_0-9_]+ | .
					
note = ("Note:" ~"\n\n") | ("Note:" ~"\r\n\r\n")

sentence = ([A-Z] [a-z-]+ (" " "("? [a-z]+ ")"? )+) | 
					([A-Z] [a-z/]+ (" " "("?  [a-z]+ ")"? )+) | 
					([A-Z] [a-z]+ (" " "("? [a-z/]+ ")"? )+) 	| 
					"("? [A-Za-z/]{2,20} ")"? |
					([A-Z] " = " [A-Z] [a-z]+) |
					([A-Z] ".")+

definition = ([A-Z]+ " " "& "? [A-Z]+ )+ | 
					([A-Z/]+ " " "& "?  [A-Z]+ )+ | 
					([A-Z]+ " " "& "?  [A-Z/]+ )+ | 
					( [A-Z/]{2, 20}  )+

description = ("Desc: " ~".\n\n") | ("Desc: " ~".\r\n\r\n")

codeelement = [A-Z]{1} [0-9]{3}

codeline = [0-9]{3} "   "

codedef = [0-9]{4}

state = "M " | "C "

size = a?n? ".." [0-9]{1,3} | a?n? [0-9]{1,3}

/////////////////////////////////////////////////////////////////
%%
  
   
<YYINITIAL> {

	{note}
		{}

	{newline}
		{}

	"----------------------------------------------------------------------" 		
		{ 
			begtag = true;
			return symbol(SymCDE.BEGTAG);
		}
		
	
	{codeelement}		       
		{ 
			if (begtag) {
				return symbol(SymCDE.CODE, yytext());
			}
		}

	{codeline}		   
		{ 
			codeline = true;
			return symbol(SymCDE.INDEX, yytext());
		} 	
	
	{codedef}		   
		{ 
			if (codeline){
				return symbol(SymCDE.VALUE, yytext());
			}		
		}
	
	{size}		       
		{
			if (codeline){
				codeline = false;
				return symbol(SymCDE.SIZE, yytext());
			}
		}

    {description}         
    	{
			return symbol(SymCDE.DESC, yytext());
		} 
		
    {definition}         
    	{
    		if (begtag){
    			begtag = false;
				return symbol(SymCDE.DEFINITION, yytext());
			}
		}
                         
    {sentence}         
    	{
			if (codeline){			
				return symbol(SymCDE.ENTITLE, yytext());
			}
		} 
		
    {state}
    	{
    		if (codeline){    		
				return symbol(SymCDE.STATE, yytext());
			}
		}
		
    {anything}         
    	{}
    
}

//[^]                    { throw new Error("Illegal character <"+yytext()+">"); }
