package org.ebmwebsourcing.b2bgw.edifact.lexers_parsers.edifact_standard.segment_element;
import java_cup.runtime.*;
      
%%////////////////////////////////////////////////////////////////////////

%class LexerSE

%public
%line
%column
%16bit

%cup
   
%{   
    private Symbol symbol(int type) {
        return new Symbol(type, yyline, yycolumn);
    }

    private Symbol  symbol(int type, Object value) {
        return new Symbol(type, yyline, yycolumn, value);
    }  
    
    private boolean codeelem = false;
    
    private boolean begtag = false;
	private boolean wasCodeSeg = false;
	private boolean wasCodeElem = false;	
	private boolean wasIndex = false;	
    
%}

newline = \r|\n|\r\n
   
anything = [A-Za-z_][A-Za-z_0-9]* | 
					[A-Za-Z_0-9_]+ | . | " "{4,15}
					
note = ("Note:" ~".\n") | ("Note:" ~".\r\n") | "DEPENDENCY NOTES:" | "OTHER NOTES:" | "NOTES:"

sentence = ([A-Z] [a-z-]+ ","? (" " "("? [a-z]+ ")"? ","? )+) | 
					([A-Z] [a-z/]+ ","? (" " "("?  [a-z]+ ")"? ","? )+) | 
					([A-Z] [a-z]+ ","? (" " "("? [a-z/]+ ")"? ","? )+) 	| 
					"("? [A-Za-z/]{2,20} ")"? |
					([A-Z] " = " [A-Z] [a-z]+) |
					([A-Z] ".")+

definition = ([A-Z-]+ ","? " " "& "? [A-Z] [A-Z]+ )+ | 
					([A-Z/]+ ","? " " "& "?  [A-Z] [A-Z]+ )+ | 
					([A-Z]+ ","? " " "& "?  [A-Z/] [A-Z/]+ )+ | 
					( [A-Z,] [A-Z/]+)+ |
					[A-Z/-]+  (" " "& "? [A-Z] [A-Z/]+ )+

function = ("Function: " ~".\n\n") | ("Function: " ~".\r\n\r\n")

codeelement = [A-Z]{1} [0-9]{3}

codeline = ([0-9]{3} "   ") | ([0-9]{3} " + ") | ([0-9]{3} " X ")

codedef = [0-9]{4}

state = "M " | "C "

size = a?n? ".." [0-9]{1,3} | a?n? [0-9]{1,3}

codeseg = [A-Z]{3}

repeat =  ("  " [1-9]{1,3}) | ([1-9]{1,3} "   ") | ([1-9]{1,3} "\t") 

begtag = "----------------------------------------------------------------------" | "\r\n\r\n\r\n\r\n" | "\n\n\n\n" 

rebegtag = "Pos   TAG   Name"

/////////////////////////////////////////////////////////////////
%%
  
   
<YYINITIAL> {

	{note}
		{
			begtag = false;
		}
		
	{rebegtag}
		{
			begtag = true;
		}		
		
	{repeat}
		{
			if (begtag && codeelem) {
				return symbol(SymSE.REPEAT, yytext());
			}
		}		

	{newline}
		{}

	 {begtag}
		{ 
			begtag = true;
			codeelem = false;
			wasCodeSeg = false;
			return symbol(SymSE.BEGTAG);
		}

	{codeseg}		       
		{ 
			if (begtag && !wasCodeSeg) {
				wasCodeSeg = true;
				return symbol(SymSE.CODESEG, yytext());
			}
		}		
	
	{codeelement}		       
		{ 
			if (begtag) {
				wasCodeElem = true;
				return symbol(SymSE.CODE, yytext());
			}
		}

	{codeline}		   
		{ 
			if (begtag){
				wasCodeElem = false;
				codeelem = true;
				wasIndex = true;
				return symbol(SymSE.INDEX, yytext());
			}
		} 	
	
	{codedef}		   
		{ 
			if (begtag){
				codeelem = true;
				wasCodeElem = false;
				return symbol(SymSE.VALUE, yytext());
			}		
		}
	
	{size}		       
		{
			if (begtag && codeelem){
				if  (wasIndex){
					wasIndex = false;
					return symbol(SymSE.REPEAT, yytext());
				}
				else if (!wasCodeElem){	
					codeelem = false;	
					wasIndex = false;
					return symbol(SymSE.SIZE, yytext());										
				}
			}
		}
		
    {function}         
    	{	
			if (begtag){
				return symbol(SymSE.FUNCTION, yytext());
			}
		} 		
		
    {definition}         
    	{			
			if (begtag){
				if (codeelem){
					return symbol(SymSE.DEFTAG, yytext());
				}
				else {
					return symbol(SymSE.DEFINITION, yytext());
				}
			}
		}
                         
    {sentence}         
    	{
			if (codeelem){			
				return symbol(SymSE.ENTITLE, yytext());
			}
		} 
		
    {state}
    	{
    		if (begtag && codeelem){    		
				return symbol(SymSE.STATE, yytext());
			}
		}
		
    {anything}         
    	{}
    
}

//[^]                    { throw new Error("Illegal character <"+yytext()+">"); }
