/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2006 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: ExternalListenerManager.java 154 27 sept. 06 ofabre $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.component.common.basic.binding;

/**
 * This manager is used to enable a binding component to handle external service
 * call. It defines methods to register and unregister listeners for external
 * services.
 * <p>
 * This interface have to be implemented if the related binding component is
 * used to listen to incoming messages from external services, in order to
 * forward these messages to internal JBI Components.
 * 
 * @author ofabre - EBM Websourcing
 * 
 */
public interface ExternalListenerManager {

    /**
     * Start a listener for the given address.
     * 
     * @param address
     *            the external address to listen to.
     * @throws Exception
     *             if an error occurs during listener starting
     */
    public void startListening(String address) throws Exception;

    /**
     * Stop a listener for the given address.
     * 
     * @param address
     *            the external address to listen to.
     */
    public void stopListening(String address);

}
