/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2006 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: JBIListener.java 154 27 sept. 06 ofabre $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.component.common.basic.binding;

import java.net.URI;

import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.xml.namespace.QName;

import org.objectweb.petals.component.common.MEPConstants;

/**
 * This interface defines a method that forward internal JBI messages to
 * external services.
 * 
 * @author alouis, ofabre - EBM Websourcing
 * 
 */
public interface JBIListener {

    /**
     * Send an incoming JBI message to an outside service. The address
     * associated with the requested endpoint is resolved by the abstract
     * component.
     * 
     * @param address
     *            the external service address associated to the endpoint
     *            given in the incoming message exchange
     * @param in
     *            the "in" message from the MessageExchange sent by the JBI
     *            consumer component
     * @param out
     *            the "out" message from the MessageExchange. This message have
     *            to be set in case of IN_OUT_PATTERN. It can be set in case of
     *            IN_OPTIONAL_OUT_PATTERN. null in case of IN_ONLY_PATTERN or
     *            ROBUST_IN_ONLY_PATTERN
     * @param operation
     *            the operation set for the MessageExchange. 
     * @param messageExchangePattern
     *            The exchange pattern of the MessageExchange : IN_OUT_PATTERN,
     *            IN_OPTIONAL_OUT_PATTERN, IN_ONLY_PATTERN,
     *            ROBUST_IN_ONLY_PATTERN.
     * @see MEPConstants
     * 
     * @return boolean used in case of IN_OPTIONAL_OUT_PATTERN. Set to true if
     *         there's an out message to send, false otherwise
     * @throws MessagingException
     *             If an error occurs during processing. If the MEP allows it,
     *             a Fault is returned on the MessageExchange
     */
    public boolean onJBIMessage(String address, NormalizedMessage in,
            NormalizedMessage out, QName operation, URI messageExchangePattern)
        throws MessagingException;

}
