/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: InstallationService.java 154 2006-03-27 15:30:10Z wjoseph $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.component.common.serviceunitmanager.handler;

import java.io.File;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jbi.component.ComponentContext;
import javax.jbi.management.DeploymentException;

import org.objectweb.petals.component.common.serviceunitmanager.manager.PetalsServiceUnitManager;
import org.objectweb.petals.component.common.util.ManagementMessageUtil;
import org.objectweb.petals.component.common.util.WSDLHelper;

import org.w3c.dom.Document;

/**
 * Handle service units for external services
 * 
 * @version $Rev: 250 $ $Date: 2006-04-21 14:20:57 +0200 (ven, 21 avr 2006) $
 * @since Petals 1.0
 * @author wjoseph - eBMWebsourcing
 */

public class ExternalServiceManager extends PetalsServiceUnitHandler {

    /**
     * Default contructor, this must be implemented
     */
    public ExternalServiceManager() {
        super();
        SERVICE_UNIT_TYPE = "external_service";
    }

    /**
     * Creates a new service unit manager
     * 
     * @param context
     *                Component context of the callind component
     * @param logger
     *                Logger of the calling component
     */
    public ExternalServiceManager(ComponentContext context, Logger logger) {
        super(context, logger);
        SERVICE_UNIT_TYPE = "external_service";
    }

    /* (non-Javadoc)
     * @see org.objectweb.petals.component.common.serviceunitmanager.handler.PetalsServiceUnitHandler#deploy(java.lang.String, java.lang.String, java.lang.String)
     */
    public String deploy(String serviceUnitName, String serviceUnitType
            , String serviceUnitRootPath) {
        String result = null;
        if (SERVICE_UNIT_TYPE.equals(serviceUnitType)) {
            logger.log(Level.FINE, "deploy  serviceUnitName  "
                    + serviceUnitName + "serviceUnitRootPath" + serviceUnitRootPath);
            result = ManagementMessageUtil.getComponentTaskResult(context
                    .getComponentName(), "deploy",
                    ManagementMessageUtil.TASK_RESULT_SUCCESS);
        }
        return result;
    }

    public void init(String serviceUnitName, String serviceUnitRootPath)
        throws DeploymentException {

        logger.log(Level.FINE, "init  serviceUnitName  " + serviceUnitName
                + "  serviceUnitRootPath " + serviceUnitRootPath);
        
        if(serviceUnitInstallationRootPath.put(serviceUnitName, serviceUnitRootPath) != null){
            logger.log(Level.SEVERE,ERROR_DUPLICATE_SERVICE_UNIT);
            throw new DeploymentException(ERROR_DUPLICATE_SERVICE_UNIT);
        }
    }

    protected Document getServiceUnitWSDL(String suRootPath) {
        Document result = null;
        File[] files = new File(suRootPath).listFiles();
        for (File file : files) {
            if (file.getName().endsWith(".wsdl")) {
                result = WSDLHelper.createDocumentFromWSDL(file);
                break;
            }
        }
        return result;
    }
    
    public void shutDown(String serviceUnitName,
            PetalsServiceUnitManager epHandler) throws DeploymentException {
        logger.log(Level.FINE, "shutdown  serviceUnitName  " + serviceUnitName);
    }

    public void start(String serviceUnitName, PetalsServiceUnitManager epHandler)
        throws DeploymentException {
        logger.log(Level.FINE, "start  serviceUnitName  " + serviceUnitName);
        String suRootPath = serviceUnitInstallationRootPath.get(serviceUnitName); 
        Document serviceDesc = getServiceUnitWSDL(suRootPath);
        if (serviceDesc != null) {
            try {
                activateEndpointsFromJBIDescription(epHandler, serviceDesc,
                        serviceUnitName,suRootPath);
            } catch (Exception ex) {
                logger.log(Level.SEVERE, FAILED_ACTIVATE_ENDPOINT + ex);
                throw new DeploymentException(FAILED_ACTIVATE_ENDPOINT, ex);
            }
        } else {
            throw new DeploymentException(INCOMPLETE_SERVICE_UNIT_PACKAGE);
        }
    }

    public void stop(String serviceUnitName, PetalsServiceUnitManager epHandler)
        throws DeploymentException {
        try {
            serviceUnitInstallationRootPath.remove(serviceUnitName);
            deactivateEndpointsFromJBIDescription(serviceUnitName, epHandler);
        } catch (Exception e) {
            throw new DeploymentException(FAILED_DEACTIVATE_ENDPOINT,e);
        }
        logger.log(Level.FINE, "stop  serviceUnitName  " + serviceUnitName);
    }

    public String undeploy(String serviceUnitName, String serviceUnitRootPath)
        throws DeploymentException {
        return ManagementMessageUtil.getComponentTaskResult(context
                .getComponentName(), "undeploy",
                ManagementMessageUtil.TASK_RESULT_SUCCESS);
    }

}
