/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: InstallationService.java 154 2006-03-27 15:30:10Z wjoseph $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.component.common.serviceunitmanager.handler;

import java.io.File;
import java.util.HashMap;
import java.util.Map;
import java.util.logging.Logger;

import javax.jbi.JBIException;
import javax.jbi.component.ComponentContext;
import javax.jbi.management.DeploymentException;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.xml.namespace.QName;

import org.objectweb.petals.component.common.serviceunitmanager.manager.PetalsServiceUnitManager;
import org.objectweb.petals.component.common.util.WSDLHelper;
import org.objectweb.petals.component.common.util.XMLHelper;
import org.objectweb.petals.tools.jbicommon.descriptor.JBIDescriptor;
import org.w3c.dom.Document;
import org.w3c.dom.Node;

/**
 * This is the abstract class of all handlers managed by the
 * PetalsServiceUnitManager
 * 
 * 
 * @version $Rev: 250 $ $Date: 2006-04-21 14:20:57 +0200 (ven, 21 avr 2006) $
 * @since Petals 1.0
 * @author wjoseph - EBM WebSourcing
 * 
 */
public abstract class PetalsServiceUnitHandler {

    protected final static String SERVICE_UNIT_PROPERTY_FILE = "serviceUnit.xml";

    protected static final String FAILED_ACTIVATE_ENDPOINT = "Activate new endpoint failed ";

    protected static final String FAILED_DEACTIVATE_ENDPOINT = "Deactivate endpoint failed ";

    protected static final String INCOMPLETE_SERVICE_UNIT_PACKAGE = "The deployed service package is incomplete ";
    
    protected static final String INVALID_SERVICE_DESCRIPTION = "The provided service description is not valid ";
    
    protected static final String ERROR_DUPLICATE_SERVICE_UNIT = "A service unit with the given name is already registered ";

    protected ComponentContext context;

    protected Logger logger;

    protected String SERVICE_UNIT_TYPE;

    protected JBIDescriptor descriptor;
    
    protected Map<String,String> serviceUnitInstallationRootPath = new HashMap<String, String>();

    public PetalsServiceUnitHandler(){
    }
            
    
    public PetalsServiceUnitHandler(ComponentContext context,
            Logger logger) {
        this.context = context;
        this.logger = logger;
    }
    
   
    /**
     * Deploy a service unit. Before deploying, the service unit type if verified
     * @param serviceUnitName name of the service unit to deploy
     * @param serviceUnitType type of the service unit to deploy
     * @param serviceUnitRootPath root path of the service unit to deploy
     * @return a valid management message
     * @throws DeploymentException If an error occurs during deployment
     */
    abstract public String deploy(String serviceUnitName,
            String serviceUnitType, String serviceUnitRootPath) throws DeploymentException;

    public ComponentContext getContext() {
        return context;
    }

    public Logger getLogger() {
        return logger;
    }

    public String getSERVICE_UNIT_TYPE() {
        return SERVICE_UNIT_TYPE;
    }

    /* (non-Javadoc)
     * @see org.objectweb.petals.component.common.serviceunitmanager.handler.PetalsServiceUnitHandler#init(java.lang.String, java.lang.String)
     */
    abstract public void init(String serviceUnitName, String serviceUnitRootPath)
        throws DeploymentException;

    /* (non-Javadoc)
     * @see org.objectweb.petals.component.common.serviceunitmanager.handler.PetalsServiceUnitHandler#setContext(javax.jbi.component.ComponentContext)
     */
    public void setContext(ComponentContext context) {
        this.context = context;
    }

    /* (non-Javadoc)
     * @see org.objectweb.petals.component.common.serviceunitmanager.handler.PetalsServiceUnitHandler#setLogger(java.util.logging.Logger)
     */
    public void setLogger(Logger logger) {
        this.logger = logger;
    }

    public void setSERVICE_UNIT_TYPE(String service_unit_type) {
        SERVICE_UNIT_TYPE = service_unit_type;
    }

    /* (non-Javadoc)
     * @see org.objectweb.petals.component.common.serviceunitmanager.handler.PetalsServiceUnitHandler#shutDown(java.lang.String, org.objectweb.petals.component.common.serviceunitmanager.manager.PetalsServiceUnitManager)
     */
    abstract public void shutDown(String serviceUnitName,
            PetalsServiceUnitManager epHandler) throws DeploymentException;
    
    /* (non-Javadoc)
     * @see org.objectweb.petals.component.common.serviceunitmanager.handler.PetalsServiceUnitHandler#start(java.lang.String, org.objectweb.petals.component.common.serviceunitmanager.manager.PetalsServiceUnitManager)
     */
    abstract public void start(String serviceUnitName,
            PetalsServiceUnitManager epHandler) throws DeploymentException;

    /* (non-Javadoc)
     * @see org.objectweb.petals.component.common.serviceunitmanager.handler.PetalsServiceUnitHandler#stop(java.lang.String, org.objectweb.petals.component.common.serviceunitmanager.manager.PetalsServiceUnitManager)
     */
    abstract public void stop(String serviceUnitName,
            PetalsServiceUnitManager epHandler) throws DeploymentException;

    /* (non-Javadoc)
     * @see org.objectweb.petals.component.common.serviceunitmanager.handler.PetalsServiceUnitHandler#undeploy(java.lang.String, java.lang.String)
     */
    abstract public String undeploy(String serviceUnitName,
            String serviceUnitRootPath) throws DeploymentException;

    /**
     * Activate an endpoint in Petals platform, put the activate endpoint add the service description of this endpoint in HashMap,
     * add the service unit name and the endpoint in a HashMap and add the interface QName and the endpoint in a HashMap 
     * @param epHandler the component service unit manager
     * @param serviceDesc the service unit service description
     * @param serviceUnitName the service unit name
     * @param suRootPath the service unit root path
     * @throws Exception If errors occur during processing
     */
    protected void activateEndpointsFromJBIDescription(
            PetalsServiceUnitManager epHandler, Document serviceDesc,
            String serviceUnitName, String suRootPath) throws Exception {
        File jbiXmlFile = new File(suRootPath + File.separator
                + "META-INF" + File.separator + "jbi.xml");
        /*JBIDescriptorBuilder descriptorBuilder = new JBIDescriptorBuilder(
                jbiXmlFile.toURI(), null);
        JBIDescriptor descriptor = descriptorBuilder.build();
        Services services = descriptor.getServices();
        for (Provides provides : services.getProvides()) {*/
        /*
         * TODO here only the first provides node is activated, if there is more problem
         */ 
            QName interfaceName = getInterfaceNameFromJbiXml(jbiXmlFile);
            String endpointName = getEndpointNameFromJbiXml(jbiXmlFile);
            QName serviceName = getServiceNameFromJbiXml(jbiXmlFile);
            ServiceEndpoint ep = context.activateEndpoint(serviceName,
                    endpointName);
            epHandler.setServiceDescription(ep, serviceDesc);
            epHandler.setServiceUnitEndpoint(serviceUnitName, ep);
            epHandler.addServiceForInterface(interfaceName, ep);
            epHandler.addSUHandlerForEndpoint(ep, this);
        //}
    }
    
    /**
     * Deactivate an endpoint in Petals platform, remove the activate endpoint from various HashMap in which it was registered
     * @param epHandler the component service unit manager
     * @param ep the service endpoint to be deactivated
     * @throws Exception If errors occur during processing
     */
    protected void deactivateEndpointsFromJBIDescription( String serviceUnitName,
            PetalsServiceUnitManager epHandler) throws JBIException {
            ServiceEndpoint ep = epHandler.getServiceUnitEndpoint(serviceUnitName);
            if(ep != null) {
                context.deactivateEndpoint(ep);
                epHandler.removeServiceDescription(ep);
                epHandler.removeServiceUnitEndpoint(serviceUnitName);
                epHandler.removeServiceForInterface(ep);
                epHandler.removeSUHandlerForEndpoint(ep);
            }
    }
    
    /**
     * Return the endpoint name parse from the jbi descriptor file
     * @param jbiXml jbi descriptor file
     * @return the endpoint name if found, else null
     */
    protected String getEndpointNameFromJbiXml(File jbiXml){
        String result = new String();
        Document jbiDoc = WSDLHelper.createDocumentFromWSDL(jbiXml);
        Node providesNode = XMLHelper.findChild(jbiDoc, "provides", true);
        result = XMLHelper.getAttributeValue(providesNode, "endpoint-name");
        return result;
    }
    
    /**
     * Return the interface QName parse from the jbi descriptor file
     * @param jbiXml jbi descriptor file
     * @return the interface QName if found, else null
     */
    protected QName getInterfaceNameFromJbiXml(File jbiXml){
        QName result = null;
        Document jbiDoc = WSDLHelper.createDocumentFromWSDL(jbiXml);
        Node providesNode = XMLHelper.findChild(jbiDoc, "provides", true);
        result = QName.valueOf(XMLHelper.getAttributeValue(providesNode, "interface-name"));
        return result;
    }

    /**
     * Return the service QName parse from the jbi descriptor file
     * @param jbiXml jbi descriptor file
     * @return the service QName if found, else null
     */
    protected QName getServiceNameFromJbiXml(File jbiXml){
        QName result = null;
        Document jbiDoc = WSDLHelper.createDocumentFromWSDL(jbiXml);
        Node providesNode = XMLHelper.findChild(jbiDoc, "provides", true);
        result = QName.valueOf(XMLHelper.getAttributeValue(providesNode, "service-name"));
        return result;
    }

}
