/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2006 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: Axis2BC.java 154 25 sept. 06 alouis $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.binding.axis2;

import java.io.File;
import java.util.logging.Level;

import javax.jbi.JBIException;

import org.apache.axis2.AxisFault;
import org.apache.axis2.context.ConfigurationContext;
import org.apache.axis2.context.ConfigurationContextFactory;
import org.objectweb.petals.binding.axis2.listener.jbi.JBIListener;
import org.objectweb.petals.binding.axis2.listener.soap.SOAPListenerManager;
import org.objectweb.petals.component.common.bc.AbstractBindingComponent;
import org.objectweb.petals.component.common.su.ServiceUnitListener;
import org.objectweb.petals.component.common.su.SimpleServiceUnitManager;

/**
 * The Axis2 Component.
 * 
 * @author alouis - EBM Websourcing
 * 
 */
public class Axis2BC extends AbstractBindingComponent {

    /**
     * The listener processing request providing from the NMR
     */
    protected JBIListener listener;

    /**
     * The listener processing SOAP request providing from Petals outside.
     */
    protected SOAPListenerManager externalListenerManager;

    // //////////////////////////////////////////////////////:
    // Component implementation
    // //////////////////////////////////////////////////////:

    @Override
    public void start() throws JBIException {
        super.start();
        try {
            externalListenerManager.start();
        } catch (AxisFault e) {
            getLogger().severe(e.getMessage());
            throw new JBIException(e);
        }
    }

    @Override
    public void stop() throws JBIException {
        super.stop();
        try {
            externalListenerManager.stop();
        } catch (AxisFault e) {
            getLogger().severe(e.getMessage());
            throw new JBIException(e);
        }
    }

    @Override
    protected void init(BindingComponentInitializer initializer) {

        // get the BindingSUManager
        SimpleServiceUnitManager bindingSUM = (SimpleServiceUnitManager) getServiceUnitManager();

        // get axis configuration context
        ConfigurationContext axisConfigurationContext = null;
        try {
            axisConfigurationContext = ConfigurationContextFactory
                .createConfigurationContextFromFileSystem(getContext()
                    .getInstallRoot(), getContext().getInstallRoot()
                    + File.separator + "axis2.xml");
        } catch (AxisFault e) {
            getLogger()
                .log(
                    Level.INFO,
                    "Can not create AXIS configuration context. "
                        + "Please verify that the axis2.xml file is in your component install root path !");
        }

        // set service unit listener
        ServiceUnitListener suListener = new Axis2BCSUListener(
            axisConfigurationContext, getContext(), getLogger());
        setServiceUnitListener(suListener);

        // create internal listener, will handle internal WS queries to external
        // services
        listener = new JBIListener(bindingSUM, getLogger());

        // create external listener, will handle incoming SOAP queries
        externalListenerManager = new SOAPListenerManager(getContext(),
            getChannel(), bindingSUM, axisConfigurationContext, getLogger());

        initializer.setExternalListenerManager(externalListenerManager);
        initializer.setJbiListener(listener);

    }
}
