package org.objectweb.petals.binding.axis2;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Properties;

import javax.jbi.JBIException;
import javax.jbi.component.InstallationContext;

import org.objectweb.petals.component.common.SimpleBootstrap;
import org.objectweb.petals.tools.jbicommon.util.XMLUtil;
import org.w3c.dom.DocumentFragment;
import org.w3c.dom.Node;

public class Axis2Bootstrap extends SimpleBootstrap {

    InstallationContext installContext = null;

    /*
     * non-Javadoc)
     * 
     * @see org.objectweb.petals.component.common.basic.AbstractBasicComponent#init(javax.jbi.component.InstallationContext)
     */
    @Override
    public void init(InstallationContext installContext) throws JBIException {
        this.installContext = installContext;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.petals.component.common.basic.AbstractBasicComponent#cleanUp()
     */
    @Override
    public void cleanUp() throws JBIException {
        installContext = null;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.petals.component.common.basic.AbstractBasicComponent#onInstall()
     */
    @Override
    public void onInstall() throws JBIException {

        // install modules and services to correct directories
        copyAxisConfFiles();

        // retrieve the specified htmlPort for http server
        String htmlPort = retrieveHttpServerPort();

        if (htmlPort == null) {
            htmlPort = "8080";
        }

        // store htmlPort for later execution
        Properties p = new Properties();
        p.put("axis2.server.port", htmlPort);

        File propertieFile = new File(installContext.getInstallRoot()
            + File.separator + "axis2.properties");
        try {
            FileOutputStream fos = new FileOutputStream(propertieFile);
            p.store(fos, "html port used to start the axis2 http server");
        } catch (IOException e) {
            throw new JBIException(e);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.petals.component.common.basic.AbstractBasicComponent#onUninstall()
     */
    @Override
    public void onUninstall() throws JBIException {
    }

    /**
     * Axis2 needs a repository composed of a "modules" and "services"
     * directory. Modules (.mar) must be copies into it
     */
    protected void copyAxisConfFiles() {
        File installPath = new File(installContext.getInstallRoot());

        File modules = new File(installPath, "modules");
        File services = new File(installPath, "services");

        modules.mkdirs();
        services.mkdirs();

        // copy .mar files to the modules directory

        File[] metaInfFiles = new File(installPath, "META-INF").listFiles();
        for (File file : metaInfFiles) {
            if (file.getName().endsWith(".mar")) {
                file.renameTo(new File(modules, file.getName()));
            }
            if (file.getName().equals("axis2.xml")) {
                file.renameTo(new File(installPath, file.getName()));
            }
        }
    }

    /**
     * Retrieve from the jbi.xml extension the HTML port to be used for Axis2
     * server. The port is defined in the installation context (jbi.xml file)
     * like this :
     * 
     * <code><petals-extensions:httpPort>8084</petals-extensions:httpPort></code>
     * 
     * @return axis2 html port, null if no one is specified
     */
    protected String retrieveHttpServerPort() {
        String result = null;
        DocumentFragment ext = installContext
            .getInstallationDescriptorExtension();
        Node httpPort = ext.getFirstChild();
        if (httpPort != null && httpPort.getNodeName().contains("http")) {
            result = XMLUtil.getTextContent(httpPort);
        }
        return result;
    }

}
