/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2006 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: SOAPHelper.java 154 25 sept. 06 wjoseph $
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.binding.axis2;

import java.util.Set;

import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.xml.namespace.QName;

import org.apache.axiom.attachments.Attachments;
import org.apache.axis2.AxisFault;
import org.apache.axis2.context.MessageContext;
import org.apache.axis2.description.AxisService;

/**
 * SOAP helper
 * 
 * @author Christophe HAMERLING - eBMWebSourcing
 * 
 */
public class SOAPHelper {

    /**
     * 
     */
    private SOAPHelper() {
    }

    /**
     * Transform an axis service name into a qname. (check if the
     * targetnamespace is part of the name of the service or as a real
     * targetnamespace
     * 
     * @param service
     *            can be null
     * @return
     */
    public static QName getServiceName(AxisService service) {

        QName name = null;

        if (service != null) {
            if (service.getName().startsWith("{")) {
                name = new QName(service.getName());
            } else {
                name = new QName(service.getTargetNamespace(), service
                    .getName());
            }
        }
        return name;
    }

    /**
     * Add the attachements from the incoming soap request to the Normalized
     * message
     * 
     * @param messageContext
     * @param nm
     */
    public static void copyAttachmentsFromSOAPToJBI(
        MessageContext messageContext, NormalizedMessage nm)
        throws MessagingException {

        if (nm != null && messageContext != null) {
            Attachments attachments = messageContext.getAttachmentMap();
            if (attachments != null) {
                String[] ids = attachments.getAllContentIDs();
                for (String id : ids) {
                    nm.addAttachment(id, attachments.getDataHandler(id));
                }
            }
        }
    }

    /**
     * Add the normalized message attachments to the Axis message context.
     * 
     * @param nm
     * @param messageContext
     * 
     * @throws AxisFault
     */
    public static void copyAttachmentsFromJBIToSOAP(NormalizedMessage nm,
        MessageContext messageContext) {
        if (nm != null && messageContext != null) {
            Set attachmentNames = nm.getAttachmentNames();
            for (Object id : attachmentNames) {
                String name = (String) id;
                messageContext.addAttachment(name, nm.getAttachment(name));
            }
        }
    }

}
