/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2006 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: Axis2ReceiverUtil.java 154 19 mai 2006 ofabre $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.binding.axis2.listener.soap;

import java.io.ByteArrayOutputStream;
import java.util.Iterator;

import javax.jbi.messaging.DeliveryChannel;
import javax.jbi.messaging.InOnly;
import javax.jbi.messaging.InOut;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.xml.namespace.QName;
import javax.xml.transform.Source;

import org.apache.axiom.om.OMElement;
import org.apache.axiom.om.OMNamespace;
import org.apache.axiom.soap.SOAPBody;
import org.apache.axiom.soap.SOAPEnvelope;
import org.apache.axis2.context.MessageContext;
import org.apache.axis2.description.AxisService;

import org.objectweb.petals.binding.axis2.SOAPHelper;
import org.objectweb.petals.component.common.util.SourceHelper;

/**
 * 
 * @author ofabre - EBM Websourcing
 * 
 */
public class Axis2ReceiverUtil {

    /**
     * Create a new Source object containing the axis message Soap envelope
     * 
     * @param message
     *            the axis message
     * @return the source content
     */
    public Source createSourceContent(MessageContext message) throws Exception {
        String source = getBodyElement(message);
        return SourceHelper.createSource(source);
    }

    /**
     * Create a new String containing the normalized message content
     * 
     * @param message
     *            the normalized message
     * @return the String content
     * @throws Exception
     */
    public String createStringContent(NormalizedMessage message)
        throws Exception {
        return SourceHelper.createString(message.getContent());
    }

    @SuppressWarnings("unchecked")
    public String getBodyElement(MessageContext message) throws Exception {
        String result = new String();
        SOAPEnvelope env = message.getEnvelope();
        SOAPBody body = env.getBody();
        OMNamespace namespace = env.getNamespace();
        Iterator<OMNamespace> envNS = env.getAllDeclaredNamespaces();
        Iterator<OMNamespace> bodyNS = body.getAllDeclaredNamespaces();
        OMElement element = body.getFirstElement();
        if (element != null) {
            element.declareNamespace(namespace);
            while (envNS.hasNext()) {
                element.declareNamespace(envNS.next());
            }
            while (bodyNS.hasNext()) {
                element.declareNamespace(bodyNS.next());
            }
            ByteArrayOutputStream os = new ByteArrayOutputStream();
            element.serialize(os);
            result = os.toString();
        }
        return result;
    }

   
    /**
     * Return the operation QName of the requested axis operation
     * 
     * @param message
     *            the Axis message
     * @return the operation QName, null if there's no Operation setted in the
     *         message
     */
    public QName getOperationName(MessageContext message) {
        return message.getOperationContext().getAxisOperation().getName();
    }

    /**
     * Return the service name of the requested axis service
     * 
     * @param message
     *            the Axis message
     * @return the service name, null if there's no service setted in the
     *         message
     */
    public QName getServiceName(MessageContext message) {
        QName name = null;
        AxisService axisService = message.getServiceContext().getAxisService();
        name = SOAPHelper.getServiceName(axisService);
        return name;
    }

   
    /**
     * Create a new MessageExchange with the given Operation, Service and
     * content. MEP is InOnly if inOnly boolean is set to true, InOut if it's
     * set to false
     * 
     * @param channel 
     *            the deliveryChannel to use
     * @param operationName
     *            the targeted operation QName
     * @param serviceName
     *            the targeted service QName
     * @param content
     *            the Source content
     * @param inOnly
     *            boolean true --> MEP=InOnly, false --> MEP=InOut
     * @return a new MessageExchange, must be non null
     * @throws MessagingException
     *             if MessageExchange and NormalizedMessage can't be created or
     *             filled or if delivery channel can't be retrieved
     */
    protected MessageExchange createMessageExchange(DeliveryChannel channel, QName operationName,
            QName serviceName, Source content, boolean inOnly)
        throws MessagingException {
        MessageExchange messageExchange = null;

        if (channel != null) {
            if (inOnly) {
                messageExchange = channel.createExchangeFactory()
                        .createInOnlyExchange();
            } else {
                messageExchange = channel.createExchangeFactory()
                        .createInOutExchange();
            }
            NormalizedMessage nm = messageExchange.createMessage();
            nm.setContent(content);
            if (inOnly) {
                ((InOnly) messageExchange).setInMessage(nm);
            } else {
                ((InOut) messageExchange).setInMessage(nm);
            }
            messageExchange.setService(serviceName);
            messageExchange.setOperation(operationName);
        } else {
            throw new MessagingException(
                    "Can't retrieve the delivery channel of the component");
        }

        return messageExchange;
    }
}
