/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.binding.axis2.listener.soap;

import java.net.SocketException;

import javax.xml.namespace.QName;

import org.apache.axis2.AxisFault;
import org.apache.axis2.Constants;
import org.apache.axis2.addressing.EndpointReference;
import org.apache.axis2.context.ConfigurationContext;
import org.apache.axis2.description.Parameter;
import org.apache.axis2.description.TransportInDescription;
import org.apache.axis2.engine.ListenerManager;
import org.apache.axis2.transport.TransportListener;
import org.apache.axis2.transport.http.server.HttpUtils;
import org.mortbay.jetty.Server;
import org.mortbay.jetty.handler.ContextHandlerCollection;
import org.mortbay.jetty.servlet.Context;
import org.mortbay.jetty.servlet.ServletHolder;

/**
 * Create a Jetty server that will be the axis2 transportlistener. This Jetty
 * server is used to replace the SimpleHTTPServer provided by axis that have
 * quite bad performances.
 * 
 * @author Christophe Hamerling - EBM WebSourcing
 * 
 */
public class AxisJettyServer implements TransportListener {

    /**
     * The Jetty server
     */
    private Server server;

    /**
     * The host address
     */
    private String hostAddress = null;

    /**
     * The server port
     */
    protected int port;

    /**
     * The axis2 configuration context (built from axis2.xml config file)
     */
    protected ConfigurationContext configurationContext;

    /**
     * Creates a new instance of AxisJettyServer
     * 
     * @param configurationContext
     * @param port
     */
    public AxisJettyServer(ConfigurationContext configurationContext, int port)
        throws AxisFault {
        this.port = port;
        this.configurationContext = configurationContext;
        this.server = new Server(this.port);
        
        // create context handlers
        ContextHandlerCollection contexts = new ContextHandlerCollection();
        server.setHandler(contexts);

        // create the axis context
        Context axis2 = new Context(contexts, "/axis2", Context.SESSIONS);

        // create servlet holders
        ServletHolder axisServlet = new ServletHolder(new AxisJettyServlet(
            configurationContext));
        axisServlet.setName("AxisServlet");
        axisServlet.setInitOrder(1);

        // REST servlet holder
        /*
         * ServletHolder axisRESTServlet = new ServletHolder(
         * org.apache.axis2.transport.http.AxisRESTServlet.class);
         * axisRESTServlet.setName("AxisRESTServlet");
         */

        // admin servlet holder
        /*
         * ServletHolder axisAdminServlet = new ServletHolder(
         * org.apache.axis2.transport.http.AxisAdminServlet.class);
         * axisAdminServlet.setName("AxisAdminServlet");
         */

        // do servlet mappings
        axis2.addServlet(axisServlet, "/services/*");
        // axis2.addServlet(axisRESTServlet, "/rest/*");
        axis2.addServlet(axisServlet, "/servlet/AxisServlet");
        // axis2.addServlet(axisAdminServlet, "/axis2-admin/*");

        // AXIS
        ListenerManager listenerManager = configurationContext
            .getListenerManager();
        TransportInDescription trsIn = new TransportInDescription(new QName(
            Constants.TRANSPORT_HTTP));
        trsIn.setReceiver(this);
        if (listenerManager == null) {
            listenerManager = new ListenerManager();
            listenerManager.init(configurationContext);
        }
        listenerManager.addListener(trsIn, true);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.apache.axis2.transport.TransportListener#start()
     */
    public void start() throws AxisFault {
        // start the jetty server in a separate thread
        Runnable httpListenerRunnable = new Runnable() {
            public void run() {
                try {
                    server.start();
                } catch (Exception e) {
                    e.printStackTrace(System.err);
                }
            }
        };
        Thread httpThread = new Thread(httpListenerRunnable,
            "Jetty started Thread");

        try {
            httpThread.start();
            httpThread.join();
        } catch (InterruptedException e) {
            throw new AxisFault(e.getMessage());
        } catch (Exception e) {
            throw new AxisFault(e.getMessage());
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.apache.axis2.transport.TransportListener#stop()
     */
    public void stop() throws AxisFault {
        try {
            // stop jetty server
            server.stop();
        } catch (InterruptedException e) {
            throw AxisFault.makeFault(e);
        } catch (Exception e) {
            throw AxisFault.makeFault(e);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.apache.axis2.transport.TransportListener#init(org.apache.axis2.context.ConfigurationContext,
     *      org.apache.axis2.description.TransportInDescription)
     */
    public void init(ConfigurationContext axisConf,
        TransportInDescription transprtIn) throws AxisFault {
        try {
            this.configurationContext = axisConf;

            Parameter param = transprtIn.getParameter(PARAM_PORT);

            if (param != null) {
                this.port = Integer.parseInt((String) param.getValue());
            }
            param = transprtIn.getParameter(HOST_ADDRESS);
            if (param != null) {
                hostAddress = ((String) param.getValue()).trim();
            }
        } catch (Exception e) {
            throw AxisFault.makeFault(e);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.apache.axis2.transport.TransportListener#getEPRsForService(java.lang.String,
     *      java.lang.String)
     */
    public EndpointReference[] getEPRsForService(String serviceName, String ip)
        throws AxisFault {
        // if host address is present
        if (hostAddress != null) {
            if (server != null) {
                return new EndpointReference[] {new EndpointReference(
                    hostAddress + "/"
                        + configurationContext.getServiceContextPath() + "/"
                        + serviceName)};
            } else {
                throw new AxisFault(
                    "Unable to generate EPR for the transport : http");
            }
        }
        // if the host address is not present
        String localAddress;
        if (ip != null) {
            localAddress = ip;
        } else {
            try {
                localAddress = HttpUtils.getIpAddress();
                if (localAddress == null) {
                    localAddress = "127.0.0.1";
                }
            } catch (SocketException e) {
                throw AxisFault.makeFault(e);
            }
        }
        if (server != null) {
            return new EndpointReference[] {new EndpointReference("http://"
                + localAddress + ":" + port + "/"
                + configurationContext.getServiceContextPath() + "/"
                + serviceName)};
        } else {
            throw new AxisFault(
                "Unable to generate EPR for the transport : http");
        }
    }

    /**
     * 
     * @return
     */
    public ConfigurationContext getConfigurationContext() {
        return this.configurationContext;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.apache.axis2.transport.TransportListener#getEPRsForService(java.lang.String,
     *      java.lang.String)
     */
    public EndpointReference getEPRForService(String serviceName, String ip)
        throws AxisFault {
        return getEPRsForService(serviceName, ip)[0];
    }
}
