/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.binding.axis2.listener.soap;

import java.io.IOException;
import java.io.OutputStream;

import javax.jbi.component.ComponentContext;
import javax.jbi.servicedesc.ServiceEndpoint;

import org.apache.axis2.AxisFault;
import org.apache.axis2.description.AxisService;
import org.apache.axis2.engine.AxisConfiguration;
import org.objectweb.petals.component.common.util.WSDLHelper;
import org.w3c.dom.Document;

/**
 * The {@link PetalsAxisService} extends an {@link AxisService}. It is
 * registered during SU deployment into the {@link AxisConfiguration}.
 * 
 * This class override the printWSDL method to dynamically generate the WSDL
 * from the JBI container.
 * 
 * @author Christophe HAMERLING - eBMWebSourcing
 * 
 */
public class PetalsAxisService extends AxisService {

    private ComponentContext componentContext;

    private ServiceEndpoint endpoint;

    /**
     * Creates a new instance of {@link PetalsAxisService}
     * 
     * @param name
     * @param componentContext
     * @param endpoint
     */
    public PetalsAxisService(String name, ComponentContext componentContext,
        ServiceEndpoint endpoint) {
        super(name);
        this.componentContext = componentContext;
        this.endpoint = endpoint;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.apache.axis2.description.AxisService#printWSDL(java.io.OutputStream,
     *      java.lang.String, java.lang.String)
     */
    @Override
    public void printWSDL(OutputStream out, String requestIP, String servicePath)
        throws AxisFault {
        try {
            Document document = componentContext
                .getEndpointDescriptor(endpoint);
            if (document == null) {
                printWSDLError(out);
            } else {
                WSDLHelper.printWSDL(document, out);
            }
        } catch (Exception e) {
            throw new AxisFault(e);
        }
    }
    
    /* (non-Javadoc)
     * @see org.apache.axis2.description.AxisService#printWSDL2(java.io.OutputStream, java.lang.String, java.lang.String)
     */
    @Override
    public void printWSDL2(OutputStream out, String requestIP, String servicePath) throws AxisFault {
        printWSDL(out, requestIP, servicePath);
    }

    /**
     * Print the WSDL error on the output stream
     * 
     * @param out
     * @throws AxisFault
     */
    private void printWSDLError(OutputStream out) throws AxisFault {
        try {
            String wsdlntfound = "<error>"
                + "<description>Unable to get WSDL for this service</description>"
                + "<reason>The PEtALS endpoint did not returns its descriptor. </reason>"
                + "</error>";
            out.write(wsdlntfound.getBytes());
            out.flush();
            out.close();
        } catch (IOException e) {
            throw new AxisFault(e);
        }
    }
}
