/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2006 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: Axis2BCListener.java 154 19 avr. 2006 wjoseph $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.binding.axis2.listener.soap;

import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jbi.component.ComponentContext;
import javax.jbi.messaging.DeliveryChannel;
import javax.xml.namespace.QName;

import org.apache.axis2.AxisFault;
import org.apache.axis2.addressing.AddressingConstants;
import org.apache.axis2.addressing.EndpointReference;
import org.apache.axis2.context.MessageContext;
import org.apache.axis2.description.AxisOperation;
import org.apache.axis2.description.AxisService;
import org.apache.axis2.description.HandlerDescription;
import org.apache.axis2.description.InOutAxisOperation;
import org.apache.axis2.engine.AbstractDispatcher;
import org.objectweb.petals.component.common.su.SimpleServiceUnitManager;
import org.objectweb.petals.tools.jbicommon.descriptor.Consumes;

/**
 * Dispatcher for JBI container. This dispatcher is used to return the service
 * that has been registered on SU deployment.
 * 
 * @version $Rev: 250 $ $Date: 2006-04-21 14:20:57 +0200 (ven, 21 avr 2006) $
 * @since Petals 1.0
 * @author alouis,wjoseph,chamerling - EBM Websourcing
 * 
 */
public class PetalsDispatcher extends AbstractDispatcher implements
    AddressingConstants {

    protected Logger log = null;

    protected ComponentContext componentContext;

    protected DeliveryChannel channel;

    protected SimpleServiceUnitManager bindingSUM;

    protected PetalsReceiver jbiReceiver;

    protected AxisService jbiService;

    protected AxisOperation jbiOperation;

    public static final String NAME = "PetalsDispatcher";

    private static final long serialVersionUID = 10983L;

    protected static final QName OPERATION_QNAME = QName
        .valueOf("PetalsGenericOperation");

    /**
     * Init the PetalsDispatcher after it has been created by Axis.
     * 
     * @param componentContext
     * @param channel
     * @param bindingSUM
     * @param log
     */
    public void init(ComponentContext componentContext,
        DeliveryChannel channel, SimpleServiceUnitManager bindingSUM, Logger log) {
        this.bindingSUM = bindingSUM;
        this.componentContext = componentContext;
        this.log = log;
        this.channel = channel;

        this.jbiService = new AxisService();
        this.jbiReceiver = new PetalsReceiver(componentContext, channel,
            bindingSUM, log);
        this.jbiOperation = new InOutAxisOperation(OPERATION_QNAME);
        this.jbiOperation.setMessageReceiver(jbiReceiver);
        this.jbiService.addOperation(jbiOperation);
    }

    /**
     * Called by Axis Engine to find the operation. TODO : try to retrieve the
     * service description of the JBI service engine and search this description
     * for an operation with the same name
     * 
     * @param service
     *            The service for which we search the operation
     * @param messageContext
     *            Current MessageContext
     * @return Returns an AxisOperation if found in the service description file
     *         or else null.
     * @throws AxisFault
     */
    public AxisOperation findOperation(AxisService service,
        MessageContext messageContext) throws AxisFault {
        return jbiOperation;
    }

    /**
     * Called by Axis Engine to find the service. Asks the JBI container if
     * there is a service registered with the name of the computed called
     * service.
     * 
     * @param messageContext
     *            Current Messagecontext
     * @return Returns an AxisService if found on the JBI container or else
     *         null.
     * @throws AxisFault
     */
    public AxisService findService(MessageContext messageContext)
        throws AxisFault {
        AxisService axisService = null;
        String serviceName = null;

        // Get Service name using endpoint
        EndpointReference toEPR = messageContext.getTo();
        log.log(Level.FINE,
            "PetalsDispatcher - Checking for Service using target endpoint address : "
                + toEPR.getAddress());
        String filePart = toEPR.getAddress();
        serviceName = extractWebServiceName(filePart);

        // Retrieve the JBI ServiceEndpoint related to the request serviceName
        Consumes consumes = bindingSUM.getConsumesFromAddress(serviceName);
        if (consumes != null) {
            // try to find if the axisService has already been registered
            axisService = messageContext.getConfigurationContext()
                .getAxisConfiguration().getService(serviceName);
        }
        return axisService;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.apache.axis2.engine.AbstractDispatcher#initDispatcher()
     */
    public void initDispatcher() {
        init(new HandlerDescription(NAME));
    }

    /**
     * Parse request URL to catch the service name:
     * ...server/services/[SERVICENAME]?method=...
     * 
     * @param serviceName
     * @return
     */
    protected String extractWebServiceName(String serviceName) {
        int indexOfOperation = serviceName.indexOf('?');
        indexOfOperation = (indexOfOperation > 0) ? indexOfOperation
            : serviceName.length();

        int indexOfService = serviceName.lastIndexOf('/');
        indexOfService = (indexOfService > 0) ? indexOfService + 1 : 0;

        return serviceName.substring(indexOfService, indexOfOperation);
    }
}
