/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2006 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.component.common.bc;

import org.objectweb.petals.component.common.PEtALSComponentSDKException;
import org.objectweb.petals.tools.jbicommon.descriptor.Extensions;

/**
 * This manager is used to enable a binding component to handle external service
 * call. It defines methods to register and unregister listeners for external
 * services.
 * <p>
 * This interface have to be implemented if the related binding component is
 * used to listen to incoming messages from external services, in order to
 * forward these messages to internal JBI Components.
 * 
 * @author ofabre, alouis - EBM Websourcing
 * 
 */
public interface ExternalListenerManager {

    /**
     * Start a listener for the given address.
     * 
     * @param address
     *            the external address to listen to. Not null
     * @param extensions
     *            JBI descriptor extensions. The message exchange targets a
     *            specific endpoint, this endpoint has been activated during the
     *            processing of a provides node of a service unit jbi
     *            descriptor. Theses extensions are the extensions of this
     *            consumes node. Can be null
     * @throws PEtALSComponentSDKException
     *             if an error occurs during listener starting
     */
    void startListening(String address, Extensions extensions) throws PEtALSComponentSDKException;

    /**
     * Stop a listener for the given address.
     * 
     * @param address
     *            the external address to listen to. Not null
     * @throws PEtALSComponentSDKException
     *             if an error occurs during listener starting
     */
    void stopListening(String address) throws PEtALSComponentSDKException;

}
