/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.component.common.su;

import java.util.HashMap;
import java.util.Map;

import javax.jbi.servicedesc.ServiceEndpoint;

import org.objectweb.petals.tools.jbicommon.descriptor.Consumes;
import org.objectweb.petals.tools.jbicommon.descriptor.JBIDescriptor;
import org.objectweb.petals.tools.jbicommon.descriptor.Provides;
import org.w3c.dom.Document;

/**
 * Handles useful information about a Service Unit like :<br/> - the service
 * unit description (jbi descriptor)<br/> - links between Provides nodes and
 * activated endpoints<br/> - links between activated endpoints and
 * corresponding service description<br/> - links between Consumes nodes and
 * corresponding addresses<br/> - the installation root<br/> - the service
 * unit name<br/>
 * 
 * @author ofabre
 * 
 */
public class ServiceUnitDataHandler {

    private JBIDescriptor descriptor;

    private Map<ServiceEndpoint, Provides> epJBIDesc;

    private Map<ServiceEndpoint, Document> epServiceDesc;

    private Map<String, Consumes> addressToConsumes;

    private String installRoot;

    private String name;

    public ServiceUnitDataHandler() {
        super();
        addressToConsumes = new HashMap<String, Consumes>();
        epServiceDesc = new HashMap<ServiceEndpoint, Document>();
        epJBIDesc = new HashMap<ServiceEndpoint, Provides>();
    }

    public void addEndpointDescription(final ServiceEndpoint endpoint,
        final Document description) {
        epServiceDesc.put(endpoint, description);
    }

    public void addJBIDescription(final ServiceEndpoint endpoint,
        final Provides provides) {
        epJBIDesc.put(endpoint, provides);
    }

    public JBIDescriptor getDescriptor() {
        return descriptor;
    }

    public Map<ServiceEndpoint, Provides> getEpJBIDesc() {
        return epJBIDesc;
    }

    public Map<ServiceEndpoint, Document> getEpServiceDesc() {
        return epServiceDesc;
    }

    public String getInstallRoot() {
        return installRoot;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public void removeEndpoint(final ServiceEndpoint endpoint) {
        epServiceDesc.remove(endpoint);
        epJBIDesc.remove(endpoint);
    }

    public void setDescriptor(final JBIDescriptor descriptor) {
        this.descriptor = descriptor;
    }

    public void setEpJBIDesc(final Map<ServiceEndpoint, Provides> epJBIDesc) {
        this.epJBIDesc = epJBIDesc;
    }

    public void setEpServiceDesc(
        final Map<ServiceEndpoint, Document> epServiceDesc) {
        this.epServiceDesc = epServiceDesc;
    }

    public void setInstallRoot(final String installRoot) {
        this.installRoot = installRoot;
    }

    public Map<String, Consumes> getAddressToConsumes() {
        return addressToConsumes;
    }

    public void setAddressToConsumes(
        final Map<String, Consumes> addressToConsumes) {
        this.addressToConsumes = addressToConsumes;
    }

    public void addAddressToConsumes(final String address,
        final Consumes consumes) {
        addressToConsumes.put(address, consumes);
    }

}
