/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.component.common.util;

import java.net.URI;
import java.net.URISyntaxException;
import java.util.Map;

import org.objectweb.petals.tools.jbicommon.descriptor.Extensions;
import org.objectweb.petals.tools.jbicommon.descriptor.PetalsExtension;

/**
 * 
 * @author ofabre
 * 
 */
public final class PetalsExtensionsUtil {

    /**
     * Key node names
     */
    public static final String ADDRESS = "address";

    public static final String WSDL_LOCATION = "wsdl";

    /**
     * Namespace of the extension elements in the jbi descriptor file
     */
    private static final String KEY_VALUE_NS = "http://petals.objectweb.org/extensions/key-value/";

    private PetalsExtensionsUtil() {

    }

    /**
     * Extract a value from the key value map extension
     * 
     * @param extensions
     *            the {@link Extensions} element containing Petals key value
     *            extension
     * @param key
     *            the targeted key
     * @return the {@link String} value for the given key or null
     */
    @SuppressWarnings("unchecked")
    public static String extractValueFromKeyValueExtension(
        Extensions extensions, String key) {
        String value = null;
        if (extensions != null) {
            try {
                URI uri = new URI(KEY_VALUE_NS);
                PetalsExtension petalsExtension = extensions
                    .getPetalsExtensions().get(uri);
                if (petalsExtension != null) {
                    Map<String, String> keyValue = (Map<String, String>) petalsExtension
                        .getExtensionObject();
                    value = keyValue.get(key);
                }
            } catch (URISyntaxException e) {
                // This exception should not be raised.
                // If this exception is raised, fix the value of KEY_VALUE_NS.
                e.printStackTrace();
            }
        }

        return value;
    }

    /**
     * Test if the key is present in the extensions.
     * 
     * @param extensions
     *            the {@link Extensions} element containing Petals key value
     *            extension
     * @param key
     *            the targeted key
     * 
     * @return true if the key is present (Implies that the extensions are not
     *         null), false otherwise.
     */
    public static boolean hasKeyValueExtension(Extensions extensions, String key) {
        return (extractValueFromKeyValueExtension(extensions, key) != null);
    }

}
