/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: Csv.java 1216 2006-11-14 18:27:09Z dutoo $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.engine.csv;

import java.util.HashMap;
import java.util.Properties;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jbi.JBIException;
import javax.jbi.component.Component;
import javax.jbi.component.ComponentContext;
import javax.jbi.component.ComponentLifeCycle;
import javax.jbi.component.ServiceUnitManager;
import javax.jbi.messaging.DeliveryChannel;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.management.ObjectName;
import javax.xml.namespace.QName;

import org.objectweb.petals.component.common.listener.MessageExchangeListener;
import org.objectweb.petals.component.common.serviceunitmanager.manager.PetalsServiceUnitManager;
import org.objectweb.petals.component.common.util.ComponentLogger;
import org.objectweb.petals.component.common.util.XMLHelper;

import org.w3c.dom.Document;
import org.w3c.dom.DocumentFragment;
import org.w3c.dom.Node;

/**
 * This component transforms a CSV document in an XML document. It uses the
 * JavaCSV library to do this transformation. NB. To transform the XML output,
 * use another XSLT service engine SU.
 * 
 * @version $Rev: 1068 $Date: {date}
 * @since Petals 1.0
 * @author Marc Dutoo - Open Wide
 * 
 */
public class Csv implements Component, ComponentLifeCycle {

    protected DeliveryChannel channel;

    private HashMap<String, Properties> mapEndpointCsv;

    protected PetalsServiceUnitManager suManager;

    protected ComponentContext context;

    private MessageExchangeListener listener;

    protected Logger logger;

    public ObjectName getExtensionMBeanName() {
        return null;
    }

    public ComponentLifeCycle getLifeCycle() {
        return this;
    }

    /**
     * return the wsdl description
     */
    public Document getServiceDescription(ServiceEndpoint arg0) {
        return suManager.getServiceDescription(arg0);
    }

    public ServiceUnitManager getServiceUnitManager() {
        logger.log(Level.INFO,
            "Component instance asked to provide a ServiceUnitManager.");
        return suManager;
    }

    public void init(ComponentContext ctx) throws JBIException {
        this.context = ctx;
        Logger javaLogger = ctx.getLogger("", null);
        logger = new ComponentLogger(javaLogger, javaLogger.getName(),
            javaLogger.getResourceBundleName(), ctx.getComponentName());

        logger.log(Level.INFO, "init");
        this.channel = ctx.getDeliveryChannel();
        this.mapEndpointCsv = new HashMap<String, Properties>();
    }

    public boolean isExchangeWithConsumerOkay(ServiceEndpoint arg0,
        MessageExchange arg1) {
        return true;
    }

    public boolean isExchangeWithProviderOkay(ServiceEndpoint arg0,
        MessageExchange arg1) {
        return false;
    }

    /**
     * If requested service is HelloworldService, return the endpoint
     */
    public ServiceEndpoint resolveEndpointReference(DocumentFragment arg0) {
        ServiceEndpoint result = null;

        if (arg0 != null) {
            String prefix = arg0.getPrefix();
            if (prefix == null) {
                prefix = new String();
            } else {
                prefix += ":";
            }
            Node service = XMLHelper.findChild(arg0, prefix + "service", true);

            if (service != null) {
                String serviceRequested = XMLHelper.getAttributeValue(service,
                    "name");
                if (serviceRequested != null) {
                    result = suManager.findEndpointForService(QName
                        .valueOf(serviceRequested));
                }
            }
        }
        return result;
    }

    public void shutDown() throws JBIException {
        logger.log(Level.INFO, "shutDown");
        listener = null;
    }

    public void start() throws JBIException {
        logger.log(Level.INFO, "start");
        CsvProcessor processor = new CsvProcessor(channel, logger,
            mapEndpointCsv);
        listener = new MessageExchangeListener(channel, processor,
            MessageExchangeListener.IgnoredStatus.DONE_AND_ERROR_IGNORED, 0);
        listener.listen();
        CsvSUHandler SUHandler = new CsvSUHandler(context, this.mapEndpointCsv,
            logger);
        this.suManager = new PetalsServiceUnitManager(context, logger);

        suManager.addNewHandler(SUHandler);
    }

    public void stop() throws JBIException {
        logger.log(Level.INFO, "stop");
        this.listener.terminate();
    }

}
