/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: CsvBootstrapImplTest.java 1089 2006-09-14 19:57:55Z dutoo $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.engine.csv;

import static org.easymock.EasyMock.expect;
import static org.easymock.classextension.EasyMock.createMock;
import static org.easymock.classextension.EasyMock.replay;
import static org.easymock.classextension.EasyMock.verify;

import java.io.File;

import javax.jbi.JBIException;
import javax.jbi.component.InstallationContext;

import org.objectweb.petals.engine.csv.CsvBootstrapImpl;

import junit.framework.TestCase;


/**
 * Tests CsvBootstrapImpl
 * @version	$Rev: 1069 $Date: ${date}
 * @since	Petals 1.0
 * @author Marc Dutoo - Open Wide
 *
 */
public class CsvBootstrapImplTest extends TestCase {

    File installationRoot = null;

    private String baseDir;

    
    public void testCleanUp() {
        CsvBootstrapImpl csvBootstrap = new CsvBootstrapImpl();

        // Run the Test
        try {
            csvBootstrap.cleanUp();
        } catch (JBIException e) {
            e.printStackTrace();
            fail("Error in CsvBootstrap clean up");
        }
    }

    
    public void testGetExtensionMBeanName() {
        // Create objects
        CsvBootstrapImpl csvBootstrap = new CsvBootstrapImpl();

        // Run the Test
        Object result = null;
        result = csvBootstrap.getExtensionMBeanName();
        assertNull("Result of invocation should be null", result);
    }

    
    public void testInit() {
        // Create Mock Objects
        InstallationContext installationContext = createMock(InstallationContext.class);
        // Init mocks
        expect(installationContext.getInstallRoot()).andReturn(
            installationRoot.getAbsolutePath());
        replay(installationContext);

        CsvBootstrapImpl csvBootstrap = new CsvBootstrapImpl();
        // Run the Test
        try {
            csvBootstrap.init(installationContext);
        } catch (JBIException e) {
            e.printStackTrace();
            fail("Error in CsvBootstrap init method");
        }

        assertEquals(csvBootstrap.installContext , installationContext);
    }

    
    public void testOnInstall() {
        // Create Mock Objects
        InstallationContext installationContext = createMock(InstallationContext.class);
        // Init mocks
        CsvBootstrapImpl csvBootstrap = new CsvBootstrapImpl();
        // Run the Test
        try {
            csvBootstrap.init(installationContext);
        } catch (JBIException e) {
            e.printStackTrace();
            fail("Error in CsvBootstrap onInstall method");
        }
    }

    
    public void testOnUninstall() {
        // Create Mock Objects
        InstallationContext installationContext = createMock(InstallationContext.class);
        // Init mocks
        expect(installationContext.getInstallRoot()).andReturn(
            installationRoot.getAbsolutePath()).anyTimes();
        expect(installationContext.getInstallationDescriptorExtension()).andReturn(
                null);
        replay(installationContext);

        CsvBootstrapImpl csvBootstrap = new CsvBootstrapImpl();
        // Run the Test
        try {
            csvBootstrap.init(installationContext);
            csvBootstrap.onInstall();
        } catch (JBIException e) {
            e.printStackTrace();
            fail("Error in CsvBootstrap init or onUninstall method");
        }

        // Run the Test
        try {
            csvBootstrap.onUninstall();
        } catch (JBIException e) {
            e.printStackTrace();
            fail("Error in CsvBootstrap onuninstall method");
        }
        verify(installationContext);
    }

    
    @Override
    protected void setUp() throws Exception {
        baseDir = this.getClass().getResource(".").toString();
        baseDir = baseDir.substring(0, baseDir.indexOf("target"));
        baseDir = baseDir.substring(baseDir.indexOf(":") + 1);
        installationRoot = new File(baseDir + File.separator + "src"
            + File.separator + "test-data" + File.separator + "bootstrap");
    }

}


