/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: MockDeliveryChannel.java 1216 2006-11-14 18:27:09Z dutoo $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.engine.csv.Mock;

import javax.jbi.messaging.DeliveryChannel;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessageExchangeFactory;
import javax.jbi.messaging.MessagingException;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.xml.namespace.QName;

import org.objectweb.petals.component.common.util.SourceHelper;

import static org.easymock.EasyMock.expect;
import static org.easymock.classextension.EasyMock.createMock;
import static org.easymock.classextension.EasyMock.replay;

/**
 * Mock of DeliveryChannel
 * 
 * @version $Rev: 1069 $Date: ${date}
 * @since   Petals 1.0
 * @author Marc Dutoo - Open Wide
 * 
 */
public class MockDeliveryChannel implements DeliveryChannel {

    public MessageExchange accept() throws MessagingException {
        return null;
    }

    public MessageExchange accept(long timeoutMS) throws MessagingException,
        IllegalArgumentException {
        return null;
    }

    public void close() throws MessagingException {
    }

    public MessageExchangeFactory createExchangeFactory() {
        MessageExchangeFactory messageExchangeFactory = createMock(MessageExchangeFactory.class);
        try {
            expect(messageExchangeFactory.createInOnlyExchange()).andReturn(
                new MockInOnly()).anyTimes();
            expect(messageExchangeFactory.createRobustInOnlyExchange())
                .andReturn(new MockRobustInOnly()).anyTimes();
            expect(messageExchangeFactory.createInOutExchange()).andReturn(
                new MockInOut()).anyTimes();
        } catch (MessagingException e) {
        }
        replay(messageExchangeFactory);
        return messageExchangeFactory;
    }

    public MessageExchangeFactory createExchangeFactory(QName interfaceName) {
        return null;
    }

    public MessageExchangeFactory createExchangeFactory(ServiceEndpoint endpoint) {
        return null;
    }

    public MessageExchangeFactory createExchangeFactoryForService(
        QName serviceName) {
        return null;
    }

    private boolean send;

    public boolean isSend() {
        return send;
    }

    public void setSend(boolean send) {
        this.send = send;
    }

    public void send(MessageExchange exchange) throws MessagingException {
        send = true;
    }

    private boolean sendSync;

    public boolean sendSync(MessageExchange exchange) throws MessagingException {
        if (exchange instanceof MockInOut){
            MockNormalizedMessage mockNormalizedMessage = new MockNormalizedMessage();
            mockNormalizedMessage.setContent(SourceHelper.createContentSource("test"));
            ((MockInOut) exchange).setOutMessage(mockNormalizedMessage);
        }
        sendSync = true;
        return false;
    }

    private boolean sendSyncLong;

    public boolean sendSync(MessageExchange exchange, long timeoutMS)
        throws MessagingException {
        sendSyncLong = true;
        return false;
    }

    public boolean isSendSync() {
        return sendSync;
    }

    public void setSendSync(boolean sendSync) {
        this.sendSync = sendSync;
    }

    public boolean isSendSyncLong() {
        return sendSyncLong;
    }

    public void setSendSyncLong(boolean sendSyncLong) {
        this.sendSyncLong = sendSyncLong;
    }

}
