/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * Id : EdifactMappingHandler.java 154 ${date}  rbarraza 
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.engine.edifact;

import java.io.File;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jbi.component.ComponentContext;
import javax.jbi.management.DeploymentException;

import org.ebmwebsourcing.b2bgw.edifact.dictionary.Dictionary;
import org.objectweb.petals.component.common.serviceunitmanager.handler.PetalsServiceUnitHandler;
import org.objectweb.petals.component.common.serviceunitmanager.manager.PetalsServiceUnitManager;
import org.objectweb.petals.component.common.util.ManagementMessageUtil;

/**
 * 
 * @version $Rev: 250 $Date: {date}
 * @since Petals 1.0
 * @author Rocio BARRAZA - eBMWebsourcing
 * 
 */
public class EdifactMappingHandler extends PetalsServiceUnitHandler {

    public EdifactMappingHandler(ComponentContext context, Logger logger) {
        this.context = context;
        this.logger = logger;
        SERVICE_UNIT_TYPE = "EdiMappingServiceUnit";
    }

    private EdifactMappingHandler() {
        SERVICE_UNIT_TYPE = "EdiMappingServiceUnit";
    }

    public void init(String serviceUnitName, String serviceUnitRootPath)
        throws DeploymentException {
        logger.fine("init  serviceUnitName  " + serviceUnitName);
        this.serviceUnitInstallationRootPath.put(serviceUnitName,
                serviceUnitRootPath);
    }

    public void setContext(ComponentContext context) {
        this.context = context;
    }

    public void setLogger(Logger logger) {
        this.logger = logger;
    }

    public void shutDown(String serviceUnitName,
            PetalsServiceUnitManager epHandler) throws DeploymentException {
    }

    public void start(String serviceUnitName, PetalsServiceUnitManager epHandler)
        throws DeploymentException {
        String serviceUnitRootPath = serviceUnitInstallationRootPath
                .get(serviceUnitName);

        File[] files = new File(serviceUnitRootPath).listFiles();

        for (File file : files) {
            if (file.getName().endsWith(".xml")) {
                try {
                    Dictionary.getInstance().reloadMapping(
                            file.getAbsolutePath());
                } catch (Exception e) {
                    String msg = "Error loading mapping in Edifact Dictionary";
                    logger.log(Level.SEVERE, msg);
                    throw new DeploymentException(msg, e);
                }

            }
        }
    }

    public void stop(String serviceUnitName, PetalsServiceUnitManager epHandler)
        throws DeploymentException {
        /*
         * When service unit is stopped, so properties of directory are deleted
         */
        try {
            serviceUnitInstallationRootPath.remove(serviceUnitName);
            deactivateEndpointsFromJBIDescription(serviceUnitName, epHandler);
        } catch (Exception e) {
            throw new DeploymentException(FAILED_DEACTIVATE_ENDPOINT, e);
        }
    }

    public String undeploy(String serviceUnitName, String serviceUnitRootPath)
        throws DeploymentException {
        return ManagementMessageUtil.getComponentTaskResult(context
                .getComponentName(), "undeploy",
                ManagementMessageUtil.TASK_RESULT_SUCCESS);
    }

    @Override
    public String deploy(String serviceUnitName, String serviceUnitType,
            String serviceUnitRootPath) throws DeploymentException {
        if (SERVICE_UNIT_TYPE.equals(serviceUnitType)) {
            logger.fine(" DEPLOY  serviceUnitName  " + serviceUnitName
                    + "  serviceUnitRootPath " + serviceUnitRootPath);
            return ManagementMessageUtil.getComponentTaskResult(context
                    .getComponentName(), "deploy",
                    ManagementMessageUtil.TASK_RESULT_SUCCESS);
        } else {
            return null;
        }
    }

}
