/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id:  EdifactProcessor.java
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.engine.edifact;

import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jbi.component.ComponentContext;
import javax.jbi.messaging.DeliveryChannel;
import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.Fault;
import javax.jbi.messaging.InOut;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.xml.namespace.QName;
import javax.xml.transform.Source;

import org.ebmwebsourcing.b2bgw.edifact.edifact_beans.InterchangeEDI;
import org.objectweb.petals.component.common.listener.AbstractInternalMEProcessor;
import org.objectweb.petals.component.common.util.SourceHelper;
import org.objectweb.petals.component.common.util.StringHelper;

/**
 * 
 * @version $Rev: 250 $Date: {date}
 * @since Petals 1.0
 * @author Rocio BARRAZA - eBMWebsourcing
 * 
 */
public class EdifactProcessor extends AbstractInternalMEProcessor {

    private ComponentContext ctx;

    private Map<QName, QName> endpointXslt;

    public EdifactProcessor(DeliveryChannel channel, Logger logger,
            ComponentContext context, Map<QName, QName> EndpointXslt) {
        super(channel, logger);

        this.ctx = context;
        this.endpointXslt = EndpointXslt;
    }

    protected String callXsltTransformation(Source content, QName endpoint)
        throws Exception {

        String xml = null;
        String ediMessage = null;

        QName service = findXsltEndpointForEndpoint(endpoint);
        if (service != null) {
            ServiceEndpoint[] xsltEndpoint = this.ctx
                    .getEndpointsForService(service);

            if (xsltEndpoint.length > 0) {
                NormalizedMessage nm = null;
                try {

                    /*
                     * Call xslt service to be xml transformation The service
                     * Unit that activate endpoint with Invoice xslt sheet must
                     * be
                     */
                    InOut inOut = channel.createExchangeFactory()
                            .createInOutExchange();
                    nm = inOut.createMessage();

                    nm.setContent(content);
                    inOut.setMessage(nm, "IN");
                    inOut.setService(service);
                    inOut.setEndpoint(xsltEndpoint[0]);
                    channel.sendSync(inOut);

                    /*
                     * Get transformation or exception
                     */

                    ExchangeStatus status = inOut.getStatus();

                    String fault = null;

                    if (ExchangeStatus.ACTIVE.equals(status)) {
                        NormalizedMessage nm2 = inOut.getMessage("OUT");
                        if (nm2 != null) {
                            xml = SourceHelper.createString(nm2.getContent());
                            if (xml != null) {
                                ediMessage = xmlToEdifact(xml);
                                if (ediMessage == null) {
                                    fault = "Edifact messsage is empty";
                                }
                            }
                        } else {
                            Fault sourceFault = inOut.getFault();
                            if (sourceFault != null) {
                                fault = SourceHelper.createString(sourceFault
                                        .getContent());
                            }
                        }
                    } else {
                        if (ExchangeStatus.DONE.equals(status)) {
                            throw new Exception(
                                    "The exchange has been end too soon");
                        } else {
                            if (ExchangeStatus.ERROR.equals(status)) {
                                Exception exception = inOut.getError();
                                if (exception != null) {
                                    throw new Exception(exception);
                                }
                            }

                        }
                    }

                    inOut.setStatus(ExchangeStatus.DONE);
                    channel.send(inOut);

                    if (fault != null) {
                        throw new Exception("Error in xslt transformation :  "
                                + fault);
                    }

                } catch (MessagingException e) {
                    throw new Exception(e);
                }
            }
        } else {
            throw new Exception(
                    "This endpoint doesn't have any xslt endpoint match");
        }
        return "<message><![CDATA[" + ediMessage + "]]></message>";
    }

    protected QName findXsltEndpointForEndpoint(QName endpoint) {

        return this.endpointXslt.get(endpoint);
    }

    @Override
    protected void sendInMessage(QName service, QName operation,
            NormalizedMessage in) throws Exception {
        String content = SourceHelper.createString(in.getContent());

        if (StringHelper.isEmpty(content)) {
            l.log(Level.WARNING, "edifactToXML operation : message is empty");
            // this will be automaticly added to a Fault if the message
            throw new Exception("message is empty");
        } else {
            throw new Exception("Only InOut message is supported");
        }
    }

    @Override
    protected boolean sendInOutMessage(QName service, QName operation,
            NormalizedMessage in, NormalizedMessage out, boolean optionalOut)
        throws Exception {
        Source content = in.getContent();

        if (content == null) {
            l.log(Level.WARNING,
                    "edifactToString operation : any string to be transformed");
            // this will be automaticly added to a Fault if the message
            throw new Exception(
                    "edifactToString operation : any string to be transformed");
        } else {
            String response = null;
            if (!service.getLocalPart().equals("EdifactService")) {
                try {
                    response = callXsltTransformation(content, service);
                } catch (Exception e) {
                    throw new Exception(e);
                }
            } else {
                if (operation.getLocalPart().equals("edifactToXml")) {
                    response = edifactToXml(content);
                } else if (operation.getLocalPart().equals("xmlToEdifact")) {
                    String edi = SourceHelper.createString(content);
                    response = xmlToEdifact(edi);
                    response = "<message><![CDATA[" + response + "]]></message>";
                } else {
                    throw new Exception(
                            "Only 'edifactToXml' and 'xmlToEdifact' operations are supported with 'EdifactService'");
                }
            }
            if (response != null) {
                out.setContent(SourceHelper.createSource(response));
            } else {
                out.setContent(SourceHelper.createSource("<message/>"));
            }
            return true;

        }

    }

    private String edifactToXml(Source content) throws Exception {

        String edi = null;
        InterchangeEDI interchange = null;
        try {
            edi = SourceHelper.createString(content);
            if (edi.indexOf("<message><![CDATA[") > -1) {
                edi = edi.substring(18, edi.indexOf("]]></"));
                edi = edi.replace("!", " ");
                edi = edi.replace("#", " ");
                interchange = InterchangeEDI.loadFromEDIFACTString(edi);
            } else {
                throw new Exception(
                        "You must provide the edifact message between two 'message' xml tags and a CDATA section (Example : <message><![CDATA[edi]]></message>");
            }
        } catch (NullPointerException e) {
            throw new Exception("You did not provide a valid EDIFACT message");
        } catch (Exception e) {
            throw new Exception(e.getMessage());
        } catch (Error e) {
            throw new Exception(e.getMessage());
        }

        try {
            interchange.validateAndEnrich();
            return interchange.toXmlString();
        } catch (Exception e) {
            throw new Exception(e.getMessage());
        }
    }

    protected String xmlToEdifact(String xml) throws Exception {

        String edi = null;
        InterchangeEDI interchange = null;
        try {
            xml = xml.replace(">", ">\n");
            interchange = InterchangeEDI.loadFromXmlString(xml);

            l.log(Level.INFO, interchange.toString());

            try {
                if (interchange != null) {
                    interchange.buildAndValidate();
                    edi = interchange.toEDIFACTString();
                }

            } catch (Exception e) {
                throw new Exception("ERROR: Edifact message is not validate"
                        + e.getMessage());
            }
        } catch (Exception e) {
            throw new Exception("ERROR: Create Edifact message"
                    + e.getMessage());
        }

        return edi;
    }

}
