/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * Id : EdifactSUHandler.java 154 ${date}  rbarraza 
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.engine.edifact;

import java.io.File;

import java.util.HashMap;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jbi.component.ComponentContext;
import javax.jbi.management.DeploymentException;
import javax.xml.namespace.QName;

import org.objectweb.petals.component.common.serviceunitmanager.handler.PetalsServiceUnitHandler;
import org.objectweb.petals.component.common.serviceunitmanager.manager.PetalsServiceUnitManager;
import org.objectweb.petals.component.common.util.ManagementMessageUtil;
import org.objectweb.petals.component.common.util.WSDLHelper;
import org.objectweb.petals.component.common.util.XMLHelper;

import org.w3c.dom.Document;
import org.w3c.dom.Node;

/**
 * 
 * @version $Rev: 250 $Date: {date}
 * @since Petals 1.0
 * @author Rocio BARRAZA - eBMWebsourcing
 * 
 */
public class EdifactSUHandler extends PetalsServiceUnitHandler {

    protected Map<QName, QName> mapEndpointXslt;

    protected Map<String, QName> mapServiceUnitEndpoint;

    public EdifactSUHandler(ComponentContext context,
            Map<QName, QName> mapServiceXslt, Logger logger) {
        this.context = context;
        this.logger = logger;
        this.mapEndpointXslt = mapServiceXslt;
        this.mapServiceUnitEndpoint = new HashMap<String, QName>();
        SERVICE_UNIT_TYPE = "EdiServiceUnit";
    }

    private EdifactSUHandler() {
        SERVICE_UNIT_TYPE = "EdiServiceUnit";
    }

    public void init(String serviceUnitName, String serviceUnitRootPath)
        throws DeploymentException {
        logger.log(Level.FINE, "init  serviceUnitName  " + serviceUnitName);
        this.serviceUnitInstallationRootPath.put(serviceUnitName,
                serviceUnitRootPath);
    }

    public void setContext(ComponentContext context) {
        this.context = context;
    }

    public void setLogger(Logger logger) {
        this.logger = logger;
    }

    public void shutDown(String serviceUnitName,
            PetalsServiceUnitManager epHandler) throws DeploymentException {
        QName service = this.mapServiceUnitEndpoint.remove(serviceUnitName);
        this.mapEndpointXslt.remove(service);
    }

    public void start(String serviceUnitName, PetalsServiceUnitManager epHandler)
        throws DeploymentException {
        logger.log(Level.FINE, "start  serviceUnitName  " + serviceUnitName);
        String serviceUnitRootPath = serviceUnitInstallationRootPath
                .get(serviceUnitName);
        File[] files = new File(serviceUnitRootPath).listFiles();
        Document serviceDesc = null;
        QName service = null;
        String xsltEndpoint = null;
        File jbiXml = new File(serviceUnitRootPath + File.separator
                + "META-INF" + File.separator + "jbi.xml");

        for (File file : files) {
            if (file.getName().endsWith(".wsdl")) {
                serviceDesc = WSDLHelper.createDocumentFromWSDL(file);
            }
        }
        if (serviceDesc != null) {
            try {
                activateEndpointsFromJBIDescription(epHandler, serviceDesc,
                        serviceUnitName, serviceUnitRootPath);

                xsltEndpoint = loadEndpointFromFile(jbiXml);
                service = getServiceNameFromJbiXml(jbiXml);

            } catch (Exception ex) {
                logger.log(Level.SEVERE, FAILED_ACTIVATE_ENDPOINT + ex);
                throw new DeploymentException(FAILED_ACTIVATE_ENDPOINT, ex);
            }
        } else {
            throw new DeploymentException(INCOMPLETE_SERVICE_UNIT_PACKAGE);
        }
        if (service != null && xsltEndpoint != null) {
            mapServiceUnitEndpoint.put(serviceUnitName, service);
            mapEndpointXslt.put(service, QName.valueOf(xsltEndpoint));
        }
    }

    public void stop(String serviceUnitName, PetalsServiceUnitManager epHandler)
        throws DeploymentException {
        /*
         * When service unit is stopped, so properties of directory are deleted
         */
        QName endpoint = this.mapServiceUnitEndpoint.remove(serviceUnitName);
        this.mapEndpointXslt.remove(endpoint);
        try {
            serviceUnitInstallationRootPath.remove(serviceUnitName);
            deactivateEndpointsFromJBIDescription(serviceUnitName, epHandler);
        } catch (Exception e) {
            throw new DeploymentException(FAILED_DEACTIVATE_ENDPOINT, e);
        }
    }

    public String undeploy(String serviceUnitName, String serviceUnitRootPath)
        throws DeploymentException {
        return ManagementMessageUtil.getComponentTaskResult(context
                .getComponentName(), "undeploy",
                ManagementMessageUtil.TASK_RESULT_SUCCESS);
    }

    private String loadEndpointFromFile(File configFile) {
        String value = null;
        Document jbiDoc = WSDLHelper.createDocumentFromWSDL(configFile);
        Node providesNode = XMLHelper.findChild(jbiDoc, "consumes", true);
        value = XMLHelper.getAttributeValue(providesNode, "service-name");
        return value;
    }

    @Override
    public String deploy(String serviceUnitName, String serviceUnitType,
            String serviceUnitRootPath) throws DeploymentException {
        if (SERVICE_UNIT_TYPE.equals(serviceUnitType)) {
            logger.fine(" DEPLOY  serviceUnitName  " + serviceUnitName
                    + "  serviceUnitRootPath " + serviceUnitRootPath);
            return ManagementMessageUtil.getComponentTaskResult(context
                    .getComponentName(), "deploy",
                    ManagementMessageUtil.TASK_RESULT_SUCCESS);
        } else {
            return null;
        }
    }

}
