/**
 * 
 */
package org.objectweb.petals.binding.filetransferbc.listeners;

import java.util.HashMap;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.objectweb.petals.binding.filetransferbc.FileTransferBCException;
import org.objectweb.petals.component.common.PEtALSComponentSDKException;
import org.objectweb.petals.component.common.bc.AbstractBindingComponent;
import org.objectweb.petals.component.common.bc.ExternalListenerManager;
import org.objectweb.petals.component.common.util.PetalsExtensionsUtil;
import org.objectweb.petals.tools.jbicommon.descriptor.Extensions;

/**
 * The external listener manager for file transfer BC.
 * 
 * @author chamerling - eBM WebSourcing
 * 
 */
public class FileTransferBCExternalListenerManager implements
    ExternalListenerManager {

    private Logger logger;

    private AbstractBindingComponent component;

    private Map<String, FileTransferBCListener> listeners;

    /**
     * Creates a new instance of {@link FileTransferBCExternalListenerManager}
     * 
     * @param logger
     * @param channel
     * @param context
     * @param jbiListener
     * @param workDir
     * @param mapDirConfig
     */
    public FileTransferBCExternalListenerManager(Logger logger,
        AbstractBindingComponent component) {
        this.logger = logger;
        this.component = component;
        this.listeners = new HashMap<String, FileTransferBCListener>();
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.petals.component.common.bc.ExternalListenerManager#startListening(java.lang.String)
     */
    public void startListening(String address, Extensions extensions)
        throws PEtALSComponentSDKException {
        logger.log(Level.INFO, "Starting polling directory " + address);
        
        long pollingPeriod = retrievePollingPeriod(extensions);

        // Create and register the listener
        FileTransferBCListener listener = null;
        if (!listeners.containsKey(address)) {
            listener = new FileTransferBCListener(logger, component, address, pollingPeriod);
            try {
                listener.init();
            } catch (FileTransferBCException e) {
                throw new PEtALSComponentSDKException(e);
            }
            listeners.put(address, listener);
        } else {
            throw new PEtALSComponentSDKException(
                "A file transfer listener is already started for the following address:"
                    + address);
        }

        // Start listening for new files to transfer. Scanning period is set to
        // 2 seconds.
        listener.startProcessing();
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.petals.component.common.bc.ExternalListenerManager#stopListening(java.lang.String)
     */
    public void stopListening(String address) {
        logger.log(Level.FINE, "Stopping listening on address " + address);
        FileTransferBCListener listener = listeners.get(address);
        if (listener != null) {
            listener.stopProcessing();
        }
        listeners.remove(address);
    }

    /**
     * Retrieve plooing period from the extensions. If polling is not found, the
     * default value is set to 2000 ms.
     * 
     * @param extensions
     * @return the polling period in ms
     */
    private long retrievePollingPeriod(Extensions extensions) {
        long polling = 2000;

        String tmp = PetalsExtensionsUtil.extractValueFromKeyValueExtension(
            extensions, "polling-period");
        if (tmp != null) {
            try {
                polling = Long.parseLong(tmp);
            } catch (NumberFormatException e) {
                logger.log(Level.FINE,
                    "Polling period parse exception, value is set to "
                        + polling);
            }
        }

        return polling;
    }

}
