/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * Id : ${file_name} 154 ${date}  rbarraza 
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.binding.filetransferbc.listeners;

import java.io.File;
import java.util.HashSet;
import java.util.Set;
import java.util.Timer;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.activation.DataHandler;
import javax.activation.FileDataSource;
import javax.jbi.JBIException;
import javax.jbi.messaging.DeliveryChannel;
import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.InOnly;
import javax.jbi.messaging.InOut;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.jbi.messaging.RobustInOnly;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.xml.namespace.QName;

import org.objectweb.petals.binding.filetransferbc.FileTransferBCException;
import org.objectweb.petals.binding.filetransferbc.IOUtils;
import org.objectweb.petals.component.common.bc.AbstractBindingComponent;
import org.objectweb.petals.component.common.su.SimpleServiceUnitManager;
import org.objectweb.petals.component.common.util.PetalsExtensionsUtil;
import org.objectweb.petals.component.common.util.SourceHelper;
import org.objectweb.petals.component.common.util.WSDLHelper;
import org.objectweb.petals.tools.jbicommon.descriptor.Consumes;
import org.objectweb.petals.tools.jbicommon.descriptor.Extensions;
import org.objectweb.petals.tools.jbicommon.util.PeriodicTask;
import org.objectweb.petals.tools.jbicommon.util.StringHelper;
import org.w3c.dom.Document;

/**
 * The task which scans the file transfer working directories. It builds JBI
 * message with files as attachments and send them to the defined endpoint.
 * 
 * @version $Rev: 250 $Date: {date}
 * @since Petals 1.0
 * @author Christophe HAMERLING - eBM WebSourcing
 * 
 */
public class FileTransferBCListener extends PeriodicTask {

    private String address;

    private Logger logger;

    private AbstractBindingComponent component;

    private File outputDir;
    
    private File wasteDir;

    private String operation;

    private QName service;

    private QName interfaceName;

    private ServiceEndpoint endpoint;

    /**
     * Creates a new instance of {@link FileTransferBCListener}
     * 
     * @param logger
     * @param channel
     * @param context
     * @param jbiListener
     * @param mapDirConfig
     * @param dir
     */
    public FileTransferBCListener(Logger logger,
        AbstractBindingComponent component, String address, long pollingPeriod) {
        super(0, pollingPeriod);
        this.component = component;
        this.logger = logger;
        this.address = address;
    }

    /*
     * (non-Javadoc)
     * 
     * @see java.lang.Runnable#run()
     */
    public void run() {
        File inDir = new File(address);
        if (inDir.exists() && inDir.listFiles().length > 0)
            processDirectory(inDir);
    }

    /**
     * Scan the current directory. Get the files and send them to destination
     * through a JBI message.
     * 
     * @param currentDir
     *            the directory to get files from
     */
    protected void processDirectory(File currentDir) {
        logger.log(Level.FINE, "Processing directory : "
            + currentDir.getAbsolutePath());

        boolean isInOutOperation = false;
        boolean operationFound = false;
        
        Document serviceDesc = null;
        try {
            serviceDesc = component.getContext()
                .getEndpointDescriptor(endpoint);

            // Determines if the operation exists and if it is an inOnly or
            // inOut operation
            try {
                operationFound = WSDLHelper.hasOperationNamed(serviceDesc,
                    operation, service);
                isInOutOperation = WSDLHelper.isInOutOperation(serviceDesc,
                    operation, service);
            } catch (Exception e) {
                operationFound = false;
                isInOutOperation = false;
                this.logger.log(Level.WARNING, e.getMessage());
            }

            if (operationFound)
                handleNewFiles(currentDir, isInOutOperation);

        } catch (JBIException e) {
            this.logger.log(Level.INFO,
                "Error verifying operation definition ", e.getMessage());
        }
    }

    /**
     * Init data
     * 
     */
    public void init() throws FileTransferBCException {

        // Get required datas
        SimpleServiceUnitManager suManager = (SimpleServiceUnitManager) component
            .getServiceUnitManager();
        Consumes consumes = suManager.getConsumesFromAddress(address);

        // get output directory
        String out = retrieveOutputDir(consumes.getExtensions());

        // create the output direcotry if it not exists
        outputDir = new File(out);
        if (!outputDir.exists())
            outputDir.mkdirs();

        // create the waste dir used to save transfered files
        wasteDir = new File(outputDir, "waste");
        if (!wasteDir.exists())
            wasteDir.mkdirs();

        // get operation name
        this.operation = PetalsExtensionsUtil
            .extractValueFromKeyValueExtension(consumes.getExtensions(),
                "operation");

        // get servive name
        this.service = consumes.getServiceName();

        // get service endpoint
        if (consumes.getEndpointName() != null
            && consumes.getServiceName() != null) {
            this.endpoint = component.getContext().getEndpoint(service,
                consumes.getEndpointName());
            
            if (endpoint == null) {
                throw new FileTransferBCException("Could not get endpoint " + consumes.getEndpointName());
            }
        }

        // get interface name
        this.interfaceName = consumes.getInterfaceName();
    }

    /**
     * Retrieve the output directory from the consumes bean. If the output
     * directory has not been set, set the default directory to the component
     * installation root.
     * 
     * @param extensions
     */
    protected String retrieveOutputDir(Extensions extensions) {
        String tmp = PetalsExtensionsUtil.extractValueFromKeyValueExtension(
            extensions, "output-dir");

        // output dir has not been set, get JBI output directory
        if (tmp == null) {
            tmp = getDefaultDirectoryName();
        }

        return tmp;
    }
    
    /**
     * Get the default directory name
     * 
     * @return
     */
    protected String getDefaultDirectoryName() {
        String baseDir = component.getContext().getInstallRoot();
        StringBuffer sb = new StringBuffer(baseDir.substring(0, baseDir
            .length()
            - "install".length()));
        sb.append("work");
        sb.append(File.separator);
        return sb.toString();
    }

    /**
     * Handle an InOnly message
     * 
     * @param body
     * @param service
     * @param operation
     * @return
     * @throws FileTransferBCException
     */
    public boolean handleInOnlyMessage(String body, Set<DataHandler> attachments)
        throws FileTransferBCException {
        boolean result = false;

        try {
            InOnly msg = component.getChannel().createExchangeFactory()
                .createInOnlyExchange();

            // Create and send messageExchange
            NormalizedMessage nm = msg.createMessage();
            nm.setContent(SourceHelper.createContentSource(body));

            // add attachments
            for (DataHandler dh : attachments) {
                nm.addAttachment(dh.getName(), dh);
            }
            msg.setInMessage(nm);

            sendMessage(msg);
            if (ExchangeStatus.DONE.equals(msg.getStatus())) {
                result = true;
            }

        } catch (Exception ex) {
            throw new FileTransferBCException(ex.getMessage());
        }

        return result;
    }

    /**
     * Handle an inout message. Send the files through JBI and wait for a
     * response. If the response contains files, the files are written to disk.
     * 
     * @param body
     * @param service
     * @param operation
     * @param destDir
     * @return
     * @throws FileTransferBCException
     */
    public boolean handleInOutMessage(String body, Set<DataHandler> attachments)
        throws FileTransferBCException {
        boolean result = true;
        InOut msg = null;
        String response = "";
        String fileName = "";
        DeliveryChannel channel = component.getChannel();

        try {
            msg = channel.createExchangeFactory().createInOutExchange();

            NormalizedMessage nm = msg.createMessage();
            nm.setContent(SourceHelper.createContentSource(body));

            // add attachments
            for (DataHandler dh : attachments) {
                nm.addAttachment(dh.getName(), dh);
            }
            msg.setInMessage(nm);

            // send the message
            sendMessage(msg);

            // Verify if request was success,if not, handle the error message
            if (msg.getStatus().equals(ExchangeStatus.ERROR)) {
                fileName = "fault";
                response = SourceHelper.createString(msg.getFault()
                    .getContent());

            } else {
                fileName = "content";
                // if there wasn't errors , binding component handle out reponse
                NormalizedMessage nmOut = msg.getMessage("OUT");
                response = SourceHelper.createString(nmOut.getContent());

                IOUtils.writeAttachmentsToFiles(nmOut, outputDir);
            }

            IOUtils.writeStringToFile(response, outputDir, fileName);

            // Send status DONE to complete the exchange
            msg.setStatus(ExchangeStatus.DONE);
            channel.send(msg);

        } catch (Exception ex) {
            throw new FileTransferBCException(ex.getMessage());
        }

        return result;
    }

    /**
     * Handle the files of the directory
     * 
     * @param srcDir
     * @param inOut
     * @param service
     * @param operation
     * @param destDir
     */
    private void handleNewFiles(File srcDir, boolean inOut) {

        // Create body and attachements
        File[] srcFiles = srcDir.listFiles();
        String body = "<?xml version='1.0' encoding='UTF-8'?><info>attached file(s)</info>";
        Set<DataHandler> attachments = new HashSet<DataHandler>();
        for (File src : srcFiles) {
            if (!src.isDirectory()) {
                DataHandler dhFile = new DataHandler(new FileDataSource(src));
                attachments.add(dhFile);
            }
        }

        // send files through JBI
        try {
            if (!inOut) {
                handleInOnlyMessage(body, attachments);
            } else {
                handleInOutMessage(body, attachments);
            }
        } catch (FileTransferBCException e) {
            this.logger.log(Level.WARNING, e.getMessage());
        } finally {
            moveFilesToWaste(srcFiles);
        }
    }

    /**
     * Handle RobustInOnly messages
     * 
     * @param mailCorps
     * @param service
     * @param operation
     * @throws SMTPBCException
     */
    public void handleRobustInOnlyMessage(String body,
        Set<DataHandler> attachments, QName service, String operation,
        String dirDest) throws FileTransferBCException {

        DeliveryChannel channel = component.getChannel();

        try {
            RobustInOnly msg = channel.createExchangeFactory()
                .createRobustInOnlyExchange();
            NormalizedMessage nm = msg.createMessage();
            nm.setContent(SourceHelper.createContentSource(body));

            // add attachments
            for (DataHandler dh : attachments) {
                nm.addAttachment(dh.getName(), dh);
            }

            // send message
            sendMessage(msg);

            // if error occured, write it to file
            if (msg.getStatus().equals(ExchangeStatus.ERROR)) {
                IOUtils.writeStringToFile(SourceHelper.createString(msg
                    .getFault().getContent()), new File(dirDest), "fault");
                msg.setStatus(ExchangeStatus.DONE);
                channel.send(msg);
            }

        } catch (Exception ex) {
            throw new FileTransferBCException(ex.getMessage());
        }
    }

    /**
     * Move the files to the waste directory.
     * 
     * @param files
     *            the files to move
     * @param parentDir
     *            the parent directory (waste directory is included)
     */
    private void moveFilesToWaste(File[] files) {
        for (File file : files) {
            file.renameTo(new File(wasteDir, file.getName()));
        }
    }

    /**
     * Send message to endpoint
     * 
     * @param messageExchange
     * @throws MessagingException
     */
    private void sendMessage(final MessageExchange messageExchange)
        throws MessagingException {

        if (StringHelper.isNullOrEmpty(address)) {
            throw new MessagingException(
                "You must provide a non null and non empty address");
        }
        if (messageExchange == null) {
            throw new MessagingException(
                "You must provide a non null MessageExchange");
        }

        // set message exchange properties
        messageExchange.setInterfaceName(interfaceName);
        messageExchange.setService(service);
        messageExchange.setEndpoint(endpoint);
        messageExchange.setOperation(QName.valueOf(operation));

        // send message
        try {
            component.getChannel().sendSync(messageExchange);
        } catch (JBIException e) {
            throw new MessagingException(
                "Failed to send the mapped message to the delivery channel.");
        }

    }

    /**
     * Get the timer for this task
     * 
     * @return
     */
    public Timer getTimer() {
        return this.timer;
    }

}
