/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: JmsBC.java 676 2006-06-27 15:44:03Z alouis $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.binding.jms;

import java.util.HashMap;
import java.util.Map;

import org.objectweb.petals.binding.jms.incoming.JMSExternalListenerManager;
import org.objectweb.petals.binding.jms.outgoing.JBIListener;
import org.objectweb.petals.binding.jms.su.JMSServiceUnitListener;
import org.objectweb.petals.component.common.bc.AbstractBindingComponent;
import org.objectweb.petals.component.common.bc.ExternalListenerManager;

/**
 * This component is a JMS BC
 * 
 * @author alouis - eBMWebsourcing
 */
public class JmsBC extends AbstractBindingComponent {

    /**
     * Contains jmsProducerConnections, stored by 'address' (ie jms url
     * provider)
     */
    private Map<String, JMSConnection> jmsProducerConnections;

    /**
     * Contains jmsConsumerConnections, stored by 'address' (ie jms url
     * provider)
     */
    private Map<String, JMSConnection> jmsConsumerConnections;

    @Override
    protected void init(BindingComponentInitializer initializer) {

        jmsProducerConnections = new HashMap<String, JMSConnection>();
        jmsConsumerConnections = new HashMap<String, JMSConnection>();

        // set the SU listener that will perform extra process on SU deployment
        JMSServiceUnitListener suListener = new JMSServiceUnitListener(
            jmsProducerConnections, jmsConsumerConnections);
        setServiceUnitListener(suListener);

        org.objectweb.petals.component.common.bc.JBIListener jbiListener = new JBIListener(
            jmsProducerConnections, getLogger());
        ExternalListenerManager jmsListenerManager = new JMSExternalListenerManager(
            this, jmsConsumerConnections);

        initializer.setExternalListenerManager(jmsListenerManager);
        initializer.setJbiListener(jbiListener);

    }

}
