/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: JMSExternalListenerManager.java 12:18:02 alouis $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.binding.jms.incoming;

import java.util.Map;

import javax.jms.JMSException;
import javax.jms.MessageConsumer;

import org.objectweb.petals.binding.jms.JMSConnection;
import org.objectweb.petals.binding.jms.JmsBC;
import org.objectweb.petals.component.common.PEtALSComponentSDKException;
import org.objectweb.petals.component.common.bc.ExternalListenerManager;
import org.objectweb.petals.tools.jbicommon.descriptor.Extensions;

/**
 * This class is the JMS message listener manager for the JmsBC component
 * 
 * @version $Revision: $ $Date: $
 * @since Petals 1.1
 * @author alouis
 */
public class JMSExternalListenerManager implements ExternalListenerManager {

    public static final String PROPERTY_OPERATION = "javax.jbi.messaging.operation";

    protected JmsBC jmsBC;

    private Map<String, JMSConnection> jmsConnections;

    public JMSExternalListenerManager(JmsBC jmsBC,
        Map<String, JMSConnection> jmsConnections) {
        this.jmsConnections = jmsConnections;
        this.jmsBC = jmsBC;
    }

    /**
     * Retrieve the <code>JMSConnection</code> object previously created and
     * stored in the <code>jmsConnections</code> map, with
     * <code>address</code> as key.<br>
     * Start it.<br>
     * Set a new <code>JMSMessageListener</code> object for the
     * <code>MessageConsumer</code> of this <code>JMSConnection</code>.
     */
    public void startListening(String address, Extensions extensions)
        throws PEtALSComponentSDKException {
        JMSConnection conn = jmsConnections.get(address);
        
        JMSMessageListener jmsMessageListener = new JMSMessageListener(jmsBC,
            address);
        MessageConsumer consumer = conn.getMessageConsumer();
        try {
            consumer.setMessageListener(jmsMessageListener);
        } catch (JMSException e) {
            throw new PEtALSComponentSDKException(
                "Can not set the jms messageListener fro the jms consumer.", e);
        }
    }

    /**
     * Retrieve the <code>JMSConnection</code> object previously created and
     * stored in the <code>jmsConnections</code> map, with
     * <code>address</code> as key.<br>
     * Stop it.<br>
     */
    public void stopListening(String address)
        throws PEtALSComponentSDKException {
        JMSConnection conn = jmsConnections.get(address);
        conn.stop();
    }
}
