/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: PojoServiceUnitListener.java 676 2006-06-27 15:44:03Z alouis $
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.engine.pojo;

import static org.objectweb.petals.component.common.util.ClassLoaderUtil.createClassLoader;

import java.util.List;
import java.util.Map;

import org.objectweb.petals.component.common.PEtALSComponentSDKException;
import org.objectweb.petals.component.common.su.ServiceUnitListener;
import org.objectweb.petals.component.common.util.PetalsExtensionsUtil;
import org.objectweb.petals.tools.jbicommon.descriptor.Extensions;
import org.objectweb.petals.tools.jbicommon.descriptor.JBIDescriptor;
import org.objectweb.petals.tools.jbicommon.descriptor.Provides;

public class PojoServiceUnitListener implements ServiceUnitListener {

    private Map<String, Pojo> pojos;
    
    private PojoSE pojoSE;

    
    public PojoServiceUnitListener(PojoSE pojoSE, Map<String, Pojo> pojos){
        this.pojos = pojos;
        this.pojoSE = pojoSE;
    }
    
    public void onSUDeployed(String serviceUnitName, String suRootPath,
        JBIDescriptor descriptor) throws PEtALSComponentSDKException {

        // Retrieve provides extensions
        List<Provides> providesList = descriptor.getServices().getProvides();
        if(providesList.size() != 1){
            throw new PEtALSComponentSDKException(
            "One, and only one, 'provides' section is required un the JBI.XML descriptor.");
        }
        Extensions extensions = providesList.get(0).getExtensions();
        
        // retrieve pojo' class name
        String className =null;
        className = PetalsExtensionsUtil.extractValueFromKeyValueExtension(
                extensions, "class-name");
        if(className == null){
            throw new PEtALSComponentSDKException(
            "'class-name' property not found in the JBI.XML descriptor");
        }
        
        // create a classloader from the SU path
        ClassLoader classLoader = createClassLoader(suRootPath, this.getClass().getClassLoader());
        
        // load the pojo class from the classloader
        Class clazz =null;
        try {
            clazz = classLoader.loadClass(className);
        } catch (ClassNotFoundException e) {
            throw new PEtALSComponentSDKException(
            className+ " class not found in the service-unit libraries.",e);
        }

        // create the pojo
        Pojo pojo=null;
        try {
            pojo = new Pojo(clazz.newInstance(), pojoSE);
        } catch (Exception e) {
            throw new PEtALSComponentSDKException(
                className+ " found, but can not create an instance.",e);
        }
        
        //init the pojo
        pojo.init();
        pojos.put(serviceUnitName,pojo);
    }

    public void onSUStarted(String serviceUnitName)
        throws PEtALSComponentSDKException {
        pojos.get(serviceUnitName).start();
    }

    public void onSUStopped(String serviceUnitName)
        throws PEtALSComponentSDKException {
        pojos.get(serviceUnitName).stop();
    }

    public void onSUUndeployed(String serviceUnitName, String suRootPath,
        JBIDescriptor descriptor) throws PEtALSComponentSDKException {
        pojos.remove(serviceUnitName);
    }

}
