/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: SampleClient.java 16:40:52 dutoo $
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.engine.sampleclient;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.activation.DataHandler;
import javax.activation.FileDataSource;
import javax.jbi.JBIException;
import javax.jbi.component.Component;
import javax.jbi.component.ComponentContext;
import javax.jbi.component.ComponentLifeCycle;
import javax.jbi.component.ServiceUnitManager;
import javax.jbi.messaging.DeliveryChannel;
import javax.jbi.messaging.InOnly;
import javax.jbi.messaging.InOptionalOut;
import javax.jbi.messaging.InOut;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.jbi.messaging.RobustInOnly;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.management.ObjectName;
import javax.xml.namespace.QName;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.Source;

import org.objectweb.petals.component.common.util.ComponentLogger;
import org.objectweb.petals.component.common.util.SourceHelper;
import org.objectweb.petals.engine.sampleclient.gui.Console;
import org.objectweb.petals.tools.jbicommon.util.XMLUtil;
import org.w3c.dom.Document;
import org.w3c.dom.DocumentFragment;
import org.xml.sax.InputSource;

/**
 * Main class of the SampleClient component, this client is used to manage a
 * container and to test it
 * 
 * @author alouis, ddesjardins - eBMWebsourcing
 * @author Marc Dutoo - Open Wide
 */
public class SampleClient implements Component, ComponentLifeCycle {

    private ComponentContext context;

    private DeliveryChannel channel;

    @SuppressWarnings("unused")
    private ServiceEndpoint endpointReference;

    private SampleClientListener listener;

    private Console console;

    private ComponentLogger logger;

    public void init(ComponentContext context) throws JBIException {

        this.context = context;
        Logger logger = context.getLogger("", null);
        this.logger = new ComponentLogger(logger, logger.getName(), logger
            .getResourceBundleName(), context.getComponentName());
        logger.log(Level.INFO, "");
    }

    public void shutDown() throws JBIException {
        logger.log(Level.INFO, "shutDown");
        this.listener.stopProcessing();
        console.setVisible(false);
        console.dispose();

    }

    public void start() throws JBIException {
        logger.log(Level.INFO, "start");

        try {
            console = new Console(this);
            this.channel = this.context.getDeliveryChannel();
            this.listener = new SampleClientListener(this.channel, console,
                logger);
            Thread listenerThread = new Thread(this.listener, context
                .getComponentName()
                + "-JBI listener thread");
            listenerThread.start();
            console.setVisible(true);
        } catch (MessagingException e) {
            throw new JBIException(e);
        }

    }

    public void stop() throws JBIException {
        logger.log(Level.INFO, "stop");
        listener.stopProcessing();
        console.dispose();
        channel.close();
        this.listener.stopProcessing();

    }

    public ComponentLifeCycle getLifeCycle() {
        return this;
    }

    public Document getServiceDescription(ServiceEndpoint arg0) {
        return null;
    }

    public ServiceUnitManager getServiceUnitManager() {
        return null;
    }

    public boolean isExchangeWithConsumerOkay(ServiceEndpoint arg0,
        MessageExchange arg1) {
        return false;
    }

    public boolean isExchangeWithProviderOkay(ServiceEndpoint arg0,
        MessageExchange arg1) {
        logger.log(Level.INFO, "SampleClient accept the exchange");
        return true;
    }

    public ServiceEndpoint resolveEndpointReference(DocumentFragment arg0) {
        return null;
    }

    public ObjectName getExtensionMBeanName() {
        return null;
    }

    public static final String INONLY = "InOnly";

    public static final String INOUT = "InOut";

    public static final String INOPTIONALOUT = "InOptionalOut";

    public static final String ROBUSTINONLY = "RobustInOnly";

    /**
     * Sends a message through Petals JBI
     * 
     * @param service
     *            the target JBI service
     * @param op
     * @param content
     *            the message content
     * @param type
     * @param attachmentFiles
     *            the message's attached files (if any)
     * @param syncTime
     */
    public void send(String service, String op, String content, String type,
        List<File> attachmentFiles, long syncTime) {
        logger.log(Level.INFO, "SampleClient try to send");
        try {
            Source source = SourceHelper.createSource(content);

            MessageExchange msg = null;

            if (type.equals(INONLY))
                msg = channel.createExchangeFactory().createInOnlyExchange();
            else if (type.equals(INOUT))
                msg = channel.createExchangeFactory().createInOutExchange();
            else if (type.equals(INOPTIONALOUT))
                msg = channel.createExchangeFactory()
                    .createInOptionalOutExchange();
            else if (type.equals(ROBUSTINONLY))
                msg = channel.createExchangeFactory()
                    .createRobustInOnlyExchange();

            NormalizedMessage nm = msg.createMessage();

            // setting content
            nm.setContent(source);

            // setting attachments if any
            if (attachmentFiles != null) {
                for (File attachmentFile : attachmentFiles) {
                    try {
                        nm
                            .addAttachment(attachmentFile.getName(),
                                new DataHandler(new FileDataSource(
                                    attachmentFile)));
                    } catch (MessagingException e) {
                        throw new Exception("Error when attaching file "
                            + attachmentFile.getName());
                    }
                }
            }

            msg.setService(QName.valueOf(service));

            msg.setOperation(QName.valueOf(op));

            if (type.equals(INONLY))
                ((InOnly) msg).setInMessage(nm);
            else if (type.equals(INOUT))
                ((InOut) msg).setInMessage(nm);
            else if (type.equals(INOPTIONALOUT))
                ((InOptionalOut) msg).setInMessage(nm);
            else if (type.equals(ROBUSTINONLY))
                ((RobustInOnly) msg).setInMessage(nm);

            // test if a service endpoint satisfy the service
            ServiceEndpoint[] eps = context.getEndpointsForService(msg
                .getService());
            if (eps == null || eps.length == 0) {
                console
                    .showWarning("No Endpoint satisfy the following service : "
                        + service);
            } else {
                if (syncTime < 0) {
                    channel.send(msg);
                } else {
                    boolean ok = channel.sendSync(msg, syncTime);
                    if (ok) {
                        listener.process(msg);
                    } else {
                        console.setResponse("Time out !");
                    }
                }
            }
        } catch (Exception e) {
            logger.log(Level.SEVERE, e.getMessage());
            console.setResponse(e.getMessage());
            console.showError(e);
        }
    }

    public ServiceEndpoint[] getEndpoints() {
        return context.getEndpoints(null);
    }

    public ServiceEndpoint resolveDescription(String text) {
        InputSource source = createSource(text);

        DocumentFragment result = null;

        try {
            DocumentBuilderFactory factory = DocumentBuilderFactory
                .newInstance();

            DocumentBuilder builder = factory.newDocumentBuilder();

            Document document = builder.parse(source);

            result = document.createDocumentFragment();
        } catch (Exception e) {
            e.printStackTrace();
        }

        return context.resolveEndpointReference(result);

    }

    public String getDescription(ServiceEndpoint se) {
        Document doc;
        try {
            doc = context.getEndpointDescriptor(se);
            return XMLUtil.parseToString(doc);
        } catch (Throwable e) {
            console.showError(e);
        }
        return null;
    }

    public String getInterfaces(ServiceEndpoint se) {
        String out = "";
        try {
            for (QName interfaces : se.getInterfaces()) {
                out += interfaces;
            }
            return out;
        } catch (Throwable e) {
            console.showError(e);
        }
        return null;
    }

    public String getAsReference(ServiceEndpoint se) {
        try {
            DocumentFragment doc = se.getAsReference(null);
            return XMLUtil.parseToString(doc);
        } catch (Throwable e) {
            console.showError(e);
        }
        return null;
    }

    protected InputSource createSource(String msg) {
        InputSource source = new InputSource();

        byte[] msgByte = msg.getBytes();

        ByteArrayInputStream in = new ByteArrayInputStream(msgByte);

        source.setByteStream(in);

        return source;
    }
}
