/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2006 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: MailBCImpl.java 154 25 sept. 06 ofabre $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.binding.mail;

import org.objectweb.petals.binding.mail.listeners.MailBCExternalListenerManager;
import org.objectweb.petals.binding.mail.listeners.MailBCJBIListener;
import org.objectweb.petals.binding.mail.listeners.MailSessionManager;
import org.objectweb.petals.binding.mail.listeners.MimeMessageManager;
import org.objectweb.petals.binding.mail.listeners.SessionDescriptorBuilder;
import org.objectweb.petals.component.common.bc.AbstractBindingComponent;
import org.objectweb.petals.component.common.bc.ExternalListenerManager;
import org.objectweb.petals.component.common.bc.JBIListener;

/**
 * The Mail Binding Component.
 * <p>
 * It creates some sub components like :
 * <p>
 * the <b>{@link MailSessionManager}</b> used to manage session with mail
 * servers : create and open sessions, send and receive mails,
 * <p>
 * the <b>{@link MimeMessageManager}</b> used to process (reading or creating)
 * mail messages,
 * <p>
 * the <b>{@link SessionDescriptorBuilder}</b> used to extract information
 * from a Mail URI String,
 * <p>
 * the <b>{@link MailBCExternalListenerManager}</b> used to manage listeners
 * for external mail addresses registered during SU deployment,
 * <p>
 * the <b>{@link MailBCJBIListener}</b> used to process incoming JBI messages
 * <p>
 * This class defines some Default Value for mail features like :
 * <p>
 * <b>SMTP</b> (25), <b>IMAP</b> (143) and <b>POP3</b> (110) default ports,
 * <p>
 * default mail folder (INBOX),
 * <p>
 * default mail checking period (60 seconds).
 * 
 * @author ofabre - EBM Websourcing
 * 
 */
public class MailBCImpl extends AbstractBindingComponent {

    public final static String FROM_PATHELEMENT = "from:";

    public final static String MAIL_FROM_KEY = "mail.from";

    public final static String MAIL_HOST_KEY = "mail.host";

    public final static String MAIL_USER_KEY = "mail.user";

    public final static String MAIL_STORE_PROTOCOL_KEY = "mail.store.protocol";

    public final static String MAIL_SMTP_PORT_DEFAULT = "25";

    public final static String MAIL_IMAP_PORT_DEFAULT = "143";

    public final static String MAIL_POP3_PORT_DEFAULT = "110";

    public final static String MAIL_SCHEME_SMTP = "smtp";

    public final static String MAIL_SCHEME_IMAP = "imap";

    public final static String MAIL_SCHEME_POP3 = "pop3";

    public final static String MAIL_TRANSPORT_PROTOCOL_KEY = "mail.transport.protocol";

    public final static String TO_PATHELEMENT = "to:";

    public final static String PERIOD_QUERYELEMENT = "period";

    public final static String PERIOD_DEFAULT = "60000";

    public final static String FOLDER_QUERYELEMENT = "folder";

    public final static String FOLDER_DEFAULT = "INBOX";

    public final static String QUERY_SEPARATOR = "&";
    
    public final static String DESTINATION_ADDRESS = "destinationAddress";

    protected SessionDescriptorBuilder sessionDescriptorBuilder;

    protected MimeMessageManager mimeMessageManager;

    protected MailSessionManager mailSessionManager;


    @Override
    protected void init(BindingComponentInitializer initializer) {
        this.sessionDescriptorBuilder = new SessionDescriptorBuilder(
            getLogger());
        this.mimeMessageManager = new MimeMessageManager(getLogger(), this);
        this.mailSessionManager = new MailSessionManager(getLogger());
        ExternalListenerManager externalListenerManager = new MailBCExternalListenerManager(
            getLogger(), mailSessionManager, sessionDescriptorBuilder,
            mimeMessageManager);
        JBIListener listener = new MailBCJBIListener(mimeMessageManager,
            mailSessionManager, sessionDescriptorBuilder, getLogger());
        
        initializer.setExternalListenerManager(externalListenerManager);
        initializer.setJbiListener(listener);

    }

}
