/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2006 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: MailBCExternalListenerManager.java 154 27 sept. 06 ofabre $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.binding.mail.listeners;

import java.util.HashMap;
import java.util.Map;
import java.util.Timer;
import java.util.logging.Logger;

import javax.jbi.JBIException;

import org.objectweb.petals.component.common.PEtALSComponentSDKException;
import org.objectweb.petals.component.common.bc.ExternalListenerManager;
import org.objectweb.petals.tools.jbicommon.descriptor.Extensions;

/**
 * This class is used to manage listeners for external mail addresses registered
 * during SU deployments
 * 
 * @author ofabre - EBM Websourcing
 * 
 */
public class MailBCExternalListenerManager implements ExternalListenerManager {

    protected Logger log;

    protected MailSessionManager mailSessionManager;

    protected SessionDescriptorBuilder sessionDescriptorBuilder;

    protected MimeMessageManager mimeMessageManager;

    protected Map<String, MailBCExternalListener> listeners;

    public MailBCExternalListenerManager(Logger log,
        MailSessionManager mailSessionManager,
        SessionDescriptorBuilder sessionDescriptorBuilder,
        MimeMessageManager mimeMessageManager) {
        super();
        this.log = log;
        this.mailSessionManager = mailSessionManager;
        this.sessionDescriptorBuilder = sessionDescriptorBuilder;
        this.mimeMessageManager = mimeMessageManager;
        this.listeners = new HashMap<String, MailBCExternalListener>();
    }

    public void startListening(String address, Extensions extensions)
        throws PEtALSComponentSDKException {

        // Extract session information from address
        SessionDescriptor sessionDescriptor;
        try {
            sessionDescriptor = sessionDescriptorBuilder.build(address, null);
        } catch (JBIException e) {
            throw new PEtALSComponentSDKException(e);
        }

        // Create and register the listener
        Timer t = null;
        MailBCExternalListener listener = null;
        if (!listeners.containsKey(address)) {
            t = new Timer();
            listener = new MailBCExternalListener(t, log, sessionDescriptor,
                mailSessionManager, mimeMessageManager, address);
            listeners.put(address, listener);
        } else {
            throw new PEtALSComponentSDKException(
                "A listerner is already started for the following address:"
                    + address);
        }

        // Start listening for new mails
        t
            .schedule(listener, 0, Integer.parseInt(sessionDescriptor
                .getPeriod()));
    }

    public void stopListening(String address) {
        MailBCExternalListener listener = listeners.get(address);
        if (listener != null) {
            listener.getT().cancel();
        }
        listeners.remove(address);
    }

}
