/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2006 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: MailBCJBIListener.java 154 27 sept. 06 ofabre $
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.binding.mail.listeners;

import java.util.logging.Logger;

import javax.mail.Session;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeMessage;

import org.objectweb.petals.component.common.HandlingException;
import org.objectweb.petals.component.common.bc.JBIListener;
import org.objectweb.petals.component.common.util.MessageExchangeWrapper;
import org.objectweb.petals.tools.jbicommon.descriptor.Extensions;

/**
 * Mail BC listener for incoming JBI messages
 * 
 * @author ofabre - EBM Websourcing
 * 
 */
public class MailBCJBIListener implements JBIListener {

	protected Logger log;

	protected MailSessionManager mailSessionManager;

	protected MimeMessageManager mimeMessageManager;

	protected SessionDescriptorBuilder sessionDescriptorBuilder;

    /**
     * Creates a new instance of {@link MailBCJBIListener}
     *
     * @param mimeMessageManager
     * @param mailSessionManager
     * @param sessionDescriptorBuilder
     * @param log
     */
	public MailBCJBIListener(MimeMessageManager mimeMessageManager,
			MailSessionManager mailSessionManager,
			SessionDescriptorBuilder sessionDescriptorBuilder, Logger log) {
		super();
		this.log = log;
		this.mimeMessageManager = mimeMessageManager;
		this.mailSessionManager = mailSessionManager;
		this.sessionDescriptorBuilder = sessionDescriptorBuilder;
	}

    /*
     * (non-Javadoc)
     * @see org.objectweb.petals.component.common.bc.JBIListener#onJBIMessage(java.lang.String, org.objectweb.petals.component.common.util.MessageExchangeWrapper, org.objectweb.petals.tools.jbicommon.descriptor.Extensions)
     */
	public boolean onJBIMessage(String address, MessageExchangeWrapper exchange,
			Extensions extensions) throws HandlingException {

		if (!exchange.isInOnlyPattern()) {
			throw new HandlingException(
					"Mail BC can only handle InOnly message exchanges");
		}

		try {
			// Build a mail session descriptor from the given address (URI)
			SessionDescriptor sessionDescriptor = sessionDescriptorBuilder
					.build(address, exchange);

			// Build the destination address
			InternetAddress internetAddress = new InternetAddress(
					sessionDescriptor.getToAddress());

			// Build the mail session
			Session session = mailSessionManager
					.createSessionPropertiesFromDescriptor(sessionDescriptor);

			// Create a mail message with the incoming jbi message
			MimeMessage mimeMessage = mimeMessageManager
					.mapNormalizedMessageToMimeMessage(session,
							internetAddress, exchange.getInMessage());

			// Send this mail
			mailSessionManager
					.sendMail(mimeMessage, sessionDescriptor, session);

		} catch (Throwable e) {
			throw new HandlingException(e);
		}
		return false;
	}

}
