/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id : $ 
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.binding.xquarebc;

import java.util.logging.Logger;

import javax.jbi.messaging.MessageExchange;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.management.ObjectName;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import junit.framework.TestCase;

import org.easymock.classextension.EasyMock;
import org.objectweb.petals.binding.xquarebc.mock.MockServiceEndpoint;
import org.objectweb.petals.binding.xquarebc.mock.MockXQuareBC;
import org.objectweb.petals.component.common.serviceunitmanager.manager.PetalsServiceUnitManager;
import org.w3c.dom.Document;

import static org.easymock.EasyMock.expect;
import static org.easymock.classextension.EasyMock.createMock;
import static org.easymock.classextension.EasyMock.replay;
import static org.easymock.classextension.EasyMock.verify;

/**
 * 
 * Tests the XQuareBC
 * 
 * @version $Rev: 250 $Date: {date}
 * @since Petals 1.0
 * @author Marc Dutoo - Open Wide
 * 
 */
public class XQuareBCTest extends TestCase {

    Logger logger;

    @Override
    public void setUp() throws Exception {
        logger = EasyMock.createMock(Logger.class);
    }

    public void testGetExtensionMBeanName() {
        // Run Test Case
        XQuareBC xquareBC = new XQuareBC();
        ObjectName objectName = xquareBC.getExtensionMBeanName();
        assertNull(objectName);
    }

    public void testGetServiceDescription() {
        // Create mock object and set suManager in a new method

        // Create Mock Objects
        PetalsServiceUnitManager suManager = createMock(PetalsServiceUnitManager.class);
        MockXQuareBC xquareBC = new MockXQuareBC();

        // Create test parameters
        Document document = null;
        ServiceEndpoint fooService = createMock(ServiceEndpoint.class);
        // new InternalEndpoint(new QName(
        // "fooService"), "fooServiceEndpoint", null, null, null);
        ServiceEndpoint barService = createMock(ServiceEndpoint.class);
        // new InternalEndpoint(new QName(
        // "barService"), "barServiceEndpoint", null, null, null);
        try {
            document = DocumentBuilderFactory.newInstance()
            .newDocumentBuilder().newDocument();
            expect(suManager.getServiceDescription(fooService)).andReturn(
                document).anyTimes();
            expect(suManager.getServiceDescription(barService)).andReturn(null)
            .anyTimes();
            expect(suManager.getServiceDescription(null)).andReturn(null)
            .anyTimes();
        } catch (ParserConfigurationException e2) {
            e2.printStackTrace();
        }
        replay(suManager);

        xquareBC.setServiceUnitManager(suManager);
        // Run the test case 1
        // Asking service description for foo service
        Document doc = null;
        doc = xquareBC.getServiceDescription(fooService);
        assertEquals(document, doc);

        // Run the test case 2
        // Asking service description for bar service
        doc = null;
        doc = xquareBC.getServiceDescription(barService);
        assertNull("This service description should be null", doc);

        // Run the test case 3
        // Asking service description for null service
        doc = null;
        doc = xquareBC.getServiceDescription(null);
        assertNull("This service description should be null", doc);

        verify(suManager);
    }


    public void testIsExchangeWithConsumerOkay() {
        // Create Mock Objects
        MessageExchange msgExchange = createMock(MessageExchange.class);

        XQuareBC xquareBC = new XQuareBC();

        // Create test parameters
        ServiceEndpoint se = new MockServiceEndpoint();

        // Run the Test
        boolean result = xquareBC.isExchangeWithConsumerOkay(se,
            msgExchange);
        assertEquals(result, true);
    }

    public void testIsExchangeWithProviderOkay() {
        // Create Mock Objects
        MessageExchange msgExchange = createMock(MessageExchange.class);

        XQuareBC xquareBC = new XQuareBC();

        // Create test parameters
        ServiceEndpoint se = new MockServiceEndpoint();

        // Run the Test
        boolean result = xquareBC.isExchangeWithProviderOkay(se,
            msgExchange);
        assertEquals(result, true);
    }

}