/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id : $ 
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.binding.xquarebc.listeners;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.HashMap;
import java.util.Properties;
import java.util.logging.Logger;

import javax.jbi.component.ComponentContext;
import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.xml.namespace.QName;
import javax.xml.transform.Source;

import junit.framework.TestCase;

import org.easymock.IArgumentMatcher;
import org.easymock.classextension.EasyMock;
import org.objectweb.petals.binding.xquarebc.XQuareBCException;
import org.objectweb.petals.binding.xquarebc.XQuareSUHandler;
import org.objectweb.petals.binding.xquarebc.mock.MockDeliveryChannel;
import org.objectweb.petals.binding.xquarebc.mock.MockServiceEndpoint;
import org.objectweb.petals.component.common.MEPConstants;
import org.objectweb.petals.component.common.util.SourceHelper;

import static org.easymock.EasyMock.expect;
import static org.easymock.classextension.EasyMock.createMock;
import static org.easymock.classextension.EasyMock.replay;
import static org.easymock.classextension.EasyMock.verify;

/**
 * 
 * This is a helper class for testing the XQuareBC capabilities.
 * It sets up the test sample environment and provides helper
 * methods.
 * 
 * @version $Rev: 250 $Date: {date}
 * @since Petals 1.0
 * @author Marc Dutoo - Open Wide
 * 
 */
public class XQuareBCTestTemplate extends TestCase {

    protected String baseDir;

    protected String workDir;

    protected String auctionDir;

    protected Logger logger;
    
    protected Properties testXquareAuctionProperties;
    
    protected Connection dbConn = null;

    @Override
    public void setUp() throws Exception {
        baseDir = this.getClass().getResource(".").toString();
        baseDir = baseDir.substring(0, baseDir.indexOf("target"));
        baseDir = baseDir.substring(baseDir.indexOf(":") + 1);
        workDir = baseDir + "src" + File.separator + "test-data"
            + File.separator;
        auctionDir = workDir + "auction" + File.separator;
        
        this.logger = EasyMock.createMock(Logger.class);


        this.testXquareAuctionProperties = new Properties();
        //this.testXquareAuctionProperties.load(new FileInputStream(auctionDir + "xquarebc_auction.properties"));
        String databaseUrl = "jdbc:mysql://localhost/petals_samples_xquare_auction";
        String databaseUser = "petals_samples";
        String databasePassword = "petals_samples";
        String databaseMappingFilepath = auctionDir + "auction.map";
        testXquareAuctionProperties.put(XQuareSUHandler.DATABASE_URL, databaseUrl);
        testXquareAuctionProperties.put(XQuareSUHandler.DATABASE_USER, databaseUser);
        testXquareAuctionProperties.put(XQuareSUHandler.DATABASE_PASSWORD, databasePassword);
        testXquareAuctionProperties.put(XQuareSUHandler.DATABASE_MAPPING, databaseMappingFilepath);
        testXquareAuctionProperties.setProperty(XQuareSUHandler.XML_CONNECTION_BASE_URI, "file://" + auctionDir);
        testXquareAuctionProperties.setProperty(XQuareSUHandler.WRAP_QUERY_RESULTS_IN_ROOT_TAG, "true");
        //databaseUrl = testXquareAuctionProperties.getProperty(XQuareSUHandler.DATABASE_URL, "jdbc:mysql://localhost/petals_samples_xquare_auction");
        //databaseUser = testXquareAuctionProperties.getProperty(XQuareSUHandler.DATABASE_USER, "petals_samples_xquare");
        //databasePassword = testXquareAuctionProperties.getProperty(XQuareSUHandler.DATABASE_PASSWORD, "petals_samples_xquare");
        //String databaseMappingFilepath = testXquareAuctionProperties.getProperty(XQuareSUHandler.DATABASE_MAPPING, "auction.map");
        try {
            Class.forName("com.mysql.jdbc.Driver");
            dbConn = DriverManager.getConnection(databaseUrl, databaseUser, databasePassword);
            dbConn.setAutoCommit(false);
        } catch (ClassNotFoundException e) {
            throw new XQuareBCException("Unable to load database driver", e);
        } catch (SQLException e) {
            throw new XQuareBCException("Unable to connect to database", e);
        }

        String delimiterPrefix = XQuareSUHandler.LISTENER_PROP_PREFIX + DelimiterNewDataStrategy.DELIMITER_PROP_PREFIX;
        testXquareAuctionProperties.put(XQuareSUHandler.LISTENER_SERVICE_PROP, "{http://petals.objectweb.org/}HelloworldService");
        testXquareAuctionProperties.put(XQuareSUHandler.LISTENER_OPERATION_PROP, "sayHello");
        testXquareAuctionProperties.put(delimiterPrefix + DelimiterNewDataStrategy.OLD_DATA_DELIMITER_SELECT_PROP, "select max(userid) from users_xqbc");
        testXquareAuctionProperties.put(delimiterPrefix + DelimiterNewDataStrategy.NEW_DATA_DELIMITER_SELECT_PROP, "select max(userid) from users");
        testXquareAuctionProperties.put(delimiterPrefix + DelimiterNewDataStrategy.INIT_DATA_DELIMITER_PROP, "U00");
        testXquareAuctionProperties.put(delimiterPrefix + DelimiterNewDataStrategy.NEW_DATA_DELIMITER_UPDATES_PROP, "delete from users_xqbc;\ninsert into users_xqbc (userid) values (\"##NEW_DATA_DELIMITER##\")");
        testXquareAuctionProperties.put(delimiterPrefix + DelimiterNewDataStrategy.NEW_DATA_XQUERY_PROP,
            "for $i in collection(\"users\")/users \n" +
            "where $i/userid > \"##OLD_DATA_DELIMITER##\" and $i/userid <= \"##NEW_DATA_DELIMITER##\" return \n" +
            "<user>{ $i/userid }</user>");
        
        super.setUp();
    }


    @Override
    protected void tearDown() throws Exception {
        super.tearDown();
        
        try {
            if (dbConn != null) {
                dbConn.close();
            }
        } catch (SQLException e) {
            throw new XQuareBCException("Unable to close connection to database", e);
        }
        
    }
    

    protected int getSelectRowNb(String selectStatementString) throws Exception {
        Statement stmt = dbConn.createStatement();
        stmt.execute(selectStatementString);
        ResultSet rs = stmt.getResultSet();
        int rowNb = 0;
        while (rs.next()) {
            rowNb++;
        }
        stmt.close();
        return rowNb;
    }

    
    protected void cleanDatabase() throws Exception {
        Statement stmt = dbConn.createStatement();
        stmt.executeUpdate("delete from bids;");
        stmt.close();
        stmt = dbConn.createStatement();
        stmt.executeUpdate("delete from items;");
        stmt.close();
        stmt = dbConn.createStatement();
        stmt.executeUpdate("delete from users;");
        stmt.close();
        stmt = dbConn.createStatement();
        stmt.executeUpdate("delete from users_xqbc;");
        stmt.close();
        //dbConn.commit(); // not needed when autocommit
        assert(getSelectRowNb("select * from bids") == 0);
        assert(getSelectRowNb("select * from items") == 0);
        assert(getSelectRowNb("select * from users") == 0);
        assert(getSelectRowNb("select * from users_xqbc") == 0);
    }

    


    public void testOperation(String testOperation, String testInput,
           String testOutput, Properties testServiceProperties) {
        MockDeliveryChannel mockDeliveryChannel = new MockDeliveryChannel();
        ComponentContext mockComponentContext = EasyMock.createMock(ComponentContext.class); // NB. null is ok

        HashMap<String, Properties> mapEndpointProps = new HashMap<String, Properties>();
        mapEndpointProps.put("myServiceName", testServiceProperties);
        XQuareBCJBIProcessor xquareProcessor = new XQuareBCJBIProcessor(
            mockComponentContext, mockDeliveryChannel,
            logger, mapEndpointProps);

        MockServiceEndpoint mockServiceEndpoint = createMock(MockServiceEndpoint.class);
        // expect(mockServiceEndpoint.getEndpointName()).andReturn("myEndpointName");
        expect(mockServiceEndpoint.getServiceName()).andReturn(
            new QName("myServiceName"));

        QName myOperation = new QName(testOperation);

        NormalizedMessage mockInNormalizedMessage = createMock(NormalizedMessage.class);
        Source contentSource = SourceHelper.createSource(testInput);
        expect(mockInNormalizedMessage.getContent()).andReturn(contentSource);
        NormalizedMessage mockOutNormalizedMessage = createMock(NormalizedMessage.class);

        MessageExchange mockMessageExchange = createMock(MessageExchange.class);
        expect(mockMessageExchange.getStatus())
            .andReturn(ExchangeStatus.ACTIVE);
        expect(mockMessageExchange.getRole()).andReturn(
            MessageExchange.Role.PROVIDER);
        expect(mockMessageExchange.getPattern()).andReturn(
            MEPConstants.IN_OUT_PATTERN).atLeastOnce();
        expect(mockMessageExchange.getFault()).andReturn(null);
        try {
            expect(mockMessageExchange.createMessage()).andReturn(
                mockOutNormalizedMessage);
        } catch (Exception e) {
            e.printStackTrace();
        }
        expect(mockMessageExchange.getEndpoint())
            .andReturn(mockServiceEndpoint);
        expect(mockMessageExchange.getOperation()).andReturn(myOperation);
        expect(mockMessageExchange.getMessage("in")).andReturn(
            mockInNormalizedMessage);

        try {
            mockOutNormalizedMessage.setContent(eqSource(SourceHelper
                .createSource(testOutput)));
            // expectLastCall().andAnswer(new IAnswer<void>() {
            mockMessageExchange.setMessage(mockOutNormalizedMessage, "out");
        } catch (MessagingException e) {
            e.printStackTrace();
        }

        replay(mockServiceEndpoint);
        replay(mockInNormalizedMessage);
        replay(mockOutNormalizedMessage);
        replay(mockMessageExchange);

        try {
            // Function to test
            xquareProcessor.process(mockMessageExchange);
        } catch (XQuareBCException e) {
            e.printStackTrace();
        } catch (Exception e) {
            e.printStackTrace();
        }

        verify(mockServiceEndpoint);
        verify(mockInNormalizedMessage);
        verify(mockOutNormalizedMessage);
        verify(mockMessageExchange);
    }


    public void testEmitNewData(String testOperation, String testInput,
           String testOutput, Properties testServiceProperties) {
        MockDeliveryChannel mockDeliveryChannel = new MockDeliveryChannel();
        ComponentContext mockComponentContext = EasyMock.createMock(ComponentContext.class); // NB. null is ok

        HashMap<String, Properties> mapEndpointProps = new HashMap<String, Properties>();
        mapEndpointProps.put("myServiceName", testServiceProperties);
        XQuareBCJBIProcessor xquareProcessor = new XQuareBCJBIProcessor(
            mockComponentContext, mockDeliveryChannel,
            logger, mapEndpointProps);


        XQuareBCListener xquareListener = new XQuareBCListener(
            mockComponentContext, mockDeliveryChannel,
            logger, mapEndpointProps, xquareProcessor);
        
        xquareListener.process(testServiceProperties);
        boolean isInOut = true;
        String tagetServiceName = "targetServiceName";
        String targetOperation = "targetOperation";
        try {
            xquareListener.handleNewData(isInOut, QName.valueOf(tagetServiceName),
                targetOperation, testServiceProperties);
        } catch (Exception e1) {
            // TODO Bloc catch auto-généré
            e1.printStackTrace();
        }
        
        MockServiceEndpoint mockServiceEndpoint = createMock(MockServiceEndpoint.class);
        // expect(mockServiceEndpoint.getEndpointName()).andReturn("myEndpointName");
        expect(mockServiceEndpoint.getServiceName()).andReturn(
            new QName("myServiceName"));

        QName myOperation = new QName(testOperation);

        NormalizedMessage mockInNormalizedMessage = createMock(NormalizedMessage.class);
        Source contentSource = SourceHelper.createSource(testInput);
        expect(mockInNormalizedMessage.getContent()).andReturn(contentSource);
        NormalizedMessage mockOutNormalizedMessage = createMock(NormalizedMessage.class);

        MessageExchange mockMessageExchange = createMock(MessageExchange.class);
        expect(mockMessageExchange.getStatus())
            .andReturn(ExchangeStatus.ACTIVE);
        expect(mockMessageExchange.getRole()).andReturn(
            MessageExchange.Role.PROVIDER);
        expect(mockMessageExchange.getPattern()).andReturn(
            MEPConstants.IN_OUT_PATTERN).atLeastOnce();
        expect(mockMessageExchange.getFault()).andReturn(null);
        try {
            expect(mockMessageExchange.createMessage()).andReturn(
                mockOutNormalizedMessage);
        } catch (Exception e) {
            e.printStackTrace();
        }
        expect(mockMessageExchange.getEndpoint())
            .andReturn(mockServiceEndpoint);
        expect(mockMessageExchange.getOperation()).andReturn(myOperation);
        expect(mockMessageExchange.getMessage("in")).andReturn(
            mockInNormalizedMessage);

        try {
            mockOutNormalizedMessage.setContent(eqSource(SourceHelper
                .createSource(testOutput)));
            // expectLastCall().andAnswer(new IAnswer<void>() {
            mockMessageExchange.setMessage(mockOutNormalizedMessage, "out");
        } catch (MessagingException e) {
            e.printStackTrace();
        }

        replay(mockServiceEndpoint);
        replay(mockInNormalizedMessage);
        replay(mockOutNormalizedMessage);
        replay(mockMessageExchange);

        try {
            // Function to test
            xquareProcessor.process(mockMessageExchange);
        } catch (XQuareBCException e) {
            e.printStackTrace();
        } catch (Exception e) {
            e.printStackTrace();
        }

        verify(mockServiceEndpoint);
        verify(mockInNormalizedMessage);
        verify(mockOutNormalizedMessage);
        verify(mockMessageExchange);
    }



    /**
     * Custom argument matcher for Source
     * 
     * @author mdutoo
     * 
     */
    public static class SourceArgumentMatcher implements IArgumentMatcher {
        private Source expectedSource;

        public SourceArgumentMatcher(Source expectedSource) {
            this.expectedSource = expectedSource;
        }

        public boolean matches(Object found) {
            if (!(found instanceof Source)) {
                return false;
            }
            Source foundSource = (Source) found;
            try {
                String foundSourceContent = SourceHelper
                    .createString(foundSource);
                String expectedSourceContent = SourceHelper
                    .createString(expectedSource);
                return (foundSourceContent == null) ?
                    (expectedSourceContent == null) :
                        foundSourceContent.equals(expectedSourceContent);
            } catch (Exception e) {
                return false;
            }
        }

        public void appendTo(StringBuffer sbuf) {
            String expectedSourceContent;
            try {
                expectedSourceContent = SourceHelper
                    .createString(expectedSource);
            } catch (Exception e) {
                expectedSourceContent = "<error creating representation>";
            }
            sbuf.append("eqSource(with content\"");
            sbuf.append(expectedSourceContent);
            sbuf.append("\")");
        }

    }

    /**
     * Helps using the source argument matcher
     * 
     * @param in
     * @return
     */
    public static Source eqSource(Source in) {
        EasyMock.reportMatcher(new SourceArgumentMatcher(in));
        return in;
    }
    
    /**
     * Helper method in place of IOUtils.getStringFromReader()
     * that has the advantage of not adding any EOL at the end
     * of the file.
     * @param in
     * @return
     * @throws IOException
     */
    public static final String getStringFromInputStream(InputStream in)
            throws IOException {
        //in.reset();
        byte[] byteBuf = new byte[1024];
        int byteReadNb;
        ByteArrayOutputStream bos = new ByteArrayOutputStream();
        while ((byteReadNb = in.read(byteBuf)) != -1) {
            bos.write(byteBuf, 0, byteReadNb);
        }
        String res = bos.toString();
        return res;
    }
}
