package org.objectweb.rentacar.agency.database;

import javax.servlet.http.HttpServlet;

import org.objectweb.rentacar.persistance.database.DatabaseException;
import org.objectweb.rentacar.persistance.database.DatabaseManager;
import org.objectweb.rentacar.util.PropertiesLoader;
import org.objectweb.rentacar.util.PropertyLoadingException;

/**
 * This class is used to start and load data in the database
 */
public class AgencyDatabase extends HttpServlet {

	private static final long serialVersionUID = 1L;

	/**
	 * Database manager
	 */
	private DatabaseManager databaseManager = new DatabaseManager();

	/**
	 * Start the dabase and build the schema and fill it
	 */
	public void init() {
		try {
			String host = PropertiesLoader.getProperty("database.properties", "database.host");
			String port = PropertiesLoader.getProperty("database.properties", "database.port");
			String dbName = PropertiesLoader.getProperty("database.properties", "database.dbname");
			String dbPath = PropertiesLoader.getProperty("database.properties", "database.pathname");
			String dataset = PropertiesLoader.getProperty("database.properties", "database.dataset");
			String url = PropertiesLoader.getProperty("database.properties", "hibernate.connection.url");
			String driver = PropertiesLoader.getProperty("database.properties", "hibernate.connection.driver_class");
			String dialect = PropertiesLoader.getProperty("database.properties", "hibernate.dialect");
			String user = PropertiesLoader.getProperty("database.properties", "hibernate.connection.username");
			String pass = PropertiesLoader.getProperty("database.properties", "hibernate.connection.password");
			databaseManager.runServer(host, port, dbName, dbPath);
			databaseManager.buildDatabaseSchema(url, driver, dialect, user, pass);
			databaseManager.fillDatabase(host, port, dbName, dataset);
		}
		catch (PropertyLoadingException e) {
			e.printStackTrace();
		}
		catch (DatabaseException e) {
			e.printStackTrace();
		}
	}

	/**
	 * Destroy the database
	 */
	public void destroy() {
		databaseManager.shutdownServer();
	}
}
