package org.objectweb.rentacar.agency.services.reservation;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import javax.jws.WebMethod;
import javax.jws.WebParam;
import javax.jws.WebService;
import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.XMLGregorianCalendar;
import javax.xml.namespace.QName;

import org.apache.log4j.Logger;
import org.objectweb.rentacar.AgencyException;
import org.objectweb.rentacar.persistance.bo.Agency;
import org.objectweb.rentacar.persistance.bo.Car;
import org.objectweb.rentacar.persistance.bo.CarCriteria;
import org.objectweb.rentacar.persistance.bo.CarVO;
import org.objectweb.rentacar.persistance.bo.CentralOffice;
import org.objectweb.rentacar.persistance.bo.Contact;
import org.objectweb.rentacar.persistance.bo.Reservation;
import org.objectweb.rentacar.persistance.dao.AgencyDAO;
import org.objectweb.rentacar.persistance.dao.CarDAO;
import org.objectweb.rentacar.persistance.dao.CentralOfficeDAO;
import org.objectweb.rentacar.persistance.dao.ContactDAO;
import org.objectweb.rentacar.persistance.dao.DAOException;
import org.objectweb.rentacar.persistance.dao.ReservationDAO;
import org.objectweb.rentacar.services.client.AgencyCheckAvailabilityServiceImpl;
import org.objectweb.rentacar.services.client.AgencyCheckAvailabilityServiceImplService;
import org.objectweb.rentacar.services.client.AgencyCriteria;
import org.objectweb.rentacar.services.client.AgencyException_Exception;
import org.objectweb.rentacar.services.client.AgencyReservationServiceImplService;
import org.objectweb.rentacar.services.client.AgencyVO;
import org.objectweb.rentacar.services.client.CentralOfficeException;
import org.objectweb.rentacar.services.client.CentralOfficeException_Exception;
import org.objectweb.rentacar.services.client.ConsultationImpl;
import org.objectweb.rentacar.services.client.ConsultationImplService;
import org.objectweb.rentacar.services.client.UpdateCentralDBImpl;
import org.objectweb.rentacar.services.client.UpdateCentralDBImplService;
import org.objectweb.rentacar.util.PropertiesLoader;
import org.objectweb.rentacar.util.PropertyLoadingException;

import sun.management.resources.agent;


/**
 * This class expose web services 
 * that allow to control Reservation lifecycle (create, cancel, close...)
 * @author wjoseph
 */
@WebService
public class AgencyReservationServiceImpl {
	
	private Logger logger = Logger.getLogger(AgencyReservationServiceImpl.class);
	
	/**
	 * Create a new reservation in the agency database 
	 * and update the central office database
	 * @param carId the reserved car Id
	 * @param start the starting date of the reservation
	 * @param end the ending date of the reservation
	 * @param customerId the customer Id
	 * @param startAgencyId the starting agency Id
	 * @param endAgencyId the ending agency Id
	 * @return
	 * @throws DAOException
	 * @throws AgencyException
	 */
	@SuppressWarnings("unchecked")
	@WebMethod
	public String createReservation(@WebParam(name="carId")String carId, 
			@WebParam(name="startDate")Date start, 
			@WebParam(name="endDate")Date end, 
			@WebParam(name="customerId")String customerId, 
			@WebParam(name="startAgencyId")String startAgencyId,
			@WebParam(name="endAgencyId")String endAgencyId) throws AgencyException {
		
		logger.info("Agency . createReservation : "+carId+", "+start.toString()+
				", "+end.toString()+", "+customerId+
				", "+startAgencyId+", "+endAgencyId);
		
		ReservationDAO reservationDAO = ReservationDAO.getInstance();
		String result = null;
		Set<Car> carsInitially = new HashSet<Car>();
		
		try {
			CarCriteria criterium= new CarCriteria();
			criterium.setCarId(carId);
			
			org.objectweb.rentacar.agency.services.availability.AgencyCheckAvailabilityServiceImpl check= new org.objectweb.rentacar.agency.services.availability.AgencyCheckAvailabilityServiceImpl();
			CarVO[] availableCars = check.consultAvailability(criterium, start, end);
			
			if(availableCars.length >= 1){ 
				logger.debug("The car with Id "+ carId+"is disponible");
				carsInitially.add((Car) CarDAO.getInstance().retrieveById(Car.class, carId));
				
				
				Contact cont =(Contact)  ContactDAO.getInstance().retrieveById(Contact.class, customerId);
				Agency stAg = (Agency)AgencyDAO.getInstance().retrieveById(Agency.class, startAgencyId);
				Agency enAg = (Agency)AgencyDAO.getInstance().retrieveById(Agency.class, endAgencyId);
				
				Reservation res = new Reservation(cont,	carsInitially, start, end, stAg, enAg);
				
				
				result = reservationDAO.create(res);
				String centralOfficePresent= PropertiesLoader.getProperty("agency.properties", "central_office");
				if(centralOfficePresent != null && centralOfficePresent.equals("1")){
					//////// Update to central office
					List<CentralOffice> centralOffices = new ArrayList<CentralOffice>();
					for (Object obj : CentralOfficeDAO.getInstance().retrieveAll(CentralOffice.class)) {
						centralOffices.add((CentralOffice) obj);
					}
					// There's only one central office in database
					CentralOffice centralOffice = centralOffices.get(0);
					String centralOfficeHost = centralOffice.getHost();
					String centralOfficePort = centralOffice.getPort();
					
					URL updateWsdlLocation = new URL("http://"+centralOfficeHost+":"+centralOfficePort+"/rentacar-central-office/updateDB?wsdl");
					QName updateName = new QName("{http://update.services.centraloffice.rentacar.objectweb.org/}UpdateCentralDBImplPort");
					
					
					
					UpdateCentralDBImpl port = new UpdateCentralDBImplService(updateWsdlLocation,updateName).getUpdateCentralDBImplPort();
					
					//////// converts Dates
					
					XMLGregorianCalendar stDa = getDate(res.getStartingDate().getTime());
					
					XMLGregorianCalendar enDa = getDate(res.getEndingDate().getTime());
					
					port.addReservationInCentralDB(res.getCustomer().getContactId()
							,carId,stDa,enDa,res.getStartingAgency().getAgencyId(),res.getEndingAgency().getAgencyId());
				}else{
					logger.debug("This car is already renting");
				}
			}
		} catch (DAOException e) {
			logger.error("Error creating Reservation", e);
			throw new AgencyException("Error creating Reservation", e);
		} catch (CentralOfficeException_Exception e) {
			logger.error("Error while updating central DB",e);
			throw new AgencyException("Error while updating central DB",e);
		} catch (MalformedURLException e) {
			logger.error("Error while proccessing WSDL URL",e);
			throw new AgencyException("Error while proccessing WSDL URL",e);
		} catch (PropertyLoadingException e) {
			logger.error("Error while loading the properties",e);
			throw new AgencyException("Error while loading the properties",e);
		}
		return result;
	}
	
	@SuppressWarnings("unchecked")
	@WebMethod
	 public String[] createReservationPartner(@WebParam(name="criterion")CarCriteria criterium, 
			 	@WebParam(name="quantity")int quantity, @WebParam(name="startDate")Date start, 
				@WebParam(name="endDate")Date end, 
				@WebParam(name="partnerId")String partnerId, 
				@WebParam(name="startAgencyId")String startAgencyId,
				@WebParam(name="endAgencyId")String endAgencyId) throws DAOException {
	 String ids[] = {};
	 ArrayList<String> idList= new ArrayList<String>();
	 ArrayList<ElementsToReserve> carsToReserve = new ArrayList<ElementsToReserve>();
	 
	
	 try {
		 XMLGregorianCalendar startDate = getDate(start.getTime());		
			
		XMLGregorianCalendar endDate = getDate(end.getTime());	 
	 org.objectweb.rentacar.agency.services.availability.AgencyCheckAvailabilityServiceImpl check= new org.objectweb.rentacar.agency.services.availability.AgencyCheckAvailabilityServiceImpl();
	 CarVO[] availableCars = check.consultAvailability(criterium, start, end);
	 
	 //If there are enough car in the agency, the cars are reserve here
	 if (availableCars.length >= quantity){
		 for (CarVO carVO : availableCars) {
			carsToReserve.add(new ElementsToReserve(null, carVO.getCarId()));
		 }
	 }
	 else{
		 //If it wants any cars, the agency search in close agencies
		 System.out.println(new String("Search in close Agencies \n"));
		 for (CarVO carVO : availableCars) {
				carsToReserve.add(new ElementsToReserve(null, carVO.getCarId()));
			 }
		ArrayList<ElementsToReserve> availableAgencyBCars= availableCarsInCloseAgencies(criterium, quantity- availableCars.length,start,end);
	 	carsToReserve.addAll(availableAgencyBCars);
	 	System.out.println(new String("cars to rent\n" + carsToReserve.size()));
		}
	 	
	 	if(carsToReserve.size()>= quantity){
	 		int i=0;
	 	    while  (i < quantity){
	 	    	String id;
	 	    	ElementsToReserve current=carsToReserve.get(i);
	 	    if (current.getResaImpl()==null){
	 	    	//Reservation in local agency
	 	    	id=createReservation(current.getCarVOId(),start,end,partnerId,startAgencyId,endAgencyId);
	 	    }else{
	 	    	//Reservation in close agencies
				id= current.getResaImpl().createReservation(current.getCarVOId(),startDate,endDate,partnerId,startAgencyId,endAgencyId);
	 	    }
	 	    	idList.add(id);
		    	i++;
		    }
	 	}else
	 		idList.add("There aren't enough cars matching with the criteria");
		    
	  }
	    catch (AgencyException e) {
	    	throw new DAOException("Error creating Reservation", e);
	    }
	 	catch (AgencyException_Exception e) {
	 		throw new DAOException("Error creating Reservation", e);
		}
	    return idList.toArray(ids);
	    }
	   
	
	/**
	 * Delete the reservation matching the given id
	 * TODO : modify a reservation statut instead of deleting the reservation
	 * @param reservationCode the reservationId of the reservation to close 
	 * @throws AgencyException 
	 */
	@WebMethod
	public void closeReservation(@WebParam(name="reservationCode")String reservationCode) throws AgencyException {
		
		logger.info("Agency . closeReservation : "+reservationCode);
		
		ReservationDAO reservationDAO = ReservationDAO.getInstance();
		
		try {
			Reservation res = (Reservation) reservationDAO.retrieveById(Reservation.class, reservationCode);
			reservationDAO.delete(res);
			
		}
		catch (DAOException e) {
			logger.error("Error closing Reservation", e);
			throw new AgencyException("Error closing Reservation", e);
		}
	}
	
	/**
	 * Delete the reservation matching the given id
	 * TODO : modify a reservation statut instead of deleting the reservation
	 * @param reservationId the reservationId of the reservation to cancel
	 * @return
	 * @throws AgencyException 
	 */
	@WebMethod
	public boolean cancelReservation(@WebParam(name="reservationId")String reservationId) throws AgencyException {
		
		logger.info("Agency . cancelReservation : "+reservationId);
		
		try {
			Reservation resToCancel = (Reservation) ReservationDAO.getInstance().retrieveById(Reservation.class, reservationId);
			ReservationDAO.getInstance().delete(resToCancel);
			return true;
		}
		catch (DAOException e) {
			logger.error("Error closing Reservation", e);
			throw new AgencyException("Error closing Reservation", e);
		}
		
	}
	
	/**
	 * Get a XMLGregorianCalendar from a time in milli second
	 * @param timeInMillis time in millis
	 * @return an xmlgregorian date corresponding to the timeInMillis
	 */
	private XMLGregorianCalendar getDate(long timeInMillis) {
		try {
			GregorianCalendar cal = new GregorianCalendar();
			cal.setTimeInMillis(timeInMillis);
			return DatatypeFactory.newInstance().newXMLGregorianCalendar(cal);
		}
		catch (DatatypeConfigurationException e) {
			throw new Error(e);
		}
	}
////////Ask to close agency for available cars
	private ArrayList<ElementsToReserve> availableCarsInCloseAgencies(CarCriteria carCrit, int quantity, Date start, Date end) throws AgencyException{
		
		List<Object> centralOffices;
		ArrayList<ElementsToReserve> availableCarsAgency = new ArrayList<ElementsToReserve>();
		AgencyCriteria agencyCriteria= new AgencyCriteria();
		try {
		centralOffices =  CentralOfficeDAO.getInstance().retrieveAll(CentralOffice.class);
		// There's only one central office in database
		CentralOffice centralOffice = (CentralOffice)centralOffices.get(0);
		String centralOfficeHost = centralOffice.getHost();
		String centralOfficePort = centralOffice.getPort();
		
		ConsultationImpl consultPort;
		AgencyCheckAvailabilityServiceImpl availabilityPort;
		org.objectweb.rentacar.services.client.AgencyReservationServiceImpl reservationPort;
		
		URL consultWsdlLocation;
		
		consultWsdlLocation = new URL("http://"+centralOfficeHost+":"+centralOfficePort+"/rentacar-central-office/consultation?wsdl");
		logger.debug("http://"+centralOfficeHost+":"+centralOfficePort+"/rentacar-central-office/consultation?wsdl");
		QName consultName = new QName("{http://consultation.services.centraloffice.rentacar.objectweb.org/}ConsultationImplPor");
				
		consultPort= new ConsultationImplService(consultWsdlLocation, consultName).getConsultationImplPort();
		
		//////// converts Dates
		
		XMLGregorianCalendar stDa = getDate(start.getTime());
		
		XMLGregorianCalendar enDa = getDate(end.getTime());
		
		agencyCriteria.setAgencyId("2");
		
		//Ask to central office the list of close agencies
		List<org.objectweb.rentacar.services.client.AgencyVO> agencyClose = consultPort.retreiveAgencies(agencyCriteria);
		
		for (org.objectweb.rentacar.services.client.AgencyVO agencyVO : agencyClose) {
			logger.debug("agencyVO "+  agencyVO);
		}
		String closeAgencyHost;
		String closeAgencyPort;
		String closeAgencyWarName;
		Iterator<AgencyVO> it= agencyClose.iterator();
		
		while(it.hasNext() && (availableCarsAgency.size()< quantity)){
			//While all cars aren't complete, it searches availables cars in close agencies 
			
			org.objectweb.rentacar.services.client.AgencyVO currentAgency= it.next();
			logger.debug("agency "+ currentAgency.getAgencyId());
			closeAgencyHost = currentAgency.getHost();
			closeAgencyPort = currentAgency.getPort();
			closeAgencyWarName = currentAgency.getWarName();
		 	
			//URL consultCloseAgenceLocation = new URL("http://"+closeAgencyHost+":"+closeAgencyPort+"/rentacar-agency-reservation2/availability?wsdl");
			URL consultCloseAgenceLocation = new URL("http://"+closeAgencyHost+":"+closeAgencyPort+"/"+closeAgencyWarName+"/availability?wsdl");
			QName consultAgencyName = new QName("{http://availability.services.agency.rentacar.objectweb.org/}AgencyCheckAvailabilityServiceImplPort");
			availabilityPort = new AgencyCheckAvailabilityServiceImplService(consultCloseAgenceLocation, consultAgencyName).getAgencyCheckAvailabilityServiceImplPort();
			//URL resaWsdlLocation = new URL("http://"+closeAgencyHost+":"+closeAgencyPort+"/rentacar-agency-reservation2/reservation?wsdl");
			URL resaWsdlLocation = new URL("http://"+closeAgencyHost+":"+closeAgencyPort+"/"+closeAgencyWarName+"/reservation?wsdl");
			QName resaName = new QName("{http://reservation.services.agency.rentacar.objectweb.org/}AgencyReservationServiceImplPort");
			reservationPort = (org.objectweb.rentacar.services.client.AgencyReservationServiceImpl) new AgencyReservationServiceImplService(resaWsdlLocation, resaName).getAgencyReservationServiceImplPort();
			// Get available cars
			
			org.objectweb.rentacar.services.client.CarCriteria carCritCl= new org.objectweb.rentacar.services.client.CarCriteria();
			carCritCl.setType(carCrit.getType());
			carCritCl.setModel(carCrit.getModel());
			carCritCl.setBrand(carCrit.getBrand());
			
			List<org.objectweb.rentacar.services.client.CarVO> result;
			result = availabilityPort.consultAvailability(carCritCl, stDa, enDa);
			
			
				if (result.size() < 0 ){
					System.out.println("No car available for your criteria");
				}
				else{
					
					System.out.println(" There is "+ result.size() + " cars matching your criteria");
					
					for (org.objectweb.rentacar.services.client.CarVO carVO : result) {
						//logger.debug(carVO);
						//add available cars in list of cars to resereve 
						availableCarsAgency.add(new ElementsToReserve(reservationPort, carVO.getCarId()));
					}
				}
		}
		return availableCarsAgency;
		}catch (DAOException e1) {
			throw new AgencyException("Error retriving all central offices ", e1);
		}
		catch (MalformedURLException e) {
			throw new AgencyException("Error in service URL"+e);
		} catch (CentralOfficeException_Exception e) {
			throw new AgencyException("Error retriving all agencies matching to agency criteria "+ agencyCriteria, e);
		} catch (AgencyException_Exception e) {
			throw new AgencyException("Error consulting available cars"+e);
		}
	}
	
	
	
}
