package org.objectweb.rentacar.persistance.bo;

import org.apache.commons.lang.builder.CompareToBuilder;
import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;
import org.apache.commons.lang.builder.ToStringBuilder;

/**
 * 
 * @author ofabre
 *
 */
public class AddressVO implements Comparable {
	
	private String addressId;
	
	private String address;
	
	private String city;
	
	private String zip;
	
	private String country;

	public AddressVO() {
		super();
	}

	/**
	 * Class contructor
	 * @param address The number of the street and the street of the address
	 * @param city The city of the address
	 * @param zip The zip code of the address
	 * @param country The country of the address
	 */
	public AddressVO(String address, String city, String zip, String country) {
		super();
		this.address = address;
		this.city = city;
		this.zip = zip;
		this.country = country;
	}
	
	/**
	 * Class contructor
	 * @param addressId The Id of the address
	 * @param address The number of the street and the street of the address
	 * @param city The city of the address
	 * @param zip The zip code of the address
	 * @param country The country of the address
	 */
	public AddressVO(String addressId, String address, String city, String zip, String country) {
		super();
		this.addressId = addressId;
		this.address = address;
		this.city = city;
		this.zip = zip;
		this.country = country;
	}
	
	/**
	 * Creates an AddressVO object from an Address object
	 * @param address 
	 */
	public AddressVO(Address address) {
		super();
		this.addressId = address.getAddressId();
		this.address = address.getAddress();
		this.city = address.getCity();
		this.zip = address.getZip();
		this.country = address.getCountry();
	}

	public String getAddress() {
		return address;
	}

	public void setAddress(String address) {
		this.address = address;
	}

	/**
	 * @return
	 */
	public String getAddressId() {
		return addressId;
	}

	public void setAddressId(String addressId) {
		this.addressId = addressId;
	}

	public String getCity() {
		return city;
	}

	public void setCity(String city) {
		this.city = city;
	}

	public String getCountry() {
		return country;
	}

	public void setCountry(String country) {
		this.country = country;
	}

	public String getZip() {
		return zip;
	}

	public void setZip(String zip) {
		this.zip = zip;
	}

	/**
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		return new ToStringBuilder(this).append("country", this.country)
				.append("address", this.address).append("addressId",
						this.addressId).append("zip", this.zip).append("city",
						this.city).toString();
	}

	/**
	 * @see java.lang.Object#equals(Object)
	 */
	public boolean equals(Object object) {
		if (!(object instanceof AddressVO)) {
			return false;
		}
		AddressVO rhs = (AddressVO) object;
		return new EqualsBuilder().append(this.addressId, rhs.addressId)
				.append(this.country, rhs.country).append(this.address,
						rhs.address).append(this.zip, rhs.zip).append(
						this.city, rhs.city).isEquals();
	}

	/**
	 * @see java.lang.Object#hashCode()
	 */
	public int hashCode() {
		return new HashCodeBuilder(-1574159667, 218892883).append(
				this.addressId).append(this.country).append(this.address)
				.append(this.zip).append(this.city).toHashCode();
	}

	/**
	 * @see java.lang.Comparable#compareTo(Object)
	 */
	public int compareTo(Object object) {
		AddressVO myClass = (AddressVO) object;
		return new CompareToBuilder().append(this.addressId, myClass.addressId)
				.append(this.country, myClass.country).append(this.address,
						myClass.address).append(this.zip, myClass.zip).append(
						this.city, myClass.city).toComparison();
	}
	
	

}
