package org.objectweb.rentacar.persistance.bo;

import java.util.HashSet;
import java.util.Set;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;
import org.apache.commons.lang.builder.ToStringBuilder;

/**
 * 
 * @author ofabre
 *
 */
public class AgencyVO {
	
	private String agencyId;
	
	private String host;
	
	private String port;
	
	private String warName;
	
	private ContactVO contact;
	
	private Set<CarVO> cars;

	public AgencyVO() {
		super();
	}
	
	public AgencyVO(ContactVO contact, Set<CarVO> cars, CentralOfficeVO centralOffice) {
		super();
		this.contact = contact;
		this.cars = cars;
	}

	public AgencyVO(String agencyId, ContactVO contact, Set<CarVO> cars, CentralOfficeVO centralOffice) {
		super();
		this.agencyId = agencyId;
		this.contact = contact;
		this.cars = cars;
	}
	
	public AgencyVO(Agency agency) {
		super();
		this.agencyId = agency.getAgencyId();
		this.contact = new ContactVO(agency.getContact());
		Set<CarVO> cars = new HashSet<CarVO>();
		for (Car car : agency.getCars()) {
			cars.add(new CarVO(car));
		}
		this.cars = cars;
		this.host = agency.getHost();
		this.port = agency.getPort();
		this.warName = agency.getWarName();
	}

	public String getAgencyId() {
		return agencyId;
	}

	public void setAgencyId(String agencyId) {
		this.agencyId = agencyId;
	}

	public Set<CarVO> getCars() {
		return cars;
	}

	public void setCars(Set<CarVO> cars) {
		this.cars = cars;
	}
	
	public ContactVO getContact() {
		return contact;
	}

	public void setContact(ContactVO contact) {
		this.contact = contact;
	}
	public int getNumberOfCarByType(String type){
		int maxNumberCarsOfType=0;
		for (CarVO car : cars) {
			if(car.getType().equals(type))
				maxNumberCarsOfType++;
		}
		return maxNumberCarsOfType;
	}
	
	public String getHost() {
		return host;
	}

	public void setHost(String host) {
		this.host = host;
	}

	public String getPort() {
		return port;
	}

	public void setPort(String port) {
		this.port = port;
	}
	
	public String getWarName() {
		return warName;
	}

	public void setWarName(String warName) {
		this.warName = warName;
	}

	/**
	 * @see java.lang.Object#equals(Object)
	 */
	public boolean equals(Object object) {
		if (!(object instanceof AgencyVO)) {
			return false;
		}
		AgencyVO rhs = (AgencyVO) object;
		return new EqualsBuilder().append(this.contact, rhs.contact).append(
				this.cars, rhs.cars).append(this.agencyId, rhs.agencyId)
				.append(this.host, rhs.host).append(this.port, rhs.port)
				.append(this.warName, rhs.warName).isEquals();
	}

	/**
	 * @see java.lang.Object#hashCode()
	 */
	public int hashCode() {
		return new HashCodeBuilder(-1800697853, -175234907)
				.append(this.contact).append(this.cars).append(this.agencyId)
				.append(this.host).append(this.port).append(this.warName).toHashCode();
	}

	/**
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		return new ToStringBuilder(this).append("host", this.host).append(
				"agencyId", this.agencyId).append("contact", this.contact)
				.append("port", this.port).append("cars", this.cars).append(this.warName).toString();
	}
}
