package org.objectweb.rentacar.persistance.bo;

import java.util.HashSet;
import java.util.Set;

import javax.persistence.CascadeType;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.OneToMany;

import org.hibernate.annotations.GenericGenerator;
import org.objectweb.rentacar.persistance.dao.AgencyDAO;
import org.objectweb.rentacar.persistance.dao.DAOException;
import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;
import org.apache.commons.lang.builder.ToStringBuilder;

/**
 * 
 * @author ofabre
 *
 */
@Entity
public class CentralOffice {
	
	// Fields
	private String centralOfficeId;
	
	private String host;
	
	private String port;
	
	// Relations
	private Set<Agency> agencies;

	public CentralOffice() {
		super();
		// TODO Auto-generated constructor stub
	}

	public CentralOffice(Set<Agency> agencies) {
		super();
		this.agencies = agencies;
	}
	
	public CentralOffice(String centralOfficeId, Set<Agency> agencies) {
		super();
		// TODO Auto-generated constructor stub
		this.centralOfficeId = centralOfficeId;
		this.agencies = agencies;
	}
	
	public CentralOffice(CentralOfficeVO centralOfficeVO) {
		super();
		// TODO Auto-generated constructor stub
		this.centralOfficeId = centralOfficeVO.getCentralOfficeId();
		Set<Agency> agencies = new HashSet<Agency>();
		for (String agencyId : centralOfficeVO.getAgencies()) {
			try {
				agencies.add((Agency)AgencyDAO.getInstance().retrieveById(Agency.class, agencyId));
			} catch (DAOException e) {
				e.printStackTrace();
			}
		}
		this.agencies = agencies;
		this.host = centralOfficeVO.getHost();
		this.port = centralOfficeVO.getPort();
	}

	@OneToMany(cascade=CascadeType.ALL)
	@JoinColumn(name = "centralOffice_centralOfficeId")
	public Set<Agency> getAgencies() {
		return agencies;
	}

	public void setAgencies(Set<Agency> agencies) {
		this.agencies = agencies;
	}

	@Id	@GeneratedValue(generator = "system-uuid")
	@GenericGenerator(name="system-uuid", strategy = "uuid")
	public String getCentralOfficeId() {
		return centralOfficeId;
	}

	public void setCentralOfficeId(String centralOfficeId) {
		this.centralOfficeId = centralOfficeId;
	}

	public String getHost() {
		return host;
	}

	public void setHost(String host) {
		this.host = host;
	}

	public String getPort() {
		return port;
	}

	public void setPort(String port) {
		this.port = port;
	}

	/**
	 * @see java.lang.Object#equals(Object)
	 */
	public boolean equals(Object object) {
		if (!(object instanceof CentralOffice)) {
			return false;
		}
		CentralOffice rhs = (CentralOffice) object;
		return new EqualsBuilder().append(this.centralOfficeId,
				rhs.centralOfficeId).append(this.host, rhs.host).append(
				this.agencies, rhs.agencies).append(this.port, rhs.port)
				.isEquals();
	}

	/**
	 * @see java.lang.Object#hashCode()
	 */
	public int hashCode() {
		return new HashCodeBuilder(135413657, -892320915).append(
				this.centralOfficeId).append(this.host).append(this.agencies)
				.append(this.port).toHashCode();
	}

	/**
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		return new ToStringBuilder(this).append("centralOfficeId",
				this.centralOfficeId).append("agencies", this.agencies).append(
				"host", this.host).append("port", this.port).toString();
	}
	
	

}
