package org.objectweb.rentacar.persistance.bo;

import java.util.Date;
import java.util.HashSet;
import java.util.Set;

import javax.persistence.CascadeType;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToMany;
import javax.persistence.ManyToOne;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;
import org.apache.commons.lang.builder.ToStringBuilder;
import org.hibernate.annotations.GenericGenerator;

/**
 * 
 * @author ofabre
 *
 */
@Entity
public class Reservation {
	
	private String reservationId;
	
	private Contact customer;
	
	private Set<Car> cars;
	
	private Date startingDate;
	
	private Date endingDate;
	
	private Agency startingAgency;
	
	private Agency endingAgency;
	

	public Reservation() {
		super();
	}
	
	public Reservation(Contact customer, Set<Car> cars, Date startingDate, Date endingDate, Agency startingAgency, Agency endingAgency) {
		super();
		this.customer = customer;
		this.cars = cars;
		this.startingDate = startingDate;
		this.endingDate = endingDate;
		this.startingAgency = startingAgency;
		this.endingAgency = endingAgency;
	}

	public Reservation(String reservationId, Contact customer, Set<Car> cars, Date startingDate, Date endingDate, Agency startingAgency, Agency endingAgency) {
		super();
		this.reservationId = reservationId;
		this.customer = customer;
		this.cars = cars;
		this.startingDate = startingDate;
		this.endingDate = endingDate;
		this.startingAgency = startingAgency;
		this.endingAgency = endingAgency;
	}
	
	public Reservation(ReservationVO reservationVO) {
		super();
		this.reservationId = reservationVO.getReservationId();
		this.customer = new Contact(reservationVO.getCustomer());
		Set<Car> cars = new HashSet<Car>();
		for (CarVO car : reservationVO.getCars()) {
			cars.add(new Car(car));
		}
		this.cars = cars;
		this.startingDate = reservationVO.getStartingDate();
		this.endingDate = reservationVO.getEndingDate();
		this.startingAgency = new Agency(reservationVO.getStartingAgency());
		this.endingAgency = new Agency(reservationVO.getEndingAgency());
	}


	@ManyToMany(fetch=FetchType.EAGER)
	@JoinColumn(name="reservation_reservaionId")
	public Set<Car> getCars() {
		return cars;
	}

	public void setCars(Set<Car> cars) {
		this.cars = cars;
	}

	@ManyToOne(cascade={CascadeType.PERSIST,CascadeType.MERGE,CascadeType.REFRESH})
	public Contact getCustomer() {
		return customer;
	}

	public void setCustomer(Contact customer) {
		this.customer = customer;
	}

	@ManyToOne
	@JoinColumn(name="ending_agency")
	public Agency getEndingAgency() {
		return endingAgency;
	}

	public void setEndingAgency(Agency endingAgency) {
		this.endingAgency = endingAgency;
	}

	@Temporal(TemporalType.TIMESTAMP)
	public Date getEndingDate() {
		return endingDate;
	}

	public void setEndingDate(Date endingDate) {
		this.endingDate = endingDate;
	}

	@Id	@GeneratedValue(generator = "system-uuid")
	@GenericGenerator(name="system-uuid", strategy = "uuid")
	public String getReservationId() {
		return reservationId;
	}

	public void setReservationId(String reservationId) {
		this.reservationId = reservationId;
	}

	@ManyToOne
	@JoinColumn(name="starting_agency")
	public Agency getStartingAgency() {
		return startingAgency;
	}

	public void setStartingAgency(Agency startingAgency) {
		this.startingAgency = startingAgency;
	}

	@Temporal(TemporalType.TIMESTAMP)
	public Date getStartingDate() {
		return startingDate;
	}

	public void setStartingDate(Date startingDate) {
		this.startingDate = startingDate;
	}

	/**
	 * @see java.lang.Object#equals(Object)
	 */
	public boolean equals(Object object) {
		if (!(object instanceof Reservation)) {
			return false;
		}
		Reservation rhs = (Reservation) object;
		return new EqualsBuilder().append(this.endingDate, rhs.endingDate)
				.append(this.cars, rhs.cars).append(this.startingDate, rhs.startingDate).append(
						this.reservationId, rhs.reservationId).append(
						this.customer, rhs.customer).append(this.endingAgency,
						rhs.endingAgency).append(this.startingAgency,
						rhs.startingAgency).isEquals();
	}

	/**
	 * @see java.lang.Object#hashCode()
	 */
	public int hashCode() {
		return new HashCodeBuilder(2130538209, 1222600301).append(
				this.endingDate).append(this.cars).append(
				this.startingDate).append(this.reservationId).append(
				this.customer).append(this.endingAgency).append(
				this.startingAgency).toHashCode();
	}

	/**
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		return new ToStringBuilder(this).append("startingDate",
				this.startingDate).append("car", this.cars).append(
				"reservationId", this.reservationId).append("customer",
				this.customer).append("startingAgency", this.startingAgency)
				.append("endingAgency",
						this.endingAgency)
				.append("endingDate", this.endingDate).toString();
	}
	
}
