package org.objectweb.rentacar.persistance.dao;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.apache.log4j.Logger;
import org.hibernate.Query;
import org.hibernate.Session;
import org.objectweb.rentacar.persistance.bo.Agency;
import org.objectweb.rentacar.persistance.bo.AgencyCriteria;
import org.objectweb.rentacar.persistance.bo.Car;
import org.objectweb.rentacar.persistance.bo.Contact;
import org.objectweb.rentacar.persistance.util.HibernateUtil;

/**
 * @author ofabre
 */
public class AgencyDAO extends GenericDAO {

	Logger logger = Logger.getLogger(AgencyDAO.class);

	private static AgencyDAO theInstance;

	private AgencyDAO() {

	}

	public static synchronized AgencyDAO getInstance() {
		if (null == theInstance) {
			theInstance = new AgencyDAO();
		}
		return theInstance;
	}

	public Agency getAgencyByContact(Contact contact) throws DAOException {
		Session session = null;
		Agency agency = null;
		try {
			session = HibernateUtil.getSession();
		}
		catch (ExceptionInInitializerError e) {
			throw new DAOException("Error in Hibernate Session instanciation", e);
		}
		catch (Exception e) {
			throw new DAOException("Error in Hibernate Session instanciation", e);
		}

		Query q = session.createQuery("from Agency a where a.contact >= :contact");
		q.setParameter("contact", contact);
		agency = (Agency) q.uniqueResult();

		return agency;
	}

	/**
	 * Retrieve all agencies that match criteria given in parameter. If all criteria are
	 * null, it returns all agencies.
	 * @param AgencyCriteria criteria of the searched agencies
	 * @return The list of all agencies that match criteria
	 * @throws DAOException
	 */
	public List<Agency> retreiveAgencies(AgencyCriteria agencyCriteria) throws DAOException {
		Session session = null;
		try {
			session = HibernateUtil.getSession();
		}
		catch (ExceptionInInitializerError e) {
			throw new DAOException("Error in Hibernate Session instanciation", e);
		}
		catch (Exception e) {
			throw new DAOException("Error in Hibernate Session instanciation", e);
		}

		// Create the query for all the selected criteria
		String query = "";
		if (agencyCriteria.getZipPattern() == null && agencyCriteria.getLastName() == null && agencyCriteria.getAgencyId() == null) {
			query = "from Agency";
		}
		else {
			query = "from Agency a where ";
			if (agencyCriteria.getAgencyId()!=null){
				query+="a.agencyId = '"+agencyCriteria.getAgencyId()+"' and ";
			}
			if (agencyCriteria.getZipPattern() != null) {
				query += "a.contact.address.zip like '" + agencyCriteria.getZipPattern() + "' and ";
			}
			if (agencyCriteria.getLastName() != null) {
				query += "a.contact.lastName = '" + agencyCriteria.getLastName() + "' and ";
			}
			query = query.substring(0, query.length() - 5);
		}

		List<Agency> agencies = new ArrayList<Agency>();
		logger.debug(query);
		Query q = session.createQuery(query);
		for (Iterator iter = q.list().iterator(); iter.hasNext();) {
			agencies.add((Agency) iter.next());
		}
		return agencies;
	}

	/**
	 * Update the cars set of an agency following the best practice given by Hibernate
	 * @param agencyId
	 * @param newCars
	 * @throws DAOException
	 */
	public void updateAgencyCars(String agencyId, Set<Car> newCars) throws DAOException {

		Agency agency = null;

		agency = (Agency) retrieveById(Agency.class, agencyId);

		agency.getCars().clear();
		agency.getCars().addAll(newCars);

		update(agency);
	}
}
