package org.objectweb.rentacar.persistance.dao;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.hibernate.ObjectNotFoundException;
import org.hibernate.Session;
import org.hibernate.Transaction;
import org.objectweb.rentacar.persistance.util.HibernateUtil;

public class GenericDAO {
	
	public String create(Object o) throws DAOException {
		Session session = null;
		try{
			session =  HibernateUtil.getSession();
		}catch (ExceptionInInitializerError e){
			throw new DAOException("Error in Hibernate Session instanciation", e);
		}catch (Exception e){
			throw new DAOException("Error in Hibernate Session instanciation", e);
		}
		
		Transaction tx = null;
		try{
			tx = session.beginTransaction();
			Serializable s = session.save(o);
			tx.commit();
			return (String) s;
		}catch (Exception e) {
			if (tx!=null) tx.rollback();
			if (o!=null){
				throw new DAOException("Error creating new "+o.getClass().getName(), e);
			}
			throw new DAOException("Error creating new Object of type null", e);
		}
	}
	
	public String persist(Object o) throws DAOException {
		Session session = null;
		try{
			session =  HibernateUtil.getSession();
		}catch (ExceptionInInitializerError e){
			throw new DAOException("Error in Hibernate Session instanciation", e);
		}catch (Exception e){
			throw new DAOException("Error in Hibernate Session instanciation", e);
		}
		
		Transaction tx = null;
		try{
			tx = session.beginTransaction();
			session.persist(o);
			Serializable s = session.save(o);
			
			tx.commit();
			return (String) s;
		}catch (Exception e) {
			if (tx!=null) tx.rollback();
			if (o!=null){
				throw new DAOException("Error creating new "+o.getClass().getName(), e);
			}
			throw new DAOException("Error creating new Object of type null", e);
		}
	}
	
	public void delete(Object o) throws DAOException {
		Session session = null;
		try{
			session =  HibernateUtil.getSession();
		}catch (ExceptionInInitializerError e){
			throw new DAOException("Error in Hibernate Session instanciation", e);
		}catch (Exception e){
			throw new DAOException("Error in Hibernate Session instanciation", e);
		}
		
		Transaction tx = null;
		try{
			tx = session.beginTransaction();
			session.delete(o);
			tx.commit();
		}catch (Exception e) {
			if (tx!=null) tx.rollback();
			if (o!=null){
				throw new DAOException("Error deleting "+o.getClass().getName(), e);
			}
			throw new DAOException("Error deleting Object of type null", e);
		}
	}
	
	public void update(Object o) throws DAOException {
		Session session = null;
		try{
			session =  HibernateUtil.getSession();
		}catch (ExceptionInInitializerError e){
			throw new DAOException("Error in Hibernate Session instanciation", e);
		}catch (Exception e){
			throw new DAOException("Error in Hibernate Session instanciation", e);
		}
		
		Transaction tx = null;
		try{
			tx = session.beginTransaction();
			session.update(o);
			tx.commit();
		}catch (Exception e) {
			if (tx!=null) tx.rollback();
			if (o!=null){
				throw new DAOException("Error updating "+o.getClass().getName(), e);
			}
			throw new DAOException("Error updating Object of type null", e);
		}
	}
	
	public Object retrieveById(Class clazz, Serializable s) throws DAOException {
		Session session = null;
		try{
			session =  HibernateUtil.getSession();
		}catch (ExceptionInInitializerError e){
			throw new DAOException("Error in Hibernate Session instanciation", e);
		}catch (Throwable e){
			throw new DAOException("Error in Hibernate Session instanciation", e);
		}
		
		Object result;
		try {
			result = session.load(clazz, s);
		}catch (ObjectNotFoundException e){
			throw new DAOException("Error loading "+clazz.getName()+" with id : "+s, e);
		}
		catch (Throwable e){
			if (clazz !=null){
				throw new DAOException("Error loading "+clazz.getName()+" with id : "+s, e);
			}
			throw new DAOException("Error retrieving an Object of type null", e);
		}

		return result;
	}
	
	public List<Object> retrieveAll(Class clazz) throws DAOException {
		Session session = null;
		try{
			session =  HibernateUtil.getSession();
		}catch (ExceptionInInitializerError e){
			throw new DAOException("Error in Hibernate Session instanciation", e);
		}
		catch (Exception e){
			throw new DAOException("Error in Hibernate Session instanciation", e);
		}
		try {
			List list = session.createQuery("from "+clazz.getName()).list();
			List<Object> result = new ArrayList<Object>();
			for (Iterator iter = list.iterator();iter.hasNext();){
				result.add((Object) iter.next());
			}			
			return result;
		}catch (Exception e){
			if (clazz !=null){
				throw new DAOException("Error retrieving all"+clazz.getName(), e);
			}
			throw new DAOException("Error retrieving all Objects of type null", e);
		}
		//return session.createQuery("from :clazz").setString("clazz", clazz.getName()).list();
	}

}
