package org.objectweb.petals.demo.icare.service.spring;

import java.util.Collection;
import java.util.Date;
import java.util.LinkedList;
import java.util.List;

import org.objectweb.petals.demo.icare.dao.CarDao;
import org.objectweb.petals.demo.icare.model.Car;
import org.objectweb.petals.demo.icare.model.Category;
import org.objectweb.petals.demo.icare.model.Reservation;
import org.objectweb.petals.demo.icare.model.dto.CarSearchDto;
import org.objectweb.petals.demo.icare.service.CarService;

/**
 * The implementation of the services
 * 
 * @author Cristian Spiescu
 * @author Imad Zarzouri
 * @author Omar Senhaji Rhazi
 * @author Anass Ouazzani Chahdi
 * 
 * @since 15.01.2007
 */
public class CarServiceImpl implements CarService {

	private CarDao carDao = null;
	
	public CarServiceImpl() {
		//populate();
	}
	
	public CarDao getCarDao() {
		return carDao;
	}

	public void setCarDao(CarDao carDao) {
		this.carDao = carDao;
		populate();
	}

	private void populate() {
		List<Car> cars;
		List<Category> categories;
		List<Reservation> reservations;
		
		cars = new LinkedList<Car>();
		categories = new LinkedList<Category>();
		reservations = new LinkedList<Reservation>();
		
		categories.add(new Category("A"));
		categories.add(new Category("B"));
		categories.add(new Category("C"));

		carDao.storeAllCategories(categories);
		
		Car[] carTable = new Car[6];
		carTable[0] = new Car(categories.get(0), "Renault", "Clio");
		carTable[1] = new Car(categories.get(0), "Peugeut", "207");
		carTable[2] = new Car(categories.get(0), "Opel", "Corsa");
		carTable[3] = new Car(categories.get(1), "Peugeot", "607");
		carTable[4] = new Car(categories.get(1), "Mercedess", "C");
		carTable[5] = new Car(categories.get(1), "Audi", "A4");
		
		for(int i=0; i<6; i++)
			cars.add(carTable[i]);
		carDao.storeAllCars(cars);
		
		Reservation[] reservationTable = new Reservation[4];
		reservationTable[0] = new Reservation();
		reservationTable[0].setClientName("anass");
		reservationTable[0].setStartDate(new Date(0));
		reservationTable[0].setEndDate(new Date(1));
		reservationTable[0].setCar(carTable[0]);
		
		reservationTable[1] = new Reservation();
		reservationTable[1].setClientName("omar");
		reservationTable[1].setStartDate(new Date(2));
		reservationTable[1].setEndDate(new Date(3));
		reservationTable[1].setCar(carTable[1]);
		
		reservationTable[2] = new Reservation();
		reservationTable[2].setClientName("imad");
		reservationTable[2].setStartDate(new Date(4));
		reservationTable[2].setEndDate(new Date(5));
		reservationTable[2].setCar(carTable[2]);
		
		reservationTable[3] = new Reservation();
		reservationTable[3].setClientName("cristian");
		reservationTable[3].setStartDate(new Date(6));
		reservationTable[3].setEndDate(new Date(7));
		reservationTable[3].setCar(carTable[3]);
		
		for(int i=0; i<4; i++)
			reservations.add(reservationTable[i]);
		
		carDao.storeAllReservation(reservations);
		
	}
	
	public Collection<Car> getAllCars() {
		return carDao.loadAllCars();
	}

	public Collection<Car> getAvailableCars(CarSearchDto carSearchDto) {
		return carDao.loadAllCars();
	}

	public Collection<Category> getAllCategories() {
		return carDao.loadAllCategories();
	}

	public Category findCategoryById(long id) {
		return (Category) carDao.getEntityById(Category.class, id);
	}

	public Car findCarById(long id) {
		return (Car) carDao.getEntityById(Car.class, id);
	}

	public void storeCar(Car car) {
		carDao.storeEntity(car);
	}

	public Reservation findReservationById(long id) {
		return (Reservation) carDao.getEntityById(Reservation.class, id);
	}

	public Collection<Reservation> getAllReservations() {
		return carDao.loadAllReservations();
	}

	public void storeReservation(Reservation reservation) {
		carDao.storeEntity(reservation);
	}
}
