package org.objectweb.petals.demo.icare.web;

import java.beans.PropertyEditorSupport;
import java.text.NumberFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.objectweb.petals.demo.icare.model.Car;
import org.objectweb.petals.demo.icare.model.Category;
import org.objectweb.petals.demo.icare.service.CarService;
import org.springframework.beans.propertyeditors.CustomDateEditor;
import org.springframework.beans.propertyeditors.CustomNumberEditor;
import org.springframework.validation.BindException;
import org.springframework.web.bind.RequestUtils;
import org.springframework.web.bind.ServletRequestDataBinder;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.mvc.SimpleFormController;

/**
 * The controller for editing a car
 *
 * @author Cristian Spiescu
 * @since 17.01.2007
 */
public class CarFormController extends SimpleFormController {

	private CarService carService;
	
	public CarService getCarService() {
		return carService;
	}

	public void setCarService(CarService carService) {
		this.carService = carService;
	}
	
	public CarFormController() {
		setSessionForm(true);
		setBindOnNewForm(true);
		setValidateOnBinding(true);
	}

	@Override
	protected void initBinder(HttpServletRequest request, ServletRequestDataBinder binder) throws Exception {
		//binder.registerCustomEditor(null, "category", new PropertyEditorSupport());
        // convert java.lang.Long
        NumberFormat nf = NumberFormat.getNumberInstance();
        binder.registerCustomEditor(Long.class, null,
                new CustomNumberEditor(Long.class, nf, true));
        
        binder.registerCustomEditor(null, "category", new PropertyEditorSupport() {

			@Override
			public void setAsText(String text) throws IllegalArgumentException {
				Integer id = new Integer(text);
				Category cat = carService.findCategoryById(id);
				setValue(cat);					
			}
        	
        });

		SimpleDateFormat dateFormat = new SimpleDateFormat("dd/MM/yy HH:mm");
		dateFormat.setLenient(false);
		binder.registerCustomEditor(Date.class, new CustomDateEditor(dateFormat, true));
        
        super.initBinder(request, binder);
	}

	public ModelAndView processFormSubmission(HttpServletRequest request,
			HttpServletResponse response, Object command, BindException errors)
			throws Exception {
		if (request.getParameter("cancel") != null) {
			return new ModelAndView(getSuccessView());
		}

		return super.processFormSubmission(request, response, command, errors);
	}
	
	@SuppressWarnings("unchecked")
	protected Map referenceData(HttpServletRequest request) throws Exception {
		Map refData = new HashMap();
		refData.put("categories", carService.getAllCategories());
		return refData;
	}
	
	protected Object formBackingObject(HttpServletRequest request)
			throws Exception {
		long carId = RequestUtils.getIntParameter(request, "id", -1);
		Car car = null;
		if (carId == -1) {
			car = new Car();
		} else {
			car = carService.findCarById(carId);
		}
		return car;
	}
	
	protected ModelAndView onSubmit(HttpServletRequest request,
			HttpServletResponse response, Object command, BindException errors)
			throws Exception {

		Car car = (Car) command;
		if (request.getParameter("delete") != null) {
			//carService.deleteCar(car.getId());
		} else {
			carService.storeCar(car);
		}

		return new ModelAndView(getSuccessView());
	}
	
	
		
}
