package org.objectweb.petals.demo.icare.web;

import java.beans.PropertyEditorSupport;
import java.text.NumberFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.objectweb.petals.demo.icare.model.Car;
import org.objectweb.petals.demo.icare.model.Reservation;
import org.objectweb.petals.demo.icare.service.CarService;
import org.springframework.beans.propertyeditors.CustomDateEditor;
import org.springframework.beans.propertyeditors.CustomNumberEditor;
import org.springframework.validation.BindException;
import org.springframework.web.bind.RequestUtils;
import org.springframework.web.bind.ServletRequestDataBinder;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.mvc.SimpleFormController;

/**
 * @author Cristian Spiescu
 * @author Imad Zarzouri
 * @author Omar Senhaji Rhazi
 * @author Anass Ouazzani Chahdi
 *
 * @since 21.01.2007
 */
public class MakeReservationController extends SimpleFormController{

private CarService carService;
	
	public CarService getCarService() {
		return carService;
	}

	public void setCarService(CarService carService) {
		this.carService = carService;
	}
	
	public MakeReservationController() {
		setSessionForm(true);
		setBindOnNewForm(true);
		setValidateOnBinding(true);
	}

	@Override
	protected void initBinder(HttpServletRequest request, ServletRequestDataBinder binder) throws Exception {
		//binder.registerCustomEditor(null, "category", new PropertyEditorSupport());
        // convert java.lang.Long
        NumberFormat nf = NumberFormat.getNumberInstance();
        binder.registerCustomEditor(Long.class, null,
                new CustomNumberEditor(Long.class, nf, true));
        
        binder.registerCustomEditor(null, "car", new PropertyEditorSupport() {

			@Override
			public void setAsText(String text) throws IllegalArgumentException {
				Integer id = new Integer(text);
				Car car = carService.findCarById(id);
				setValue(car);					
			}
        	
        });

		SimpleDateFormat dateFormat = new SimpleDateFormat("dd/MM/yy");
		dateFormat.setLenient(false);
		binder.registerCustomEditor(Date.class, new CustomDateEditor(dateFormat, true));
        
        super.initBinder(request, binder);
	}

	public ModelAndView processFormSubmission(HttpServletRequest request,
			HttpServletResponse response, Object command, BindException errors)
			throws Exception {
		if (request.getParameter("cancel") != null) {
			return new ModelAndView(getSuccessView());
		}

		return super.processFormSubmission(request, response, command, errors);
	}
	
	@SuppressWarnings("unchecked")
	protected Map referenceData(HttpServletRequest request) throws Exception {
		Map refData = new HashMap();
		refData.put("cars", carService.getAllCars());
		return refData;
	}
	
	protected Object formBackingObject(HttpServletRequest request)
			throws Exception {
		long reservationId = RequestUtils.getIntParameter(request, "id", -1);
		Reservation reservation = null;
		if (reservationId == -1) {
			reservation = new Reservation();
		} else {
			reservation = carService.findReservationById(reservationId);
		}
		return reservation;
	}
	
	protected ModelAndView onSubmit(HttpServletRequest request,
			HttpServletResponse response, Object command, BindException errors)
			throws Exception {

		Reservation reservation = (Reservation) command;
		if (request.getParameter("delete") != null) {
			//carService.deleteCar(car.getId());
		} else {
			carService.storeReservation(reservation);
		}
		return new ModelAndView(getSuccessView());
	}
}
