/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id:MortgageClient.java 10:42:15 ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.demo.mortgage.gui;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.apache.axiom.om.OMAbstractFactory;
import org.apache.axiom.om.OMElement;
import org.apache.axiom.om.OMFactory;
import org.apache.axiom.om.OMNamespace;
import org.apache.axiom.soap.SOAP11Constants;
import org.apache.axis2.Constants;
import org.apache.axis2.addressing.AddressingConstants;
import org.apache.axis2.addressing.EndpointReference;
import org.apache.axis2.client.Options;
import org.apache.axis2.client.ServiceClient;
import org.objectweb.petals.component.common.HandlingException;
import org.objectweb.petals.demo.mortgage.gui.gui.Console;
import org.objectweb.petals.tools.jbicommon.util.XMLUtil;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.xml.sax.SAXException;

/**
 * This class is the client of the mortgage demonstration TODO : it's just a
 * test for the moment
 * 
 * @author ofabre - eBMWebsourcing
 */
public class Axis2MortgageClient implements MortgageClient {

	private Console console;

	protected Logger logger = Logger.getLogger(this.getClass().toString());

	private static EndpointReference targetEPR = new EndpointReference(
			"http://localhost:8084/axis2/services/MortgageProfilingService");

	private static String operation = "orchestrate";

	public void submit(String firstName, String lastName, String address,
			String salary, String propertyTaxes, String insurrance,
			String autoPayment, String creditCards, String otherPayments) {
		logger.log(Level.INFO, "");
		try {

			// uncomment to use axis generated stub (known bug : Multiple
			// headers encountered!)
			/*
			 * Create Port java binding
			 */
			// MortgageProfilingStub port = new MortgageProfilingStub(
			// "http://localhost:8083/axis2/services/MortgageProfilingService");
			/*
			 * Create mortgage request object
			 */
			// MortgageRequest mortgageRequest =
			// createMortgageRequest(firstName,
			// lastName, address, salary, propertyTaxes, insurrance,
			// autoPayment, creditCards, otherPayments);
			//
			// MortgageProfilingStub.Orchestrate orchestrate = new
			// MortgageProfilingStub.Orchestrate();
			// orchestrate.setParam0(mortgageRequest);
			/*
			 * send the request
			 */
			// MortgageProfilingStub.OrchestrateResponse response = port
			// .orchestrate(orchestrate);
			/*
			 * Create the string response and print it on the console
			 */
			// String stringResponse =
			// createStringResponse(response.get_return());
			OMElement payload = createPayload(firstName, lastName, address,
					salary, propertyTaxes, insurrance, autoPayment,
					creditCards, otherPayments);

			Options options = createOptions();

			ServiceClient client = new ServiceClient();
			client.setOptions(options);

			OMElement result = client.sendReceive(payload);

			String stringResponse = createStringResponse(result);

			console.setResponse(stringResponse);

		} catch (Exception e) {
			logger.log(Level.SEVERE, e.getMessage());
		}
	}

	private String createStringResponse(OMElement result)
			throws HandlingException {

		String firstName = "";
		String lastName = "";
		String rateString = "";

		Document doc = parseXml(result.toString());
		Node responseValues = doc.getDocumentElement().getFirstChild();

		List<Node> values = XMLUtil.getNodeChildren(responseValues);
		for (Node node : values) {
			if ("firstName".equalsIgnoreCase(node.getLocalName())) {
				firstName = node.getTextContent();
			} else if ("lastName".equalsIgnoreCase(node.getLocalName())) {
				lastName = node.getTextContent();
			} else if ("mortgageRate".equalsIgnoreCase(node.getLocalName())) {
				rateString = node.getTextContent();
			}
		}

		String stringResponse = "";
		if (new Float(rateString).floatValue() < 0) {
			stringResponse = "Mortgage refused for the customer " + lastName
					+ " " + firstName;
		} else {
			stringResponse = "Mortgage accepted for the customer " + lastName
					+ " " + firstName + " with the following rate : "
					+ rateString;
		}

		return stringResponse;

	}

	private Document parseXml(final String xmlString) throws HandlingException {

		DocumentBuilderFactory docBuilderFactory = DocumentBuilderFactory
				.newInstance();
		docBuilderFactory.setNamespaceAware(true);

		DocumentBuilder docBuilder = null;

		try {
			docBuilder = docBuilderFactory.newDocumentBuilder();
		} catch (ParserConfigurationException pce) {
			throw new HandlingException("Bad XML parser configuration", pce);
		}

		Document document = null;

		try {
			document = docBuilder.parse(new ByteArrayInputStream(xmlString
					.getBytes()));
		} catch (SAXException saxe) {
			throw new HandlingException(
					"JBI descriptor is not well formed XML", saxe);
		} catch (IOException ioe) {
			throw new HandlingException("Can't read JBI descriptor", ioe);
		}
		return document;
	}

	/**
	 * Create the orchestrate OM element
	 * 
	 * @param _firstName
	 *            The first name of the customer
	 * @param _lastName
	 *            The last name of the customer
	 * @param _address
	 *            The address of the customer
	 * @param _salary
	 *            The salary of the customer
	 * @param _propertyTaxes
	 *            Property Taxes of the customer
	 * @param _insurrance
	 *            Amount of insurrance of the customer
	 * @param _autoPayment
	 *            All the auto payment of the customer
	 * @param _creditCards
	 *            Amount of the credit cards of the customer
	 * @param _otherPayments
	 *            Other payments (credits, subscriptions...) of the customer
	 * @return
	 */
	private OMElement createPayload(String _firstName, String _lastName,
			String _address, String _salary, String _propertyTaxes,
			String _insurrance, String _autoPayment, String _creditCards,
			String _otherPayments) {
		OMFactory fac = OMAbstractFactory.getOMFactory();
		OMNamespace omNs = fac.createOMNamespace(
				"http://petals.objectweb.org/", getServiceName());
		OMElement method = fac.createOMElement(operation, omNs);
		OMElement value = fac.createOMElement("param0", omNs);

		OMElement address = fac.createOMElement("address", omNs);
		address.addChild(fac.createOMText(value, _address));
		value.addChild(address);

		OMElement autoPayment = fac.createOMElement("autoPayment", omNs);
		autoPayment.addChild(fac.createOMText(value, _autoPayment));
		value.addChild(autoPayment);

		OMElement creditCards = fac.createOMElement("creditCards", omNs);
		creditCards.addChild(fac.createOMText(value, _creditCards));
		value.addChild(creditCards);

		OMElement firstName = fac.createOMElement("firstName", omNs);
		firstName.addChild(fac.createOMText(value, _firstName));
		value.addChild(firstName);

		OMElement insurance = fac.createOMElement("insurance", omNs);
		insurance.addChild(fac.createOMText(value, _insurrance));
		value.addChild(insurance);

		OMElement otherPayments = fac.createOMElement("otherPayments", omNs);
		otherPayments.addChild(fac.createOMText(value, _otherPayments));
		value.addChild(otherPayments);

		OMElement propertyTaxes = fac.createOMElement("propertyTaxes", omNs);
		propertyTaxes.addChild(fac.createOMText(value, _propertyTaxes));
		value.addChild(propertyTaxes);

		OMElement salary = fac.createOMElement("salary", omNs);
		salary.addChild(fac.createOMText(value, _salary));
		value.addChild(salary);

		OMElement lastName = fac.createOMElement("lastName", omNs);
		lastName.addChild(fac.createOMText(value, _lastName));
		value.addChild(lastName);

		method.addChild(value);

		return method;
	}

	/**
	 * Get the service name from the target EPR
	 * 
	 * @return
	 */
	private static String getServiceName() {
		String eprName = targetEPR.getAddress();
		return eprName
				.substring(eprName.lastIndexOf("/") + 1, eprName.length());
	}

	/**
	 * Create the Axis2 options
	 * 
	 * @return
	 */
	protected static Options createOptions() {

		Options options = new Options();

		// set destination address
		options.setTo(targetEPR);

		// set operation
		options.setAction(operation);

		options.setSoapVersionURI(SOAP11Constants.SOAP_ENVELOPE_NAMESPACE_URI);
		options.setTransportInProtocol(Constants.TRANSPORT_HTTP);

		// FIXME
		options
				.setProperty(
						org.apache.axis2.Constants.Configuration.CHARACTER_SET_ENCODING,
						"UTF-8");
		options.setProperty(
				org.apache.axis2.transport.http.HTTPConstants.CHUNKED,
				org.apache.axis2.Constants.VALUE_FALSE);
		options.setProperty(
				AddressingConstants.DISABLE_ADDRESSING_FOR_OUT_MESSAGES,
				Boolean.TRUE);
		return options;
	}

	// /**
	// * Create a MortgageRequest object setted with the customer information.
	// *
	// * @param firstName
	// * The first name of the customer
	// * @param lastName
	// * The last name of the customer
	// * @param address
	// * The address of the customer
	// * @param salary
	// * The salary of the customer
	// * @param propertyTaxes
	// * Property Taxes of the customer
	// * @param insurrance
	// * Amount of insurrance of the customer
	// * @param autoPayment
	// * All the auto payment of the customer
	// * @param creditCards
	// * Amount of the credit cards of the customer
	// * @param otherPayments
	// * Other payments (credits, subscriptions...) of the customer
	// * @return A <code>MortgageRequest</code> object. Must not be null.
	// */
	// protected MortgageRequest createMortgageRequest(String firstName,
	// String lastName, String address, String salary,
	// String propertyTaxes, String insurrance, String autoPayment,
	// String creditCards, String otherPayments) {
	// MortgageRequest mortgageRequest = new MortgageRequest();
	// mortgageRequest.setAddress(address);
	// mortgageRequest.setAutoPayment(new Float(autoPayment).floatValue());
	// mortgageRequest.setCreditCards(new Float(creditCards).floatValue());
	// mortgageRequest.setFirstName(firstName);
	// mortgageRequest.setInsurance(new Float(insurrance).floatValue());
	// mortgageRequest.setLastName(lastName);
	// mortgageRequest.setOtherPayments(new Float(otherPayments).floatValue());
	// mortgageRequest.setPropertyTaxes(new Float(propertyTaxes).floatValue());
	// mortgageRequest.setSalary(new Float(salary).floatValue());
	// return mortgageRequest;
	// }

	// /**
	// * Create a response message that have to be print into the response text
	// * box of the Console. This response is based on customer firstname and
	// * lastname and mortgage rate included in the MortgageResponse.
	// *
	// * @param response
	// * The <code>MortgageResponse</code> bo
	// * @return a <code>String</code> response message
	// */
	// protected String createStringResponse(MortgageResponse response) {
	// String stringResponse = "ERROR";
	//
	// String firstName = response.getFirstName();
	//
	// String lastName = response.getLastName();
	//
	// float rate = response.getMortgageRate();
	//
	// if (rate < 0) {
	// stringResponse = "Mortgage refused for the customer " + lastName
	// + " " + firstName;
	// } else {
	// stringResponse = "Mortgage accepted for the customer " + lastName
	// + " " + firstName + " with the following rate : " + rate;
	// }
	//
	// return stringResponse;
	// }

	public Console getConsole() {
		return console;
	}

	public void setConsole(Console console) {
		this.console = console;
	}

}
