/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id:Console.java 10:42:15 ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.demo.mortgage.gui.gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.DataInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JDesktopPane;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.JTextPane;

import org.objectweb.petals.demo.mortgage.gui.MortgageClient;

/**
 * This class is the gui of the mortgage demonstration
 * 
 * @author ddesjardins - eBMWebsourcing
 */
public class Console extends javax.swing.JFrame {
    private static final long serialVersionUID = -8117896325113377153L;

    private JTextArea address;

    private JTextPane addressTxt;

    private JTextField carPayment;

    private JTextPane carPaymentTxt;

    private JTextField creditCards;

    private JTextPane creditCardsTxt;

    private JTextPane custTxt;

    private JTextPane expensesTxt;

    private JTextField firstName;

    private JTextPane firstNameTxt;

    private JTextField insurance;

    private JTextPane insuranceTxt;

    private JDesktopPane jDesktopPane;

    private JTextField lastName;

    private JTextPane lastNameTxt;

    private JTextPane monthlyIncomeTxt;

    private MortgageClient mortgageClient;

    private JTextPane otherExpensesTxt;

    private JTextField otherPayments;

    private JTextPane otherPaymentsTxt;

    private JTextField propertyTaxes;

    private JButton reset;

    private JTextArea result;

    private JTextPane resultTxt;

    private JTextField salary;

    private JTextPane salaryTxt;

    private JButton submit;

    private JTextPane taxesTxt;

    public Console() {
        super();
        initialize();
    }

    public Console(MortgageClient sampleClient) {
        this();
        this.mortgageClient = sampleClient;
    }

    public static void main(String[] args) {
        Console inst = new Console();
        inst.setVisible(true);
        inst.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    }

    public void setResponse(String msg) {
        result.setText(msg);
    }

    /**
     * Reset all the fields of the form
     * 
     */
    protected void resetForm() {
        salary.setText("0");
        lastName.setText("");
        firstName.setText("");
        address.setText("");
        carPayment.setText("0");
        insurance.setText("0");
        creditCards.setText("0");
        propertyTaxes.setText("0");
        otherPayments.setText("0");
    }

    /**
     * Tests the parameters and submits the data
     * 
     */
    protected void submit() {
        // Test the data in the form
        String firstName = this.firstName.getText();
        String lastName = this.lastName.getText();
        String address = this.address.getText();
        String salary = this.salary.getText();
        String taxes = this.propertyTaxes.getText();
        String insurance = this.insurance.getText();
        String auto = this.carPayment.getText();
        String creditCards = this.creditCards.getText();
        String otherPayments = this.otherPayments.getText();

        String error = "";
        try {
            Float.parseFloat(salary);
            if (Float.parseFloat(salary) <= 0) {
                error += "The salary must be positive! ";
            }
        } catch (NumberFormatException e) {
            error += "The salary must be a number! ";
        }
        try {
            Float.parseFloat(taxes);
        } catch (NumberFormatException e) {
            error += "The property taxes must be a number! ";
        }
        try {
            Float.parseFloat(insurance);
        } catch (NumberFormatException e) {
            error += "The insurance must be a number! ";
        }
        try {
            Float.parseFloat(auto);
        } catch (NumberFormatException e) {
            error += "The car payments must be a number! ";
        }
        try {
            Float.parseFloat(creditCards);
        } catch (NumberFormatException e) {
            error += "The credit cards payments must be a number! ";
        }
        try {
            Float.parseFloat(otherPayments);
        } catch (NumberFormatException e) {
            error += "The other payments must be a number! ";
        }

        if (!"".equals(error)) {
            JOptionPane.showMessageDialog(this, error, "Error",
                    JOptionPane.ERROR_MESSAGE);
        } else {
            if (mortgageClient != null) {
                mortgageClient.submit(firstName, lastName, address, salary,
                        taxes, insurance, auto, creditCards, otherPayments);
            } else {
                JOptionPane.showMessageDialog(this,
                        "The component is not deployed in a JBI Container",
                        "Error", JOptionPane.ERROR_MESSAGE);
            }
        }
    }

    /**
     * This method initializes address
     * 
     * @return javax.swing.JTextField
     */
    private JTextArea getAddress() {
        if (address == null) {
            address = new JTextArea(2, 20);
            address.setWrapStyleWord(true);
            address.setLineWrap(true);
            address.setBounds(new Rectangle(150, 72, 150, 36));
            address.setWrapStyleWord(true);
            address.setBorder(new JTextField().getBorder());
        }
        return address;
    }

    /**
     * This method initializes jTextPane3
     * 
     * @return javax.swing.JTextPane
     */
    private JTextPane getAddressTxt() {
        if (addressTxt == null) {
            addressTxt = new JTextPane();
            addressTxt.setBounds(new Rectangle(10, 70, 83, 22));
            addressTxt.setEditable(false);
            addressTxt.setText("Address:");
            addressTxt.setOpaque(false);
        }
        return addressTxt;
    }

    /**
     * This method initializes carPayment
     * 
     * @return javax.swing.JTextField
     */
    private JTextField getCarPayment() {
        if (carPayment == null) {
            carPayment = new JTextField();
            carPayment.setText("0");
            carPayment.setBounds(new Rectangle(150, 262, 100, 18));
        }
        return carPayment;
    }

    /**
     * This method initializes jTextPane62
     * 
     * @return javax.swing.JTextPane
     */
    private JTextPane getCarPaymentTxt() {
        if (carPaymentTxt == null) {
            carPaymentTxt = new JTextPane();
            carPaymentTxt.setBounds(new Rectangle(8, 260, 94, 22));
            carPaymentTxt.setEditable(false);
            carPaymentTxt.setText("Car payment:");
            carPaymentTxt.setOpaque(false);
        }
        return carPaymentTxt;
    }

    /**
     * This method initializes creditCards
     * 
     * @return javax.swing.JTextField
     */
    private JTextField getCreditCards() {
        if (creditCards == null) {
            creditCards = new JTextField();
            creditCards.setText("0");
            creditCards.setBounds(new Rectangle(150, 282, 100, 18));
        }
        return creditCards;
    }

    /**
     * This method initializes jTextPane621
     * 
     * @return javax.swing.JTextPane
     */
    private JTextPane getCreditCardsTxt() {
        if (creditCardsTxt == null) {
            creditCardsTxt = new JTextPane();
            creditCardsTxt.setBounds(new Rectangle(8, 280, 90, 22));
            creditCardsTxt.setEditable(false);
            creditCardsTxt.setText("Credit cards:");
            creditCardsTxt.setOpaque(false);
        }
        return creditCardsTxt;
    }

    /**
     * This method initializes custTxt
     * 
     * @return javax.swing.JTextPane
     */
    private JTextPane getCustTxt() {
        if (custTxt == null) {
            custTxt = new JTextPane();
            custTxt.setBounds(new Rectangle(8, 2, 185, 24));
            custTxt.setFont(new Font("Dialog", Font.BOLD, 14));
            custTxt.setEditable(false);
            custTxt.setFont(new Font("Dialog", Font.BOLD, 14));
            custTxt.setText("Customer Information:");
            custTxt.setOpaque(false);
        }
        return custTxt;
    }

    /**
     * This method initializes expensesTxt
     * 
     * @return javax.swing.JTextPane
     */
    private JTextPane getExpensesTxt() {
        if (expensesTxt == null) {
            expensesTxt = new JTextPane();
            expensesTxt.setBounds(new Rectangle(8, 160, 203, 24));
            expensesTxt.setText("Monthly housing expense:");
            expensesTxt.setFont(new Font("Dialog", Font.BOLD, 14));
            expensesTxt.setOpaque(false);
        }
        return expensesTxt;
    }

    /**
     * This method initializes firstName
     * 
     * @return javax.swing.JTextField
     */
    private JTextField getFirstName() {
        if (firstName == null) {
            firstName = new JTextField();
            firstName.setBounds(new Rectangle(150, 52, 150, 18));
        }
        return firstName;
    }

    /**
     * This method initializes jTextPane2
     * 
     * @return javax.swing.JTextPane
     */
    private JTextPane getFirstNameTxt() {
        if (firstNameTxt == null) {
            firstNameTxt = new JTextPane();
            firstNameTxt.setBounds(new Rectangle(10, 50, 95, 22));
            firstNameTxt.setEditable(false);
            firstNameTxt.setText("First name:");
            firstNameTxt.setOpaque(false);
        }
        return firstNameTxt;
    }

    /**
     * This method initializes insurance
     * 
     * @return javax.swing.JTextField
     */
    private JTextField getInsurance() {
        if (insurance == null) {
            insurance = new JTextField();
            insurance.setText("0");
            insurance.setBounds(new Rectangle(150, 212, 100, 18));
        }
        return insurance;
    }

    /**
     * This method initializes jTextPane61
     * 
     * @return javax.swing.JTextPane
     */
    private JTextPane getInsuranceTxt() {
        if (insuranceTxt == null) {
            insuranceTxt = new JTextPane();
            insuranceTxt.setBounds(new Rectangle(8, 211, 118, 22));
            insuranceTxt.setEditable(false);
            insuranceTxt.setText("Hazard insurance:");
            insuranceTxt.setOpaque(false);
        }
        return insuranceTxt;
    }

    /**
     * This method initializes jDesktopPane
     * 
     * @return javax.swing.JDesktopPane
     */
    private final JDesktopPane getJDesktopPane() {
        if (jDesktopPane == null) {
            jDesktopPane = new JDesktopPane();
            jDesktopPane.add(getCustTxt(), null);
            jDesktopPane.add(getLastNameTxt(), null);
            jDesktopPane.add(getFirstNameTxt(), null);
            jDesktopPane.add(getFirstName(), null);
            jDesktopPane.add(getLastName(), null);
            jDesktopPane.add(getAddressTxt(), null);
            jDesktopPane.add(getAddress(), null);
            jDesktopPane.add(getMonthlyIncomeTxt(), null);
            jDesktopPane.add(getSalaryTxt(), null);
            jDesktopPane.add(getSalary(), null);
            jDesktopPane.add(getTaxesTxt(), null);
            jDesktopPane.add(getPropertyTaxes(), null);
            jDesktopPane.add(getExpensesTxt(), null);
            jDesktopPane.add(getInsuranceTxt(), null);
            jDesktopPane.add(getInsurance(), null);
            jDesktopPane.add(getOtherExpensesTxt(), null);
            jDesktopPane.add(getCarPaymentTxt(), null);
            jDesktopPane.add(getCarPayment(), null);
            jDesktopPane.add(getCreditCardsTxt(), null);
            jDesktopPane.add(getCreditCards(), null);
            jDesktopPane.add(getOtherPaymentsTxt(), null);
            jDesktopPane.add(getOtherPayments(), null);
            jDesktopPane.add(getSubmit(), null);
            jDesktopPane.add(getReset(), null);
            jDesktopPane.add(getResult(), null);
            jDesktopPane.add(getResultTxt(), null);

            byte[] buffer = null;
            InputStream in = Console.class.getClassLoader()
                    .getResourceAsStream("Petals pwd by EBM.jpg");
            try {
                if (in == null) {
                    in = new FileInputStream("src" + File.separator + "img"
                            + File.separator + "Petals pwd by EBM.jpg");
                }
                DataInputStream dis = new DataInputStream(in);
                buffer = new byte[dis.available()];
                dis.readFully(buffer);
            } catch (Exception e) {
                JOptionPane.showMessageDialog(this, e.getMessage(), "Error",
                        JOptionPane.ERROR_MESSAGE);
            }
            ImageIcon img1 = new ImageIcon(buffer);
            JLabel background1 = new JLabel(img1);
            background1.setBounds(0, 0, img1.getIconWidth(), img1
                    .getIconHeight());
            jDesktopPane.add(background1, Integer.MIN_VALUE);
        }
        return jDesktopPane;
    }

    /**
     * This method initializes lastName
     * 
     * @return javax.swing.JTextField
     */
    private JTextField getLastName() {
        if (lastName == null) {
            lastName = new JTextField();
            lastName.setBounds(new Rectangle(150, 32, 150, 18));
        }
        return lastName;
    }

    /**
     * This method initializes lastNameTxt
     * 
     * @return javax.swing.JTextPane
     */
    private JTextPane getLastNameTxt() {
        if (lastNameTxt == null) {
            lastNameTxt = new JTextPane();
            lastNameTxt.setBounds(new Rectangle(10, 30, 84, 22));
            lastNameTxt.setEditable(false);
            lastNameTxt.setText("Last name:");
            lastNameTxt.setOpaque(false);
        }
        return lastNameTxt;
    }

    /**
     * This method initializes jTextPane4
     * 
     * @return javax.swing.JTextPane
     */
    private JTextPane getMonthlyIncomeTxt() {
        if (monthlyIncomeTxt == null) {
            monthlyIncomeTxt = new JTextPane();
            monthlyIncomeTxt.setBounds(new Rectangle(10, 110, 172, 22));
            monthlyIncomeTxt.setFont(new Font("Dialog", Font.BOLD, 14));
            monthlyIncomeTxt.setText("Monthly Income:");
            monthlyIncomeTxt.setEditable(false);
            monthlyIncomeTxt.setOpaque(false);
        }
        return monthlyIncomeTxt;
    }

    /**
     * This method initializes jTextPane411
     * 
     * @return javax.swing.JTextPane
     */
    private JTextPane getOtherExpensesTxt() {
        if (otherExpensesTxt == null) {
            otherExpensesTxt = new JTextPane();
            otherExpensesTxt.setBounds(new Rectangle(8, 237, 194, 24));
            otherExpensesTxt.setText("Other monthly expenses:");
            otherExpensesTxt.setFont(new Font("Dialog", Font.BOLD, 14));
            otherExpensesTxt.setEditable(false);
            otherExpensesTxt.setOpaque(false);
        }
        return otherExpensesTxt;
    }

    /**
     * This method initializes otherPayments
     * 
     * @return javax.swing.JTextField
     */
    private JTextField getOtherPayments() {
        if (otherPayments == null) {
            otherPayments = new JTextField();
            otherPayments.setText("0");
            otherPayments.setBounds(new Rectangle(150, 302, 100, 18));
        }
        return otherPayments;
    }

    /**
     * This method initializes jTextPane6211
     * 
     * @return javax.swing.JTextPane
     */
    private JTextPane getOtherPaymentsTxt() {
        if (otherPaymentsTxt == null) {
            otherPaymentsTxt = new JTextPane();
            otherPaymentsTxt.setBounds(new Rectangle(8, 300, 110, 22));
            otherPaymentsTxt.setEditable(false);
            otherPaymentsTxt.setText("Other payments:");
            otherPaymentsTxt.setOpaque(false);
        }
        return otherPaymentsTxt;
    }

    /**
     * This method initializes propertyTaxes
     * 
     * @return javax.swing.JTextField
     */
    private JTextField getPropertyTaxes() {
        if (propertyTaxes == null) {
            propertyTaxes = new JTextField();
            propertyTaxes.setText("0");
            propertyTaxes.setBounds(new Rectangle(150, 192, 100, 18));
        }
        return propertyTaxes;
    }

    /**
     * This method initializes reset
     * 
     * @return javax.swing.JButton
     */
    private JButton getReset() {
        if (reset == null) {
            reset = new JButton();
            reset.setBounds(new Rectangle(210, 340, 120, 28));
            reset.setText("Reset");
            reset.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent evt) {
                    resetForm();
                }
            });
        }
        return reset;
    }

    /**
     * This method initializes result
     * 
     * @return javax.swing.JTextPane
     */
    private JTextArea getResult() {
        if (result == null) {
            result = new JTextArea(2, 20);
            result.setWrapStyleWord(true);
            result.setLineWrap(true);
            result.setBounds(new Rectangle(10, 396, 290, 34));
            result.setEditable(false);
            result.setOpaque(false);
            result.setBorder(new JTextField().getBorder());
        }
        return result;
    }

    /**
     * This method initializes resultTxt
     * 
     * @return javax.swing.JTextPane
     */
    private JTextPane getResultTxt() {
        if (resultTxt == null) {
            resultTxt = new JTextPane();
            resultTxt.setBounds(new Rectangle(10, 370, 86, 24));
            resultTxt.setFont(new Font("Dialog", Font.BOLD, 14));
            resultTxt.setText("Result:");
            resultTxt.setEditable(false);
            resultTxt.setOpaque(false);
        }
        return resultTxt;
    }

    /**
     * This method initializes salary
     * 
     * @return javax.swing.JTextField
     */
    private JTextField getSalary() {
        if (salary == null) {
            salary = new JTextField();
            salary.setText("0");
            salary.setBounds(new Rectangle(150, 140, 100, 18));
        }
        return salary;
    }

    /**
     * This method initializes jTextPane5
     * 
     * @return javax.swing.JTextPane
     */
    private JTextPane getSalaryTxt() {
        if (salaryTxt == null) {
            salaryTxt = new JTextPane();
            salaryTxt.setBounds(new Rectangle(10, 140, 90, 22));
            salaryTxt.setEditable(false);
            salaryTxt.setText("Salary:");
            salaryTxt.setOpaque(false);
        }
        return salaryTxt;
    }

    /**
     * This method initializes submit
     * 
     * @return javax.swing.JButton
     */
    private JButton getSubmit() {
        if (submit == null) {
            submit = new JButton();
            submit.setBounds(new Rectangle(30, 340, 120, 26));
            submit.setText("Submit");
            submit.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent evt) {
                    submit();
                }
            });
        }
        return submit;
    }

    /**
     * This method initializes jTextPane6
     * 
     * @return javax.swing.JTextPane
     */
    private JTextPane getTaxesTxt() {
        if (taxesTxt == null) {
            taxesTxt = new JTextPane();
            taxesTxt.setBounds(new Rectangle(8, 190, 94, 22));
            taxesTxt.setText("Property taxes:");
            taxesTxt.setOpaque(false);
            taxesTxt.setEditable(false);
        }
        return taxesTxt;
    }

    private void initialize() {
        this.setSize(new Dimension(410, 460));
        this.setTitle("Petals Mortgage Demonstration");
        // this.setContentPane(getJContentPane());
        this.getContentPane().add(getJDesktopPane(), BorderLayout.CENTER);
        this.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
        this.setResizable(false);
    }

} // @jve:decl-index=0:visual-constraint="10,10"
