/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2006 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: AbstractMortgageProfiling.java 154 7 juin 2006 ofabre $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.demo.mortgage.workflow.orchestration;

import java.util.logging.Logger;

import javax.jbi.messaging.DeliveryChannel;
import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.InOnly;
import javax.jbi.messaging.InOut;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.NormalizedMessage;
import javax.xml.namespace.QName;
import javax.xml.transform.Source;

import org.objectweb.petals.component.common.HandlingException;
import org.objectweb.petals.component.common.util.SourceHelper;

public class MortgageProfilingUtil {

	public final static String DEFAULT_NS = "http://petals.objectweb.org/";

	public final static String EVALRATE_SERVICE = "EvalRateService";

	public final static String SENDSUMMARY_SERVICE = "SendSummaryService";

	public final static String EVALRATE_SERVICE_OP = "evaluate";

	public final static String OK_PROFILE = "ok";

	protected DeliveryChannel channel;

	protected Logger l;

	public MortgageProfilingUtil(DeliveryChannel channel, Logger l) {
		super();
		this.channel = channel;
		this.l = l;
	}

	protected MortgageResponse createMortgageResponse(float rate,
			MortgageRequest request) {
		MortgageResponse response = new MortgageResponse(
				request.getFirstName(), request.getLastName(), rate);
		return response;
	}

	protected float doEvalMortgageRate(String profile) throws HandlingException {

		/*
		 * Create xml content from given profile
		 */
		profile = "<profile>" + profile + "</profile>";

		String response = "RateNotDefined";
		try {
			/*
			 * Create message content from given profile
			 */
			Source source = SourceHelper.createSource(profile);

			/*
			 * Create a new message exchange for MortgageService
			 */
			MessageExchange msg = channel.createExchangeFactory()
					.createInOutExchange();
			msg.setService(new QName(DEFAULT_NS, EVALRATE_SERVICE));
			msg.setOperation(new QName(EVALRATE_SERVICE_OP));

			/*
			 * Create the inMessage to send
			 */
			NormalizedMessage inNM = msg.createMessage();
			inNM.setContent(source);
			((InOut) msg).setInMessage(inNM);

			/*
			 * Send the message exchange
			 */
			channel.sendSync(msg);

			/*
			 * Get response from EvalRate service
			 */
			NormalizedMessage outNM = ((InOut) msg).getOutMessage();
			if (outNM != null) {
				response = SourceHelper.createString(outNM.getContent());
			}

			// create string content from xml content
			response = response.substring(response.indexOf("<response>") + 10,
					response.indexOf("</response>"));
		} catch (Exception e) {
			throw new HandlingException("Error evaluating mortgage rate", e);
		}

		l.info("##### Evaluated rate : " + response + " #####");
		return new Float(response).floatValue();
	}

	protected void doSendSummary(MortgageResponse response)
			throws HandlingException {
		/*
		 * Create String summary
		 */
		String stringResponse = "ERROR";
		String firstName = response.getFirstName();

		String lastName = response.getLastName();

		float rate = response.getMortgageRate();

		if (rate < 0) {
			stringResponse = "Mortgage refused for the customer " + lastName
					+ " " + firstName;
		} else {
			stringResponse = "Mortgage accepted for the customer " + lastName
					+ " " + firstName + " with the following rate : " + rate;
		}

		/*
		 * Create xml content from string content
		 */
		stringResponse = "<response>" + stringResponse + "</response>";

		try {
			/*
			 * Create message content from given string summary
			 */
			Source source = SourceHelper.createSource(stringResponse);

			/*
			 * Create a new message exchange for SendSummaryService
			 */
			MessageExchange msg = channel.createExchangeFactory()
					.createInOnlyExchange();
			msg.setService(new QName(DEFAULT_NS, SENDSUMMARY_SERVICE));

			/*
			 * Create the inMessage to send
			 */
			NormalizedMessage inNM = msg.createMessage();
			inNM.setContent(source);
			((InOnly) msg).setInMessage(inNM);

			/*
			 * Send the message exchange
			 */
			channel.sendSync(msg);

			/*
			 * Check if summary has been correctly sent
			 */
			if (!ExchangeStatus.DONE.equals(msg.getStatus())) {
				l.warning("Profiling summary isn't correctly sent !");
			}
		} catch (Exception e) {
			throw new HandlingException("Error sending summary", e);
		}

	}
}
