/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2006 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: SimpleMortgageProfiling.java 154 7 juin 2006 ofabre $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.demo.mortgage.workflow.orchestration;

import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jbi.messaging.DeliveryChannel;
import javax.jbi.messaging.InOut;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.NormalizedMessage;
import javax.xml.namespace.QName;
import javax.xml.transform.Source;

import org.objectweb.petals.component.common.HandlingException;
import org.objectweb.petals.component.common.PEtALSComponentSDKException;
import org.objectweb.petals.component.common.util.SourceHelper;

/**
 * Workflow for simple mortgage
 * 
 * @author ofabre - EBM Websourcing
 * 
 */
public class SimpleMortgageProfiling implements MortgageProfiling {

	public final static String MORTGAGE_SERVICE = "ProfilerImplService";

	public final static String MORTGAGE_SERVICE_OP = "evaluateSimpleProfile";

	public final static String OK_PROFILE = "yes";

	protected DeliveryChannel channel;

	protected Logger l;

	protected MortgageProfilingUtil profilingUtil;

	public SimpleMortgageProfiling() {

	}

	public SimpleMortgageProfiling(DeliveryChannel channel, Logger l) {
		super();
		this.channel = channel;
		this.l = l;
		profilingUtil = new MortgageProfilingUtil(channel, l);
	}

	public MortgageResponse orchestrate(MortgageRequest request)
			throws HandlingException {
		MortgageResponse response = null;

		l.info("#################################################");
		l.info("#### PROCESSING A SIMPLE MORTGAGE RATE EVALUATION");
		l.info("#################################################");

		/*
		 * Evaluate Mortgage Profile
		 */
		l.info("##### First step : Evalute Mortgage Profile #####");
		String profile = doEvalProfile(request.getSalary(), request
				.getPropertyTaxes(), request.getInsurance(), request
				.getAutoPayment(), request.getCreditCards(), request
				.getOtherPayments());

		float rate = -1;
		if (OK_PROFILE.equals(profile)) {
			/*
			 * Evaluate Mortgage rate
			 */
			l.info("##### Second Step : Evaluate Mortgage Rate #####");
			rate = profilingUtil.doEvalMortgageRate(profile);
		}

		/*
		 * Create Mortgage Response
		 */
		response = profilingUtil.createMortgageResponse(rate, request);
		l
				.info("##### Last Step : Send the response to the Mortgage requester#####");

		/*
		 * Send a summary of the profiling
		 */
		profilingUtil.doSendSummary(response);

		return response;
	}

	protected Source createInMessageContent(float salary, float propertyTaxes,
			float insurance, float autoPayment, float creditCards,
			float otherPayments) throws PEtALSComponentSDKException {

		StringBuffer soap = new StringBuffer();
		soap.append("<ns1:" + MORTGAGE_SERVICE_OP
				+ " xmlns:ns1=\"http://petals.objectweb.org/\">");
		soap.append("<salary>" + salary + "</salary>");
		soap.append("<propertyTaxes>" + propertyTaxes + "</propertyTaxes>");
		soap.append("<insurance>" + insurance + "</insurance>");
		soap.append("<autoPayment>" + autoPayment + "</autoPayment>");
		soap.append("<creditCards>" + creditCards + "</creditCards>");
		soap.append("<otherPayments>" + otherPayments + "</otherPayments>");
		soap.append("</ns1:" + MORTGAGE_SERVICE_OP + ">");

		return SourceHelper.createSource(soap.toString());
	}

	protected String doEvalProfile(float salary, float propertyTaxes,
			float insurance, float autoPayment, float creditCards,
			float otherPayments) throws HandlingException {

		MessageExchange msg = null;
		try {
			/*
			 * Create message content from given data
			 */
			Source source = createInMessageContent(salary, propertyTaxes,
					insurance, autoPayment, creditCards, otherPayments);

			/*
			 * Create a new message exchange for MortgageService
			 */
			msg = channel.createExchangeFactory().createInOutExchange();
			msg.setService(new QName(MortgageProfilingUtil.DEFAULT_NS,
					MORTGAGE_SERVICE));
			msg.setOperation(new QName(MORTGAGE_SERVICE_OP));

			/*
			 * Create the inMessage to send
			 */
			NormalizedMessage inNM = msg.createMessage();
			inNM.setContent(source);
			((InOut) msg).setInMessage(inNM);

			/*
			 * Send the message exchange
			 */
			channel.sendSync(msg);
		} catch (Exception e) {
			throw new HandlingException("Error evaluating profile", e);
		}
		/*
		 * Get response from EvalRate service
		 */
		NormalizedMessage outNM = ((InOut) msg).getOutMessage();
		String response = "ProfileNotDefined";
		if (outNM != null) {
			try {
				response = SourceHelper.createString(outNM.getContent());
			} catch (PEtALSComponentSDKException e) {
				throw new HandlingException(e);
			}
			if (response.indexOf("return") > -1) {
				response = response.substring(response.indexOf("<return>") + 8,
						response.indexOf("</return>"));
				l.log(Level.INFO, response);
			}
		} else {
			throw new HandlingException(
					"Mortgage profiler web service response is null");
		}
		l.info("##### Evaluated profile : " + response + " #####");
		return response;
	}

}
