/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2006 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: MD5SumCalculator.java 154 2 aot 06 ofabre $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.util;

import java.io.File;
import java.io.IOException;
import java.net.URI;

import com.twmacinta.util.MD5;

/**
 * 
 * @author ofabre - EBM Websourcing
 * 
 */
public final class MD5SumCalculator {

    private MD5SumCalculator() {
        // do nothing
    }

    /**
     * Calculate the MD5Sum of an archive
     * 
     * @return the {@link String} hash code
     * @throws PetalsRuntimeException
     *             if MD5Sum calculation fails
     */
    public static String calculateMD5Sum(URI archiveURI)
        throws PetalsRuntimeException {

        String hash = null;
        try {
            hash = MD5.asHex(MD5.getHash(new File(archiveURI)));
        } catch (IOException e) {
            String msg = "Error when calculating MD5 sum of the archive : "
                    + archiveURI + ".";
            throw new PetalsRuntimeException(msg, e);
        }
        return hash;
    }

}
