/**
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005 EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): Adrien LOUIS
 * --------------------------------------------------------------------------
 * $Id: SourceHelper.java 94 2006-03-26 17:11:05Z alouis $
 * --------------------------------------------------------------------------  
 */

package org.objectweb.petals.util;

import java.io.BufferedInputStream;
import java.io.ByteArrayInputStream;
import java.io.StringWriter;
import java.io.UnsupportedEncodingException;
import java.util.Properties;

import javax.xml.transform.OutputKeys;
import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.TransformerFactoryConfigurationError;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

/**
 * Sample class used to manage the mapping between
 * <code>javax.xml.transform.Source</code> from String. the mapping is a
 * quicly "string" -> "<content>string</content>" conversion.
 * 
 * @author Adrien LOUIS - EBM WebSourcing
 * @author wjoseph - eBMWebSourcing
 */
public final class SourceHelper {

    private static Transformer transformer;

    private SourceHelper() {
        // avoid to be built
    }

    /**
     * Create a Source from the String.
     * The string must be in UTF-8 format, otherwise, the Source is not created.
     * @param msg an UTF-8 String 
     * @return the Source built from the String, null if not an UTF-8
     */
    public static Source createSource(String msg) {
        StreamSource source = new StreamSource();
        try {
            byte[] msgByte= msg.getBytes("UTF-8");
            ByteArrayInputStream in = new ByteArrayInputStream(msgByte);
            source.setInputStream(in);
        } catch (UnsupportedEncodingException e) {
            // nothing, return null
        }
        return source;
    }

    public static String createString(Source s) throws Exception {

        String result = null;

        if (s != null) {

                if (s instanceof StreamSource) {
                    StreamSource ss = (StreamSource) s;
                    if (ss.getInputStream() != null) {
                        if (ss.getInputStream().markSupported()) {
                                ss.getInputStream().reset();
                            }
                            byte[] buf = new byte[ss.getInputStream().available()];
                            BufferedInputStream bis = new BufferedInputStream(ss
                                    .getInputStream());
                            bis.read(buf);
                            result = new String(buf);
                    }
                    else if (ss.getReader() != null) {
                        StringBuffer stringBuffer = new StringBuffer();
                        int readThisTime = 0;
                        char[] cbuf = new char[4096];
                        while (readThisTime != -1) {
                                readThisTime = ss.getReader().read(cbuf, 0, 4096);
                                if (readThisTime!= -1) {
                                        stringBuffer.append(cbuf, 0, readThisTime);
                                }
                        }
                        result = stringBuffer.toString();
                    }
                    
                } else {

                    StringWriter buffer = new StringWriter();
                    Result sresult = new StreamResult(buffer);
                    Transformer transform = getTransformer();
                    transform.transform(s, sresult);
                    result = buffer.toString();
                }
        }
        
        return result;
    }
    public static Transformer getTransformer()
        throws TransformerConfigurationException,
        TransformerFactoryConfigurationError {
        if (transformer == null) {
            transformer = TransformerFactory.newInstance().newTransformer();
            Properties props = new Properties();
            props.put(OutputKeys.OMIT_XML_DECLARATION, "yes");
            transformer.setOutputProperties(props);
        }
        return transformer;
    }

}
