/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: WSDLUtil.java 14:50:31 ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.util;

import java.util.ArrayList;
import java.util.List;

import javax.xml.namespace.QName;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
 * This is used to extract the interfaces name of a WSDL file
 * 
 * @author ddesjardins - eBMWebsourcing
 */
public final class WSDLUtil {

    protected static final String WSDL1_1_NAMESPACE = "http://schemas.xmlsoap.org/wsdl";

    protected static final String WSDL2_0_NAMESPACE = "http://www.w3.org/2006/01/wsdl";

    private WSDLUtil() {
        // avoid to be built
    }

    public static List<QName> getInterfacesForService(Document document,
            QName serviceName) {
        List<QName> interfaces = new ArrayList<QName>();

        // Retreive the namespace
        String targetNameSpace = document.getDocumentElement().getAttribute(
                "targetNamespace");
        String nodeName = document.getDocumentElement().getNodeName();
        String wsdlPrefix = null;
        int idx = nodeName.indexOf(':');
        if (idx > -1) {
            wsdlPrefix = nodeName.substring(0, idx);
        }
        if ("".equals(wsdlPrefix) || wsdlPrefix == null) {
            wsdlPrefix = "xmlns";
        } else {
            wsdlPrefix = "xmlns:" + wsdlPrefix;
        }
        String wsdlNamespace = document.getDocumentElement().getAttribute(
                wsdlPrefix);
        if (targetNameSpace != null
                && serviceName.getNamespaceURI().equals(targetNameSpace)) {

            Element rootElement = document.getDocumentElement();
            String prefix = "";
            String rootName = rootElement.getNodeName();
            if (rootName.indexOf(":") > -1) {
                prefix = rootName.substring(0, rootName.indexOf(":")) + ":";
            }

            List<String> bindingNames = new ArrayList<String>();
            // Retreive the services
            NodeList services = rootElement.getElementsByTagName(prefix
                    + "service");

            if (wsdlNamespace.contains(WSDL1_1_NAMESPACE)) {
                /*
                 * Search the interface in a WSDL 1.1
                 */
                for (int i = 0; i < services.getLength(); i++) {
                    Element service = (Element) services.item(i);
                    if (service.getAttribute("name").equals(
                            serviceName.getLocalPart())) {
                        NodeList ports = service.getElementsByTagName(prefix
                                + "port");
                        for (int j = 0; j < ports.getLength(); j++) {
                            Element port = (Element) ports.item(j);
                            String name = port.getAttribute("binding");
                            if (name.indexOf(":") > -1) {
                                name = name.substring(name.indexOf(":") + 1);
                            }
                            bindingNames.add(name);
                        }
                    }
                }

                List<String> portNames = new ArrayList<String>();
                // Retreive the bindings
                NodeList bindings = rootElement.getElementsByTagName(prefix
                        + "binding");
                for (int i = 0; i < bindings.getLength(); i++) {
                    Element binding = (Element) bindings.item(i);
                    String name = binding.getAttribute("name");
                    if (bindingNames.contains(name)) {
                        String type = binding.getAttribute("type");
                        if (type.indexOf(":") > -1) {
                            type = type.substring(type.indexOf(":") + 1);
                        }
                        portNames.add(type);
                    }
                }

                // Retreive the port types
                NodeList portTypes = rootElement.getElementsByTagName(prefix
                        + "portType");
                for (int i = 0; i < portTypes.getLength(); i++) {
                    Element portType = (Element) portTypes.item(i);
                    String name = portType.getAttribute("name");
                    if (portNames.contains(name)) {
                        interfaces.add(new QName(targetNameSpace, name));
                    }
                }
            } else {
                if (wsdlNamespace.contains(WSDL2_0_NAMESPACE)) {
                    /*
                     * Search the interface in a WSDL 2.0
                     */
                    for (int i = 0; i < services.getLength(); i++) {
                        Element service = (Element) services.item(i);
                        if (service.getAttribute("name").equals(
                                serviceName.getLocalPart())) {
                            String itf = QName.valueOf(service
                                .getAttribute("interface")).getLocalPart();
                            int prefixIndex = itf.indexOf(':');
                            if(prefixIndex>0){
                                itf = itf.substring(prefixIndex+1,itf.length());
                            }
                            interfaces.add(new QName(targetNameSpace,itf));
                        }
                    }
                }
            }
        }

        return interfaces;
    }

}
