/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: SystemUtilTest.java 10:27:27 ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.util;

import java.io.File;

import junit.framework.TestCase;

/**
 * Test of the SystemUtil
 * 
 * @author ddesjardins - eBMWebsourcing
 */
public class SystemUtilTest extends TestCase {

    public void setUp() {
        SystemUtil.petalsInstallDirectory = null;
    }

    public void testGetPetalsInstallDirectory() {
        File dir = SystemUtil.petalsInstallDirectory;
        assertNull(dir);
    }

    public void testResolveInstallDirectoryFromEnvironment() {
        File dir = SystemUtil.resolveInstallDirectoryFromEnvironment();
        if (System.getenv().get("PETALS_HOME") != null) {
            assertNotNull(dir);
        } else {
            assertNull(dir);
        }
    }

    public void testResolveInstallDirectoryFromEnvironment1() {
        System.setProperty("petals.home", new File(".").getAbsolutePath());
        File dir = SystemUtil.resolveInstallDirectoryFromEnvironment();
        assertNotNull(dir);
    }

    public void testResolveInstallDirectoryFromFramework() {
        File dir = SystemUtil.resolveInstallDirectoryFromFramework();
        assertNotNull(dir);
    }

    public void testGettersSetters() {
        assertNotNull(SystemUtil.getContainerName());
        SystemUtil.setContainerName("cont");
        assertEquals(SystemUtil.getContainerName(), "cont");

        assertNotNull(SystemUtil.getContainerUID());
        SystemUtil.setContainerUID(1);
        assertEquals(SystemUtil.getContainerUID(), 1);

        assertNotNull(SystemUtil.getHost());
        SystemUtil.setHost("127.0.0.1");
        assertEquals(SystemUtil.getHost(), "127.0.0.1");

        assertNotNull(SystemUtil.getHtmlPort());
        SystemUtil.setHtmlPort("8082");
        assertEquals(SystemUtil.getHtmlPort(), "8082");

        assertNotNull(SystemUtil.getJmxPassword());
        SystemUtil.setJmxPassword("pass");
        assertEquals(SystemUtil.getJmxPassword(), "pass");

        assertNotNull(SystemUtil.getJmxPort());
        SystemUtil.setJmxPort("8081");
        assertEquals("8081", SystemUtil.getJmxPort());

        assertNotNull(SystemUtil.getJmxUser());
        SystemUtil.setJmxUser("user");
        assertEquals(SystemUtil.getJmxUser(), "user");

        assertNotNull(SystemUtil.getJndiFactory());
        SystemUtil.setJndiFactory("test");
        assertEquals(SystemUtil.getJndiFactory(), "test");

        assertNotNull(SystemUtil.getJndiPort());
        SystemUtil.setJndiPort("16400");
        assertEquals(SystemUtil.getJndiPort(), "16400");

        assertNotNull(SystemUtil.getJoramDomain());
        SystemUtil.setJoramDomain("test");
        assertEquals(SystemUtil.getJoramDomain(), "test");

        assertNotNull(SystemUtil.getJoramDomainPort());
        SystemUtil.setJoramDomainPort("16200");
        assertEquals(SystemUtil.getJoramDomainPort(), "16200");

        assertNotNull(SystemUtil.getJoramId());
        SystemUtil.setJoramId("1");
        assertEquals(SystemUtil.getJoramId(), "1");

        assertNotNull(SystemUtil.getJoramPassword());
        SystemUtil.setJoramPassword("foo");
        assertEquals(SystemUtil.getJoramPassword(), "foo");

        assertNotNull(SystemUtil.getJoramTCPPort());
        SystemUtil.setJoramTCPPort("16300");
        assertEquals(SystemUtil.getJoramTCPPort(), "16300");

        assertNotNull(SystemUtil.getJoramUser());
        SystemUtil.setJoramUser("bar");
        assertEquals(SystemUtil.getJoramUser(), "bar");

        SystemUtil.setPetalsInstallDirectory(new File("."));
        assertEquals(SystemUtil.getPetalsInstallDirectory(), new File("."));

        assertNotNull(SystemUtil.getTribeMulticastAddress());
        SystemUtil.setTribeMulticastAddress("214.0.0.1");
        assertEquals(SystemUtil.getTribeMulticastAddress(), "214.0.0.1");

        SystemUtil.setExchangeValidation("true");
        assertTrue(SystemUtil.isExchangeValidation());
    }
}
