/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: ZipUtilTest.java 15:35:28 ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.util;

import java.io.File;
import java.io.IOException;
import java.net.URISyntaxException;
import java.util.zip.ZipFile;

import junit.framework.TestCase;

import org.objectweb.petals.PetalsException;

/**
 * Class used to test the zip util
 * 
 * @author ddesjardins - eBMWebsourcing
 */
public class ZipUtilTest extends TestCase {

    /**
     * Base directory of the project
     */
    private String baseDir;

    /**
     * Set up the base directory for the test
     */
    public void setUp() {
        baseDir = this.getClass().getResource(".").toString();
        baseDir = baseDir.substring(0, baseDir.indexOf("target"));
        baseDir = baseDir.substring(baseDir.indexOf(":") + 1);
    }

    /**
     * Try to extract a zip into a directory
     * 
     * @throws PetalsException
     * @throws IOException
     */
    public void testExplodeZip() throws PetalsException, IOException {
        File extractedDir = new File(baseDir + "src" + File.separator
            + "test-data" + File.separator + "extracted");
        extractedDir.delete();
        extractedDir.mkdir();
        ZipUtil.explodeIntoDirectory(new ZipFile(baseDir + "src"
            + File.separator + "test-data" + File.separator
            + "test-ziputil.zip"), extractedDir);
        if (extractedDir.listFiles().length != 3) {
            fail();
        }
    }

    /**
     * Test exception throwing
     * 
     * @throws PetalsException
     * @throws IOException
     */
    public void testExplodeZip1() {
        File extractedDir = new File(baseDir + "src" + File.separator
            + "test-data" + File.separator + "extracted");
        try {
            ZipUtil.explodeIntoDirectory(null, extractedDir);
            fail();
        } catch (Exception e) {

        }
    }

    /**
     * Test exception throwing
     * 
     * @throws PetalsException
     * @throws IOException
     */
    public void testExplodeZip2() {
        try {
            ZipUtil.explodeIntoDirectory(new ZipFile(baseDir + "src"
                + File.separator + "test-data" + File.separator
                + "test-ziputil.zip"), null);
            fail();
        } catch (Exception e) {

        }
    }

    /**
     * Try to get an entry of a zip as a temporary file
     * 
     * @throws IOException
     */
    public void testGetEntryAsTemp() throws IOException {
        assertNotNull(ZipUtil.getEntryAsTemp(new ZipFile(baseDir + "src"
            + File.separator + "test-data" + File.separator
            + "test-ziputil.zip"), "META-INF/jbi.xml"));
    }

    /**
     * Try to get an entry that does not exist in zip as a temporary file
     * 
     * @throws IOException
     */
    public void testGetEntryAsTemp1() throws IOException {
        File entryFile = ZipUtil.getEntryAsTemp(new ZipFile(baseDir + "src"
            + File.separator + "test-data" + File.separator
            + "test-ziputil.zip"), "META-INF" + File.separator + "manifest.mf");
        if (entryFile != null) {
            fail();
        }
    }

    /**
     * Test exception throwing
     * 
     */
    public void testGetEntryAsTemp2() {
        try {
            ZipUtil.getEntryAsTemp(null, "META-INF" + File.separator
                + "manifest.mf");
            fail();
        } catch (Exception e) {

        }
    }

    /**
     * Test exception throwing
     * 
     */
    public void testGetEntryAsTemp3() {
        try {
            ZipUtil.getEntryAsTemp(new ZipFile(baseDir + "src" + File.separator
                + "test-data" + File.separator + "test-ziputil.zip"), null);
            fail();
        } catch (Exception e) {
        }
    }

    /**
     * Test exception throwing
     * 
     */
    public void testGetEntryAsTemp4() {
        try {
            ZipUtil.getEntryAsTemp(new ZipFile(baseDir + "src" + File.separator
                + "test-data" + File.separator + "test-ziputil.zip"), "");
            fail();
        } catch (Exception e) {
        }
    }

    /**
     * Test if a zip file has a specific entry (the entry exists)
     * 
     * @throws IOException
     */
    public void testHasEntry() throws IOException {
        assertTrue(ZipUtil.hasEntry(new ZipFile(baseDir + "src"
            + File.separator + "test-data" + File.separator
            + "test-ziputil.zip"), "META-INF/jbi.xml"));
    }

    /**
     * Test if a zip file has a specific entry (the entry does not exist)
     * 
     * @throws IOException
     */
    public void testHasEntry1() throws IOException {
        assertFalse(ZipUtil.hasEntry(new ZipFile(baseDir + "src"
            + File.separator + "test-data" + File.separator
            + "test-ziputil.zip"), "test.xml"));
    }

    /**
     * Try to open a zip file from a URI
     * 
     * @throws PetalsException
     * @throws URISyntaxException
     */
    public void testOpenZipFile() throws PetalsException, URISyntaxException {
        assertNotNull(ZipUtil.openZipFile(new File(baseDir + "src"
            + File.separator + "test-data" + File.separator
            + "test-ziputil.zip").toURI()));
    }

}
