/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: DeploymentServiceImpl.java 154 2006-03-27 15:30:10Z ofabre $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.jbi.management.deployment;

import java.net.URI;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.jbi.management.AdminServiceMBean;
import javax.jbi.management.LifeCycleMBean;

import org.objectweb.fractal.fraclet.annotation.FractalComponent;
import org.objectweb.fractal.fraclet.annotation.Interface;
import org.objectweb.fractal.fraclet.annotation.LifeCycle;
import org.objectweb.fractal.fraclet.annotation.LifeCycleType;
import org.objectweb.fractal.fraclet.annotation.Monolog;
import org.objectweb.fractal.fraclet.annotation.Provides;
import org.objectweb.fractal.fraclet.annotation.Requires;
import org.objectweb.petals.PetalsException;
import org.objectweb.petals.jbi.component.lifecycle.ComponentLifeCycle;
import org.objectweb.petals.jbi.component.lifecycle.LifeCycleAbstract;
import org.objectweb.petals.jbi.component.lifecycle.ServiceAssemblyLifeCycle;
import org.objectweb.petals.jbi.management.service.EndpointService;
import org.objectweb.petals.jbi.management.service.LifeCycleManagerService;
import org.objectweb.petals.jbi.management.service.PackageHandler;
import org.objectweb.petals.jbi.management.service.util.XMLResult;
import org.objectweb.petals.jbi.management.service.util.XMLResult.CauseFramework;
import org.objectweb.petals.jbi.management.service.util.XMLResult.MessageType;
import org.objectweb.petals.jbi.management.service.util.XMLResult.TaskResult;
import org.objectweb.petals.jbi.management.systemstate.SystemState;
import org.objectweb.petals.processor.TaskProcessor;
import org.objectweb.petals.repository.RepositoryService;
import org.objectweb.petals.tools.jbicommon.descriptor.JBIDescriptorBuilder;
import org.objectweb.petals.tools.jbicommon.descriptor.ServiceUnit;
import org.objectweb.petals.util.LoggingUtil;
import org.objectweb.petals.util.ParameterCheckHelper;
import org.objectweb.util.monolog.api.Logger;

/**
 * JBI deployment service implementation. TODO map recovery service with fractal
 * 
 * @author Olivier Fabre - EBM Websourcing
 * @author ddesjardins - eBMWebsourcing
 * @author wjoseph - eBMWebsourcing
 */
@FractalComponent
@Provides(interfaces = @Interface(name = "service", signature = org.objectweb.petals.jbi.management.deployment.DeploymentService.class))
public class DeploymentServiceImpl implements DeploymentService {

	/**
	 * JBI Container :: Admin Service
	 */
	@Requires(name = "admin", signature = javax.jbi.management.AdminServiceMBean.class)
	protected AdminServiceMBean adminSrv;

	/**
	 * List of already deployed service assembly.
	 */
	protected Map<String, ServiceAssemblyDataHandler> deployedSACache;

	@Requires(name = "endpoint-service", signature = org.objectweb.petals.jbi.management.service.EndpointService.class)
	protected EndpointService endpointService;

	/**
	 * logger wrapper
	 */
	protected LoggingUtil log;

	/**
	 * Monolog Logger instance.
	 */
	@Monolog(name = "logger")
	protected Logger logger;

	/**
	 * JBI Container :: JMX Agent Admin
	 */
	@Requires(name = "lifecyclemanager", signature = org.objectweb.petals.jbi.management.service.LifeCycleManagerService.class)
	protected LifeCycleManagerService managerService;

	/**
	 * Package Handler
	 */
	protected PackageHandler packageHandler;

	/**
	 * JBI Container :: Recovery Service
	 */
	@Requires(name = "systemstate", signature = org.objectweb.petals.jbi.management.systemstate.SystemState.class)
	protected SystemState recoverySrv;

	/**
	 * Platform Component :: Repository Service
	 */
	@Requires(name = "repository", signature = org.objectweb.petals.repository.RepositoryService.class)
	protected RepositoryService repositorySrv;

	protected DeploymentTaskFactory deploymentTaskFactory;

	/**
	 * Default constructor.
	 */
	public DeploymentServiceImpl() {
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.jbi.management.DeploymentServiceMBean#canDeployToComponent(java.lang.String)
	 */
	public boolean canDeployToComponent(String componentName) {
		ParameterCheckHelper.isNullParameterWithLog(componentName,
				"componentName", log);
		ParameterCheckHelper.isEmptyParameterWithLog(componentName,
				"componentName", log);
		boolean result = false;
		ComponentLifeCycle clc = managerService
				.getComponentByName(componentName);
		result = clc != null
				&& LifeCycleAbstract.STARTED.equals(clc.getCurrentState())
				&& clc.getComponent().getServiceUnitManager() != null;
		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.jbi.management.DeploymentServiceMBean#deploy(java.lang.String)
	 */
	public String deploy(String serviceAssemblyZipUrl) throws Exception {
		log.start();
		String result = null;

		/*
		 * Checking parameters
		 */
		ParameterCheckHelper.isNullParameterWithLog(serviceAssemblyZipUrl,
				"serviceAssemblyZipUrl", log);
		ParameterCheckHelper.isEmptyParameterWithLog(serviceAssemblyZipUrl,
				"serviceAssemblyZipUrl", log);

		try {

			/*
			 * Create deployment execution context
			 */
			HashMap<String, Object> deploymentContext = createDeploymentContext(serviceAssemblyZipUrl);

			/*
			 * Create deployment processor and fill with tasks
			 */
			TaskProcessor processor = createDeploymentProcessor(deploymentContext);

			/*
			 * Launch the process
			 */
			if (!processor.process()) {
				String msg = "An error occured during the deployment "
						+ "of the following service assembly: "
						+ serviceAssemblyZipUrl;
				throw new Exception(msg);
			}

			/*
			 * Create the string result
			 */
			result = ((XMLResult) deploymentContext
					.get(DeploymentContextConstants.XML_RESULT)).getString();

		} catch (Throwable e) {
			// Catch all throwables to prevent exotic exception type (not easily
			// deserializable) throwing
			log.error(e.getMessage(), e);
			throw new Exception(e.getMessage());
		}

		//log.info(result);
		log.end();
		return result;
	}

	/**
	 * Create the deployment processor and add all required task for the
	 * complete deployment of a service assembly
	 * 
	 * @param deploymentContext
	 *            the deployment context
	 * @return the filled deployment processor
	 */
	protected TaskProcessor createDeploymentProcessor(
			HashMap<String, Object> deploymentContext) {
		TaskProcessor processor = new TaskProcessor(deploymentContext, log);

		processor.addTask(deploymentTaskFactory
				.getDescriptorAndURIRetrievalTask());

		processor.addTask(deploymentTaskFactory.getPackageCheckingTask());

		processor.addTask(deploymentTaskFactory.getResultCreationTask());
		processor.addTask(deploymentTaskFactory.getExtractionTask());
		processor.addTask(deploymentTaskFactory
				.getAllConnectionRegistrationTask());

		processor.addTask(deploymentTaskFactory.getAllSUDeploymentTask());
		processor.addTask(deploymentTaskFactory
				.getSaLifeCycleRegistrationTask());

		processor.addTask(deploymentTaskFactory.getStateHolderCreationTask());
		processor.addTask(deploymentTaskFactory
				.getCopyArchiveToInstalledDirTask());

		processor.addTask(deploymentTaskFactory.getDeploymentSuccessTask());

		return processor;
	}

	/**
	 * Create the deployment context
	 * 
	 * @param serviceAssemblyZipUrl
	 *            String containing the location URL of the Service Assembly ZIP
	 *            file; must be non-null, non-empty, and a legal URL.
	 * @return the Deployment Context HashMap
	 */
	protected HashMap<String, Object> createDeploymentContext(
			String serviceAssemblyZipUrl) {
		HashMap<String, Object> deploymentContext = new HashMap<String, Object>();
		deploymentContext.put(DeploymentContextConstants.ARCHIVE_URL,
				serviceAssemblyZipUrl);
		return deploymentContext;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.jbi.management.DeploymentServiceMBean#getComponentsForDeployedServiceAssembly(java.lang.String)
	 */
	public String[] getComponentsForDeployedServiceAssembly(
			String serviceAssemblyName) throws Exception {
		ParameterCheckHelper.isNullParameterWithLog(serviceAssemblyName,
				"serviceAssemblyName", log);
		ParameterCheckHelper.isEmptyParameterWithLog(serviceAssemblyName,
				"serviceAssemblyName", log);
		List<String> result = new ArrayList<String>();
		ServiceAssemblyLifeCycle saLifeCycle = managerService
				.getServiceAssemblyByName(serviceAssemblyName);
		if (saLifeCycle != null) {
			List<ServiceUnit> sus = saLifeCycle.getServiceAssembly()
					.getServiceUnits();
			for (ServiceUnit unit : sus) {
				result.add(unit.getTargetComponentName());
			}
		} else {
			String msg = "Cannot retrieve service assembly with the specified name : "
					+ serviceAssemblyName;
			log.error(msg);
			throw new Exception(msg);
		}
		return result.toArray(new String[0]);
	}

	public Map<String, ServiceAssemblyDataHandler> getDeployedSACache() {
		return deployedSACache;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.jbi.management.DeploymentServiceMBean#getDeployedServiceAssemblies()
	 */
	public String[] getDeployedServiceAssemblies() throws Exception {
		Set<String> sas = managerService.getServiceAssemblyLifeCycles()
				.keySet();
		return sas.toArray(new String[0]);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.jbi.management.DeploymentServiceMBean#getDeployedServiceAssembliesForComponent(java.lang.String)
	 */
	public String[] getDeployedServiceAssembliesForComponent(
			String componentName) throws Exception {
		ParameterCheckHelper.isNullParameterWithLog(componentName,
				"componentName", log);
		ParameterCheckHelper.isEmptyParameterWithLog(componentName,
				"componentName", log);
		Set<String> result = new HashSet<String>();
		Map<String, ServiceAssemblyLifeCycle> saLifeCycles = managerService
				.getServiceAssemblyLifeCycles();
		for (ServiceAssemblyLifeCycle salc : saLifeCycles.values()) {
			List<ServiceUnit> sus = salc.getServiceAssembly().getServiceUnits();
			for (ServiceUnit unit : sus) {
				if (componentName.equals(unit.getTargetComponentName())) {
					result.add(salc.getServiceAssembly().getIdentification()
							.getName());
				}
			}
		}
		return result.toArray(new String[0]);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.jbi.management.DeploymentServiceMBean#getDeployedServiceUnitList(java.lang.String)
	 */
	public String[] getDeployedServiceUnitList(String componentName)
			throws Exception {
		ParameterCheckHelper.isNullParameterWithLog(componentName,
				"componentName", log);
		ParameterCheckHelper.isEmptyParameterWithLog(componentName,
				"componentName", log);
		Set<String> result = new HashSet<String>();
		Map<String, ServiceAssemblyLifeCycle> saLifeCycles = managerService
				.getServiceAssemblyLifeCycles();
		for (ServiceAssemblyLifeCycle salc : saLifeCycles.values()) {
			List<ServiceUnit> sus = salc.getServiceAssembly().getServiceUnits();
			for (ServiceUnit unit : sus) {
				if (componentName.equals(unit.getTargetComponentName())) {
					result.add(unit.getIdentification().getName());
				}
			}
		}
		return result.toArray(new String[0]);
	}

	public String getServiceAssemblyDescriptor(String serviceAssemblyName)
			throws Exception {
		ParameterCheckHelper.isNullParameterWithLog(serviceAssemblyName,
				"serviceAssemblyName", log);
		ParameterCheckHelper.isEmptyParameterWithLog(serviceAssemblyName,
				"serviceAssemblyName", log);
		String result = "Service Assembly doesn't exist :"
				+ serviceAssemblyName;
		ServiceAssemblyLifeCycle saLifeCycle = managerService
				.getServiceAssemblyByName(serviceAssemblyName);
		if (saLifeCycle != null) {
			URI installationRoot = saLifeCycle.getInstallationRoot();
			// Load JBI descriptor as string
			URI jbiXmlUri = installationRoot
					.resolve(JBIDescriptorBuilder.JBI_DESCRIPTOR_RESOURCE
							.replace('\\', '/'));
			result = getJbiDescriptorAsString(jbiXmlUri);
		}
		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.jbi.management.DeploymentServiceMBean#getState(java.lang.String)
	 */
	public String getState(String serviceAssemblyName) throws Exception {
		ParameterCheckHelper.isNullParameterWithLog(serviceAssemblyName,
				"serviceAssemblyName", log);
		ParameterCheckHelper.isEmptyParameterWithLog(serviceAssemblyName,
				"serviceAssemblyName", log);
		String result = null;
		String msg;
		ServiceAssemblyLifeCycle saLifeCycle = managerService
				.getServiceAssemblyByName(serviceAssemblyName);
		if (saLifeCycle != null) {
			result = saLifeCycle.getCurrentState();
		} else {
			msg = "Cannot retrieve service "
					+ "assembly with the specified name : "
					+ serviceAssemblyName;
			log.error(msg);
			throw new Exception(msg);
		}
		if (!(LifeCycleMBean.SHUTDOWN.equals(result)
				|| LifeCycleMBean.STARTED.equals(result) || LifeCycleMBean.STOPPED
				.equals(result))) {
			msg = "Service Assembly state isn't correctly defined";
			log.error(msg);
			throw new Exception(msg);
		}
		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.jbi.management.DeploymentServiceMBean#isDeployedServiceUnit(java.lang.String,
	 *      java.lang.String)
	 */
	public boolean isDeployedServiceUnit(String componentName,
			String serviceUnitName) throws Exception {
		ParameterCheckHelper.isNullParameterWithLog(serviceUnitName,
				"serviceUnitName", log);
		ParameterCheckHelper.isEmptyParameterWithLog(serviceUnitName,
				"serviceUnitName", log);
		ParameterCheckHelper.isNullParameterWithLog(componentName,
				"componentName", log);
		ParameterCheckHelper.isEmptyParameterWithLog(componentName,
				"componentName", log);
		boolean result = false;
		Map<String, ServiceAssemblyLifeCycle> saLifeCycles = managerService
				.getServiceAssemblyLifeCycles();
		for (Iterator iter = saLifeCycles.values().iterator(); iter.hasNext()
				&& !result;) {
			ServiceAssemblyLifeCycle salc = (ServiceAssemblyLifeCycle) iter
					.next();
			List<ServiceUnit> sus = salc.getServiceAssembly().getServiceUnits();
			for (Iterator iter2 = sus.iterator(); iter2.hasNext() && !result;) {
				ServiceUnit unit = (ServiceUnit) iter2.next();
				if (componentName.equals(unit.getTargetComponentName())
						&& serviceUnitName.equals(unit.getIdentification()
								.getName())) {
					result = true;
				}
			}
		}
		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.jbi.management.DeploymentServiceMBean#shutDown(java.lang.String)
	 */
	public String shutDown(String serviceAssemblyName) throws Exception {
		ParameterCheckHelper.isNullParameterWithLog(serviceAssemblyName,
				"serviceAssemblyName", log);
		ParameterCheckHelper.isEmptyParameterWithLog(serviceAssemblyName,
				"serviceAssemblyName", log);
		log.start();

		String msg = null;
		XMLResult xmlResult = new XMLResult("shutdown");

		ServiceAssemblyLifeCycle saLifeCycle = managerService
				.getServiceAssemblyByName(serviceAssemblyName);
		if (saLifeCycle != null) {
			try {
				saLifeCycle.shutDown();
			} catch (Throwable e) {
				msg = e.getMessage();
				log.error(msg, e);
				throw new Exception(msg);
			}
		} else {
			msg = "The service assembly \"" + serviceAssemblyName
					+ "\" is not deployed";
			throw new Exception(msg);
		}
		xmlResult.addFrameworkTaskResult(TaskResult.SUCCESS, MessageType.INFO,
				"Successfully shutdown service assembly : {1}",
				new String[] { serviceAssemblyName }, CauseFramework.YES);
		String output = xmlResult.getString();
		try {
			XMLResult.validateManagementMessage(output, this.getClass()
					.getClassLoader().getResourceAsStream(
							"schema/management-message.xsd"));
		} catch (Exception e1) {
			log.error("Problem while validating management message", e1);
		}
		log.info(serviceAssemblyName+" successfully shutdown");
		log.end();
		return output;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.jbi.management.DeploymentServiceMBean#start(java.lang.String)
	 */
	public String start(String serviceAssemblyName) throws Exception {
		ParameterCheckHelper.isNullParameterWithLog(serviceAssemblyName,
				"serviceAssemblyName", log);
		ParameterCheckHelper.isEmptyParameterWithLog(serviceAssemblyName,
				"serviceAssemblyName", log);
		log.start();

		String msg = null;
		XMLResult xmlResult = new XMLResult("start");

		ServiceAssemblyLifeCycle saLifeCycle = managerService
				.getServiceAssemblyByName(serviceAssemblyName);
		if (saLifeCycle != null) {
			try {
				saLifeCycle.start();
			} catch (Exception e) {
				e.printStackTrace();
				msg = e.getMessage();
				throw new Exception(msg);
			}
		} else {
			msg = "The service assembly \"" + serviceAssemblyName
					+ "\" is not deployed";
			throw new Exception(msg);
		}
		xmlResult.addFrameworkTaskResult(TaskResult.SUCCESS, MessageType.INFO,
				"Successfully started service assembly : {1}",
				new String[] { serviceAssemblyName }, CauseFramework.YES);
		String output = xmlResult.getString();
		try {
			XMLResult.validateManagementMessage(output, this.getClass()
					.getClassLoader().getResourceAsStream(
							"schema/management-message.xsd"));
		} catch (Exception e1) {
			log.error("Problem while validating management message", e1);
		}

        log.info(serviceAssemblyName+" successfully started");
		log.end();
		return output;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.jbi.management.DeploymentServiceMBean#stop(java.lang.String)
	 */
	public String stop(String serviceAssemblyName) throws Exception {
		ParameterCheckHelper.isNullParameterWithLog(serviceAssemblyName,
				"serviceAssemblyName", log);
		ParameterCheckHelper.isEmptyParameterWithLog(serviceAssemblyName,
				"serviceAssemblyName", log);
		log.start();

		String msg = null;
		XMLResult xmlResult = new XMLResult("stop");

		ServiceAssemblyLifeCycle saLifeCycle = managerService
				.getServiceAssemblyByName(serviceAssemblyName);
		if (saLifeCycle != null) {
			try {
				saLifeCycle.stop();
			} catch (Throwable e) {
				msg = e.getMessage();
				log.error(msg, e);
				throw new Exception(msg);
			}
		} else {
			msg = "The service assembly \"" + serviceAssemblyName
					+ "\" is not deployed";
			throw new Exception(msg);
		}
		xmlResult.addFrameworkTaskResult(TaskResult.SUCCESS, MessageType.INFO,
				"Successfully stopped service assembly : {1}",
				new String[] { serviceAssemblyName }, CauseFramework.YES);
		String output = xmlResult.getString();
		try {
			XMLResult.validateManagementMessage(output, this.getClass()
					.getClassLoader().getResourceAsStream(
							"schema/management-message.xsd"));
		} catch (Exception e1) {
			log.error("Problem while validating management message", e1);
		}

        log.info(serviceAssemblyName+" successfully stoped");
		log.end();
		return output;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.jbi.management.DeploymentServiceMBean#undeploy(java.lang.String)
	 */
	public String undeploy(String serviceAssemblyName) throws Exception {
		log.start();

		String output = "";

		ParameterCheckHelper.isNullParameterWithLog(serviceAssemblyName,
				"serviceAssemblyZipUrl", log);
		ParameterCheckHelper.isEmptyParameterWithLog(serviceAssemblyName,
				"serviceAssemblyZipUrl", log);
		try {

			/*
			 * Create undeployment execution context
			 */
			HashMap<String, Object> undeploymentContext = createUndeploymentContext(serviceAssemblyName);

			/*
			 * Create undeployment processor and fill with tasks
			 */
			TaskProcessor processor = createUndeploymentProcessor(undeploymentContext);

			/*
			 * Launch the process
			 */
			if (!processor.process()) {
				String msg = "An error occured during the undeployment "
						+ "of the following service assembly: "
						+ serviceAssemblyName;
				throw new Exception(msg);
			}

			/*
			 * Create the string result
			 */
			output = ((XMLResult) undeploymentContext
					.get(DeploymentContextConstants.XML_RESULT)).getString();

		} catch (Throwable e) {
			// Catch all throwables to prevent exotic exception type (not easily
			// deserializable) throwing
			log.error(e.getMessage(), e);
			throw new Exception(e.getMessage());
		}

        log.info(serviceAssemblyName+" successfully undeployed");
		log.end();
		return output;
	}

	/**
	 * Create the undeployment processor and add all required task for the
	 * complete undeployment of a service assembly
	 * 
	 * @param undeploymentContext
	 *            the deployment context
	 * @return filled undeployment processor
	 */
	protected TaskProcessor createUndeploymentProcessor(
			HashMap<String, Object> undeploymentContext) {
		TaskProcessor processor = new TaskProcessor(undeploymentContext, log);

		processor.addTask(deploymentTaskFactory
				.getXmlUndeploymentResultCreationTask());
		processor.addTask(deploymentTaskFactory.getSaLifeCycleRetrievalTask());
		processor.addTask(deploymentTaskFactory.getAllSUUndeploymentTask());
		processor.addTask(deploymentTaskFactory.getAllConnectionsRemovalTask());
		processor.addTask(deploymentTaskFactory
				.getSaLifeCycleUnregistrationTask());
		processor.addTask(deploymentTaskFactory.getSaStateHolderRemovalTask());
		processor.addTask(deploymentTaskFactory.getSaCacheRemovalTask());
		processor.addTask(deploymentTaskFactory.getSaFolderRemovalTask());
		processor.addTask(deploymentTaskFactory.getDeleteArchiveFromInstalledDirTask());

		return processor;
	}

	/**
	 * Create the undeployment context
	 * 
	 * @param serviceAssemblyName
	 *            the service assembly name
	 * @return the undeployment context
	 * @throws PetalsException
	 *             if error occured during JBI descriptor loading
	 */
	protected HashMap<String, Object> createUndeploymentContext(
			String serviceAssemblyName) throws PetalsException {
		HashMap<String, Object> undeploymentContext = new HashMap<String, Object>();

		ServiceAssemblyDataHandler handler = deployedSACache
				.get(serviceAssemblyName);

		undeploymentContext.put(DeploymentContextConstants.SA_DATA_HANDLER,
				handler);
		return undeploymentContext;
	}

	/**
	 * Return a string representation of the JBI descriptor. This descriptor in
	 * found thanks to the jbiXmlUri
	 * 
	 * @param jbiXmlUri
	 *            the URI of the deployment descriptor
	 * @return a string representing the JBI deployment descriptor
	 */
	protected String getJbiDescriptorAsString(URI jbiXmlUri) {
		JBIDescriptorBuilder builder = new JBIDescriptorBuilder(jbiXmlUri,
				(java.util.logging.Logger) logger);
		return builder.getJbiDescriptorAsString();
	}

	@LifeCycle(on = LifeCycleType.START)
	protected void start() {
		log = new LoggingUtil(logger);
		log.start();

		/*
		 * initialize packages loading cache
		 */
		deployedSACache = new HashMap<String, ServiceAssemblyDataHandler>();

		/*
		 * Initialize package handler
		 */
		packageHandler = new PackageHandler(logger, repositorySrv);

		/*
		 * Initialize DeploymentTaskFactory
		 */
		deploymentTaskFactory = new DeploymentTaskFactory(packageHandler,
				endpointService, this, recoverySrv, repositorySrv, log,
				managerService);

		log.end();
	}

	@LifeCycle(on = LifeCycleType.STOP)
	public void stop() {
		log.start();

		logger = null;

		deployedSACache = null;

		log.end();
	}

	/**
	 * Shutdown the DeploymentService. All service assemblies are undeployed.
	 */
	public void shutdown() throws Exception {
		// Undeploy all service assemblies
		Map<String, ServiceAssemblyDataHandler> removeList = new HashMap<String, ServiceAssemblyDataHandler>(
				deployedSACache);
		for (String saName : removeList.keySet()) {
			undeploy(saName);
		}
	}

	public DeploymentTaskFactory getDeploymentTaskFactory() {
		return deploymentTaskFactory;
	}

}
