/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2006 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: DescriptorAndURIRetrievalTask.java 154 6 oct. 06 ofabre $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.jbi.management.deployment.deploy;

import java.net.URI;
import java.util.HashMap;

import org.objectweb.petals.jbi.management.deployment.DeploymentContextConstants;
import org.objectweb.petals.jbi.management.service.PackageHandler;
import org.objectweb.petals.processor.Task;
import org.objectweb.petals.tools.jbicommon.descriptor.JBIDescriptor;

/**
 * This task loads service assembly descriptor and transforms archive URL into
 * URI. Used only by DeploymentService.
 * 
 * @author ofabre - EBM Websourcing
 * 
 */
public class DescriptorAndURIRetrievalTask implements Task {

    /**
     * Package Handler
     */
    protected PackageHandler packageHandler;

    public DescriptorAndURIRetrievalTask(PackageHandler packageHandler) {
        super();
        this.packageHandler = packageHandler;
    }

    @SuppressWarnings("unchecked")
    public void execute(HashMap context) throws Exception {

        String archiveURL = (String) context
                .get(DeploymentContextConstants.ARCHIVE_URL);

        /*
         * Transforming URL into URI
         */
        URI archiveURI = packageHandler.processAndGetPackageURI(archiveURL,
                true);

        /*
         * Loading descriptor
         */
        JBIDescriptor descriptor = packageHandler.loadDescriptor(archiveURI);

        /*
         * Fill context
         */
        context.put(DeploymentContextConstants.ARCHIVE_URI, archiveURI);
        context.put(DeploymentContextConstants.SA_DESCRIPTOR, descriptor);

    }

    public void undo(HashMap context) {
        // Nothing to do

    }

}
