/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2006 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: SAAndSUInstallRootRetrievalTask.java 154 6 oct. 06 ofabre $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.jbi.management.deployment.deploy;

import java.net.URI;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.objectweb.petals.jbi.management.deployment.DeploymentContextConstants;
import org.objectweb.petals.jbi.management.deployment.DeploymentUtils;
import org.objectweb.petals.jbi.management.service.PackageHandler;
import org.objectweb.petals.processor.Task;
import org.objectweb.petals.repository.RepositoryService;
import org.objectweb.petals.tools.jbicommon.descriptor.JBIDescriptor;
import org.objectweb.petals.tools.jbicommon.descriptor.ServiceUnit;

/**
 * This task retrieves the SA install root and all SUs install roots
 * 
 * @author ofabre - EBM Websourcing
 * 
 */
public class SAAndSUInstallRootRetrievalTask implements Task {

    /**
     * Platform Component :: Repository Service
     */
    protected RepositoryService repositorySrv;

    /**
     * Package Handler
     */
    protected PackageHandler packageHandler;

    public SAAndSUInstallRootRetrievalTask(RepositoryService repositorySrv,
            PackageHandler packageHandler) {
        super();
        this.repositorySrv = repositorySrv;
        this.packageHandler = packageHandler;
    }

    @SuppressWarnings("unchecked")
    public void execute(HashMap context) throws Exception {
        JBIDescriptor descriptor = (JBIDescriptor) context
                .get(DeploymentContextConstants.SA_DESCRIPTOR);

        URI installationRoot = repositorySrv.getServiceAssemblyInstallRoot(
                DeploymentUtils.getServiceAssemblyName(descriptor)).toURI();
        Map<String, URI> suInstallRoots = retrieveSUInstallRoots(
                installationRoot, descriptor);

        /*
         * Fill context
         */
        context.put(DeploymentContextConstants.SA_INSTALL_ROOT,
                installationRoot);
        context
                .put(DeploymentContextConstants.SU_INSTALL_ROOTS,
                        suInstallRoots);

    }

    /**
     * Retrieve all su installation root {@link URI}
     * 
     * @param descriptor
     *            the service assembly descriptor
     * @param installationRoot
     *            the sa intallation root {@link URI}
     * @return the service unit install roots {@link Map}
     */
    protected Map<String, URI> retrieveSUInstallRoots(URI installationRoot,
            JBIDescriptor descriptor) {
        Map<String, URI> suInstallRoots = new HashMap<String, URI>();
        List<ServiceUnit> sus = descriptor.getServiceAssembly()
                .getServiceUnits();
        for (ServiceUnit unit : sus) {
            String suName = unit.getIdentification().getName();
            suInstallRoots.put(suName, packageHandler.processAndGetPackageURI(
                    "file://" + installationRoot.getPath() + suName, true));
        }
        return suInstallRoots;
    }

    public void undo(HashMap context) {
        // Nothing to do

    }

}
