/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2006 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: SADeploymentSuccessTask.java 154 6 oct. 06 ofabre $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.jbi.management.deployment.deploy;

import java.io.File;
import java.io.IOException;
import java.net.URI;
import java.util.HashMap;

import org.objectweb.petals.jbi.management.autoload.AutoLoaderImpl;
import org.objectweb.petals.jbi.management.deployment.DeploymentContextConstants;
import org.objectweb.petals.jbi.management.deployment.DeploymentServiceImpl;
import org.objectweb.petals.jbi.management.deployment.DeploymentUtils;
import org.objectweb.petals.jbi.management.deployment.ServiceAssemblyDataHandler;
import org.objectweb.petals.processor.Task;
import org.objectweb.petals.tools.jbicommon.descriptor.JBIDescriptor;
import org.objectweb.petals.util.LoggingUtil;

/**
 * This task adds sa name to the list of successfuly deployed sa and creates
 * success file in workdir.
 * 
 * @author ofabre - EBM Websourcing
 * 
 */
public class SADeploymentSuccessTask implements Task {

    protected DeploymentServiceImpl deploymentService;

    /**
     * logger wrapper
     */
    protected LoggingUtil log;

    public SADeploymentSuccessTask(DeploymentServiceImpl deploymentService,
        LoggingUtil log) {
        super();
        this.deploymentService = deploymentService;
        this.log = log;
    }

    public void execute(HashMap context) throws Exception {
        JBIDescriptor descriptor = (JBIDescriptor) context
            .get(DeploymentContextConstants.SA_DESCRIPTOR);

        URI saArchiveURI = (URI) context
            .get(DeploymentContextConstants.ARCHIVE_URI);

        File file = DeploymentUtils.getArchiveFile(saArchiveURI);
        File installedArchive = DeploymentUtils.getInstalledArchive(file);

        /*
         * Add sa name to the list of successfuly deployed sa
         */
        ServiceAssemblyDataHandler handler = new ServiceAssemblyDataHandler(
            DeploymentUtils.getServiceAssemblyName(descriptor), descriptor,
            saArchiveURI);
        deploymentService.getDeployedSACache().put(
            DeploymentUtils.getServiceAssemblyName(descriptor), handler);

        /*
         * Create success file in workdir
         */
        createSuccessFile(installedArchive);
        
        /*
         * Log deployment success
         */
        log.info(DeploymentUtils.getServiceAssemblyName(descriptor)+" successfully deployed");

    }

    /**
     * Create a success file into the work directory to confirm the sa
     * successful deployment
     * 
     * @param installedArchive
     *            the installed archive {@link File}
     * @return true if succes file creation is ok, false otherwise
     * @throws IOException
     *             if success file creation fails
     */
    protected boolean createSuccessFile(File installedArchive)
        throws IOException {
        return new File(AutoLoaderImpl.getWorkDirectory(), installedArchive
            .getName()
            + ".success").createNewFile();
    }

    public void undo(HashMap context) {
        // Last task, nothing to do

    }

}
