/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2006 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: XMLDeploymentResultCreationTask.java 154 10 oct. 06 ofabre $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.jbi.management.deployment.undeploy;

import java.util.HashMap;

import org.objectweb.petals.jbi.management.deployment.DeploymentContextConstants;
import org.objectweb.petals.jbi.management.deployment.ServiceAssemblyDataHandler;
import org.objectweb.petals.jbi.management.service.util.XMLResult;
import org.objectweb.petals.jbi.management.service.util.XMLResult.CauseFramework;
import org.objectweb.petals.jbi.management.service.util.XMLResult.MessageType;
import org.objectweb.petals.jbi.management.service.util.XMLResult.TaskResult;
import org.objectweb.petals.processor.Task;

/**
 * This task create framework task result.
 * 
 * @author ofabre - EBM Websourcing
 * 
 */
public class XMLUndeploymentResultCreationTask implements Task {

	@SuppressWarnings("unchecked")
	public void execute(HashMap context) throws Exception {
		ServiceAssemblyDataHandler handler = (ServiceAssemblyDataHandler) context
				.get(DeploymentContextConstants.SA_DATA_HANDLER);

		XMLResult xmlResult = new XMLResult("undeploy");
		createFrameworkTaskResult(handler.getName(), xmlResult);

		/*
		 * Fill context
		 */
		context.put(DeploymentContextConstants.XML_RESULT, xmlResult);

	}

	/**
	 * Create the framework task result of the xml result message. Framework
	 * task result is always a success because we decide that in other cases, an
	 * exception is thrown.
	 * 
	 * @param saName
	 *            the service assembly name
	 * @param result
	 *            framework task result
	 */
	protected void createFrameworkTaskResult(String saName, XMLResult result) {
		result.addFrameworkTaskResult(TaskResult.SUCCESS, MessageType.INFO,
				"Successfully undeployed service assembly : {1}",
				new String[] { saName }, CauseFramework.YES);
	}

	public void undo(HashMap context) throws Exception {
		// Nothing to do

	}

}
