/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: InstallationServiceMBean.java 154 2006-03-27 15:30:10Z ofabre $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.jbi.management.service;

import javax.jbi.JBIException;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.servicedesc.ServiceEndpoint;

import org.objectweb.petals.PetalsException;
import org.w3c.dom.Document;

/**
 * The <code>AdminService</code> mbean implementation has to have its mbean
 * interface in the same package as it.
 * 
 * @author ofabre
 * 
 */
public interface AdminServiceMBean extends
    javax.jbi.management.AdminServiceMBean {

    /**
     * Get the service description for a endpoint of a component
     * 
     * @param componentName
     *            name of the component
     * @param endpoint
     *            endpoint
     * @return Document containing the service description
     */
    Document getServiceDescription(String componentName,
        ServiceEndpoint endpoint) throws PetalsException;

    /**
     * Check if a message exchange is okay with a provider
     * 
     * @param componentName
     * @param providerEP
     * @param exchange
     * @return
     * @throws JBIException
     *             If no component is found with this component name
     */
    boolean isExchangeWithProviderOkayForComponent(String componentName,
        ServiceEndpoint internalEndpoint, MessageExchange exchange)
        throws JBIException;

    /**
     * Check if a message exchange is okay with a consumer
     * 
     * @param componentName
     * @param internalEndpoint
     * @param exchange
     * @return
     * @throws JBIException
     *             If no component is found with this component name
     */
    boolean isExchangeWithConsumerOkayForComponent(String componentName,
        ServiceEndpoint internalEndpoint, MessageExchange exchange)
        throws JBIException;

}
