/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: XMLResult.java 154 2006-03-27 15:30:10Z ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.jbi.management.service.util;

import java.io.ByteArrayInputStream;
import java.io.InputStream;

import javax.xml.XMLConstants;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamSource;
import javax.xml.validation.Schema;
import javax.xml.validation.SchemaFactory;
import javax.xml.validation.Validator;

import org.w3c.dom.Document;

/**
 * This class gathers the dtd used for the management message
 * 
 * @author ddesjardins - eBMWebsourcing
 * @author wjoseph - eBMWebsourcing
 */
public class XMLResult {

    public static enum CauseFramework {
        NO, YES
    };

    public static enum MessageType {
        ERROR, INFO, WARNING
    };

    public static enum TaskResult {
        FAILED, SUCCESS
    };

    private static final String EXCEPTION_INFO = "exception-info";

    private static final String FRMWK_TASK_RESULT = "frmwk-task-result";

    private static final String FRMWK_TASK_RESULT_DETAILS = "frmwk-task-result-details";

    private static final String IS_CAUSE_FRAMEWORK = "is-cause-framework";

    private static final String JBI_TASK = "jbi-task";

    private static final String JBI_TASK_RESULT = "jbi-task-result";

    private static final String LOC_MESSAGE = "loc-message";

    private static final String LOC_PARAM = "loc-param";

    private static final String LOC_TOKEN = "loc-token";

    private static final String LOCALE = "locale";

    private static final String MESSAGE_TYPE = "message-type";

    private static final String MSG_LOC_INFO = "msg-loc-info";

    private static final String TASK_STATUS_MSG = "task-status-msg";

    private static final String NESTING_LEVEL = "nesting-level";

    private static final String STACK_TRACE = "stack-trace";

    private static final String TASK_ID = "task-id";

    private static final String TASK_RESULT = "task-result";

    private static final String TASK_RESULT_DETAILS = "task-result-details";

    private static final String XMLNS = "http://java.sun.com/xml/ns/jbi/management-message";

    private static final String COMPONENT_TASK_RESULT = "component-task-result";

    private static final String COMPONENT_TASK_RESULT_DETAILS = "component-task-result-details";

    private static final String COMPONENT_NAME = "component-name";

    protected StringBuffer xmlBuffer;

    protected String frameworkTaskId;

    public XMLResult(String frameworkTaskId) {
        xmlBuffer = new StringBuffer();
        this.frameworkTaskId = frameworkTaskId;
        xmlBuffer
            .append("<?xml version='1.0' encoding='UTF-8' standalone='yes'?>\n");
        xmlBuffer.append("<" + XMLResult.JBI_TASK + " version='1.0' xmlns='"
            + XMLResult.XMLNS + "'>\n");
        xmlBuffer.append("\t<" + XMLResult.JBI_TASK_RESULT + ">\n");
    }

    /**
     * Add a framework task result to a xml management message
     * 
     * @param taskResult
     *            result of the task
     * @param causeFramework
     *            causeFramework
     */
    public void addFrameworkTaskResult(TaskResult taskResult,
        CauseFramework causeFramework) {
        addFrameworkTaskResult(taskResult, null, null, null, null, null, null,
            null, causeFramework);
    }

    /**
     * Add a framework task result to a xml management message
     * 
     * @param taskResult
     *            result of the task
     * @param messageType
     *            type of message
     * @param message
     *            message
     * @param messageParams
     *            list of param for the message
     * @param causeFramework
     *            causeFramework
     */
    public void addFrameworkTaskResult(TaskResult taskResult,
        MessageType messageType, String message, String[] messageParams,
        CauseFramework causeFramework) {
        addFrameworkTaskResult(taskResult, messageType, message, messageParams,
            null, null, null, null, causeFramework);
    }

    /**
     * Add a framework task result to a xml management message
     * 
     * @param taskResult
     *            result of the task
     * @param messageType
     *            type of message
     * @param message
     *            message
     * @param messageParams
     *            list of param for the message
     * @param nestingLevel
     *            nesting level of the exception
     * @param exceptionMessage
     *            message of the exception
     * @param exceptionMessageParams
     *            list of the param for the exception message
     * @param causeFramework
     *            causeFramework
     */
    public void addFrameworkTaskResult(TaskResult taskResult,
        MessageType messageType, String message, String[] messageParams,
        String nestingLevel, String exceptionMessage,
        String[] exceptionMessageParams, CauseFramework causeFramework) {
        addFrameworkTaskResult(taskResult, messageType, message, messageParams,
            nestingLevel, exceptionMessage, exceptionMessageParams, null,
            causeFramework);
    }

    /**
     * Added a component task result to this management message
     * 
     * @param componentTaskString
     *            component task result xml
     */
    public void addComponentTaskResult(StringBuffer componentTaskString) {
        xmlBuffer.append(componentTaskString);
    }

    /**
     * Add a framework task result to a xml management message
     * 
     * @param taskResult
     *            result of the task
     * @param messageType
     *            type of message
     * @param message
     *            message
     * @param messageParams
     *            list of param for the message
     * @param nestingLevel
     *            nesting level of the exception
     * @param exceptionMessage
     *            message of the exception
     * @param exceptionMessageParams
     *            list of the param for the exception message
     * @param stackStrace
     *            stacktrace
     * @param causeFramework
     *            causeFramework
     */
    public void addFrameworkTaskResult(TaskResult taskResult,
        MessageType messageType, String message, String[] messageParams,
        String nestingLevel, String exceptionMessage,
        String[] exceptionMessageParams, String stackStrace,
        CauseFramework causeFramework) {
        xmlBuffer.append("\t\t<" + XMLResult.FRMWK_TASK_RESULT + ">\n");
        xmlBuffer.append("\t\t\t<" + XMLResult.FRMWK_TASK_RESULT_DETAILS
            + ">\n");
        xmlBuffer.append("\t\t\t\t<" + XMLResult.TASK_RESULT_DETAILS + ">\n");
        xmlBuffer.append("\t\t\t\t\t<" + XMLResult.TASK_ID + ">"
            + frameworkTaskId + "</" + XMLResult.TASK_ID + ">\n");
        xmlBuffer.append("\t\t\t\t\t<" + XMLResult.TASK_RESULT + ">"
            + taskResult + "</" + XMLResult.TASK_RESULT + ">\n");
        if (messageType != null) {
            xmlBuffer.append("\t\t\t\t\t<" + XMLResult.MESSAGE_TYPE + ">"
                + messageType + "</" + XMLResult.MESSAGE_TYPE + ">\n");
            if (message != null) {
                xmlBuffer.append("\t\t\t\t\t<" + TASK_STATUS_MSG + ">\n");
                xmlBuffer.append("\t\t\t\t\t\t<" + MSG_LOC_INFO + ">\n");
                // TODO change token with a key message for looking up localized
                // text
                // for the message
                xmlBuffer.append("\t\t\t\t\t\t\t<" + XMLResult.LOC_TOKEN
                    + ">jbimsg</" + XMLResult.LOC_TOKEN + ">\n");
                xmlBuffer.append("\t\t\t\t\t\t\t<" + XMLResult.LOC_MESSAGE
                    + ">" + message + "</" + XMLResult.LOC_MESSAGE + ">\n");
                for (String string : messageParams) {
                    xmlBuffer.append("\t\t\t\t\t\t\t<" + XMLResult.LOC_PARAM
                        + ">" + string + "</" + XMLResult.LOC_PARAM + ">\n");
                }
                xmlBuffer.append("\t\t\t\t\t\t</" + MSG_LOC_INFO + ">\n");
                xmlBuffer.append("\t\t\t\t\t</" + TASK_STATUS_MSG + ">\n");
            }
            if (exceptionMessage != null) {
                xmlBuffer.append("\t\t\t\t\t<" + XMLResult.EXCEPTION_INFO
                    + ">\n");
                xmlBuffer.append("\t\t\t\t\t\t<" + XMLResult.NESTING_LEVEL
                    + ">" + nestingLevel + "</" + XMLResult.NESTING_LEVEL
                    + ">\n");
                xmlBuffer.append("\t\t\t\t\t\t<" + XMLResult.MSG_LOC_INFO
                    + ">\n");
                // TODO change token with a key message for looking up localized
                // text
                // for the message
                xmlBuffer.append("\t\t\t\t\t\t\t<" + XMLResult.LOC_TOKEN
                    + ">jbimsg</" + XMLResult.LOC_TOKEN + ">\n");
                xmlBuffer.append("\t\t\t\t\t\t\t<" + XMLResult.LOC_MESSAGE
                    + ">" + exceptionMessage + "</" + XMLResult.LOC_MESSAGE
                    + ">\n");
                for (String string : exceptionMessageParams) {
                    xmlBuffer.append("\t\t\t\t\t\t\t<" + XMLResult.LOC_PARAM
                        + ">" + string + "</" + XMLResult.LOC_PARAM + ">\n");
                }
                xmlBuffer.append("\t\t\t\t\t\t</" + XMLResult.MSG_LOC_INFO
                    + ">\n");
                if (stackStrace == null) {
                    stackStrace = "";
                }
                xmlBuffer.append("\t\t\t\t\t\t<" + XMLResult.STACK_TRACE + ">"
                    + stackStrace + "</" + XMLResult.STACK_TRACE + ">\n");
                xmlBuffer.append("\t\t\t\t</" + XMLResult.EXCEPTION_INFO
                    + ">\n");
            }
        }
        xmlBuffer.append("\t\t\t\t</" + XMLResult.TASK_RESULT_DETAILS + ">\n");
        xmlBuffer.append("\t\t\t\t<" + XMLResult.LOCALE + ">en</"
            + XMLResult.LOCALE + ">\n");
        xmlBuffer.append("\t\t\t</" + XMLResult.FRMWK_TASK_RESULT_DETAILS
            + ">\n");
        xmlBuffer.append("\t\t\t<" + XMLResult.IS_CAUSE_FRAMEWORK + ">"
            + causeFramework + "</" + XMLResult.IS_CAUSE_FRAMEWORK + ">\n");
        xmlBuffer.append("\t\t</" + XMLResult.FRMWK_TASK_RESULT + ">\n");
    }

    /**
     * Close the XMLResult and return the String
     * 
     */
    public String getString() {
        xmlBuffer.append("\t</" + XMLResult.JBI_TASK_RESULT + ">\n");
        xmlBuffer.append("</" + XMLResult.JBI_TASK + ">\n");
        return xmlBuffer.toString();
    }

    /**
     * Check if a component task result is valid against a xml schema
     * 
     * @param componentTask
     *            xml of the component task
     * @return true if valid, false if not valid
     */
    public static boolean isValidComponentTask(String componentTask,
        InputStream xsd) {
        try {
            validateManagementMessage(componentTask, xsd);
        } catch (Exception e) {
            return false;
        }
        return true;
    }

    /**
     * Check if a management message task result is valid against a xml schema
     * 
     * @param managementMessage
     *            message xml of the management message
     */
    public static void validateManagementMessage(String managementMessage,
        InputStream xsd) throws Exception {
        DocumentBuilder parser;
        parser = DocumentBuilderFactory.newInstance().newDocumentBuilder();
        Document document = parser.parse(new ByteArrayInputStream(
            managementMessage.getBytes()));
        SchemaFactory factory = SchemaFactory
            .newInstance(XMLConstants.W3C_XML_SCHEMA_NS_URI);

        StreamSource schemaFile = new StreamSource(xsd);
        Schema schema = factory.newSchema(schemaFile);
        Validator validator = schema.newValidator();
        validator.validate(new DOMSource(document));
    }

    /**
     * Return a component task result xml
     * 
     * @param componentName
     *            component name
     * @param taskId
     *            id of the task
     * @param taskResult
     *            result of the task
     * @param messageType
     *            type of message
     * @param message
     *            message
     * @param messageParams
     *            list of param for the message
     * @param nestingLevel
     *            nesting level of the exception
     * @param exceptionMessage
     *            message of the exception
     * @param exceptionMessageParams
     *            list of the param for the exception message
     * @param stackStrace
     *            stacktrace
     * @return xml string
     */
    public String wrapComponentTaskResult(String componentName, String taskId,
        String message) {
        StringBuffer xmlInput = new StringBuffer();
        xmlInput.append("\t\t<" + XMLResult.COMPONENT_TASK_RESULT + ">\n");
        xmlInput.append("\t\t\t<" + XMLResult.COMPONENT_NAME + ">"
            + componentName + "</" + XMLResult.COMPONENT_NAME + ">\n");
        xmlInput.append("\t\t\t<" + XMLResult.COMPONENT_TASK_RESULT_DETAILS
            + " xmlns='" + XMLResult.XMLNS + "'>\n");
        xmlInput.append("\t\t\t\t<" + XMLResult.TASK_RESULT_DETAILS + ">\n");
        xmlInput.append("\t\t\t\t\t<" + XMLResult.TASK_ID + ">" + taskId + "</"
            + XMLResult.TASK_ID + ">\n");
        xmlInput.append("\t\t\t\t\t<" + XMLResult.TASK_RESULT + ">"
            + XMLResult.TaskResult.SUCCESS + "</" + XMLResult.TASK_RESULT
            + ">\n");
        xmlInput.append("\t\t\t\t\t<" + XMLResult.MESSAGE_TYPE + ">"
            + XMLResult.MessageType.WARNING + "</" + XMLResult.MESSAGE_TYPE
            + ">\n");
        if (message != null) {
            xmlInput.append("\t\t\t\t\t<" + XMLResult.MSG_LOC_INFO + ">\n");
            // TODO change token with a key message for looking up localized
            // text
            // for the message
            xmlInput.append("\t\t\t\t\t\t<" + XMLResult.LOC_TOKEN + ">jbimsg</"
                + XMLResult.LOC_TOKEN + ">\n");
            xmlInput.append("\t\t\t\t\t\t<" + XMLResult.LOC_MESSAGE + ">"
                + "![CDATA[" + message + "]]" + "</" + XMLResult.LOC_MESSAGE
                + ">\n");
            xmlInput.append("\t\t\t\t\t</" + XMLResult.MSG_LOC_INFO + ">\n");
        }
        xmlInput.append("\t\t\t\t</" + XMLResult.TASK_RESULT_DETAILS + ">\n");
        xmlInput.append("\t\t\t</" + XMLResult.COMPONENT_TASK_RESULT_DETAILS
            + ">\n");
        xmlInput.append("\t\t</" + XMLResult.COMPONENT_TASK_RESULT + ">\n");
        return xmlInput.toString();
    }

}
