/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2006 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: SystemState.java 154 28 avr. 2006 ofabre $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.jbi.management.systemstate;

import java.util.List;

import org.objectweb.petals.jbi.management.service.ManagementException;

/**
 * Allows to persist, update, delete and recover component, shared lib and
 * service assembly state
 * 
 * @author Olivier Fabre - EBM Websourcing
 * 
 */
public interface SystemState {

    /**
     * Create an entry in the state holder xml file that keep component
     * information (state, install and zip root...)
     * 
     * @param componentName
     * @param installURL
     * @param zipURL
     * @param installState
     * @param lifecycleState
     * @return
     */
    ComponentState createComponentStateHolder(String componentName,
            String installURL, String zipURL, String installState,
            String lifecycleState) throws Exception;

    /**
     * Create an entry in the state holder xml file that keep service assembly
     * information (state, install and zip root...)
     * 
     * @param saName
     * @param installURL
     * @param zipURL
     * @param lifecycleState
     * @return
     */
    ServiceAssemblyState createServiceAssemblyStateHolder(String saName,
            String installURL, String zipURL, String lifecycleState)
        throws Exception;

    /**
     * Create an entry in the state holder xml file that keep shared lib
     * information (state, install and zip root...)
     * 
     * @param slName
     * @param installURL
     * @param zipURL
     * @return
     */
    SharedLibraryState createSharedLibraryStateHolder(String slName,
            String installURL, String zipURL) throws Exception;

    /**
     * Delete a component entry in the state holder xml file
     * 
     * @param componentName
     * @return
     */
    ComponentState deleteComponentStateHolder(String componentName)
        throws Exception;

    /**
     * Delete a service assembly entry in the state holder xml file
     * 
     * @param saName
     * @return
     */
    ServiceAssemblyState deleteServiceAssemblyStateHolder(String saName)
        throws Exception;

    /**
     * Delete a shared library entry in the state holder xml file
     * 
     * @param slName
     * @return
     */
    SharedLibraryState deleteSharedLibraryStateHolder(String slName)
        throws Exception;

    /**
     * Recovers all component
     * 
     * @return true if retrieval is correctly performed, false otherwise
     */
    boolean recoverAllComponent() throws ManagementException;

    /**
     * Recover all petals recoverable entities (component, shared lib and
     * service assemblies)
     * 
     * @return true if all recovers are ok, false otherwise
     * @throws ManagementException
     */
    boolean recoverAllEntities() throws ManagementException;

    /**
     * Recovers all service assembly
     * 
     * @return true if retrieval is correctly performed, false otherwise
     */
    boolean recoverAllServiceAssembly() throws ManagementException;

    /**
     * Recovers all shared library
     * 
     * @return true if retrieval is correctly performed, false otherwise
     */
    boolean recoverAllSharedLibrary() throws ManagementException;

    List<ComponentState> retrieveAllComponentStates();

    List<ServiceAssemblyState> retrieveAllServiceAssemblyStates();

    List<SharedLibraryState> retrieveAllSharedLibraryStates();

    /**
     * Update the given component installation state information
     * 
     * @param componentName
     * @param installState
     */
    void updateComponentInstallationState(String componentName,
            String installState) throws Exception;

    /**
     * Update the given component lifecycle state information
     * 
     * @param componentName
     * @param lifecycleState
     */
    void updateComponentLifeCycleState(String componentName,
            String lifecycleState) throws Exception;

    /**
     * Update the given service assembly state information
     * 
     * @param saName
     * @param lifecycleState
     */
    void updateServiceAssemblyState(String saName, String lifecycleState)
        throws Exception;

}
