/**
 * PETALS: PETALS Services Platform
 * Copyright (C) 2006 EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id: ObjectSerializer.java,v 0.3 2005/07/22 10:24:27 alouis Exp $
 * --------------------------------------------------------------------------  
 */

package org.objectweb.petals.jbi.transport;

import javax.jbi.JBIException;
import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.ObjectMessage;
import javax.jms.Session;

import org.objectweb.petals.jbi.messaging.MessageExchangeImpl;

/**
 * This class is used to transform a <code>MessageExchangeImpl</code> into an
 * <code>jms.ObjectMessage</code>. Serialization mechanism is used; so the
 * <code>MessageExchangeImpl</code> object has to be fully (de)serialized.
 * 
 * @version $Rev: 477 $ $Date: 2006-05-29 17:18:07 +0200 (lun., 29 mai 2006) $
 * @since Petals 1.0
 * @author alouis
 */
public class ObjectSerializer implements Serializer {

    public Message jbi2jms(MessageExchangeImpl jbi, Session session)
        throws TransportException {
        ObjectMessage jms;
        try {
            jms = session.createObjectMessage(jbi);
        } catch (JMSException e) {
            if(e.getMessage().indexOf("org.xml.sax.SAXParseException")>0){
                if(e.getMessage().indexOf("Content is not allowed in prolog")>0){
                    throw new TransportException("The Normalized Message is not a valid XML document.",e);
                }else if (e.getMessage().indexOf("Invalid byte")>0){
                    throw new TransportException("The Normalized Message contains invalid characters.",e);
                }
            }
            throw new TransportException(e);
        }
        return jms;
    }

    public MessageExchangeImpl jms2jbi(Message jms) throws JBIException {
        try {
            return (MessageExchangeImpl) ((ObjectMessage) jms).getObject();
        } catch (JMSException e) {
            throw new JBIException(e);
        }

    }

}
