/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: PetalsAdminService.java 16:40:52 alouis $
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.kernel.admin.service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.MessageExchange.Role;

import org.objectweb.petals.monitoring.ExchangeStateCapture;
import org.objectweb.petals.monitoring.ExchangeStateReport;

/**
 * This class is used for monitoring service
 * 
 * @author alouis - eBMWebsourcing
 */
public class MonitoringService {

    public static List<List<Map<String,Object>>> bulkData(List<ExchangeStateReport> reports) {
        List<List<Map<String,Object>>> bulk = new ArrayList<List<Map<String,Object>>>(reports.size());

        for (ExchangeStateReport report : reports) {
            bulk.add(reportToRaw(report));
        }
        return bulk;
    }

    /**
     * Transform a report to a raw format : 
     * <code>List(Map<String,Object>)</code> list of raw captures
     * 
     * @see captureToRaw(ExchangeStateCapture capture)
     * @param report
     * @return
     */
    protected static List<Map<String,Object>> reportToRaw(ExchangeStateReport report) {
        
        List<Map<String,Object>> rawCaptures = new ArrayList<Map<String,Object>>();
        for (ExchangeStateCapture capture : report.getReportList()) {
            rawCaptures.add(captureToRaw(report.getId(),capture));
        }
        return rawCaptures;
    }

    /**
     * transform a capture to a raw Map : 
     * "id" = <code>String</code> exchangeId <br>
     * "time" = <code>long</code> capture.time <br>
     * "component" = <code>String</code> capture.component<br>
     * "role" = <code>String</code> capture.role<br>
     * "type" = <code>String</code> capture.messageType <br>
     * "status" = <code>String</code> capture.status <br>
     * 
     * @param capture
     * @return
     */
    protected static Map<String,Object> captureToRaw(String exchangeId, ExchangeStateCapture capture) {
        
        Map<String,Object> rawCapture = new HashMap<String,Object>();
        rawCapture.put("id", exchangeId);
        rawCapture.put("time", capture.time);
        rawCapture.put("component", capture.component);
        rawCapture.put("role", roleToRaw(capture.role));
        rawCapture.put("type", capture.messageType);
        rawCapture.put("status", statusToRaw(capture.status));
        
        return rawCapture;
    }
    
    /**
     * Convert a Role to a string representation
     * @param role
     * @return "consumer" or "provider
     */
    protected static String roleToRaw(Role role){
        return (Role.CONSUMER.equals(role))?"consumer":"provider";
    }

    /**
     * Convert a status to a string representation
     * @param status
     * @return "active" or "done" or "error"
     */
    protected static String statusToRaw(ExchangeStatus status){
        return (ExchangeStatus.DONE.equals(status))?"done":(ExchangeStatus.ACTIVE.equals(status))?"active":"error";
    }
}
