/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: PetalsAdminService.java 16:40:52 alouis $
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.kernel.admin.service;

import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.naming.Binding;
import javax.naming.InitialContext;
import javax.naming.NamingEnumeration;
import javax.naming.NamingException;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.fraclet.annotation.FractalComponent;
import org.objectweb.fractal.fraclet.annotation.Interface;
import org.objectweb.fractal.fraclet.annotation.LifeCycle;
import org.objectweb.fractal.fraclet.annotation.LifeCycleType;
import org.objectweb.fractal.fraclet.annotation.Monolog;
import org.objectweb.fractal.fraclet.annotation.Provides;
import org.objectweb.fractal.fraclet.annotation.Requires;
import org.objectweb.util.monolog.api.Logger;

import org.objectweb.petals.jbi.management.deployment.DeploymentService;
import org.objectweb.petals.jbi.management.service.InstallationService;
import org.objectweb.petals.jbi.registry.AbstractEndpoint;
import org.objectweb.petals.jbi.registry.Registry;
import org.objectweb.petals.jbi.registry.RegistryException;
import org.objectweb.petals.jbi.transport.Transporter;
import org.objectweb.petals.kernel.admin.ContainerInformation;


import org.objectweb.petals.monitoring.ExchangeStateReport;
import org.objectweb.petals.monitoring.RouterMonitor;
import org.objectweb.petals.util.JNDIUtil;
import org.objectweb.petals.util.LoggingUtil;
import org.objectweb.petals.util.PropertyUtil;

/**
 * This class is used for Petals global Administration
 * 
 * @author alouis - eBMWebsourcing
 */
@FractalComponent
@Provides(interfaces = @Interface(name = "service", signature = org.objectweb.petals.kernel.admin.service.PetalsAdminServiceMBean.class))
public class PetalsAdminService implements PetalsAdminServiceMBean {

    protected static final String CONTAINERS_REF = "containers";

    protected static final String ENDPOINTS_REF = "endpoints";

    protected boolean monitoring=true;
    /**
     * Logger wrapper
     */
    protected LoggingUtil log;

    /**
     * Monolog
     */
    @Monolog(name = "logger")
    protected Logger logger;

    /**
     * JBI :: JMX Deployment
     */
    @Requires(name = "deployment", signature = org.objectweb.petals.jbi.management.deployment.DeploymentService.class)
    protected DeploymentService deployment;

    /**
     * JBI :: JMX Installation
     */
    @Requires(name = "installation", signature = org.objectweb.petals.jbi.management.service.InstallationService.class)
    protected InstallationService installation;

    @Requires(name = "transporter", signature = org.objectweb.petals.jbi.transport.Transporter.class)
    protected Transporter transporter;

    @Requires(name = "registry", signature = org.objectweb.petals.jbi.registry.Registry.class)
    protected Registry registry;

    @Requires(name = "router-monitor", signature = org.objectweb.petals.monitoring.RouterMonitor.class)
    protected RouterMonitor routerMonitor;


    protected Component rootComponent;

    /**
     * uninstalled serviceassemblies, components and sharelibraries endpoints
     * are removed the container is removed from the global configuration
     * 
     * @throws Exception
     */
    public void shutdownContainer() throws Exception {
        log.start();
        log
        .info("Shutdown the server. It will be removed from the Petals network.");
        try {
            log.info("  1/4 Undeploy Service Assemblies.");
            deployment.shutdown();
        } catch (Exception e) {
            log.error("Can not shutdown the DeploymentService.", e);
        }
        try {
            log.info("  2/4 Uninstall Shared Libraries and Components.");
            installation.shutdown();
        } catch (Exception e) {
            log.error("Can not shutdown the InstallationService.", e);
        }
        try {
            log
            .info("  3/4 Shutdown the Transporter (remove the server from the global configuration).");
            transporter.shutdown();
        } catch (Exception e) {
            log.error("Can not shutdown the Transporter.", e);
        }

        log.info("  4/4 Stopping the server...");
        System.exit(0);
        log.end();
    }

    public void stopContainer() throws Exception {
        log.start();
        log.info("Stopping the server...");
        System.exit(0);
        log.end();
    }

    /**
     * Retrieve the collection of all the started servers configuration from
     * this domain.
     * 
     * @return Collection of server information ContainerInformation
     */
    public Collection<Map<String, String>> getAllServersConfiguration() {
        return retrieveAllServersConfiguration(false);
    }

    /**
     * Retrieve the collection of all the started servers configuration from
     * this domain.
     * 
     * @return Collection of server information ContainerInformation
     */
    public Collection<Map<String, String>> getAllStartedServersConfiguration() {
        return retrieveAllServersConfiguration(true);
    }

    /**
     * Retrieve the collection of all the started servers configuration from
     * this domain.
     * 
     * @param startedOnly
     *            return only started servers information
     * @return Collection of server information ContainerInformation
     */
    protected Collection<Map<String, String>> retrieveAllServersConfiguration(
        boolean startedOnly) {

        log.start();
        Set<Map<String, String>> containers = new HashSet<Map<String, String>>();

        try {
            InitialContext rootContext = new InitialContext(PropertyUtil
                .retrieveJNDIProperties());

            if (JNDIUtil.isBound(rootContext, "/", CONTAINERS_REF)) {

                NamingEnumeration<Binding> enums = rootContext
                .listBindings(CONTAINERS_REF);

                while (enums.hasMoreElements()) {

                    Binding binding = (Binding) enums.next();
                    ContainerInformation containerInformation = (ContainerInformation) binding
                    .getObject();

                    if (!startedOnly
                            || (startedOnly && containerInformation.isStarted())) {

                        containers
                        .add(createConfigurationMap(containerInformation));
                    }
                }
            }
        } catch (NamingException e) {
            log
            .debug("Problem while trying to retrieve all the containers information : context may not be initialized");
        }
        log.end();
        return containers;
    }

    /**
     * Create a Map from a ContainerInformation object
     * 
     * @param info
     * @return a Map of the server properties
     */
    protected Map<String, String> createConfigurationMap(
        ContainerInformation info) {
        Map<String, String> map = new HashMap<String, String>();
        if (info != null) {
            map.put(CONF_HOST, info.getHost());
            map.put(CONF_HTML_PORT, info.getHtmlPort());
            map.put(CONF_JMX_LOGIN, info.getJmxLogin());
            map.put(CONF_JMX_PASSWORD, info.getJmxPassword());
            map.put(CONF_JMX_PORT, info.getJmxPort());
            map.put(CONF_JNDI_FACTORY, info.getJndiFactory());
            map.put(CONF_JNDI_PORT, info.getJndiPort());
            map.put(CONF_JORAM_DOMAIN, info.getJoramDomain());
            map.put(CONF_JORAM_DOMAIN_PORT, info.getJoramDomainPort());
            map.put(CONF_JORAM_ID, info.getJoramId());
            map.put(CONF_JORAM_LOGIN, info.getJoramLogin());
            map.put(CONF_JORAM_PASSWORD, info.getJoramPassword());
            map.put(CONF_JORAM_TCP_PORT, info.getJoramTCPPort());
            map.put(CONF_NAME, info.getName());
            map
            .put(
                CONF_STATE,
                (info.getState() == ContainerInformation.ContainerState.STARTED) ? "started"
                    : "stopped");
            map.put(CONF_UID, "" + info.getUid());
        }
        return map;
    }

    /**
     * Remove the specified server from the network configuration. <br>
     * The server is unbound, but not shutdown. All ServiceEndpoints activated
     * on this server are unbound. <br>
     * <b>Be very careful when using this operation, the network configuration
     * may become unstable.</b><br>
     * TODO JORAM configuration not updated
     * @param serverName
     * @throws Exception
     *             The server is not found in the network configuration, some
     *             Endpoints have not been removed, or problem while accessing
     *             the network configuration
     */
    public void removeServerFromNetwork(String serverName) throws Exception {

        log.start();

        InitialContext rootContext;
        try {
            rootContext = new InitialContext(PropertyUtil
                .retrieveJNDIProperties());
        } catch (NamingException e) {
            throw new Exception("Network configuration can not be accessed.", e);
        }

        try {
            Collection<AbstractEndpoint> eps = getServiceEndpointForContainer(serverName);

            rootContext.unbind(CONTAINERS_REF + "/" + serverName);

            for (AbstractEndpoint endpoint : eps) {
                registry.deregisterInternalEndpoint(endpoint);
            }

        } catch (NamingException e) {
            throw new Exception(
                "Server not found in the network configuration.", e);
        } catch (RegistryException e) {
            throw new Exception("A ServiceEndpoint failed to be removed.", e);
        }

        log.end();
    }

    /**
     * Return all the registered endpoints for the specified container
     * 
     * @param containerName
     * @return a collection of AbstractEndpoint. Not null, may be empty
     * @throws NamingException
     *             problem accessing JNDI
     */
    protected Collection<AbstractEndpoint> getServiceEndpointForContainer(
        String containerName) throws NamingException {

        log.start();

        InitialContext rootContext = new InitialContext(PropertyUtil
            .retrieveJNDIProperties());

        Collection<AbstractEndpoint> eps = new HashSet<AbstractEndpoint>();

        if (JNDIUtil.isBound(rootContext, "/", ENDPOINTS_REF)) {

            NamingEnumeration<Binding> enums = rootContext
            .listBindings(ENDPOINTS_REF);

            while (enums.hasMoreElements()) {

                Binding binding = (Binding) enums.next();
                AbstractEndpoint abstractEndpoint = (AbstractEndpoint) binding
                .getObject();

                if (!abstractEndpoint.getContainerName().equals(containerName)) {
                    eps.add(abstractEndpoint);
                }
            }
        }
        log.end();

        return eps;
    }


    public boolean getMonitoring(){
        return routerMonitor.isMonitoring();
    }
    public void setMonitoring(boolean activate) throws Exception{
        routerMonitor.setMonitoring(activate);
    }

    /**
     * Return information about messages exchanges since the specified date.<br>
     * Information are sort by exchange-start-date.
     * Information is returned in a bulk mode, with no specific Java Object. <br>
     * - Each element of the list represents an exchange report.<br>
     * &nbsp;&nbsp;- an exchange report is a <em>List(Map(String,Object))</em> of the different captures of the exchange<br>
     * &nbsp;&nbsp;&nbsp;&nbsp;- a capture is a Map of <em>Object</em> objects, following the structure:<br>
     * &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;- "id" <em>String</em> the id of the exchange<br>
     * &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;- "time" <em>long</em> the date of the capture<br>
     * &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;- "component" <em>String</em> the component that interact on this exchange<br>
     * &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;- "role" <em>String</em> the role of the component : [consumer|provider]<br>
     * &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;- "type" <em>String</em> the message type : [in|out|fault|ANY]<br>
     * &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;- "status"<em>String</em> the message status : [active|done|error]<br>
     *            
     * @param sinceTime <em>long</em> representation of the date, in the standard <em>long<em> format
     * @return the raw reports. not null, may be empty
     */
    public List<List<Map<String,Object>>> bulkData(long sinceTime){
        log.start();
        List<ExchangeStateReport> reports = routerMonitor.getReports(sinceTime);
        List<List<Map<String,Object>>> bulkDatas = MonitoringService.bulkData(reports);
        log.end();
        return bulkDatas;
    }


    // ----------------------------------------------------------------------
    // Fractal LifecycleController implementation
    // ----------------------------------------------------------------------

    /**
     * @see org.objectweb.fractal.api.control.LifeCycleController#startFc()
     */
    @LifeCycle(on = LifeCycleType.START)
    public void start() throws IllegalLifeCycleException {
        log = new LoggingUtil(logger);
    }

}
