/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: StopJMXServerThread.java 10:14:10 AM ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.kernel.jmx;

import javax.management.remote.JMXConnectorServer;

import org.objectweb.petals.util.LoggingUtil;

/**
 * This thread is used to stop the JMX server
 * 
 * We need to keep the JMX server alive for some seconds while all the Fractal
 * components are stopped
 * 
 * Then we can safely stop the JMX server
 * 
 * @author ddesjardins - eBMWebsourcing
 */
public class StopJMXServerThread extends Thread {

    /**
     * Instance of the JMX server
     */
    protected JMXConnectorServer cs;

    /**
     * Logger
     */
    protected LoggingUtil log;

    public StopJMXServerThread(JMXConnectorServer cs, LoggingUtil log) {
        super();
        this.cs = cs;
    }

    public void run() {
        try {
            Thread.sleep(1000);
            if (cs != null) {
                cs.stop();
                if (log != null) {
                    log.debug("JMX server at " + cs.getAddress() + " stopped");
                }
            }
        } catch (Exception e) {
            if (log != null) {
                log.error("Problem while stopping the JMX server", e);
            }
        }
    }

}
