/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: PetalsCommandProcessor.java 11:28:17 AM ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.kernel.mx4j;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import javax.management.JMException;
import javax.management.MBeanServer;
import javax.xml.parsers.DocumentBuilder;

import mx4j.tools.adaptor.http.HttpCommandProcessor;
import mx4j.tools.adaptor.http.HttpInputStream;
import mx4j.tools.adaptor.http.ServerCommandProcessor;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * Petals command processor
 * 
 * @author ddesjardins - eBMWebsourcing
 */
public class PetalsCommandProcessor implements HttpCommandProcessor {

    /**
     * Document builder
     */
    protected DocumentBuilder documentBuilder;

    /**
     * ServerCommandProcessor used to process the request
     */
    protected ServerCommandProcessor serverCommandProcessor = new ServerCommandProcessor();

    /**
     * @see HttpCommandProcessor#executeRequest(HttpInputStream)
     */
    public Document executeRequest(HttpInputStream httpInputStream)
        throws IOException, JMException {
        Document document = preProcess(httpInputStream);
        Document outDocument = documentBuilder.newDocument();
        Element petalsNode = outDocument.createElement("petals");
        outDocument.appendChild(petalsNode);
        NodeList mbeans = document.getDocumentElement().getChildNodes();
        Element servicesNode = outDocument.createElement("services");
        Element composNode = outDocument.createElement("components");
        List<Node> compos = new ArrayList<Node>();
        List<Node> services = new ArrayList<Node>();
        for (int i = 0; i < mbeans.getLength(); i++) {
            Element mbean = (Element) mbeans.item(i);
            String objectName = mbean.getAttribute("objectname");
            if (objectName.indexOf("type=engine") > -1) {
                // The mbean is a component
                compos.add(mbean);
            } else if (objectName.indexOf("type=binding") > -1) {
                // The mbean is a component
                compos.add(mbean);
            } else if (objectName.indexOf("type=installer") > -1) {
                // The mbean is a component
                compos.add(mbean);
            } else if (objectName.indexOf("type=service") > -1) {
                // The mbean is a service
                services.add(mbean);
            }
        }
        for (Node node : services) {
            Node tmpNode = outDocument.adoptNode(node);
            servicesNode.appendChild(tmpNode);
        }
        for (Node node : compos) {
            Node tmpNode = outDocument.adoptNode(node);
            composNode.appendChild(tmpNode);
        }
        petalsNode.appendChild(servicesNode);
        petalsNode.appendChild(composNode);
        return outDocument;
    }

    /**
     * @see HttpCommandProcessor#setDocumentBuilder(DocumentBuilder)
     */
    public void setDocumentBuilder(DocumentBuilder documentBuilder) {
        this.documentBuilder = documentBuilder;
        serverCommandProcessor.setDocumentBuilder(documentBuilder);
    }

    /**
     * @see HttpCommandProcessor#setMBeanServer(MBeanServer)
     */
    public void setMBeanServer(MBeanServer mBeanServer) {
        serverCommandProcessor.setMBeanServer(mBeanServer);
    }

    /**
     * Preprocess an request by the server command processor
     * 
     * @param httpInputStream
     *            request to process
     * @return Document
     * @throws IOException
     * @throws JMException
     */
    protected Document preProcess(HttpInputStream httpInputStream)
        throws IOException, JMException {
        return serverCommandProcessor.executeRequest(httpInputStream);
    }

}
