/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: JNDIConnection.java 10:47:20 AM ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.kernel.registry.jndi;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.net.InetSocketAddress;
import java.net.Socket;

import javax.naming.NamingException;
import javax.naming.ServiceUnavailableException;

import org.objectweb.petals.kernel.registry.msg.request.RegistryRequest;
import org.objectweb.petals.kernel.registry.msg.response.RegistryResponse;
import org.objectweb.petals.kernel.registry.msg.response.RegistryResponse.ResponseType;

/**
 * Connection to a RegistryServer
 * 
 * @author ddesjardins - eBMWebsourcing
 */
public class JNDIConnection implements Serializable {

    private static final long serialVersionUID = 1L;

    /**
     * Port of the RegistryServer
     */
    protected int port;

    /**
     * Host of the RegistryServer
     */
    protected String host;

    public JNDIConnection(int port, String host) {
        super();
        this.port = port;
        this.host = host;
    }

    /**
     * Send a request to a Registry server
     * 
     * @param request
     *            request to send
     * @return return of the request
     * @throws NamingException
     *             if a NamingException is thrown on the server
     */
    public Object send(RegistryRequest request) throws NamingException {
        Object out = null;
        Socket socket = new Socket();
        try {
            socket.connect(new InetSocketAddress(host, port), 1000);
            ObjectOutputStream objectOutputStream = new ObjectOutputStream(
                socket.getOutputStream());
            objectOutputStream.writeObject(request);
            objectOutputStream.flush();

            ObjectInputStream objectInputStream = new ObjectInputStream(socket
                .getInputStream());
            RegistryResponse response = (RegistryResponse) objectInputStream
                .readObject();

            if (response.getType() == ResponseType.exception) {
                throw (NamingException) response.getArg1();
            }
            out = response.getArg1();
        } catch (IOException e) {
            throw new ServiceUnavailableException(
                "Problem while sending request to server " + host + ":" + port);
        } catch (ClassNotFoundException e) {
            throw new ServiceUnavailableException(
                "Problem while sending request to server " + host + ":" + port);
        }
        return out;
    }

}
