/*
 * PETALS: PETALS Services Platform
 * Copyright (C) 2007 EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): Adrien LOUIS
 * --------------------------------------------------------------------------
 * $Id: 
 * --------------------------------------------------------------------------  
 */

package org.objectweb.petals.monitoring;

import java.util.ArrayList;
import java.util.List;

import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.MessageExchange.Role;

import org.objectweb.petals.jbi.messaging.MessageExchangeImpl;
import org.objectweb.petals.jbi.registry.AbstractEndpoint;

/**
 * @version $Rev: 617 $ $Date: 2006-06-19 17:28:41 +0200 (lun, 19 jun 2006) $
 * @since Petals 1.1
 * @author alouis
 * 
 * This class contains all the states of a messageexchange lifecyle.
 */
public class ExchangeStateReport {

    private String id;
    
    private List<ExchangeStateCapture> reportList = new ArrayList<ExchangeStateCapture>();

    public ExchangeStateReport(MessageExchangeImpl exchange){
        this.id=exchange.getExchangeId();
        
    }
    
    public void reportState(MessageExchangeImpl exchange) {
        ExchangeStateCapture state = new ExchangeStateCapture();
        state.time = System.currentTimeMillis();
        state.status = exchange.getStatus();

        state.role = exchange.getRole();
        
        if(Role.CONSUMER.equals(state.role)){
            state.component=exchange.getConsumerEndpoint().getComponentName();
        }else{
            state.component=((AbstractEndpoint)exchange.getEndpoint()).getComponentName();
        }
        // set the message type. Set to null if no message found
        if (ExchangeStatus.ACTIVE.equals(state.status)) {
            // Assume there is only one message in the exchange.
            if(exchange.getMessages() != null && exchange.getMessages().size()>0){
                state.messageType = exchange.getMessages().keySet().iterator()
                .next();
            }
            // check if the message is a fault
            if (state.messageType == null && exchange.getFault() != null) {
                state.messageType = "fault";
            }
        }
        reportList.add(state);
    }

    public String getId() {
        return id;
    }

    public List<ExchangeStateCapture> getReportList() {
        return reportList;
    }
}
