/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: FractalHelper.java 16:40:52 alouis $
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.system.launch;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.ContentController;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.api.control.LifeCycleController;
import org.objectweb.fractal.util.Fractal;

import org.objectweb.petals.jbi.transport.Transporter;

/**
 * This class helps the PetalsAdmin Service to work with Fractal
 * 
 * @author alouis
 * 
 */
public class FractalHelper {

    /**
     * Stop the Petals fractal composites
     * 
     * FIXME only jndi-server isstopped,petals will be closed without stopping
     * fractal
     * 
     * @param rootContentController
     * @throws NoSuchInterfaceException
     * @throws IllegalLifeCycleException
     */
    public static void stopPetalsComponents(ContentController rootContentController)
        throws NoSuchInterfaceException, IllegalLifeCycleException {

        // stopComponent(rootContentController, "jbi");

        // stopComponent(rootContentController, "platform");

        stopComponent(rootContentController, "jndi-server");

    }

    /**
     * Stop the fractal component Transporter
     * FIXME problem with the monitoring mode where the transporter is in fact a Composite
     * @param rootContentController
     * @return true if the transported was found and stopped, false otherwise
     */
    public static boolean stopTransporter(ContentController rootContentController) {

        boolean result = false;

        try {
            // find the jbi composite
            Component jbi = getComponentByName(rootContentController, "jbi");
            
            // Problem when stopping the transporter Composite in monitoring mode
            //ContentController jbiController = Fractal.getContentController(jbi);
            // stop the component
            //stopComponent(jbiController, "transporter-impl");
            ((Transporter) jbi.getFcInterface("transporter")).stop();

            result = true;
        } catch (Exception e) {
            // nothing to do, return false
        }
        return result;
    }

    /**
     * Stop the fractal component InstallationService
     * 
     * @param rootContentController
     * @return true if the InstallationService was found and stopped, false
     *         otherwise
     */
    public static boolean stopInstallationService(
        ContentController rootContentController) {

        boolean result = false;

        try {
            // find the jbi composite
            Component jbi = getComponentByName(rootContentController, "jbi");
            ContentController jbiController = Fractal.getContentController(jbi);

            // stop the component
            stopComponent(jbiController, "installation-impl");
            result = true;
        } catch (NoSuchInterfaceException e) {
            // nothing to do, return false
        }
        return result;
    }

    /**
     * A utility function allowing to get a component from a content controller
     * 
     * @param cc
     *            parentContentController
     * @param name
     *            component name
     * @return the component, null if not found
     */
    protected static Component getComponentByName(
        ContentController parentContentController, String name) {

        // The component to be returned
        Component c = null;

        // List content controller subcomponents
        for (Component component : parentContentController.getFcSubComponents()) {

            try {
                String componentName = Fractal.getNameController(component)
                    .getFcName();
                if (componentName.equals(name)) {
                    c = component;
                    break;
                }
            } catch (NoSuchInterfaceException e) {
                // Do nothing, return null
            }
        }
        return c;
    }

    /**
     * A utility function allowing to get a component LifeCycleController from a
     * content controller
     * 
     * @param parentContentController
     * @param name
     * @return the LifeCycleController of the component, null if not found
     */
    protected static LifeCycleController getLifeCycleControllerByName(
        ContentController parentContentController, String name) {

        LifeCycleController lifeCycleController = null;

        Component comp = getComponentByName(parentContentController, name);

        if (comp != null) {
            try {
                lifeCycleController = Fractal.getLifeCycleController(comp);
            } catch (NoSuchInterfaceException e) {
                // Do nothing, return null
            }
        }
        return lifeCycleController;
    }

    /**
     * A utility function allowing to get a component LifeCycleController from a
     * content controller
     * 
     * @param parentContentController
     * @param name
     * @return true if the component was found et stopped, false otherwise
     */
    protected static boolean stopComponent(
        ContentController parentContentController, String name) {

        boolean result = false;

        LifeCycleController lifeCycleController = getLifeCycleControllerByName(
            parentContentController, name);

        if (lifeCycleController != null) {
            try {
                lifeCycleController.stopFc();
                result = true;
            } catch (IllegalLifeCycleException e) {
                // Do nothing, return false
            }
        }
        return result;
    }

}
