/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 Fossil E-Commerce, http://www.fossilec.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: Launcher.java 461 2006-05-24 12:12:45Z ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.system.launch;

import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.rmi.UnmarshalException;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;

import javax.management.MBeanServerConnection;
import javax.management.ObjectName;

import org.objectweb.fractal.adl.Factory;
import org.objectweb.fractal.adl.FactoryFactory;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.ContentController;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.api.control.LifeCycleController;
import org.objectweb.fractal.util.Fractal;
import org.objectweb.util.monolog.Monolog;

import org.objectweb.petals.PetalsException;
import org.objectweb.petals.jbi.management.service.ManagementException;
import org.objectweb.petals.jbi.management.systemstate.SystemState;
import org.objectweb.petals.jbi.transport.TransportException;
import org.objectweb.petals.jbi.transport.Transporter;
import org.objectweb.petals.kernel.admin.service.PetalsAdminServiceMBean;
import org.objectweb.petals.util.JNDIUtil;
import org.objectweb.petals.util.SystemUtil;

/**
 * This class is used to start the Petals platform
 * 
 * @author alouis, ddesjardins, Rafael Marins
 * @version $Rev: 461 $ $Date: 2006-05-24 14:12:45 +0200 (mer., 24 mai 2006) $
 */
public class Launcher {

    /**
     * Petals composite
     */
    protected Component petalsComposite;

    /**
     * @param args
     */
    public static void main(String[] args) {
        try {
            if (args.length > 0 && "stop".equals(args[0])) {
                Launcher launcher = new Launcher();
                launcher.closeServer(false);
            } else if (args.length > 0 && "shutdown".equals(args[0])) {
                Launcher launcher = new Launcher();
                launcher.closeServer(true);
            } else if (args.length > 0 && "JNDIBrowser".equals(args[0])) {
                System.out.println(JNDIUtil.browseJNDI(args));
            } else {
                Launcher launcher = new Launcher();
                launcher.run(args);
            }
        } catch (Throwable e) {
            e.printStackTrace(System.err);
            Component kernelComp = new Launcher().getPetalsComposite();
            if (kernelComp != null) {
                try {
                    LifeCycleController lcc = Fractal
                        .getLifeCycleController(kernelComp);
                    if (lcc.getFcState().equals(LifeCycleController.STARTED)) {
                        lcc.stopFc();
                        System.out.println("Petals is " + lcc.getFcState());
                    }
                } catch (IllegalLifeCycleException e1) {
                    System.err.println("Problem while stopping Petals");
                } catch (NoSuchInterfaceException e2) {
                    System.err.println("Problem while stopping Petals");
                }
            }
            System.err.println("Petals is stopped !");
            System.exit(-1);
        }
    }

    public Component getPetalsComposite() {
        return petalsComposite;
    }

    /**
     * Initialize Fractal
     * 
     * @throws Exception
     */
    protected void initializeFractal() throws Exception {
        // check required fractal properties
        System.setProperty("fractal.provider",
            "org.objectweb.fractal.julia.Julia");
        System.setProperty("julia.loader",
            "org.objectweb.fractal.julia.loader.DynamicLoader");
        System.setProperty("julia.config", "julia.cfg");

        Map<String, Object> context = new HashMap<String, Object>();
        // Create Fractal ADL factories
        Factory factory = FactoryFactory.getFactory(
            FactoryFactory.FRACTAL_BACKEND, context);
        // System.setSecurityManager(new SecurityManager());
        // Registry.createRegistry(Registry.DEFAULT_PORT, context);
        // Create composite from ADL
        String compositeName = "Petals";
        petalsComposite = (Component) factory.newComponent(compositeName,
            context);

    }

    /**
     * 
     * @throws IOException
     */
    protected void initializeMonolog() throws IOException {
        
        // create logs directory is not present
        new File(SystemUtil
            .getPetalsInstallDirectory()
            + File.separator + "logs").mkdirs();
        
        // Configure Monolog
        URL url1 = this.getClass().getResource("/loggers.properties");
        Properties p1 = new Properties();
        p1.load(url1.openStream());
        String petalsLF = p1.getProperty("handler.petalsLF.output");
        if (petalsLF != null) {
            p1.setProperty("handler.petalsLF.output", new File(SystemUtil
                .getPetalsInstallDirectory()
                + File.separator + "logs" + File.separator + petalsLF)
                .getAbsolutePath());
        }
        String joramLF = p1.getProperty("handler.joramLF.output");
        if (joramLF != null) {
            p1.setProperty("handler.joramLF.output", new File(SystemUtil
                .getPetalsInstallDirectory()
                + File.separator + "logs" + File.separator + joramLF)
                .getAbsolutePath());
        }
        Monolog.initialize();
        Monolog.loadMonologConfiguration(p1);
    }

    protected void startPetals() throws NoSuchInterfaceException,
        IllegalLifeCycleException {
        String componentName = "jndi-server";
        ContentController contentController = Fractal
            .getContentController(petalsComposite);
        Component jndiServer = FractalHelper.getComponentByName(
            contentController, componentName);

        LifeCycleController jndiServerLifeCycleController = Fractal
            .getLifeCycleController(jndiServer);
        jndiServerLifeCycleController.startFc();

        // Start the composite
        LifeCycleController lcc = Fractal
            .getLifeCycleController(petalsComposite);
        lcc.startFc();
    }

    protected void run(String[] args) throws Exception {

        // set the compatible SUID for the Qname
        System.setProperty(
            "com.sun.xml.namespace.QName.useCompatibleSerialVersionUID", "1.0");

        try {
            initializeMonolog();
        } catch (IOException e) {
            throw new PetalsException(
                "Monolog can't be correctly initialized !");
        }

        Monolog.initialize().getLogger("root0.Petals.launcher");

        System.setProperty("javax.management.builder.initial",
            "mx4j.server.MX4JMBeanServerBuilder");

        // Initialize Fractal
        initializeFractal();

        // Start Petals
        startPetals();

        // expose JBI MBean objectin the JMX Server
        MBeanHelper.registerMBeans(petalsComposite);

        ContentController contentController = Fractal
            .getContentController(petalsComposite);

        // recover all entities
        recoverSystem(contentController);

        // tell the Transporter to start Listening
        startListening(contentController);

        // Set up the shutdown hook
        LifeCycleController lcc = Fractal
            .getLifeCycleController(petalsComposite);

        Runtime.getRuntime().addShutdownHook(
            new SystemExitHook(lcc, contentController));

        System.out.println("##### Petals container correctly started #####");

        // show the commandLine mode if asked
        if (args != null && args.length > 0
            && "-console".equalsIgnoreCase(args[0])) {
            commandLineMode(contentController);
        }
    }

    /**
     * Stop or Shutdown the Petals server.
     * We connect to the JMX server of the server that is running
     * with the server.properties entries.
     * 
     * @param shutdown the server if true, stop the server if false 
     */
    protected void closeServer(boolean shutdown) {
        try {

            System.out.println("Petals is "+(shutdown?"shutdowning...":"stopping..."));
            
            String action = (shutdown? "shutdownContainer":"stopContainer");
            
            // get the jmxServer of this container
            MBeanServerConnection connection = MBeanHelper.retrieveJMXServer();

            // search the PetalsAdmin mbean
            ObjectName result = MBeanHelper.retrieveServiceMBean("PetalsAdmin",
                connection);

            // call Stop
            if (result != null) {
                connection.invoke(result, action, new Object[0],
                    new String[0]);

            }
        } catch (UnmarshalException e) {
            // nothing to do, the JVM of the jmx server is stopped
        } catch (Exception e) {
            System.err.println("Petals can not be accessed.");
            e.printStackTrace(System.err);
        }
        System.out.println("Petals has been "+(shutdown?"shutdown.":"stopped."));
    }

   
    /**
     * Recover the system state. All state elements are recovered (components,
     * share libraries, service assemblies)
     * 
     * @param contentContoller
     * @throws NoSuchInterfaceException
     * @throws ManagementException
     */
    protected void recoverSystem(ContentController contentContoller)
        throws NoSuchInterfaceException, ManagementException {
        Component jbi = FractalHelper.getComponentByName(contentContoller,
            "jbi");
        ContentController contentControllerForJBI = Fractal
            .getContentController(jbi);
        Component systemState = FractalHelper.getComponentByName(
            contentControllerForJBI, "systemstate-impl");
        SystemState systemStateService = (SystemState) systemState
            .getFcInterface("service");
        systemStateService.recoverAllEntities();
    }

    /**
     * Start listening for incoming messages. This is only activated after all
     * the components has been restarted.
     * 
     * @param contentContoller
     * @throws NoSuchInterfaceException
     * @throws ManagementException
     * @throws TransportException
     */
    protected void startListening(ContentController contentContoller)
        throws NoSuchInterfaceException, ManagementException,
        TransportException {
        Component jbi = FractalHelper.getComponentByName(contentContoller,
            "jbi");
        ContentController contentControllerForJBI = Fractal
            .getContentController(jbi);
        Component systemState = FractalHelper.getComponentByName(
            contentControllerForJBI, "transporter-impl");
        Transporter transporter = (Transporter) systemState
            .getFcInterface("service");
        transporter.startListening();
    }

    /**
     * Show a command line to interact with Petals TODO we pass the PetalsAdmin
     * object to the CommandReader to perform some actions. Find a better way.
     * 
     * @param contentController
     *            used to retreive PetalsAdmin
     * @throws NoSuchInterfaceException
     */
    protected void commandLineMode(ContentController contentController)
        throws NoSuchInterfaceException {
        Component systemState = FractalHelper.getComponentByName(
            contentController, "petalsadmin-service");
        PetalsAdminServiceMBean petalsService = (PetalsAdminServiceMBean) systemState
            .getFcInterface("service");

        CommandReader console = new CommandReader(petalsService);

        console.read();
    }

}
