/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: ComponentContextImplTest.java 15:38:25 ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.jbi.component.context;

import java.io.IOException;
import java.lang.reflect.Method;
import java.net.URL;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.MissingResourceException;
import java.util.ResourceBundle;
import java.util.Set;
import java.util.logging.Logger;

import javax.jbi.JBIException;
import javax.jbi.component.Component;
import javax.jbi.management.MBeanNames;
import javax.jbi.messaging.DeliveryChannel;
import javax.jbi.messaging.MessagingException;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.management.MBeanServer;
import javax.management.ObjectName;
import javax.xml.namespace.QName;

import junit.framework.TestCase;

import org.objectweb.petals.jbi.component.context.ComponentContextImpl;
import org.objectweb.petals.jbi.component.lifecycle.ComponentLifeCycle;
import org.objectweb.petals.jbi.component.mock.MockEndpointService;
import org.objectweb.petals.jbi.management.service.EndpointService;
import org.objectweb.petals.jbi.management.service.EndpointServiceImpl;
import org.objectweb.petals.jbi.management.service.LifeCycleManagerImpl;
import org.objectweb.petals.jbi.management.service.MBeanNamesImpl;
import org.objectweb.petals.jbi.messaging.DeliveryChannelImpl;
import org.objectweb.petals.jbi.registry.AbstractEndpoint;
import org.objectweb.petals.jbi.registry.ConsumerEndpoint;
import org.objectweb.petals.jbi.registry.ExternalEndpoint;
import org.objectweb.petals.jbi.registry.InternalEndpoint;
import org.objectweb.petals.tools.jbicommon.descriptor.ComponentDescription;
import org.objectweb.petals.tools.jbicommon.descriptor.Identification;
import org.objectweb.petals.tools.jbicommon.descriptor.JBIDescriptor;
import org.w3c.dom.DocumentFragment;

import static org.easymock.EasyMock.expect;
import static org.easymock.classextension.EasyMock.createMock;
import static org.easymock.classextension.EasyMock.replay;
import static org.easymock.classextension.EasyMock.reset;
import static org.easymock.classextension.EasyMock.verify;

import com.sun.org.apache.xerces.internal.dom.DocumentFragmentImpl;

/**
 * Test of the ComponentContextImpl
 * 
 * @author ddesjardins - eBMWebsourcing
 * @author wjoseph - eBMWebsourcing
 */
public class ComponentContextImplTest extends TestCase {

    private ComponentContextImpl componentContextImpl;

    private MockEndpointService endpointService;

    @Override
    protected void setUp() {
        componentContextImpl = new ComponentContextImpl();
        Logger logger = Logger.getLogger("test");
        componentContextImpl.setRootLogger(logger);
        endpointService = new MockEndpointService();
        componentContextImpl.endpointService = endpointService;
    }

    public void testGetLogger() throws MissingResourceException, JBIException {
        /*
         * Test Case 1
         * suffix is null
         */
        Logger rootLogger = Logger.getLogger("rootLogger");
        componentContextImpl.rootLogger = rootLogger;
        /*
         * Run Test Case 1
         */
        String suffix = null;
        try{
            componentContextImpl.getLogger(suffix, null);
            fail("No Exception is raised");
        } catch (Exception e) {
            // Do nothing  
        }
        /*
         * Test Case 2
         * The suffix is an empty string
         */
        /*
         * Run Test Case 2
         */
        suffix = "";
        Logger result = componentContextImpl.getLogger(suffix, null);
        assertNotNull(result);
        assertEquals("Returned logger does not have the name of base componentn logger",result.getName(),rootLogger.getName());
        /*
         * Test Case 3
         * The suffix is a string, the created logger must have a name containing this suffix
         */
        /*
         * Run Test Case 3
         */
        suffix = "test";
        result = componentContextImpl.getLogger(suffix, null);
        assertNotNull(result);
        assertEquals("Returned logger does not have the name of base componentn logger",result.getName(),rootLogger.getName()+"."+suffix);
    }

    public void testGetLoggerSameSuffix() throws MissingResourceException,
        JBIException {
        Logger logger = componentContextImpl.getLogger("suffix", null);
        Logger logger1 = componentContextImpl.getLogger("suffix", null);
        assertEquals(logger, logger1);
    }

    public void testGetLoggerResourceFile() throws MissingResourceException,
        JBIException, IOException {
        URL url = this.getClass().getClassLoader().getResource(
            "myresources_fr.properties");
        assertNotNull(url);
        Logger logger = componentContextImpl.getLogger("resource",
            "myresources");
        ResourceBundle resourceBundle = logger.getResourceBundle();
        assertEquals(resourceBundle.getString("Error"), "Erreur");
    }

    public void testActivateEndpoint() {
        /*
         * Test Case 1
         * Everything goes well, a new InternalEndpoint is created
         */
        /*
         * Create Mocks
         */
        Method method1 = null;
        try {
            method1 = EndpointServiceImpl.class.getDeclaredMethod("activateEndpoint", new Class[]{QName.class,String.class,ConsumerEndpoint.class});
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not retrieve mocks methods");
        }
        Method[] mockedMethods = new Method[]{method1};
        EndpointService epService = createMock(EndpointServiceImpl.class,mockedMethods);
        String service = "fooService";
        String namespace = "namespace";
        QName serviceName = new QName(namespace,service);
        String endpointName = "fooServiceEndpoint";
        ConsumerEndpoint consumerEndpoint = createMock(ConsumerEndpoint.class);
        InternalEndpoint internalEP = createMock(InternalEndpoint.class);
        /*
         * Initialize Mocks
         */
        try {
        expect(epService.activateEndpoint(serviceName, endpointName, consumerEndpoint)).andReturn(internalEP);
        } catch (JBIException e){
            e.printStackTrace();
            fail("Could not initialize mocks");
        }
        replay(epService);
        componentContextImpl.endpointService = epService;
        componentContextImpl.internalEndpoints = new HashSet<AbstractEndpoint>();
        componentContextImpl.address = consumerEndpoint;
        /*
         * Run Test Case 1
         */
        ServiceEndpoint result = null;
        try {
            result = componentContextImpl.activateEndpoint(serviceName,endpointName);
        } catch (JBIException e){
            e.printStackTrace();
            fail("Error during method invokation");
        }
        assertNotNull("The result of activateEndpoint must not be null",result);
        assertEquals("Wrong endpoint created",internalEP, result);
        assertTrue("ComponentContext does not contain the newly created ", componentContextImpl.internalEndpoints.contains(internalEP));
        verify(epService);        

        /*
         * Test Case 2
         * The serviceName is null, an exception is raised 
         */
        /*
         * Run Test Case 2
         */
        try{
        componentContextImpl.activateEndpoint(null,endpointName);
        fail("No exception was raised");
        } catch (Exception e) {
        }
        
        /*
         * Test Case 3
         * The endpointName is null, an exception is raised 
         */
        /*
         * Run Test Case 3
         */
        try{
        componentContextImpl.activateEndpoint(serviceName,null);
        fail("No exception was raised");
        } catch (Exception e) {
        }
        
    }

    public void testDeactivateEndpoint() {
        /*
         * Test Case 1
         * Everything goes well, the InternalEndpoint is deactivated
         */
        EndpointService epService = createMock(EndpointServiceImpl.class);
        AbstractEndpoint serviceEP = createMock(InternalEndpoint.class);
        Set<AbstractEndpoint> endpoints = new HashSet<AbstractEndpoint>();
        endpoints.add(serviceEP);
        componentContextImpl.endpointService = epService;
        componentContextImpl.internalEndpoints = endpoints;
        /*
         * Run Test Case 1
         */
        try {
            componentContextImpl.deactivateEndpoint(serviceEP);
        } catch (JBIException e){
            e.printStackTrace();
            fail("Error during method invokation");
        }
        assertFalse("The service endpoint was not properly deactivated", componentContextImpl.internalEndpoints.contains(serviceEP));
        
        /*
         * Test Case 2
         * endpoint is null, an exception is raised 
         */
        /*
         * Run Test Case 2
         */
        try {
        componentContextImpl.deactivateEndpoint(null);
        fail("No exception was thrown");
        } catch (Exception e) {
        }
    }

    public void testDeregisterAllEndpoints() {
        componentContextImpl.internalEndpoints = new HashSet<AbstractEndpoint>();
        componentContextImpl.externalEndpoints = new HashSet<ServiceEndpoint>();
        InternalEndpoint internalEndpoint = new InternalEndpoint(new QName(
            "service"), "endpoint", "compo", "0", null);
        componentContextImpl.internalEndpoints.add(internalEndpoint);
        ExternalEndpoint externalEndpoint = new ExternalEndpoint(
            new ConsumerEndpoint("compo", "0"), "compo", "0");
        componentContextImpl.externalEndpoints.add(externalEndpoint);
        try {
            componentContextImpl.deregisterAllEndpoints();
        } catch (JBIException e){
            e.printStackTrace();
            fail("Error during method invokation");
        }
        assertEquals(componentContextImpl.internalEndpoints.size(), 0);
        assertEquals(componentContextImpl.externalEndpoints.size(), 0);
        assertTrue(endpointService.isDeregisterExternalEndpoint());
        assertTrue(endpointService.isDeactivateEndpoint());
    }

    public void testDeregisterExternalEndpoint() {
        ExternalEndpoint externalEndpoint = new ExternalEndpoint(
            new ConsumerEndpoint("compo", "0"), "compo", "0");
        componentContextImpl.externalEndpoints = new HashSet<ServiceEndpoint>();
        componentContextImpl.externalEndpoints.add(externalEndpoint);
        try {
        componentContextImpl.deregisterExternalEndpoint(externalEndpoint);
        } catch (JBIException e){
            e.printStackTrace();
            fail("Error during method invokation");
        }
        assertEquals(componentContextImpl.externalEndpoints.size(), 0);
        assertTrue(endpointService.isDeregisterExternalEndpoint());
    }
    
    public void testGetComponentName() {
        /*
         * Test Case 1
         */
        /*
         * Create Mock
         */
        JBIDescriptor descriptor = createMock(JBIDescriptor.class);
        ComponentDescription componentDescription = createMock(ComponentDescription.class);
        Identification identification = createMock(Identification.class);
        String componentName = "fooComponent";
        expect(identification.getName()).andReturn(componentName);
        expect(componentDescription.getIdentification()).andReturn(identification);
        expect(descriptor.getComponent()).andReturn(componentDescription);
        replay(identification);
        replay(componentDescription);
        replay(descriptor);
        /*
         * Create Test parameters
         */
        componentContextImpl.descriptor = descriptor;
        /*
         * Run Test Case 1
         */
        String result = componentContextImpl.getComponentName();
        assertNotNull("The result can not be null",result);
        assertFalse("The result can not be an empty String",result.length()==0);
    }

    public void testGetDeliveryChannel() throws MessagingException {
        /*
         * Test Case 1
         * A delivery channel already exists in the ComponentContext but is not opened
         */
        Method method1 = null;
        try {
            method1 = ComponentContextImpl.class.getDeclaredMethod("createDeliveryChannel", new Class[]{});
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not retrieve mocked methods");
        }
        Method[] mockedMethods = new Method[]{method1};
        ComponentContextImpl componentContextImpl = createMock(ComponentContextImpl.class,mockedMethods);
        DeliveryChannelImpl oldDeliveryChannel = createMock(DeliveryChannelImpl.class);
        DeliveryChannelImpl newDeliveryChannel = createMock(DeliveryChannelImpl.class);
        expect(oldDeliveryChannel.isOpened()).andReturn(false);
        expect(componentContextImpl.createDeliveryChannel()).andReturn(newDeliveryChannel);
        replay(oldDeliveryChannel);
        replay(componentContextImpl);
        componentContextImpl.deliveryChannel = oldDeliveryChannel;
        /*
         * Run Test Case 1 
         */
        DeliveryChannel result = componentContextImpl.getDeliveryChannel();
        assertNotNull("The result of GetDeliveryChannel can not be null",result);
        assertEquals("Wrong DeliveryChannel returned",newDeliveryChannel, result);
        verify(oldDeliveryChannel);
        verify(componentContextImpl);
        
        /*
         * Test Case 2
         * A delivery channel already exists in the ComponentContext but is opened
         * An exception will be raised 
         */
        reset(oldDeliveryChannel);
        componentContextImpl = createMock(ComponentContextImpl.class,mockedMethods);
        expect(oldDeliveryChannel.isOpened()).andReturn(true);
        replay(oldDeliveryChannel);
        componentContextImpl.deliveryChannel = oldDeliveryChannel;
        /*
         * Run Test Case 2 
         */
        try {
        componentContextImpl.getDeliveryChannel();
        fail("No exception was raised");
        } catch (Exception e) {
            verify(oldDeliveryChannel);
        }
        
        /*
         * Test Case 3
         * No DeliveryChannel exists a new one is created
         */
        componentContextImpl = createMock(ComponentContextImpl.class,mockedMethods);
        expect(componentContextImpl.createDeliveryChannel()).andReturn(newDeliveryChannel);
        replay(componentContextImpl);
        /*
         * Run Test Case 3 
         */
        try {
        result = componentContextImpl.getDeliveryChannel();
        } catch (Exception e) {
            fail("Exception during method invokation");
        }
        assertNotNull("The result of GetDeliveryChannel can not be null",result);
        assertEquals("Wrong DeliveryChannel returned",newDeliveryChannel, result);
        verify(componentContextImpl);
    }

    public void testGetEndpoint() {
        /*
         * Test Case 1
         * the parameter service is null, an exception is raised
         */
        /*
         * Create Test parameters
         */
        QName serviceName = new QName("fooService");
        String name = "fooService";
        /*
         * Run Test Case 1
         */
        try {
        componentContextImpl.getEndpoint(null, name);
        fail("No exception was raised");
        } catch (Exception e) {
        }
        
        /*
         * Test Case 2
         * the parameter name is null, an exception is raised
         */
        /*
         * Run Test Case 2
         */
        try {
        componentContextImpl.getEndpoint(serviceName, null);
        fail("No exception was raised");
        } catch (Exception e) {
        }
        /*
         * Test Case 3
         * the parameter service is null, an exception is raised
         */
        /*
         * Run Test Case 3
         */
        componentContextImpl.getEndpoint(serviceName, name);
        assertTrue(endpointService.isGetEndpoint());
    }

    public void testGetEndpointDescriptor() {
        /*
         * Test Case 1
         * The endpoint is null, an exception is raised
         */
        /*
         * Run Test Case 1
         */
        try {
            componentContextImpl.getEndpointDescriptor(null);
            fail("no exception was raised");
        } catch (Exception e) {
            // do nothing
        }
        /*
         * Test Case 2
         * The endpoint is not null
         */
        /*
         * Create Mocks 
         */
        Method method1 = null;
        try {
            method1 = ComponentContextImpl.class.getDeclaredMethod("getEndpoint", new Class[]{QName.class,String.class});
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not retrieve mocked methods");
        }
        Method[] mockedMethods = new Method[]{method1};
        componentContextImpl = createMock(ComponentContextImpl.class,mockedMethods);
        InternalEndpoint internalEndpoint = new InternalEndpoint(new QName(
            "test"), "endpoint", "compo", "0", null);
        internalEndpoint.setDescription("<test></test>");
        /*
         * Initialize Mocks
         */
        expect(componentContextImpl.getEndpoint(internalEndpoint.getServiceName(), internalEndpoint.getEndpointName())).andReturn(internalEndpoint);
        replay(componentContextImpl);
        /*
         * Run Test Case 2
         */
        try {
            assertNotNull(componentContextImpl.getEndpointDescriptor(internalEndpoint));
        } catch (JBIException e){
            e.printStackTrace();
            fail("Error during method invokation");
        }
        verify(componentContextImpl);
    }

    public void testGetEndpoints() {
        ServiceEndpoint[] result = componentContextImpl.getEndpoints(null);
        assertTrue(endpointService.isGetInternalEndpointsForInterface());
        assertNotNull("The result can not be null",result);
    }

    public void testGetEndpointsForService() {
        /*
         * Test Case 1
         * The service name is null, an exception is raised
         */
        /*
         * Run Test Case 1
         */
        try{
        componentContextImpl.getEndpointsForService(null);
        fail("No exception was raised");
        } catch (Exception e) {
            // Do nothing
        }
        /*
         * Test Case 2
         * The service name not null, a ServiceEndpoint array is returned
         */
        /*
         * Run Test Case 2
         */
        ServiceEndpoint[] endpoints = componentContextImpl.getEndpointsForService(new QName("fooService"));
        assertTrue(endpointService.isGetInternalEndpointsForService());
        assertNotNull("Result must not be null",endpoints);
    }

    public void testGetExternalEndpoints() {
        /*
         * Test Case 1
         * The interface name is null, an exception is raised
         */
        /*
         * Run Test Case1
         */
        try {
        componentContextImpl.getExternalEndpoints(null);
        fail("No exception was raised");
        } catch (Exception e) {
           // Do nothing
        }
        /*
         * Test Case 2
         * The service name not null, a ServiceEndpoint array is returned
         */
        /*
         * Run Test Case 2
         */
        ServiceEndpoint[] endpoints = componentContextImpl.getExternalEndpoints(new QName("fooService"));
        assertTrue(endpointService.isGetExternalEndpointsForInterface());
        assertNotNull("Result must not be null",endpoints);
    }

    public void testGetExternalEndpointsForService() {
        /*
         * Test Case 1
         * The interface name is null, an exception is raised
         */
        /*
         * Run Test Case1
         */
        try {
        componentContextImpl.getExternalEndpointsForService(null);
        fail("No exception was raised");
        } catch (Exception e) {
           // Do nothing
        }
        /*
         * Test Case 2
         * The service name not null, a ServiceEndpoint array is returned
         */
        /*
         * Run Test Case 2
         */
        ServiceEndpoint[] endpoints = componentContextImpl.getExternalEndpointsForService(new QName("fooService"));
        assertNotNull("Result must not be null",endpoints);
        assertTrue(endpointService.isGetExternalEndpointsForService());
    }

    public void testGetInstallRoot() {
        componentContextImpl.installationRoot = "root";
        String result = componentContextImpl.getInstallRoot();
        assertNotNull("The result must not be null",result);
        assertFalse("The result string must not be empty",result.length() == 0);
        assertEquals(result, "root");
    }

    public void testGetMBeanNames() {
        MBeanNames beanNames = new MBeanNamesImpl("test");
        componentContextImpl.beanNames = beanNames;
        MBeanNames result = componentContextImpl.getMBeanNames();
        assertNotNull("Result of getMBeanNames must not be null",result);
        assertEquals(result, beanNames);
    }
    
    public void testGetMBeanServer() {
     /*
      * Test Case 1
      */   
        componentContextImpl.beanServer = createMock(MBeanServer.class);
      /*
       * Run Test Case 1
       */
        MBeanServer result = componentContextImpl.getMBeanServer();
        assertNotNull("Result of getMBeanServer must not be null",result);
    }
    
    public void testGetNamingContext() {
        try {
            componentContextImpl.getNamingContext();
            fail();
        } catch (Error e) {
            // Do nothing
        }
    }

    public void testGetTransactionManager() {
        try {
            componentContextImpl.getTransactionManager();
            fail();
        } catch (Error e) {
            // Do nothing
        }
    }

    public void testGetWorkspaceRoot() {
        /*
         * Test Case 1
         */
        /*
         * Run Test Case 1
         */
        componentContextImpl.workspaceRoot = "work";
        String result = componentContextImpl.getWorkspaceRoot();
        assertNotNull("The result of getWorkspaceRoot must be non null",result);
        assertFalse("The result of getWorkspaceRoot must be non empty",result.length() == 0);
        assertEquals(result, "work");
    }

    public void testRegisterExternalEndpoint() {
        /*
         * Test Case 1
         * The externalEndpoint we want to register is null
         */
        /*
         * Run Test Case 1
         */
        try{
        componentContextImpl.registerExternalEndpoint(null);
        fail("No Exception was raised");
        } catch (Exception e) {
            //Do nothing
        }
        /*
         * Test Case 2
         * The externalEndpoint we want to register is not null
         */
        ExternalEndpoint externalEndpoint = new ExternalEndpoint(
            new InternalEndpoint(new QName("service"), "endpoint", "compo",
                "0", null), "compo", "0");
        componentContextImpl.externalEndpoints = new HashSet<ServiceEndpoint>();
        /*
         * Run Test Case 2
         */
        try {
        componentContextImpl.registerExternalEndpoint(externalEndpoint);
        } catch (JBIException e){
            e.printStackTrace();
            fail("Error during method invokation");
        }
        assertEquals(componentContextImpl.externalEndpoints.size(), 1);
        assertTrue(endpointService.isRegisterExternalEndpoint());
    }
    
    public void testResolveEndpointReference() {
        /*
         * Test Case 1
         * The document frangement parameter is not null, but the endpoint is not resolved
         * Result is null
         */
        /*
         * Create Mocks
         */
        Method method1 = null;
        Method method2 = null;
        try {
            method1 = LifeCycleManagerImpl.class.getDeclaredMethod("getEngineCompoLifeCycles", new Class[]{});
            method2 = LifeCycleManagerImpl.class.getDeclaredMethod("getBindingCompoLifeCycles", new Class[]{});
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not retrieve mocked methods");
        }
        Method[] mockedMethods = new Method[]{method1, method2};
        LifeCycleManagerImpl lyfeCycleManager = createMock(LifeCycleManagerImpl.class,mockedMethods);
        /*
         * Create Test parameters
         */
        DocumentFragment epr = createMock(DocumentFragmentImpl.class);
        ComponentLifeCycle compoLifeCycle = createMock(ComponentLifeCycle.class);
        ComponentLifeCycle engineLifeCycle = createMock(ComponentLifeCycle.class);
        Component bindingComponent = createMock(Component.class);
        Component engineComponent = createMock(Component.class);
        ObjectName engineName = createMock(ObjectName.class);
        ObjectName bindingName = createMock(ObjectName.class);
        Map<ObjectName,ComponentLifeCycle> engines = new HashMap<ObjectName, ComponentLifeCycle>();
        Map<ObjectName,ComponentLifeCycle> bindings = new HashMap<ObjectName, ComponentLifeCycle>();
        engines.put(engineName, engineLifeCycle);
        bindings.put(bindingName, compoLifeCycle);
        try {
            expect(lyfeCycleManager.getEngineCompoLifeCycles()).andReturn(engines);
            expect(lyfeCycleManager.getBindingCompoLifeCycles()).andReturn(bindings);
            expect(compoLifeCycle.getComponent()).andReturn(bindingComponent);
            expect(engineLifeCycle.getComponent()).andReturn(engineComponent);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not register testing componentLyfeCycle");
        }
        replay(lyfeCycleManager);
        replay(compoLifeCycle);
        replay(engineLifeCycle);
        componentContextImpl.manager = lyfeCycleManager;    
        /*
         * Run Test Case 1
         */
        ServiceEndpoint result = null;
        try {
            result = componentContextImpl.resolveEndpointReference(epr);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Exception during method invokation");
        }
        assertNull("the result of resolveEndpointReference should be null",result);
        verify(lyfeCycleManager);
        verify(compoLifeCycle);
        verify(engineLifeCycle);
        /*
         * Test Case 2
         * The document frangement parameter is null, an exception is raised
         */
        /*
         * Run Test Case 2
         */
        try{
        componentContextImpl.resolveEndpointReference(null);
        fail("No exception was raised");
        } catch (Exception e) {
            //Do nothing
        }
    }

}
