/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: InstallationContextImplTest.java 1:55:33 PM ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.jbi.component.context;

import java.util.ArrayList;
import java.util.List;

import javax.jbi.component.ComponentContext;

import junit.framework.TestCase;

import org.easymock.classextension.EasyMock;
import org.objectweb.petals.jbi.component.context.ComponentContextImpl;
import org.objectweb.petals.jbi.component.context.InstallationContextImpl;
import org.objectweb.petals.tools.jbicommon.descriptor.ComponentDescription;
import org.objectweb.petals.tools.jbicommon.descriptor.Identification;

/**
 * Test of the InstallationContextImpl
 * 
 * @author ddesjardins - eBMWebsourcing
 * @author wjoseph - eBM-Websourcing
 */
public class InstallationContextImplTest extends TestCase {

    private InstallationContextImpl installationContextImpl;

    private ComponentContextImpl componentContextImpl;

    public void setUp() {
        ComponentDescription componentDescription = EasyMock
            .createMock(ComponentDescription.class);
        EasyMock.expect(componentDescription.getComponentClassName())
            .andReturn("testClassName").anyTimes();
        List<String> classpaths = new ArrayList<String>();
        classpaths.add("server.jar");
        EasyMock.expect(componentDescription.getComponentClassPath())
            .andReturn(classpaths).anyTimes();

        Identification identification = EasyMock
            .createMock(Identification.class);
        EasyMock.expect(identification.getName()).andReturn("compoName")
            .anyTimes();
        EasyMock.replay(identification);

        EasyMock.expect(componentDescription.getIdentification()).andReturn(
            identification).anyTimes();
        EasyMock.replay(componentDescription);

        componentContextImpl = new ComponentContextImpl();
        componentContextImpl.installationRoot = "install";

        installationContextImpl = new InstallationContextImpl(
            componentDescription, componentContextImpl, true);
    }

    public void testGetClassPathElements() {
        List result = installationContextImpl.getClassPathElements();
        assertNotNull("The result of getClassPathElements must not be null",result);
        assertFalse("The list of classpath elements must not be empty",result.isEmpty());
        assertEquals(result.get(0),
            "server.jar");
    }

    public void testGetComponentClassName() {
        String result = installationContextImpl.getComponentClassName();
        assertNotNull("The result of getComponentClassName must not be null",result);
        assertFalse("The result of getComponentClassName must not be empty",result.length() == 0);
        assertEquals(result,"testClassName");
    }

    public void testGetComponentName() {
        String result = installationContextImpl.getComponentName();
        assertNotNull("The result of getComponentName must not be null",result);
        assertFalse("The result of getComponentName must not be empty",result.length() == 0);
        assertEquals(result, "compoName");
    }

    public void testGetContext() {
        ComponentContext result = installationContextImpl.getContext();
        assertNotNull("The result of getContext must not be null",result);
        assertEquals(result.getInstallRoot(),
            componentContextImpl.getInstallRoot());
    }

    public void testGetInstallationDescriptorExtension() {
        assertNull(installationContextImpl.getInstallationDescriptorExtension());
    }

    public void testGetInstallRoot() {
        String result = installationContextImpl.getInstallRoot();
        assertNotNull("The result of getInstallRoot must not be null",result);
        assertFalse("The result of getInstallRoot must not be empty",result.length() == 0);
        assertEquals(result, "install");
    }

    public void testIsInstall() {
        assertTrue(installationContextImpl.isInstall());
    }
    
    public void testSetClassPathElements() {
        /*
         * Test Case 1
         * The classPathElements parameter is null
         */
        List<String> classPathElements = null;
        /*
         * Run Test Case 1
         */
        try {
        installationContextImpl.setClassPathElements(classPathElements);
        fail("No exception was raised");
        } catch (Exception e) {
            // Do nothing
        }
        /*
         * Test Case 2
         * The classPathElements parameter is empty
         */
        classPathElements = new ArrayList<String>();
        /*
         * Run Test Case 2
         */
        try {
        installationContextImpl.setClassPathElements(classPathElements);
        fail("No exception was raised");
        } catch (Exception e) {
            // Do nothing
        }
        /*
         * Test Case 2
         * The classPathElements parameter is fine
         */
        classPathElements.add("some class path");
        /*
         * Run Test Case 2
         */
        try {
        installationContextImpl.setClassPathElements(classPathElements);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during method invokation");
        }
        assertEquals("The classPathElements ware not properly set",installationContextImpl.getClassPathElements(), classPathElements);
    }

}
