/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: InstallerTest.java 11:09:47 wjoseph $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.jbi.component.lifecycle;

import java.io.File;
import java.lang.reflect.Method;
import java.net.URI;

import javax.jbi.JBIException;
import javax.jbi.component.Bootstrap;
import javax.jbi.management.LifeCycleMBean;
import javax.management.MBeanNotificationInfo;
import javax.management.Notification;
import javax.management.NotificationBroadcasterSupport;
import javax.management.NotificationListener;
import javax.management.ObjectName;

import junit.framework.TestCase;

import org.objectweb.petals.classloader.LoaderManager;
import org.objectweb.petals.classloader.PetalsClassLoader;
import org.objectweb.petals.jbi.component.context.ComponentContextImpl;
import org.objectweb.petals.jbi.component.event.StateChangeFailedEvent;
import org.objectweb.petals.jbi.component.event.StateChangedEvent;
import org.objectweb.petals.jbi.component.thread.BootstrapThread;
import org.objectweb.petals.jbi.management.service.LifeCycleManagerService;
import org.objectweb.petals.jbi.management.systemstate.SystemState;
import org.objectweb.petals.jbi.mock.MockInstaller;
import org.objectweb.petals.jbi.mock.MockSystemState;
import org.objectweb.petals.tools.jbicommon.descriptor.ComponentDescription;
import org.objectweb.petals.util.LoggingUtil;
import org.objectweb.util.monolog.api.Logger;

import static org.easymock.EasyMock.expect;
import static org.easymock.EasyMock.isA;
import static org.easymock.classextension.EasyMock.createMock;
import static org.easymock.classextension.EasyMock.replay;
import static org.easymock.classextension.EasyMock.reset;
import static org.easymock.classextension.EasyMock.verify;

/**
 * Test of the Installer
 * 
 * @author wjoseph - eBMWebsourcing
 */
public class InstallerTest extends TestCase {

    private File testDataDir;
    
    public void testAddNotificationListener() {
        /*
         * Test Case 1
         */
        Installer installer = createMock(Installer.class,new Method[]{});
        NotificationBroadcasterSupport notifSupport = createMock(NotificationBroadcasterSupport.class);
        /*
         * Initialize Mocks
         */
        notifSupport.addNotificationListener(null, null, null);
        replay(notifSupport);
        installer.notifSupport = notifSupport;
        /*
         * Run Test Case 1
         */
        installer.addNotificationListener(null, null, null);
        verify(notifSupport);
    }
    
    public void testGetComponentName() {
        /*
         * Test Case 1
         */
        Installer installer = createMock(Installer.class,new Method[]{});
        ComponentContextImpl componentCtx = createMock(ComponentContextImpl.class);
       expect(componentCtx.getComponentName()).andReturn(null);
       replay(componentCtx);
       installer.componentContext = componentCtx;
       /*
        * Run Test Case 1
        */
       installer.getComponentName();
       verify(componentCtx);
    }
    
    public void testGetCurrentState() {
        /*
         * Test Case 1
         */
        Installer installer = createMock(Installer.class,new Method[]{});
        /*
         * Run Test Case 1
         */
        
        assertEquals("Wrong invokation result",LifeCycleMBean.UNKNOWN,installer.getCurrentState());
        
    }
    
    public void testGetInstallerConfigurationMBean() {
        /*
         * Test Case 1
         * The component is not in the INSTALLED state, an exception is raised
         * 
         * Create Mocks 
         */
        Method method1 = null;
        try {
            method1 = Installer.class.getDeclaredMethod("loadBootstrap", new Class[]{PetalsClassLoader.class});
        } catch (Exception e ) {
            e.printStackTrace();
            fail("Could not retrieve mocked methods");
        }
        Installer installer = createMock(Installer.class,new Method[]{method1});
        
        BootstrapThread installerThread = createMock(BootstrapThread.class);
        ObjectName oName = createMock(ObjectName.class);
        installer.componentJmxName = oName;
        
        expect(installerThread.getExtensionMBeanName()).andReturn(oName);
        replay(installerThread);
        installer.bootstrapThread = installerThread;
        
        LifeCycleManagerService lifeCycleManager = createMock(LifeCycleManagerService.class);
        installer.jmxAdmin = lifeCycleManager;
        ComponentContextImpl componentCtx = createMock(ComponentContextImpl.class);
        installer.componentContext = componentCtx;
        
        /*
         * Run Test Case 1, an exception must be raised
         */
        try {
            installer.getInstallerConfigurationMBean();
            fail("No exception was raised");
        } catch (Exception e) {
            // Do nothing
        }
        
        /*
         * Test Case 2
         * The component is in the INSTALLED state, and the component life cycle is in the SHUTDOWN state
         */
        reset(installer);
        reset(installerThread);
        reset(lifeCycleManager);
        reset(componentCtx);
        /*
         * Create Test parameters
         */
        Bootstrap bootstrap = createMock(Bootstrap.class);
        ObjectName objectName = createMock(ObjectName.class);
        PetalsClassLoader classloader = createMock(PetalsClassLoader.class);
        installer.state = Installer.INSTALLED;
        installer.bootstrapThread = installerThread;
        String componentName = "componentName";
        ComponentLifeCycle lifeCycle = createMock(ComponentLifeCycle.class);
        installer.componentJmxName = objectName;
        /*
         * Initialize Mocks
         */
        try {
            expect(bootstrap.getExtensionMBeanName()).andReturn(objectName);
            expect(installer.loadBootstrap(classloader)).andReturn(bootstrap);
            expect(installerThread.getExtensionMBeanName()).andReturn(objectName);
            expect(lifeCycleManager.getComponentByName(componentName)).andReturn(lifeCycle);
            expect(lifeCycle.getCurrentState()).andReturn(LifeCycleMBean.SHUTDOWN);
            expect(componentCtx.getComponentName()).andReturn(componentName);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not initialize Mocks");
        }
        replay(installer);
        replay(installerThread);
        replay(bootstrap);
        replay(lifeCycle);
        replay(lifeCycleManager);
        replay(componentCtx);
        installer.jmxAdmin = lifeCycleManager;
        installer.componentContext = componentCtx;
        installer.jbiBootstrap = bootstrap;
        installer.bootstrapThread = installerThread;
        ObjectName result = null;
        
        /*
         * Run Test Case 2
         */
        try {
            result = installer.getInstallerConfigurationMBean();
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during invokation");
        }
        verify(installerThread);
        verify(lifeCycle);
        verify(lifeCycleManager);
        verify(componentCtx);
        assertEquals("Wrong invokation result",result, objectName);
    }
    
    public void testGetInstallRoot() {
        /*
         * Test Case 1
         */
        Installer installer = createMock(Installer.class,new Method[]{});
        /*
         * Create Test parameters
         */
        String installationRoot = "installation/root";
        ComponentContextImpl componentCtx = createMock(ComponentContextImpl.class);
        expect(componentCtx.getInstallRoot()).andReturn(installationRoot);
        replay(componentCtx);
        installer.componentContext = componentCtx;
        /*
         * Run Test Case 1
         */
        String result = installer.getInstallRoot();
        assertNotNull("Result of getInstallRoot must not be null",result);
        assertFalse("Result of getInstallRoot must not be empty",result.length() == 0);
        verify(componentCtx);
    }
    
    public void testGetMBeanName() {
        /*
         * Test Case 1
         */
        Installer installer = createMock(Installer.class,new Method[]{});
        /*
         * Create Test parameters
         */
        installer.mBeanName = createMock(ObjectName.class);
        /*
         * Run Test Case 1
         */
        assertEquals("Wroong invokation result",installer.getMBeanName(),installer.mBeanName);
    }
    
    public void testGetNotificationInfo() {
        /*
         * Test Case 1
         */
        Installer installer = createMock(Installer.class,new Method[]{});
        /*
         * Create Test parameters
         */
        MBeanNotificationInfo[] result= null;
        /*
         * Run Test Case 1
         */
        result = installer.getNotificationInfo();
        assertTrue("Wrong  invokation result",result.length == 1);
    }
    
    public void testInstall() {
        /*
         * Test Case 1
         * The installer is in the wrong state an exception is raised
         */
        Method method1 = null;
        Method method2 = null;
        Method method3 = null;
        try {
            method1 = Installer.class.getDeclaredMethod("stateChangeFailed", new Class[]{StateChangeFailedEvent.class});
            method2 = Installer.class.getDeclaredMethod("setState", new Class[]{String.class});
            method3 = Installer.class.getDeclaredMethod("doInstall", new Class[]{});
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not retrieve mocked methods");
        }
        Method[] mockedMethods = new Method[]{method1, method2, method3};
        Installer installer = createMock(Installer.class,mockedMethods);
        LoggingUtil log = createMock(LoggingUtil.class);
        installer.log = log;
        /*
         * Create Test parameters
         */
        /*
         * Initialize Mocks
         */
        installer.stateChangeFailed(isA(StateChangeFailedEvent.class));
        replay(installer);
        installer.activitySynchronizer = new Object();
        /*
         * Run Test Case 1
         */
        try {
            installer.install();
            fail("No exception was raised");
        } catch (Exception e) {
                verify(installer);
            }
        
        /*
         * Test Case 2
         */
        reset(installer);
        /*
         * Create Test parameters
         */
        installer.state = Installer.UNINSTALLED;
        ObjectName objectName = createMock(ObjectName.class);
        installer.componentJmxName = objectName;
        ObjectName result = null;
        /*
         * Initialize Mocks
         */
        try {
            installer.setState(Installer.INSTALLING);
            installer.doInstall();
            installer.setState(Installer.INSTALLED);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not initialize Mocks");
        }
        replay(installer);
        /*
         * Run Test Case 2
         */
        try {
            result = installer.install();
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during invokation");
        }
        verify(installer);
        assertEquals("Wrong invokation result",result,objectName);
        
        /*
         * Test Case 3
         */
        //reset(installer);
        /*
         * Create Test parameters
         */
        installer = createMock(MockInstaller.class,new Method[]{method1, method2});
        installer.state = Installer.UNINSTALLED;
        installer.componentJmxName = objectName;
        installer.log = log;
        installer.activitySynchronizer = new Object();
        result = null;
        try {
           // installer.doInstall();
            installer.setState(Installer.INSTALLING);
            installer.setState(Installer.UNKNOWN);
            installer.stateChangeFailed(isA(StateChangeFailedEvent.class));
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not initialize Mocks");
        }
        replay(installer);
        /*
         * Run Test Case 3
         */
        try {
            result = installer.install();
            fail("No exception was raised");
        } catch (Exception e) {
            verify(installer);
        }
    }
    
    public void testStateChanged() {
        /*
         * Test Case 1
         */
        Installer installer = createMock(Installer.class,new Method[]{});
        /*
         * Create Test parameters
         */
        LoggingUtil log = createMock(LoggingUtil.class);
        installer.log = log;
        installer.mBeanName = createMock(ObjectName.class);
        StateChangedEvent stateChange = new StateChangedEvent("source","oldState","newState");
        NotificationBroadcasterSupport notificationBroacaster = createMock(NotificationBroadcasterSupport.class);
        /*
         * Initialize Mock
         */
        notificationBroacaster.sendNotification(isA(Notification.class));
        replay(notificationBroacaster);
        installer.notifSupport = notificationBroacaster;
        /*
         * Run Test Case 1
         */
        installer.stateChanged(stateChange);
        verify(notificationBroacaster);
        
    }
    
    public void testStateChangeFailed() {
        /*
         * Test Case 1
         */
        Installer installer = createMock(Installer.class,new Method[]{});
        /*
         * Create Test parameters
         */
        LoggingUtil log = createMock(LoggingUtil.class);
        installer.log = log;
        installer.mBeanName = createMock(ObjectName.class);
        StateChangeFailedEvent stateChangeFailed = new StateChangeFailedEvent("source",new JBIException("testStateChangeFailed testing exception"));
        NotificationBroadcasterSupport notificationBroacaster = createMock(NotificationBroadcasterSupport.class);
        /*
         * Initialize Mock
         */
        notificationBroacaster.sendNotification(isA(Notification.class));
        replay(notificationBroacaster);
        installer.notifSupport = notificationBroacaster;
        /*
         * Run Test Case 1
         */
        installer.stateChangeFailed(stateChangeFailed);
        verify(notificationBroacaster);
    }
    
    public void testInstaller() {
        /*
         * Test Case 1
         */
        /*
         * Create test parameters
         */
        Installer result = null;
        URI installationURI = new File (testDataDir, "fakearchive" + File.separator + "archive").toURI();
        LoaderManager loaderManager = createMock(LoaderManager.class);
        ComponentContextImpl componentCtx = createMock(ComponentContextImpl.class);
        ComponentDescription componentDescription = createMock(ComponentDescription.class);
        LifeCycleManagerService lifeCycleManager = createMock(LifeCycleManagerService.class);
        SystemState systemState = createMock(SystemState.class);
        Logger logger = null;
        ObjectName objectName = createMock(ObjectName.class);
        /*
         * Run Test Case 1
         */
        try {
        result = new Installer(installationURI, loaderManager,
                componentCtx, componentDescription, lifeCycleManager,
                systemState, objectName, logger);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during invokation");
        }
        assertNotNull("",result);
    }
    
    public void testIsInstalled() {
        /*
         * Test Case 1
         */
        Installer installer = createMock(Installer.class,new Method[]{});
        /*
         * Create Test parameters
         */
        installer.state = Installer.INSTALLED;
        /*
         * Run Test Case 1
         */
        assertTrue("Wrong invokation result",installer.isInstalled());
    }
    
    public void testRemoveNotificationListener() {
        /*
         * Test Case 1
         */
        Installer installer = createMock(Installer.class,new Method[]{});
        /*
         * Create Test parameters
         */
        installer.notifSupport = createMock(NotificationBroadcasterSupport.class);
        NotificationListener notificationListener = createMock(NotificationListener.class);
        /*
         * Run Test Case 1
         */
        try {
            installer.removeNotificationListener(notificationListener);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error during invokation");
        }
    }
    
    public void testSetState() {
        /*
         * Test Case 1
         * recoverySrv.updateComponentInstallationState throw an exception
         */
        Method method1 = null;
        Method method2 = null;
        try {
            method1 = Installer.class.getDeclaredMethod("stateChanged", new Class[]{StateChangedEvent.class});
            method2 = Installer.class.getDeclaredMethod("getComponentName", new Class[]{});
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not retrieve mocked methods");
        }
        Method[] mockedMethods = new Method[]{method1,method2};
        Installer installer = createMock(Installer.class,mockedMethods);
        /*
         * Create Test parameters
         */
        LoggingUtil log = createMock(LoggingUtil.class);
        installer.log = log;
        installer.recoverySrv = createMock(MockSystemState.class, new Method[]{});
        String componentName = "componentName";
        /*
         * Initialize Mocks
         */
        installer.stateChanged(isA(StateChangedEvent.class));
        expect(installer.getComponentName()).andReturn(componentName);
        /*
         * Run Test Case 1
         */
        try {
            installer.setState("state");
            fail("No exception was raised");
        } catch (Exception e) {
            // Do nothing
        }
    }
    
    public void testUninstall() {
        /*
         * Test Case 1
         * The installer state is not Installer.INSTALLED
         */
        Method method1 = null;
        Method method2 = null;
        Method method3 = null;
        try {
            method1 = Installer.class.getDeclaredMethod("stateChangeFailed", new Class[]{StateChangeFailedEvent.class});
            method2 = Installer.class.getDeclaredMethod("getComponentLifeCycle", new Class[]{});
            method3 = Installer.class.getDeclaredMethod("doUninstall", new Class[]{});
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not retrieve mocked methods");
        }
        Method[] mockedMethods = new Method[]{method1, method2, method3};
        Installer installer = createMock(Installer.class,mockedMethods);
        installer.state = Installer.UNINSTALLED;
        LoggingUtil log = createMock(LoggingUtil.class);
        installer.log = log;
        /*
         * Create Test parameters
         */
        /*
         * Initialize Mocks
         */
        installer.stateChangeFailed(isA(StateChangeFailedEvent.class));
        replay(installer);
        /*
         * Run Test Case 1
         */
        try {
        installer.uninstall();
        fail("No exception was raised");
        } catch (Exception e) {
            verify(installer);
        }
    }
    
    @Override
    protected void setUp() throws Exception {
        testDataDir = new File("src" + File.separator + "test-data");
        if (!testDataDir.exists()) {
            testDataDir = new File("container" + File.separator + "src"
                    + File.separator + "test-data");
            if (!testDataDir.exists()) {
                testDataDir = new File("platform" + File.separator
                        + "container" + File.separator + "src" + File.separator
                        + "test-data");
            }
        }
    }

   
}
