/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: ServiceUnitLifeCycleTest.java 11:09:47 ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.jbi.component.lifecycle;

import javax.jbi.JBIException;
import javax.management.ObjectName;

import junit.framework.TestCase;

import org.easymock.classextension.EasyMock;
import org.objectweb.petals.jbi.component.context.ComponentContextImpl;
import org.objectweb.petals.jbi.component.lifecycle.mock.MockComponentDescriptionBuilder;
import org.objectweb.petals.jbi.component.lifecycle.mock.MockServiceUnitBuilder;
import org.objectweb.petals.jbi.component.lifecycle.mock.ServiceUnitManagerMock;
import org.objectweb.petals.jbi.management.service.LifeCycleManagerImpl;
import org.objectweb.petals.jbi.management.systemstate.SystemStateImpl;
import org.objectweb.petals.jbi.routing.mock.ComponentMock;
import org.objectweb.petals.util.LoggingUtil;

/**
 * Test of the ServiceUnitLifeCycle
 * 
 * @author ddesjardins - eBMWebsourcing
 */
public class ServiceUnitLifeCycleTest extends TestCase {

    private ServiceUnitLifeCycle serviceUnitLifeCycle;

    private ServiceUnitManagerMock serviceUnitManagerMock;

    private String baseDir;

    public void setUp() throws Exception {
        LifeCycleManagerImpl lifeCycleManagerImpl = new LifeCycleManagerImpl();

        ComponentMock componentMock = new ComponentMock();
        serviceUnitManagerMock = new ServiceUnitManagerMock();
        componentMock.setServiceUnitManager(serviceUnitManagerMock);

        serviceUnitManagerMock.deploy("testServiceUnit", "none");

        ComponentLifeCycle componentLifeCycle = new ComponentLifeCycle(
            MockComponentDescriptionBuilder.buildComponentDescription(),
            componentMock, new ComponentContextImpl(), new SystemStateImpl(),
            new ObjectName("test@test:name=test"), null, EasyMock
                .createMock(LoggingUtil.class));

        lifeCycleManagerImpl.getBindingCompoLifeCycles().put(
            new ObjectName("test@test:name=test"), componentLifeCycle);
        lifeCycleManagerImpl.start();

        baseDir = this.getClass().getResource(".").toString();
        baseDir = baseDir.substring(0, baseDir.indexOf("target"));
        baseDir = baseDir.substring(baseDir.indexOf(":") + 1);
        serviceUnitLifeCycle = new ServiceUnitLifeCycle(MockServiceUnitBuilder
            .buildServiceUnit(), lifeCycleManagerImpl, baseDir + "target",
            EasyMock.createMock(LoggingUtil.class));
    }

    public void testGetServiceUnitManager() {
        assertEquals(serviceUnitLifeCycle.getServiceUnitManager(),
            serviceUnitManagerMock);
    }

    public void testDoStart() throws JBIException {
        serviceUnitLifeCycle.doStart();
        assertEquals(serviceUnitManagerMock.getUnits().get("testServiceUnit")
            .getState(), "Started");
    }

    public void testDoShutdown() throws JBIException {
        serviceUnitLifeCycle.doShutdown();
        assertEquals(serviceUnitManagerMock.getUnits().get("testServiceUnit")
            .getState(), "Shutdown");
    }

    public void testDoStop() throws JBIException {
        serviceUnitLifeCycle.doStop();
        assertEquals(serviceUnitManagerMock.getUnits().get("testServiceUnit")
            .getState(), "Stopped");
    }

    public void testGetSuName() {
        assertEquals(serviceUnitLifeCycle.getSuName(), "testServiceUnit");
    }

    public void testGetServiceUnitRootPath() {
        assertEquals(serviceUnitLifeCycle.getServiceUnitRootPath(), baseDir
            + "target");
    }

}
