/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2006 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: MockJBIDescriptorBuilder.java 154 16 mai 2006 ofabre $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.jbi.component.lifecycle.mock;

import java.net.URI;
import java.util.ArrayList;
import java.util.List;

import javax.xml.namespace.QName;

import org.objectweb.petals.tools.jbicommon.descriptor.Connection;
import org.objectweb.petals.tools.jbicommon.descriptor.JBIDescriptor;
import org.objectweb.petals.tools.jbicommon.descriptor.JBIDescriptorBuilder;
import org.objectweb.petals.tools.jbicommon.descriptor.ServiceUnit;
import org.objectweb.util.monolog.api.Logger;

/**
 * 
 * @author Olivier Fabre - EBM Websourcing
 * 
 */
public class MockJBIDescriptorBuilder extends JBIDescriptorBuilder {

    public MockJBIDescriptorBuilder(URI sourceUri, Logger logger) {
        super(sourceUri, (java.util.logging.Logger) logger);
    }

    /**
     * Create a light Component JBIDescriptor
     * 
     * @return an empty JBIDescriptor
     */
    public static JBIDescriptor buildComponentDescriptor() {
        JBIDescriptorMock descriptor = new JBIDescriptorMock();

        ComponentDescriptionMock componentDescription = new ComponentDescriptionMock();
        IdentificationMock identification = new IdentificationMock();
        identification.setName("testComponent");
        componentDescription.setIdentification(identification);
        descriptor.setComponent(componentDescription);

        return descriptor;
    }

    /**
     * Create a light Component JBIDescriptor
     * 
     * @return an empty JBIDescriptor
     */
    public static JBIDescriptor buildComponentDescriptorWithClasspath() {
        JBIDescriptorMock descriptor = new JBIDescriptorMock();

        ComponentDescriptionMock componentDescription = new ComponentDescriptionMock();
        IdentificationMock identification = new IdentificationMock();
        identification.setName("testComponent");
        componentDescription.setIdentification(identification);
        List<String> classpath = new ArrayList<String>();
        classpath.add("Engine1.jar");
        componentDescription.setComponentClassPath(classpath);
        componentDescription.setBootstrapClassPath(classpath);
        descriptor.setComponent(componentDescription);

        return descriptor;
    }

    /**
     * Create a light Component JBIDescriptor
     * 
     * @return an empty JBIDescriptor
     */
    public static JBIDescriptor buildComponentDescriptorWithBadClasspath() {
        JBIDescriptorMock descriptor = new JBIDescriptorMock();

        ComponentDescriptionMock componentDescription = new ComponentDescriptionMock();
        IdentificationMock identification = new IdentificationMock();
        identification.setName("testComponent");
        componentDescription.setIdentification(identification);
        List<String> classpath = new ArrayList<String>();
        classpath.add("");
        componentDescription.setComponentClassPath(classpath);
        componentDescription.setBootstrapClassPath(classpath);
        descriptor.setComponent(componentDescription);

        return descriptor;
    }

    /**
     * Create a light SharedLib JBIDescriptor
     * 
     * @return an empty JBIDescriptor
     */
    public static JBIDescriptor buildSharedLibDescriptor() {
        JBIDescriptorMock descriptor = new JBIDescriptorMock();

        SharedLibraryMock sharedLibrary = new SharedLibraryMock();
        IdentificationMock identification = new IdentificationMock();
        identification.setName("testSharedLib");
        sharedLibrary.setIdentification(identification);
        descriptor.setSharedLibrary(sharedLibrary);

        return descriptor;
    }

    /**
     * Create a light SharedLib JBIDescriptor
     * 
     * @return an empty JBIDescriptor
     */
    public static JBIDescriptor buildSharedLibDescriptorWithClasspath() {
        JBIDescriptorMock descriptor = new JBIDescriptorMock();

        SharedLibraryMock sharedLibrary = new SharedLibraryMock();
        IdentificationMock identification = new IdentificationMock();
        identification.setName("testSharedLib");
        sharedLibrary.setIdentification(identification);
        List<String> classpath = new ArrayList<String>();
        classpath.add("Lib01.jar");
        sharedLibrary.setSharedLibraryClassPath(classpath);
        descriptor.setSharedLibrary(sharedLibrary);

        return descriptor;
    }

    /**
     * Create a light ServiceAssembly JBIDescriptor
     * 
     * @return an empty JBIDescriptor
     */
    public static JBIDescriptor buildServiceAssemblyDescriptor() {
        JBIDescriptorMock descriptor = new JBIDescriptorMock();

        ServiceAssemblyMock serviceAssembly = new ServiceAssemblyMock();
        IdentificationMock identification = new IdentificationMock();
        identification.setName("testServiceAssembly");
        serviceAssembly.setIdentification(identification);
        descriptor.setServiceAssembly(serviceAssembly);

        return descriptor;
    }

    /**
     * Create a light ServiceAssembly JBIDescriptor
     * 
     * @return an empty JBIDescriptor
     */
    public static JBIDescriptor buildServiceAssemblyDescriptorWithSU() {
        JBIDescriptorMock descriptor = new JBIDescriptorMock();

        ServiceAssemblyMock serviceAssembly = new ServiceAssemblyMock();
        IdentificationMock identification = new IdentificationMock();
        identification.setName("testServiceAssembly");
        serviceAssembly.setIdentification(identification);
        List<ServiceUnit> units = new ArrayList<ServiceUnit>();
        ServiceUnitMock serviceUnit = new ServiceUnitMock();
        IdentificationMock identification1 = new IdentificationMock();
        identification1.setName("testSU");
        serviceUnit.setIdentification(identification1);
        serviceUnit.setTargetArtifactsZip("serviceUnit.zip");
        units.add(serviceUnit);
        serviceAssembly.setServiceUnits(units);
        descriptor.setServiceAssembly(serviceAssembly);

        return descriptor;
    }

    /**
     * Create a light ServiceAssembly JBIDescriptor
     * 
     * @return an empty JBIDescriptor
     */
    public static JBIDescriptor buildServiceAssemblyDescriptorWithSUFoo() {
        JBIDescriptorMock descriptor = new JBIDescriptorMock();

        ServiceAssemblyMock serviceAssembly = new ServiceAssemblyMock();
        IdentificationMock identification = new IdentificationMock();
        identification.setName("testServiceAssembly");
        serviceAssembly.setIdentification(identification);
        List<ServiceUnit> units = new ArrayList<ServiceUnit>();
        ServiceUnitMock serviceUnit = new ServiceUnitMock();
        IdentificationMock identification1 = new IdentificationMock();
        identification1.setName("testSU");
        serviceUnit.setIdentification(identification1);
        serviceUnit.setTargetArtifactsZip("serviceUnitFoo.zip");
        units.add(serviceUnit);
        serviceAssembly.setServiceUnits(units);
        descriptor.setServiceAssembly(serviceAssembly);

        return descriptor;
    }

    /**
     * Create a light ServiceAssembly JBIDescriptor with connections
     * 
     * @return an empty JBIDescriptor
     */
    public static JBIDescriptor buildServiceAssemblyDescriptorWithConnections() {
        JBIDescriptorMock descriptor = new JBIDescriptorMock();

        ServiceAssemblyMock serviceAssembly = new ServiceAssemblyMock();
        IdentificationMock identification = new IdentificationMock();
        identification.setName("testServiceAssembly");
        serviceAssembly.setIdentification(identification);
        List<Connection> connections = new ArrayList<Connection>();
        ConnectionMock connection = new ConnectionMock();
        connection.setConsumerServiceName(new QName("test"));
        connection.setConsumerEndpointName("endpoint:test");
        connection.setProviderEndpointName("endpoint:test");
        connection.setProviderServiceName(new QName("test"));
        connection.setConsumerInterfaceName(new QName("interfacename"));
        connections.add(connection);
        serviceAssembly.setConnections(connections);
        descriptor.setServiceAssembly(serviceAssembly);

        return descriptor;
    }

}
