/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.jbi.component.thread;

import javax.jbi.JBIException;
import javax.jbi.component.Component;
import javax.jbi.component.ComponentContext;
import javax.jbi.component.ComponentLifeCycle;

import junit.framework.TestCase;
import static org.easymock.EasyMock.expect;
import static org.easymock.classextension.EasyMock.createMock;
import static org.easymock.classextension.EasyMock.replay;
import static org.easymock.classextension.EasyMock.reset;
import static org.easymock.classextension.EasyMock.verify;

/**
 * Component life cycle thread unit test
 * 
 * @author Christophe Hamerling - EBM WebSourcing
 */
public class ComponentLifeCycleThreadTest extends TestCase {

    /**
     * The component life cycle thread to test
     */
    private ComponentLifeCycleThread lifeCycleThread;
    
    /**
     * The mock life cycle
     */
    private ComponentLifeCycle lifeCycleMock;

    /**
     * The mock component
     */
    private Component componentMock;

    /**
     * The mock component context
     */
    private ComponentContext contextMock;

    /**
     * 
     * 
     */
    public void testDoInit() throws Exception {
        lifeCycleThread.start();
        reset(componentMock);
        expect(componentMock.getLifeCycle()).andReturn(lifeCycleMock);
        replay(componentMock);

        lifeCycleThread.doInit(contextMock);
        verify(componentMock);
        
        JBIException jbie = lifeCycleThread.getJbiException();
        assertNull(jbie);
    }

    /**
     * 
     * 
     */
    public void testDoStart() throws Exception {
        lifeCycleThread.start();
        reset(componentMock);
        expect(componentMock.getLifeCycle()).andReturn(lifeCycleMock);
        replay(componentMock);

        lifeCycleThread.doStart();
        verify(componentMock);
        
        JBIException jbie = lifeCycleThread.getJbiException();
        assertNull(jbie);
    }

    /**
     * 
     */
    public void testDoStop() throws Exception {
        lifeCycleThread.start();

        reset(componentMock);
        expect(componentMock.getLifeCycle()).andReturn(lifeCycleMock);
        replay(componentMock);

        lifeCycleThread.doStop();
        verify(componentMock);
        
        JBIException jbie = lifeCycleThread.getJbiException();
        assertNull(jbie);
    }

    /**
     * 
     * 
     */
    public void testDoShutdown() throws Exception {
        lifeCycleThread.start();

        reset(componentMock);
        expect(componentMock.getLifeCycle()).andReturn(lifeCycleMock);
        replay(componentMock);

        lifeCycleThread.doShutdown();
        verify(componentMock);

        JBIException jbie = lifeCycleThread.getJbiException();
        assertNull(jbie);
    }

    /**
     * Test multiple calls
     *
     */
    public void testMultipleCalls() {
        lifeCycleThread.start();

        // stop component
        reset(componentMock);
        expect(componentMock.getLifeCycle()).andReturn(lifeCycleMock);
        replay(componentMock);
        try {
            lifeCycleThread.doStop();
        } catch (JBIException e) {
            fail(e.getMessage());
        }
        verify(componentMock);

        // shutdown component
        reset(componentMock);
        expect(componentMock.getLifeCycle()).andReturn(lifeCycleMock);
        replay(componentMock);
        try {
            lifeCycleThread.doShutdown();
        } catch (JBIException e) {
            fail(e.getMessage());
        }
        verify(componentMock);
        
    }
    
    /*
     * (non-Javadoc)
     * 
     * @see junit.framework.TestCase#setUp()
     */
    protected void setUp() throws Exception {
        super.setUp();

        componentMock = createMock(Component.class);
        contextMock = createMock(ComponentContext.class);
        lifeCycleMock = createMock(ComponentLifeCycle.class);

        lifeCycleThread = new ComponentLifeCycleThread(componentMock);
    }

}
