/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: AutoLoaderImplTest.java 3:54:28 PM ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.jbi.management.autoload;

import static org.easymock.EasyMock.expect;
import static org.easymock.classextension.EasyMock.createMock;
import static org.easymock.classextension.EasyMock.replay;
import static org.easymock.classextension.EasyMock.verify;

import java.io.File;
import java.io.IOException;
import java.lang.reflect.Method;
import java.net.URI;
import java.util.ArrayList;
import java.util.List;

import javax.management.InstanceNotFoundException;
import javax.management.MBeanException;
import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;
import javax.management.ReflectionException;

import junit.framework.TestCase;

import org.apache.commons.io.FileUtils;
import org.easymock.classextension.EasyMock;
import org.objectweb.petals.PetalsException;
import org.objectweb.petals.jbi.management.service.InstallationServiceImpl;
import org.objectweb.petals.jbi.management.service.LifeCycleManagerImpl;
import org.objectweb.petals.jbi.management.service.PackageHandler;
import org.objectweb.petals.jbi.management.systemstate.ComponentState;
import org.objectweb.petals.jbi.management.systemstate.ServiceAssemblyState;
import org.objectweb.petals.jbi.management.systemstate.SharedLibraryState;
import org.objectweb.petals.jbi.management.systemstate.SystemState;
import org.objectweb.petals.tools.jbicommon.descriptor.ComponentDescription;
import org.objectweb.petals.tools.jbicommon.descriptor.JBIDescriptor;
import org.objectweb.petals.util.LoggingUtil;
import org.objectweb.util.monolog.Monolog;
/**
 * Test of the AutoLoaderImpl
 * 
 * @author ddesjardins - eBMWebsourcing
 * @author wjoseph - eBMWebsourcing
 */
public class AutoLoaderImplTest extends TestCase {

    private AutoLoaderImpl autoLoaderImpl;

    private String baseDir;

    public void setUp() {
        baseDir = this.getClass().getResource(".").toString();
        baseDir = baseDir.substring(0, baseDir.indexOf("target"));
        baseDir = baseDir.substring(baseDir.indexOf(":") + 1);
        System.setProperty("petals.home", new File(baseDir + "target")
            .getAbsolutePath());
        new File(baseDir + "target" + File.separator + "lib").mkdir();
        new File(baseDir + "target" + File.separator + "lib" + File.separator
            + "petals-1.0-SNAPSHOT.jar").mkdir();
        autoLoaderImpl = new AutoLoaderImpl();
    }

    public void testStart() throws IOException {
        autoLoaderImpl.logger = Monolog.initialize().getLogger("test");
        autoLoaderImpl.start();
        assertEquals(AutoLoaderImpl.getInstallDirectory(), new File(baseDir
            + "target" + File.separator + "install"));
        assertEquals(AutoLoaderImpl.getInstalledDirectory(), new File(baseDir
            + "target" + File.separator + "installed"));
        assertEquals(AutoLoaderImpl.getWorkDirectory(), new File(baseDir
            + "target" + File.separator + "work"));
        assertEquals(AutoLoaderImpl.getUninstalledDirectory(), new File(baseDir
            + "target" + File.separator + "uninstalled"));

        //old testStart method
        autoLoaderImpl.log = new LoggingUtil(autoLoaderImpl.logger);
        assertNotNull(autoLoaderImpl.installDirectoryScanTimer);
        assertNotNull(autoLoaderImpl.installedDirectoryScanTimer);
    }

    public void testGetInstallDirectory () {
        /*
         * Test Case  1
         */
        File install = new File("");
        AutoLoaderImpl.installDirectory = install;  
        /*
         * Run Test Case  1
         */
        assertEquals("Wrong invokation result",AutoLoaderImpl.getInstallDirectory(),install);
    }
 
    public void testGetInstalledDirectory () {
        /*
         * Test Case  1
         */
        File installed = new File("");
        AutoLoaderImpl.installedDirectory = installed;  
        /*
         * Run Test Case  1
         */
        assertEquals("Wrong invokation result",AutoLoaderImpl.getInstallDirectory(),installed);
    }
    
    public void testGetUninstalDirectory () {
        /*
         * Test Case  1
         */
        File uninstalled = new File("");
        AutoLoaderImpl.uninstalledDirectory = uninstalled;  
        /*
         * Run Test Case  1
         */
        assertEquals("Wrong invokation result",AutoLoaderImpl.getUninstalledDirectory(),uninstalled);
    }
    
    public void testGetWorkDirectory () {
        /*
         * Test Case  1
         */
        File work = new File("");
        AutoLoaderImpl.workDirectory = work;  
        /*
         * Run Test Case  1
         */
        assertEquals("Wrong invokation result",AutoLoaderImpl.getWorkDirectory(),work);
    }
    
    public void tearDown() {
        System.clearProperty("petals.home");
    }

    public void testInstallComponent() throws IOException, PetalsException,
        MalformedObjectNameException, NullPointerException,
        InstanceNotFoundException, MBeanException, ReflectionException {
        FileUtils.copyFileToDirectory(new File(baseDir + "src" + File.separator
            + "test-data" + File.separator + "autoload" + File.separator
            + "toInstall.zip"), new File(baseDir + "target"));
        testStart();

        JBIDescriptor descriptor = EasyMock.createMock(JBIDescriptor.class);
        ComponentDescription componentDescription = EasyMock
            .createMock(ComponentDescription.class);
        EasyMock.replay(componentDescription);

        EasyMock.expect(descriptor.getComponent()).andReturn(
            componentDescription).anyTimes();
        EasyMock.replay(descriptor);

        PackageHandler packageHandler = EasyMock
            .createMock(PackageHandler.class);
        EasyMock.expect(
            packageHandler.loadDescriptor(new File(baseDir + "target"
                + File.separator + "work" + File.separator + "toInstall.zip")
                .toURI())).andReturn(descriptor).anyTimes();
        EasyMock.replay(packageHandler);

        LifeCycleManagerImpl lifeCycleManagerImpl = EasyMock
            .createMock(LifeCycleManagerImpl.class);

        MockMbeanServer server = new MockMbeanServer();

        EasyMock.expect(lifeCycleManagerImpl.getMBeanServer())
            .andReturn(server).anyTimes();

        EasyMock.replay(lifeCycleManagerImpl);

        InstallationServiceImpl installationService = EasyMock
            .createMock(InstallationServiceImpl.class);
        EasyMock.expect(
            installationService.loadNewInstaller(new File(baseDir + "target"
                + File.separator + "work" + File.separator + "toInstall.zip")
                .toURI().toString())).andReturn(
            new ObjectName("test@test:name=installer")).anyTimes();
        EasyMock.replay(installationService);

        autoLoaderImpl.installationService = installationService;
        autoLoaderImpl.managerService = lifeCycleManagerImpl;
        autoLoaderImpl.packageHandler = packageHandler;
        autoLoaderImpl.log = new LoggingUtil(autoLoaderImpl.logger);
        autoLoaderImpl.install(new File[] {new File(baseDir + "target"
            + File.separator + "toInstall.zip")});

        assertTrue(server.isInvokeStart());
    }
    
    public void testUninstall() {
        /*
         * Test Case 1
         */
        /*
         * Create Mocks
         */
        Method method1 = null;
        Method method2 = null;
        Method method3 = null;
        try {
            method1 = AutoLoaderImpl.class.getDeclaredMethod("performCompUninstall", new Class[]{String.class});
            method2 = AutoLoaderImpl.class.getDeclaredMethod("performSAUninstall", new Class[]{String.class});
            method3 = AutoLoaderImpl.class.getDeclaredMethod("performSLUninstall", new Class[]{String.class});
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not retrieve mocked methods");
        }
        autoLoaderImpl = createMock(AutoLoaderImpl.class,new Method[]{method1, method2, method3});
        /*
         * Create Test parameters
         */
        List<ComponentState> compStates = new ArrayList<ComponentState>();
        ComponentState compState = createMock(ComponentState.class);
        compStates.add(compState);
        List<SharedLibraryState> slStates = new ArrayList<SharedLibraryState>();
        SharedLibraryState slState = createMock(SharedLibraryState.class);
        slStates.add(slState);
        List<ServiceAssemblyState> saStates = new ArrayList<ServiceAssemblyState>();
        ServiceAssemblyState saState = createMock(ServiceAssemblyState.class);
        saStates.add(saState);
        String compName = "component Name";
        String slName = " shared library name";
        String saName = "service assembly name";
        String url = "myUrl";
        File file = new File("");
        URI uri = file.toURI();
        file = new File(uri);
        PackageHandler packageHandler = createMock(PackageHandler.class);
        List<File> fileToUninstall = new ArrayList<File>();
        fileToUninstall.add(file);
        LoggingUtil log = createMock(LoggingUtil.class);
        SystemState systemState = createMock(SystemState.class);
        /*
         * Initialize Mocks
         */
        expect(compState.getName()).andReturn(compName);
        expect(compState.getArchiveURL()).andReturn(url);
        expect(slState.getName()).andReturn(slName);
        expect(slState.getArchiveURL()).andReturn(url);
        expect(saState.getName()).andReturn(saName);
        expect(saState.getArchiveURL()).andReturn(url);
        expect(packageHandler.processAndGetPackageURI(url, false)).andReturn(uri).anyTimes();
        autoLoaderImpl.performCompUninstall(compName);
        autoLoaderImpl.performSAUninstall(saName);
        autoLoaderImpl.performSLUninstall(slName);
        expect(systemState.retrieveAllComponentStates()).andReturn(compStates);
        expect(systemState.retrieveAllSharedLibraryStates()).andReturn(slStates);
        expect(systemState.retrieveAllServiceAssemblyStates()).andReturn(saStates);
        replay(compState);
        replay(slState);
        replay(saState);
        replay(packageHandler);
        replay(autoLoaderImpl);
        replay(systemState);
        autoLoaderImpl.packageHandler = packageHandler;
        autoLoaderImpl.log = log;
        autoLoaderImpl.recoverySrv = systemState;
        /*
         * Run Test Case 1
         */
        autoLoaderImpl.uninstall(fileToUninstall);
        verify(compState);
        verify(slState);
        verify(saState);
        verify(packageHandler);
        verify(autoLoaderImpl);
        verify(systemState);
    }
}
