/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: DirectoryScannerTest.java 14:37:50 ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.jbi.management.autoload;

import java.io.File;
import java.io.IOException;

import junit.framework.TestCase;

import org.apache.commons.io.FileUtils;

/**
 * Test of the DirectoryScanner
 * 
 * @author ddesjardins - eBMWebsourcing
 */
public class InstallDirectoryScannerTest extends TestCase {

    private InstallDirectoryScanner directoryScanner;

    private String baseDir;

    private AutoLoaderImplMock autoLoadImplMock;

    public void setUp() throws IOException {
        autoLoadImplMock = new AutoLoaderImplMock();
        baseDir = this.getClass().getResource(".").toString();
        baseDir = baseDir.substring(0, baseDir.indexOf("target"));
        baseDir = baseDir.substring(baseDir.indexOf(":") + 1);
        if (new File(baseDir + "target" + File.separator + "install").exists()) {
            FileUtils.forceDelete(new File(baseDir + "target" + File.separator
                + "install"));
        }
        new File(baseDir + "target" + File.separator + "install").mkdir();
        directoryScanner = new InstallDirectoryScanner(autoLoadImplMock,
            new File(baseDir + "target" + File.separator + "install"));
    }

    public void testRunInstall() throws IOException {
        new File(baseDir + "target" + File.separator).mkdir();
        new File(baseDir + "target" + File.separator + "install").mkdir();
        FileUtils.copyFileToDirectory(new File(baseDir + "src" + File.separator
            + "test-data" + File.separator + "packages" + File.separator
            + "installation01.zip"), new File(baseDir + "target"
            + File.separator + "install"));
        directoryScanner.run();
        assertTrue(autoLoadImplMock.isInstalled());
    }

    public void testRunNothing() throws IOException {
        FileUtils.forceDelete(new File(baseDir + "target" + File.separator
            + "install"));
        new File(baseDir + "target" + File.separator + "install").mkdir();
        directoryScanner.run();
        assertFalse(autoLoadImplMock.isInstalled());
    }

}
