package org.objectweb.petals.jbi.management.service;

import javax.management.MBeanServer;
import javax.management.ObjectName;

import junit.framework.TestCase;

import org.objectweb.fractal.jmx.agent.AdminAttributes;
import org.objectweb.petals.jbi.component.lifecycle.ComponentLifeCycle;
import org.objectweb.petals.jbi.component.lifecycle.Installer;
import org.objectweb.petals.util.LoggingUtil;

import static org.easymock.EasyMock.createMock;
import static org.easymock.EasyMock.replay;
import static org.easymock.EasyMock.expect;
import static org.easymock.EasyMock.verify;

public class LifeCycleManagerImplTest extends TestCase {

    AdminAttributes jmx = null;

    LifeCycleManagerImpl manager = null;

    MBeanNamesImpl mbnames = null;

    ObjectName compName = null;

    protected void setUp() throws Exception {
        super.setUp();
        manager = new LifeCycleManagerImpl();
        manager.start();

        jmx = createMock(AdminAttributes.class);
        //replay(jmx);
        manager.adminAttributes = jmx;

        compName = new ObjectName("test:name=test");
    }

    /*
     * Test method for
     * 'org.objectweb.petals.jbi.management.service.LifeCycleManagerImpl.registerBindingComponent(ComponentLifeCycle)'
     */
    public void testRegisterBindingComponent() throws Exception {
        ComponentLifeCycle comp = new ComponentLifeCycle(null, null, null,
            null, compName, null, new LoggingUtil(null)); 

        MBeanServer beanServer  = createMock(MBeanServer.class);
        // record scenario
        expect(jmx.getRawMBeanServer()).andReturn(beanServer);
        // play
        replay(jmx);
        manager.registerBindingComponent(comp);
        verify(jmx);
    }

    /*
     * Test method for
     * 'org.objectweb.petals.jbi.management.service.LifeCycleManagerImpl.unregisterBindingComponent(ObjectName)'
     */
    public void testUnregisterBindingComponent() throws Exception {
       MBeanServer beanServer  = createMock(MBeanServer.class);
        // record scenario
        expect(jmx.getRawMBeanServer()).andReturn(beanServer);
        // play
        replay(jmx);
        manager.unregisterBindingComponent(compName);
        verify(jmx);
    }

    
    /*
     * Test method for
     * 'org.objectweb.petals.jbi.management.service.LifeCycleManagerImpl.registerEngineComponent(ComponentLifeCycle)'
     */
    public void testRegisterEngineComponent() throws Exception {
        ComponentLifeCycle comp = new ComponentLifeCycle(null, null, null,
            null, compName, null, new LoggingUtil(null));
        
        MBeanServer beanServer  = createMock(MBeanServer.class);
        // record scenario
        expect(jmx.getRawMBeanServer()).andReturn(beanServer);
        // play
        replay(jmx);
        manager.registerEngineComponent(comp);
        verify(jmx);
    }

    /*
     * Test method for
     * 'org.objectweb.petals.jbi.management.service.LifeCycleManagerImpl.unregisterEngineComponent(ObjectName)'
     */
    public void testUnregisterEngineComponent() throws Exception {
        MBeanServer beanServer  = createMock(MBeanServer.class);
        // record scenario
        expect(jmx.getRawMBeanServer()).andReturn(beanServer);
        // play
        replay(jmx);
        manager.unregisterEngineComponent(compName);
        verify(jmx);

    }

    /*
     * Test method for
     * 'org.objectweb.petals.jbi.management.service.LifeCycleManagerImpl.registerInstaller(Installer)'
     */
    public void testRegisterInstaller() throws Exception {
        Installer comp = new Installer(null, null, null, null, null, null,
            compName, null);
        MBeanServer beanServer  = createMock(MBeanServer.class);
        // record scenario
        expect(jmx.getRawMBeanServer()).andReturn(beanServer);
        // play
        replay(jmx);
        manager.registerInstaller(comp);
        verify(jmx);
    }

    /*
     * Test method for
     * 'org.objectweb.petals.jbi.management.service.LifeCycleManagerImpl.unregisterInstaller(ObjectName)'
     */
    public void testUnregisterInstaller() throws Exception {
        MBeanServer beanServer  = createMock(MBeanServer.class);
        // record scenario
        expect(jmx.getRawMBeanServer()).andReturn(beanServer);
        // play
        replay(jmx);
        manager.unregisterEngineComponent(compName);
        verify(jmx);
    }

}
