/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: MockRegistry.java 16:29:05 ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.jbi.management.service.mock;

import java.util.ArrayList;
import java.util.List;

import javax.jbi.servicedesc.ServiceEndpoint;
import javax.naming.Context;
import javax.xml.namespace.QName;

import org.objectweb.petals.jbi.registry.AbstractEndpoint;
import org.objectweb.petals.jbi.registry.ConsumerEndpoint;
import org.objectweb.petals.jbi.registry.InternalEndpoint;
import org.objectweb.petals.jbi.registry.LinkedEndpoint;
import org.objectweb.petals.jbi.registry.Registry;
import org.objectweb.petals.jbi.registry.RegistryException;

/**
 * Mock object of the Registry
 * 
 * @author ddesjardins - eBMWebsourcing
 */
public class MockRegistry implements Registry {

    private boolean deregisterConnection;

    private boolean deregisterExternalEndpoint;

    private boolean deregisterInternalEndpoint;

    private boolean getExternalEndpoint;

    private boolean getExternalEndpointsForInterface;

    private boolean getExternalEndpointsForService;

    private boolean getInternalEndpoint;

    private boolean getInternalEndpointsForInterface;

    private boolean getInternalEndpointsForService;

    private boolean registerConnection;

    private boolean registerExternalEndpoint;

    private boolean registerInternalEndpoint;

    private boolean retrieveNewEndpoints;

    private boolean validateEndpoint;

    public void deregisterConnection(LinkedEndpoint connection)
        throws RegistryException {
        deregisterConnection = true;
    }

    public void deregisterExternalEndpoint(ServiceEndpoint externalEndpoint)
        throws RegistryException {
        deregisterExternalEndpoint = true;
    }

    public void deregisterInternalEndpoint(AbstractEndpoint endpoint)
        throws RegistryException {
        deregisterInternalEndpoint = true;
    }

    public AbstractEndpoint getExternalEndpoint(QName service, String name)
        throws RegistryException {
        getExternalEndpoint = true;
        return null;
    }

    public AbstractEndpoint[] getExternalEndpointsForInterface(
        QName interfaceName) throws RegistryException {
        getExternalEndpointsForInterface = true;
        return new AbstractEndpoint[] {new ConsumerEndpoint("compo", "0")};
    }

    public AbstractEndpoint[] getExternalEndpointsForService(QName serviceName) {
        getExternalEndpointsForService = true;
        return new AbstractEndpoint[] {new ConsumerEndpoint("compo", "0")};
    }

    public AbstractEndpoint getInternalEndpoint(QName service, String name)
        throws RegistryException {
        getInternalEndpoint = true;
        InternalEndpoint internalEndpoint = new InternalEndpoint(new QName(
            "service"), "endpoint", "compo", "0", null);
        if (service.equals(new QName(
            "http://127.0.0.1:8080/axis/Helloworld.jws", "HelloworldService"))) {
            internalEndpoint.setInterfaces(new QName[] {new QName("test")});
        }
        return internalEndpoint;
    }

    public AbstractEndpoint[] getInternalEndpointsForInterface(
        QName interfaceName) throws RegistryException {
        getInternalEndpointsForInterface = true;
        AbstractEndpoint[] endpoints = new AbstractEndpoint[0];
        if (interfaceName.equals(new QName("inter"))) {
            endpoints = new AbstractEndpoint[] {new InternalEndpoint(new QName(
                "serv"), "endpoiny", "compo", "0", null)};
        }
        return endpoints;
    }

    public AbstractEndpoint[] getInternalEndpointsForService(QName serviceName) {
        getInternalEndpointsForService = true;
        AbstractEndpoint[] endpoints = new AbstractEndpoint[0];
        if (serviceName.equals(new QName("serv"))) {
            endpoints = new AbstractEndpoint[] {new InternalEndpoint(
                serviceName, "endpoint", "compo", "0", null)};
        }
        return endpoints;
    }

    public boolean isDeregisterConnection() {
        return deregisterConnection;
    }

    public boolean isDeregisterExternalEndpoint() throws RegistryException {
        return deregisterExternalEndpoint;
    }

    public boolean isDeregisterInternalEndpoint() throws RegistryException {
        return deregisterInternalEndpoint;
    }

    public boolean isGetExternalEndpoint() {
        return getExternalEndpoint;
    }

    public boolean isGetExternalEndpointsForInterface()
        throws RegistryException {
        return getExternalEndpointsForInterface;
    }

    public boolean isGetExternalEndpointsForService() throws RegistryException {
        return getExternalEndpointsForService;
    }

    public boolean isGetInternalEndpoint() throws RegistryException {
        return getInternalEndpoint;
    }

    public boolean isGetInternalEndpointsForInterface() {
        return getInternalEndpointsForInterface;
    }

    public boolean isGetInternalEndpointsForService() {
        return getInternalEndpointsForService;
    }

    public boolean isRegisterConnection() {
        return registerConnection;
    }

    public boolean isRegisterExternalEndpoint() throws RegistryException {
        return registerExternalEndpoint;
    }

    public boolean isRegisterInternalEndpoint() throws RegistryException {
        return registerInternalEndpoint;
    }

    public boolean isRetrieveNewEndpoints() throws RegistryException {
        return retrieveNewEndpoints;
    }

    public boolean isValidateEndpoint() {
        return validateEndpoint;
    }

    public void registerConnection(LinkedEndpoint linkedEndpoint)
        throws RegistryException {
        registerConnection = true;
    }

    public void registerExternalEndpoint(ServiceEndpoint externalEndpoint)
        throws RegistryException {
        registerExternalEndpoint = true;
    }

    public void registerInternalEndpoint(AbstractEndpoint endpoint)
        throws RegistryException {
        registerInternalEndpoint = true;
    }

    public List<AbstractEndpoint> retrieveNewEndpoints()
        throws RegistryException {
        retrieveNewEndpoints = true;
        List<AbstractEndpoint> absList = new ArrayList<AbstractEndpoint>();
        absList.add(new InternalEndpoint(new QName(
            "http://127.0.0.1:8080/axis/Helloworld.jws", "HelloworldService"),
            "endpoint", "compo", "0", null));
        return absList;
    }

    public void setDeregisterConnection(boolean deregisterConnection) {
        this.deregisterConnection = deregisterConnection;
    }

    public void setDeregisterExternalEndpoint(boolean deregisterExternalEndpoint) {
        this.deregisterExternalEndpoint = deregisterExternalEndpoint;
    }

    public void setDeregisterInternalEndpoint(boolean deregisterInternalEndpoint) {
        this.deregisterInternalEndpoint = deregisterInternalEndpoint;
    }

    public void setGetExternalEndpoint(boolean getExternalEndpoint) {
        this.getExternalEndpoint = getExternalEndpoint;
    }

    public void setGetExternalEndpointsForInterface(
        boolean getExternalEndpointsForInterface) {
        this.getExternalEndpointsForInterface = getExternalEndpointsForInterface;
    }

    public void setGetExternalEndpointsForService(
        boolean getExternalEndpointsForService) {
        this.getExternalEndpointsForService = getExternalEndpointsForService;
    }

    public void setGetInternalEndpoint(boolean getInternalEndpoint) {
        this.getInternalEndpoint = getInternalEndpoint;
    }

    public void setGetInternalEndpointsForInterface(
        boolean getInternalEndpointsForInterface) {
        this.getInternalEndpointsForInterface = getInternalEndpointsForInterface;
    }

    public void setGetInternalEndpointsForService(
        boolean getInternalEndpointsForService) {
        this.getInternalEndpointsForService = getInternalEndpointsForService;
    }

    public void setRegisterConnection(boolean registerConnection) {
        this.registerConnection = registerConnection;
    }

    public void setRegisterExternalEndpoint(boolean registerExternalEndpoint) {
        this.registerExternalEndpoint = registerExternalEndpoint;
    }

    public void setRegisterInternalEndpoint(boolean registerInternalEndpoint) {
        this.registerInternalEndpoint = registerInternalEndpoint;
    }

    public void setRetrieveNewEndpoints(boolean retrieveNewEndpoints) {
        this.retrieveNewEndpoints = retrieveNewEndpoints;
    }

    public void setValidateEndpoint(boolean validateEndpoint) {
        this.validateEndpoint = validateEndpoint;
    }

    public void validateEndpoint(AbstractEndpoint endpoint)
        throws RegistryException {
        validateEndpoint = true;
    }

    public void cleanNewEndpoints() throws RegistryException {
    }

    public AbstractEndpoint getInternalEndpoint(QName service, String name, boolean resolveLink) throws RegistryException {
        getInternalEndpoint = true;
        InternalEndpoint internalEndpoint = new InternalEndpoint(new QName(
            "service"), "endpoint", "compo", "0", null);
        if (service.equals(new QName(
            "http://127.0.0.1:8080/axis/Helloworld.jws", "HelloworldService"))) {
            internalEndpoint.setInterfaces(new QName[] {new QName("test")});
        }
        return internalEndpoint;

    }

    public Context getUsersContext() {
        return null;
    }

}
