/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: DeliveryChannelImplTest.java 11:10:41 AM ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.jbi.messaging;

import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.TimeUnit;

import javax.jbi.JBIException;
import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessageExchangeFactory;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.MessageExchange.Role;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.xml.namespace.QName;

import junit.framework.TestCase;

import org.easymock.classextension.EasyMock;

import org.objectweb.petals.jbi.component.context.ComponentContextImpl;
import org.objectweb.petals.jbi.messaging.mock.MockLogger;
import org.objectweb.petals.jbi.registry.ConsumerEndpoint;
import org.objectweb.petals.jbi.routing.RouterImpl;
import org.objectweb.petals.jbi.routing.RoutingException;
import org.objectweb.petals.util.LoggingUtil;

/**
 * Tests of the DeliveryChannelImpl
 * 
 * @author ddesjardins - eBMWebsourcing
 */
public class DeliveryChannelImplTest extends TestCase {

    public void testAccept() throws SecurityException, NoSuchMethodException,
        MessagingException {
        DeliveryChannelImpl deliveryChannelImpl = EasyMock.createMock(
            DeliveryChannelImpl.class, new Method[] {DeliveryChannelImpl.class
                .getDeclaredMethod("accept", new Class[] {long.class})});
        LoggingUtil loggingUtil = EasyMock.createMock(LoggingUtil.class);
        MessageExchange messageExchange = EasyMock
            .createMock(MessageExchange.class);

        loggingUtil.call();
        EasyMock.expect(deliveryChannelImpl.accept(0)).andReturn(
            messageExchange);

        EasyMock.replay(deliveryChannelImpl);
        EasyMock.replay(loggingUtil);

        deliveryChannelImpl.log = loggingUtil;

        assertEquals(deliveryChannelImpl.accept(), messageExchange);
    }

    @SuppressWarnings("unchecked")
    public void testAcceptLong() throws SecurityException,
        NoSuchMethodException, MessagingException, IllegalArgumentException,
        InterruptedException {
        DeliveryChannelImpl deliveryChannelImpl = EasyMock.createMock(
            DeliveryChannelImpl.class, new Method[] {DeliveryChannelImpl.class
                .getDeclaredMethod("checkDeliveryChannelIsOpened",
                    new Class[] {})});
        LoggingUtil loggingUtil = EasyMock.createMock(LoggingUtil.class);
        MessageExchangeImpl messageExchangeImpl = EasyMock
            .createMock(MessageExchangeImpl.class);
        LinkedBlockingQueue<MessageExchangeImpl> queue = EasyMock
            .createMock(LinkedBlockingQueue.class);
        MessageExchangeFactoryImpl messageExchangeFactoryImpl = EasyMock
            .createMock(MessageExchangeFactoryImpl.class);
        MessageExchange messageExchange = EasyMock
            .createMock(MessageExchange.class);

        loggingUtil.start();
        deliveryChannelImpl.checkDeliveryChannelIsOpened();
        loggingUtil.end();
        EasyMock.expect(messageExchangeImpl.getPattern()).andReturn(
            MessageExchangeImpl.IN_ONLY_PATTERN);
        EasyMock.expect(queue.poll(0,TimeUnit.MILLISECONDS)).andReturn(messageExchangeImpl);
        EasyMock.expect(
            messageExchangeFactoryImpl.createExchangeDecorator(
                messageExchangeImpl, MessageExchangeImpl.IN_ONLY_PATTERN))
            .andReturn(messageExchange);

        EasyMock.replay(messageExchangeImpl);
        EasyMock.replay(deliveryChannelImpl);
        EasyMock.replay(loggingUtil);
        EasyMock.replay(queue);
        EasyMock.replay(messageExchangeFactoryImpl);

        deliveryChannelImpl.log = loggingUtil;
        deliveryChannelImpl.queue = queue;
        deliveryChannelImpl.messageExchangeFactory = messageExchangeFactoryImpl;

        assertEquals(messageExchange, deliveryChannelImpl.accept(0));
    }

    @SuppressWarnings("unchecked")
    public void testAcceptLongException() throws SecurityException,
        NoSuchMethodException, MessagingException, IllegalArgumentException,
        InterruptedException {
        DeliveryChannelImpl deliveryChannelImpl = EasyMock.createMock(
            DeliveryChannelImpl.class, new Method[] {DeliveryChannelImpl.class
                .getDeclaredMethod("checkDeliveryChannelIsOpened",
                    new Class[] {})});
        LoggingUtil loggingUtil = EasyMock.createMock(LoggingUtil.class);
        LinkedBlockingQueue<MessageExchangeImpl> queue = EasyMock
            .createMock( LinkedBlockingQueue.class);

        loggingUtil.start();
        deliveryChannelImpl.checkDeliveryChannelIsOpened();
        loggingUtil.end();
        EasyMock.expect(queue.poll(0,TimeUnit.MILLISECONDS)).andThrow(
            new IllegalArgumentException("test exception"));

        EasyMock.replay(deliveryChannelImpl);
        EasyMock.replay(loggingUtil);
        EasyMock.replay(queue);

        deliveryChannelImpl.log = loggingUtil;
        deliveryChannelImpl.queue = queue;
        try {
            deliveryChannelImpl.accept(0);
            fail();
        } catch (Exception e) {
            // do nothing
        }
    }

    public void testCheckDeliveryChannelIsOpenedException() {
        DeliveryChannelImpl deliveryChannelImpl = new DeliveryChannelImpl(
            EasyMock.createMock(ComponentContextImpl.class), EasyMock
                .createMock(RouterImpl.class), null);
        deliveryChannelImpl.opened = false;

        try {
            deliveryChannelImpl.checkDeliveryChannelIsOpened();
            fail();
        } catch (MessagingException e) {
            // do nothing
        }
    }

    public void testCheckMessageExchangeExceptionMessageNull() {
        DeliveryChannelImpl deliveryChannelImpl = new DeliveryChannelImpl(
            EasyMock.createMock(ComponentContextImpl.class), EasyMock
                .createMock(RouterImpl.class), null);
        try {
            deliveryChannelImpl.checkMessageExchange(null);
            fail();
        } catch (MessagingException e) {
            // do nothing
        }
    }

    public void testCheckMessageExchangeExceptionTerminated() {
        DeliveryChannelImpl deliveryChannelImpl = new DeliveryChannelImpl(
            EasyMock.createMock(ComponentContextImpl.class), EasyMock
                .createMock(RouterImpl.class), null);
        MessageExchangeImpl messageExchangeImpl = EasyMock
            .createMock(MessageExchangeImpl.class);
        MessageExchangeDecorator messageExchangeDecorator = EasyMock
            .createMock(MessageExchangeDecorator.class);

        EasyMock.expect(messageExchangeDecorator.getMessageExchange())
            .andReturn(messageExchangeImpl);
        EasyMock.expect(messageExchangeImpl.isTerminated()).andReturn(true);

        EasyMock.replay(messageExchangeDecorator);
        EasyMock.replay(messageExchangeImpl);

        try {
            deliveryChannelImpl.checkMessageExchange(messageExchangeDecorator);
            fail();
        } catch (MessagingException e) {
            // do nothing
        }
    }

    @SuppressWarnings("unchecked")
    public void testClose() throws SecurityException, NoSuchMethodException,
        IllegalArgumentException, InterruptedException, JBIException {
        DeliveryChannelImpl deliveryChannelImpl = EasyMock.createMock(
            DeliveryChannelImpl.class, new Method[] {DeliveryChannelImpl.class
                .getDeclaredMethod("checkDeliveryChannelIsOpened",
                    new Class[] {})});
        LoggingUtil loggingUtil = EasyMock.createMock(LoggingUtil.class);
        LinkedBlockingQueue<MessageExchangeImpl> queue = EasyMock
            .createMock( LinkedBlockingQueue.class);
        ComponentContextImpl componentContextImpl = EasyMock
            .createMock(ComponentContextImpl.class);

        loggingUtil.call();
        deliveryChannelImpl.checkDeliveryChannelIsOpened();
        componentContextImpl.deregisterAllEndpoints();
        //queue.interrupt();

        EasyMock.replay(deliveryChannelImpl);
        EasyMock.replay(loggingUtil);
        EasyMock.replay(componentContextImpl);
        EasyMock.replay(queue);

        deliveryChannelImpl.log = loggingUtil;
        deliveryChannelImpl.queue = queue;
        deliveryChannelImpl.opened = true;
        deliveryChannelImpl.context = componentContextImpl;

        deliveryChannelImpl.close();
        assertFalse(deliveryChannelImpl.opened);
    }

    public void testCreateExchangeFactory() {
        ComponentContextImpl componentContextImpl = EasyMock
            .createMock(ComponentContextImpl.class);

        ConsumerEndpoint consumerEndpoint = new ConsumerEndpoint("compo", "0");
        EasyMock.expect(componentContextImpl.getAddress()).andReturn(
            consumerEndpoint).anyTimes();

        EasyMock.replay(componentContextImpl);

        DeliveryChannelImpl deliveryChannelImpl = new DeliveryChannelImpl(
            componentContextImpl, EasyMock.createMock(RouterImpl.class), null);

        deliveryChannelImpl.logger = new MockLogger();
        deliveryChannelImpl.context = componentContextImpl;

        MessageExchangeFactory messageExchangeFactory = deliveryChannelImpl
            .createExchangeFactory();
        assertEquals(
            ((MessageExchangeFactoryImpl) messageExchangeFactory).consumerEndpoint
                .getComponentName(), consumerEndpoint.getComponentName());
    }

    public void testCreateExchangeFactoryWithInterfaceName() {
        ComponentContextImpl componentContextImpl = EasyMock
            .createMock(ComponentContextImpl.class);

        ConsumerEndpoint consumerEndpoint = new ConsumerEndpoint("compo", "0");
        EasyMock.expect(componentContextImpl.getAddress()).andReturn(
            consumerEndpoint).anyTimes();

        EasyMock.replay(componentContextImpl);

        DeliveryChannelImpl deliveryChannelImpl = new DeliveryChannelImpl(
            componentContextImpl, EasyMock.createMock(RouterImpl.class), null);

        deliveryChannelImpl.logger = new MockLogger();
        deliveryChannelImpl.context = componentContextImpl;

        MessageExchangeFactory messageExchangeFactory = deliveryChannelImpl
            .createExchangeFactory(new QName("inter"));
        assertEquals(
            ((MessageExchangeFactoryImpl) messageExchangeFactory).defaultInterfaceName,
            new QName("inter"));
    }

    public void testCreateExchangeFactoryWithEndpoint() {
        ComponentContextImpl componentContextImpl = EasyMock
            .createMock(ComponentContextImpl.class);
        ServiceEndpoint serviceEndpoint = EasyMock
            .createMock(ServiceEndpoint.class);
        ConsumerEndpoint consumerEndpoint = new ConsumerEndpoint("compo", "0");
        EasyMock.expect(componentContextImpl.getAddress()).andReturn(
            consumerEndpoint).anyTimes();

        EasyMock.replay(componentContextImpl);

        DeliveryChannelImpl deliveryChannelImpl = new DeliveryChannelImpl(
            componentContextImpl, EasyMock.createMock(RouterImpl.class), null);

        deliveryChannelImpl.logger = new MockLogger();
        deliveryChannelImpl.context = componentContextImpl;

        MessageExchangeFactory messageExchangeFactory = deliveryChannelImpl
            .createExchangeFactory(serviceEndpoint);
        assertEquals(
            ((MessageExchangeFactoryImpl) messageExchangeFactory).consumerEndpoint
                .getComponentName(), consumerEndpoint.getComponentName());
        assertEquals(
            ((MessageExchangeFactoryImpl) messageExchangeFactory).defaultServiceEndpoint,
            serviceEndpoint);
    }

    public void testCreateExchangeFactoryWithServiceName() {
        ComponentContextImpl componentContextImpl = EasyMock
            .createMock(ComponentContextImpl.class);
        ConsumerEndpoint consumerEndpoint = new ConsumerEndpoint("compo", "0");
        EasyMock.expect(componentContextImpl.getAddress()).andReturn(
            consumerEndpoint).anyTimes();

        EasyMock.replay(componentContextImpl);

        DeliveryChannelImpl deliveryChannelImpl = new DeliveryChannelImpl(
            componentContextImpl, EasyMock.createMock(RouterImpl.class), null);

        deliveryChannelImpl.logger = new MockLogger();
        deliveryChannelImpl.context = componentContextImpl;

        MessageExchangeFactory messageExchangeFactory = deliveryChannelImpl
            .createExchangeFactoryForService(new QName("serv"));
        assertEquals(
            ((MessageExchangeFactoryImpl) messageExchangeFactory).consumerEndpoint
                .getComponentName(), consumerEndpoint.getComponentName());
        assertEquals(
            ((MessageExchangeFactoryImpl) messageExchangeFactory).defaultServiceName,
            new QName("serv"));
    }

    public void testIsOpened() {
        DeliveryChannelImpl deliveryChannelImpl = new DeliveryChannelImpl(
            EasyMock.createMock(ComponentContextImpl.class), EasyMock
                .createMock(RouterImpl.class), null);
        deliveryChannelImpl.opened = true;
        assertTrue(deliveryChannelImpl.isOpened());
    }

    public void testPush() throws SecurityException, NoSuchMethodException,
        IllegalArgumentException, InterruptedException, JBIException {
        DeliveryChannelImpl deliveryChannelImpl = EasyMock.createMock(
            DeliveryChannelImpl.class, new Method[] {DeliveryChannelImpl.class
                .getDeclaredMethod("checkDeliveryChannelIsOpened",
                    new Class[] {})});
        MessageExchangeImpl messageExchangeImpl = EasyMock
            .createMock(MessageExchangeImpl.class);
        LoggingUtil loggingUtil = EasyMock.createMock(LoggingUtil.class);
        MessageExchange.Role role = MessageExchange.Role.CONSUMER;
        MessageExchangeDecorator messageExchangeDecorator = EasyMock
            .createMock(MessageExchangeDecorator.class);

        messageExchangeDecorator.setMessageExchange(messageExchangeImpl);
        deliveryChannelImpl.checkDeliveryChannelIsOpened();
        messageExchangeDecorator.setWaitingOnSynchronousSend(false);
        EasyMock.expect(messageExchangeDecorator.getRole()).andReturn(role).anyTimes();
        EasyMock.expect(messageExchangeDecorator.getExchangeId()).andReturn("1").anyTimes();
        EasyMock.expect(messageExchangeDecorator.getMessageExchange()).andReturn(messageExchangeImpl).anyTimes();
        
        EasyMock.expect(messageExchangeImpl.getRole()).andReturn(role).anyTimes();
        EasyMock.expect(messageExchangeImpl.getExchangeId()).andReturn("1").anyTimes();

        EasyMock.replay(deliveryChannelImpl);
        EasyMock.replay(messageExchangeImpl);
        EasyMock.replay(messageExchangeDecorator);

        deliveryChannelImpl.log = loggingUtil;
        deliveryChannelImpl.waitingExchanges = new HashMap<String, MessageExchangeDecorator>();
        deliveryChannelImpl.waitingExchanges.put(deliveryChannelImpl.waitingExchangeKey(messageExchangeDecorator), messageExchangeDecorator);

        deliveryChannelImpl.push(messageExchangeImpl);
    }

    @SuppressWarnings("unchecked")
    public void testPush1() throws SecurityException, NoSuchMethodException,
        IllegalArgumentException, InterruptedException, JBIException {
        DeliveryChannelImpl deliveryChannelImpl = EasyMock.createMock(
            DeliveryChannelImpl.class, new Method[] {DeliveryChannelImpl.class
                .getDeclaredMethod("checkDeliveryChannelIsOpened",
                    new Class[] {})});
        MessageExchange.Role role = MessageExchange.Role.CONSUMER;
        MessageExchangeImpl messageExchangeImpl = EasyMock
            .createMock(MessageExchangeImpl.class);
        LoggingUtil loggingUtil = EasyMock.createMock(LoggingUtil.class);
        LinkedBlockingQueue<MessageExchangeImpl> queue = EasyMock
            .createMock( LinkedBlockingQueue.class);

        deliveryChannelImpl.checkDeliveryChannelIsOpened();
        EasyMock.expect(messageExchangeImpl.getRole()).andReturn(role).anyTimes();
        EasyMock.expect(messageExchangeImpl.getExchangeId()).andReturn("1").anyTimes();

        queue.put(messageExchangeImpl);

        EasyMock.replay(deliveryChannelImpl);
        EasyMock.replay(messageExchangeImpl);
        EasyMock.replay(queue);

        deliveryChannelImpl.log = loggingUtil;
        deliveryChannelImpl.waitingExchanges = new HashMap<String, MessageExchangeDecorator>();
        deliveryChannelImpl.waitingExchanges.put("1", null);
        deliveryChannelImpl.queue = queue;

        deliveryChannelImpl.push(messageExchangeImpl);
    }

    public void testSend() throws SecurityException, NoSuchMethodException,
        MessagingException, RoutingException {
        DeliveryChannelImpl deliveryChannelImpl = EasyMock.createMock(
            DeliveryChannelImpl.class, new Method[] {
                DeliveryChannelImpl.class.getDeclaredMethod(
                    "checkDeliveryChannelIsOpened", new Class[] {}),
                DeliveryChannelImpl.class
                    .getDeclaredMethod("checkMessageExchange",
                        new Class[] {MessageExchange.class})});
        MessageExchangeDecorator messageExchange = EasyMock
            .createMock(MessageExchangeDecorator.class);
        MessageExchangeImpl messageExchangeImpl = EasyMock
            .createMock(MessageExchangeImpl.class);
        RouterImpl routerImpl = EasyMock.createMock(RouterImpl.class);

        EasyMock.expect(messageExchangeImpl.getStatus()).andReturn(
            ExchangeStatus.DONE);
        messageExchangeImpl.setTerminated(true);
        messageExchange.setProperty(DeliveryChannelImpl.PROPERTY_SENDSYNC,null);
        deliveryChannelImpl.checkDeliveryChannelIsOpened();
        deliveryChannelImpl.checkMessageExchange(messageExchange);
        EasyMock.expect(messageExchange.getMessageExchange()).andReturn(
            messageExchangeImpl);
        routerImpl.send(null, messageExchangeImpl,0);

        EasyMock.replay(messageExchange);
        EasyMock.replay(routerImpl);
        EasyMock.replay(deliveryChannelImpl);
        EasyMock.replay(messageExchangeImpl);

        deliveryChannelImpl.log = EasyMock.createMock(LoggingUtil.class);
        deliveryChannelImpl.router = routerImpl;

        deliveryChannelImpl.send(messageExchange);
    }

    public void testSendSync() throws SecurityException, NoSuchMethodException,
        MessagingException, RoutingException {
        DeliveryChannelImpl deliveryChannelImpl = EasyMock.createMock(
            DeliveryChannelImpl.class, new Method[] {DeliveryChannelImpl.class
                .getDeclaredMethod("sendSync", new Class[] {
                    MessageExchange.class, long.class})});
        LoggingUtil loggingUtil = EasyMock.createMock(LoggingUtil.class);
        MessageExchange messageExchange = EasyMock
            .createMock(MessageExchange.class);
        messageExchange.setProperty(DeliveryChannelImpl.PROPERTY_SENDSYNC,"true");
        
        loggingUtil.call();
        EasyMock.expect(deliveryChannelImpl.sendSync(messageExchange, 0))
            .andReturn(Boolean.TRUE);

        EasyMock.replay(deliveryChannelImpl);
        EasyMock.replay(loggingUtil);
        EasyMock.replay(messageExchange);
        
        deliveryChannelImpl.log = loggingUtil;

        deliveryChannelImpl.sendSync(messageExchange);
    }

    public void testSendSyncTime() throws SecurityException,
        NoSuchMethodException, MessagingException, RoutingException {
        DeliveryChannelImpl deliveryChannelImpl = EasyMock.createMock(
            DeliveryChannelImpl.class,
            new Method[] {DeliveryChannelImpl.class.getDeclaredMethod("sendExchange",
                new Class[] {MessageExchange.class,Long.TYPE})});
        LoggingUtil loggingUtil = EasyMock.createNiceMock(LoggingUtil.class);
        MessageExchange.Role role = MessageExchange.Role.CONSUMER;
        MessageExchangeDecorator messageExchangeDecorator = EasyMock
            .createMock(MessageExchangeDecorator.class);
        MessageExchangeImpl messageExchangeImpl = EasyMock
            .createMock(MessageExchangeImpl.class);
        
        messageExchangeDecorator.setProperty(DeliveryChannelImpl.PROPERTY_SENDSYNC,"true");
        messageExchangeDecorator.setWaitingOnSynchronousSend(true);
        
        EasyMock.expect(messageExchangeDecorator.getExchangeId())
            .andReturn("1").anyTimes();
        EasyMock.expect(messageExchangeDecorator.getRole())
            .andReturn(role).anyTimes();
    
        deliveryChannelImpl.sendExchange(messageExchangeDecorator, 5);
        EasyMock.expect(messageExchangeDecorator.isWaitingOnSynchronousSend())
            .andReturn(true).anyTimes();
        EasyMock.expect(messageExchangeDecorator.getMessageExchange())
            .andReturn(messageExchangeImpl).anyTimes();
        EasyMock.expect(messageExchangeDecorator.getObserverRole()).andReturn(
            Role.CONSUMER);
        messageExchangeImpl.setRole(Role.CONSUMER);
        EasyMock.expect(messageExchangeImpl.getRole()).andReturn(
            Role.CONSUMER).anyTimes();
        EasyMock.expect(messageExchangeImpl.getExchangeId()).andReturn(
            "").anyTimes();
        messageExchangeDecorator.setStatus(ExchangeStatus.ERROR);
        messageExchangeImpl.setTerminated(true);
        
        EasyMock.replay(messageExchangeImpl);
        EasyMock.replay(deliveryChannelImpl);
        EasyMock.replay(messageExchangeDecorator);
        EasyMock.replay(loggingUtil);

        deliveryChannelImpl.log = loggingUtil;
        deliveryChannelImpl.waitingExchanges = new HashMap<String, MessageExchangeDecorator>();

        deliveryChannelImpl.sendSync(messageExchangeDecorator, 5);
    }
}
