/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: MessageExchangeDecoratorTest.java 11:40:23 ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.jbi.messaging;

import java.lang.reflect.Method;

import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.MessageExchange.Role;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.xml.namespace.QName;

import junit.framework.TestCase;

import org.easymock.classextension.EasyMock;
import org.objectweb.petals.jbi.registry.InternalEndpoint;

/**
 * Test of the MessageExchangeDecorator
 * 
 * @author ddesjardins - eBMWebsourcing
 */
public class MessageExchangeDecoratorTest extends TestCase {

    private MessageExchangeDecorator messageExchangeDecorator;

    public void setUp() throws MessagingException {
        MessageExchangeImpl me = new MessageExchangeImpl(null);
        me.setExchangeId("test01");
        me.setOperation(new QName("operationname"));
        me.setService(new QName("servicename"));
        me.setInterfaceName(new QName("interfacename"));
        me.setStatus(ExchangeStatus.ERROR);
        me.setEndpoint(new InternalEndpoint(new QName("servicename"),
            "endpoint", "compo", null, null));
        me.setError(new RuntimeException("test"));
        me.setProperty("test", "value");
        messageExchangeDecorator = new MessageExchangeDecorator(me,
            Role.CONSUMER);
    }

    public void testCreateFault() throws MessagingException {
        assertNotNull(messageExchangeDecorator.createFault());
    }

    public void testCreateMessage() throws MessagingException {
        assertNotNull(messageExchangeDecorator.createMessage());
    }

    public void testGetEndpoint() {
        ServiceEndpoint endpoint = messageExchangeDecorator.getEndpoint();
        assertEquals(endpoint.getServiceName(), new QName("servicename"));
    }

    public void testGetError() {
        Exception exception = messageExchangeDecorator.getError();
        assertEquals(exception.getMessage(), "test");
    }

    public void testGetError1() {
        messageExchangeDecorator.setError(new RuntimeException("test1"));
        Exception exception = messageExchangeDecorator.getError();
        assertEquals(exception.getMessage(), "test1");
    }

    public void testGetExchangeId() {
        assertEquals(messageExchangeDecorator.getExchangeId(), "test01");
    }

    public void testGetProperty() {
        assertEquals(messageExchangeDecorator.getProperty("test"), "value");
    }

    public void testGetProperty1() {
        messageExchangeDecorator.setProperty("test1", "value");
        assertEquals(messageExchangeDecorator.getProperty("test1"), "value");
    }

    public void testGetPropertyNames() {
        assertEquals(messageExchangeDecorator.getPropertyNames().iterator()
            .next(), "test");
    }

    public void testGetInterfaceName1() {
        messageExchangeDecorator.setInterfaceName(new QName("interfacename1"));
        assertEquals(messageExchangeDecorator.getInterfaceName(), new QName(
            "interfacename1"));
    }

    public void testGetInterfaceName() {
        assertEquals(messageExchangeDecorator.getInterfaceName(), new QName(
            "interfacename"));
    }

    public void testGetOperation() {
        assertEquals(messageExchangeDecorator.getOperation(), new QName(
            "operationname"));
    }

    public void testGetOperation1() {
        messageExchangeDecorator.setOperation(new QName("operationname1"));
        assertEquals(messageExchangeDecorator.getOperation(), new QName(
            "operationname1"));
    }

    public void testGetService() {
        assertEquals(messageExchangeDecorator.getService(), new QName(
            "servicename"));
    }

    public void testGetService1() {
        messageExchangeDecorator.setService(new QName("servicename1"));
        assertEquals(messageExchangeDecorator.getService(), new QName(
            "servicename1"));
    }

    public void testGetStatus() {
        assertEquals(messageExchangeDecorator.getStatus(), ExchangeStatus.ERROR);
    }

    public void testGetStatus1() throws MessagingException {
        messageExchangeDecorator.setStatus(ExchangeStatus.DONE);
        assertEquals(messageExchangeDecorator.getStatus(), ExchangeStatus.DONE);
    }

    public void testGetFault() throws SecurityException, NoSuchMethodException {
        MessageExchangeDecorator messageExchangeDecorator = EasyMock
            .createMock(MessageExchangeDecorator.class,
                new Method[] {MessageExchangeDecorator.class.getDeclaredMethod(
                    "checkComponentOwnsExchange", new Class[0])});
        MessageExchangeImpl messageExchangeImpl = EasyMock
            .createMock(MessageExchangeImpl.class);
        FaultImpl faultImpl = EasyMock.createMock(FaultImpl.class);

        messageExchangeDecorator.checkComponentOwnsExchange();
        EasyMock.expect(messageExchangeImpl.getFault()).andReturn(faultImpl);

        EasyMock.replay(messageExchangeDecorator);
        EasyMock.replay(faultImpl);
        EasyMock.replay(messageExchangeImpl);

        messageExchangeDecorator.messageExchange = messageExchangeImpl;

        assertEquals(messageExchangeDecorator.getFault(), faultImpl);
    }

    public void testGetMessageExchange() {
        MessageExchangeImpl messageExchangeImpl = EasyMock
            .createMock(MessageExchangeImpl.class);

        EasyMock.replay(messageExchangeImpl);
        messageExchangeDecorator.messageExchange = messageExchangeImpl;

        assertEquals(messageExchangeDecorator.getMessageExchange(),
            messageExchangeImpl);
    }

    public void testGetObserverRole() {
        messageExchangeDecorator.observerRole = MessageExchange.Role.CONSUMER;
        assertEquals(messageExchangeDecorator.getObserverRole(),
            MessageExchange.Role.CONSUMER);
    }

    public void testGetRole() throws SecurityException, NoSuchMethodException {
        MessageExchangeDecorator messageExchangeDecorator = EasyMock
            .createMock(MessageExchangeDecorator.class,
                new Method[] {MessageExchangeDecorator.class.getDeclaredMethod(
                    "checkComponentOwnsExchange", new Class[0])});
        MessageExchangeImpl messageExchangeImpl = EasyMock
            .createMock(MessageExchangeImpl.class);

        EasyMock.expect(messageExchangeImpl.getRole()).andReturn(
            MessageExchange.Role.PROVIDER);
        messageExchangeDecorator.checkComponentOwnsExchange();

        EasyMock.replay(messageExchangeDecorator);
        EasyMock.replay(messageExchangeImpl);

        messageExchangeDecorator.messageExchange = messageExchangeImpl;

        assertEquals(messageExchangeDecorator.getRole(),
            MessageExchange.Role.PROVIDER);
    }

    public void testIsTransacted() throws SecurityException,
        NoSuchMethodException {
        MessageExchangeDecorator messageExchangeDecorator = EasyMock
            .createMock(MessageExchangeDecorator.class,
                new Method[] {MessageExchangeDecorator.class.getDeclaredMethod(
                    "checkComponentOwnsExchange", new Class[0])});
        MessageExchangeImpl messageExchangeImpl = EasyMock
            .createMock(MessageExchangeImpl.class);

        EasyMock.expect(messageExchangeImpl.isTransacted()).andReturn(true);
        messageExchangeDecorator.checkComponentOwnsExchange();

        EasyMock.replay(messageExchangeDecorator);
        EasyMock.replay(messageExchangeImpl);

        messageExchangeDecorator.messageExchange = messageExchangeImpl;

        assertEquals(messageExchangeDecorator.isTransacted(), true);
    }

    public void testSetEndpoint() throws SecurityException,
        NoSuchMethodException {
        MessageExchangeDecorator messageExchangeDecorator = EasyMock
            .createMock(MessageExchangeDecorator.class,
                new Method[] {MessageExchangeDecorator.class.getDeclaredMethod(
                    "checkComponentOwnsExchange", new Class[0])});
        MessageExchangeImpl messageExchangeImpl = EasyMock
            .createMock(MessageExchangeImpl.class);
        ServiceEndpoint serviceEndpoint = EasyMock
            .createMock(ServiceEndpoint.class);

        messageExchangeDecorator.checkComponentOwnsExchange();
        messageExchangeImpl.setEndpoint(serviceEndpoint);

        EasyMock.replay(messageExchangeDecorator);
        EasyMock.replay(messageExchangeImpl);

        messageExchangeDecorator.messageExchange = messageExchangeImpl;

        messageExchangeDecorator.setEndpoint(serviceEndpoint);
    }

    public void testSetFault() throws SecurityException, NoSuchMethodException,
        MessagingException {
        MessageExchangeDecorator messageExchangeDecorator = EasyMock
            .createMock(MessageExchangeDecorator.class,
                new Method[] {MessageExchangeDecorator.class.getDeclaredMethod(
                    "checkComponentOwnsExchange", new Class[0])});
        MessageExchangeImpl messageExchangeImpl = EasyMock
            .createMock(MessageExchangeImpl.class);
        FaultImpl faultImpl = EasyMock.createMock(FaultImpl.class);

        messageExchangeDecorator.checkComponentOwnsExchange();
        messageExchangeImpl.setFault(faultImpl);

        EasyMock.replay(messageExchangeDecorator);
        EasyMock.replay(messageExchangeImpl);

        messageExchangeDecorator.messageExchange = messageExchangeImpl;

        messageExchangeDecorator.setFault(faultImpl);
    }

    public void testSetWaitingOnSynchronousSend() {
        messageExchangeDecorator.setWaitingOnSynchronousSend(true);
        assertTrue(messageExchangeDecorator.waitingOnSynchronousSend);
    }

    public void testSetMessageExchange() {
        MessageExchangeImpl messageExchangeImpl = EasyMock
            .createMock(MessageExchangeImpl.class);

        messageExchangeDecorator.setMessageExchange(messageExchangeImpl);

        assertEquals(messageExchangeImpl,
            messageExchangeDecorator.messageExchange);
    }

    public void testCheckComponentOwnsExchangeException() throws SecurityException,
        NoSuchMethodException {
        MessageExchangeDecorator messageExchangeDecorator = EasyMock
            .createMock(MessageExchangeDecorator.class,
                new Method[] {MessageExchangeDecorator.class.getDeclaredMethod(
                    "hasOwnership", new Class[0])});

        EasyMock.expect(messageExchangeDecorator.hasOwnership()).andReturn(
            false);

        EasyMock.replay(messageExchangeDecorator);

        try {
            messageExchangeDecorator.checkComponentOwnsExchange();
            fail();
        } catch (Exception e) {
            // do nothing
        }
    }
}
