/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: MessageExchangeImplTest.java 10:58:30 ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.jbi.messaging;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.Map;

import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.jbi.messaging.MessageExchange.Role;
import javax.xml.namespace.QName;

import junit.framework.TestCase;

import org.easymock.classextension.EasyMock;

/**
 * Test of the MessageExchangeImpl
 * 
 * @author ddesjardins - eBMWebsourcing
 */
public class MessageExchangeImplTest extends TestCase {

    private MessageExchangeImpl messageExchangeImpl;

    public void setUp() throws MessagingException {
        messageExchangeImpl = new MessageExchangeImpl(null);
        messageExchangeImpl.setPattern(MessageExchangeImpl.IN_OUT_PATTERN);
        messageExchangeImpl.setRole(Role.PROVIDER);
        NormalizedMessageImpl normalizedMessageImpl = new NormalizedMessageImpl();
        messageExchangeImpl.setMessage(normalizedMessageImpl,
            MessageExchangeImpl.OUT_MSG);
    }

    public void testSetFault() throws MessagingException {
        FaultImpl faultImpl = new FaultImpl();
        messageExchangeImpl.setFault(faultImpl);
        FaultImpl faultImpl2 = (FaultImpl) messageExchangeImpl.getFault();
        assertEquals(faultImpl.getProperty("test"), faultImpl2
            .getProperty("test"));
    }

    public void testSetStatus() throws MessagingException {
        messageExchangeImpl.setStatus(ExchangeStatus.ACTIVE);
        assertEquals(messageExchangeImpl.getStatus(), ExchangeStatus.ACTIVE);
    }

    public void testSetStatus1() throws MessagingException {
        messageExchangeImpl.setRole(Role.CONSUMER);
        messageExchangeImpl.setStatus(ExchangeStatus.DONE);
        assertEquals(messageExchangeImpl.getStatus(), ExchangeStatus.DONE);
    }

    public void testSetError() throws MessagingException {
        messageExchangeImpl.setError(new RuntimeException("test"));
        assertEquals(messageExchangeImpl.getStatus(), ExchangeStatus.ERROR);
        assertEquals(messageExchangeImpl.getError().getMessage(), "test");
    }

    public void testSetProperty() {
        messageExchangeImpl.setProperty("test", "value");
        assertEquals(messageExchangeImpl.getProperty("test"), "value");
        assertEquals(messageExchangeImpl.getPropertyNames().iterator().next(),
            "test");
    }

    public void testGetMessage() {
        assertNull(messageExchangeImpl.getMessage(null));
    }

    public void testSetOperation() {
        messageExchangeImpl.setOperation(new QName("operation"));
        assertEquals(messageExchangeImpl.getOperation(), new QName("operation"));
    }

    public void testSetExchangeId() {
        messageExchangeImpl.setExchangeId("test01");
        assertEquals(messageExchangeImpl.getExchangeId(), "test01");
    }

    public void testSetTransacted() {
        messageExchangeImpl.setTransacted(true);
        assertTrue(messageExchangeImpl.isTransacted());
    }

    public void testSetTerminated() {
        messageExchangeImpl.setTerminated(true);
        assertTrue(messageExchangeImpl.isTerminated());
    }

    public void testGetMessages() {
        Map<String, NormalizedMessage> map = new HashMap<String, NormalizedMessage>();
        messageExchangeImpl.messages = map;
        assertEquals(messageExchangeImpl.getMessages(), map);
    }

    public void testSetFaultExceptionRoleNull() {
        FaultImpl faultImpl = EasyMock.createMock(FaultImpl.class);
        messageExchangeImpl.role = null;
        try {
            messageExchangeImpl.setFault(faultImpl);
            fail();
        } catch (MessagingException e) {
            // do nothing
        }
    }

    public void testSetFaultExceptionFaultNotNull() {
        FaultImpl faultImpl = EasyMock.createMock(FaultImpl.class);
        messageExchangeImpl.role = Role.CONSUMER;
        messageExchangeImpl.fault = faultImpl;
        try {
            messageExchangeImpl.setFault(faultImpl);
            fail();
        } catch (MessagingException e) {
            // do nothing
        }
    }

    public void testSetFaultExceptionStatusDone() {
        FaultImpl faultImpl = EasyMock.createMock(FaultImpl.class);
        messageExchangeImpl.role = Role.CONSUMER;
        messageExchangeImpl.fault = null;
        messageExchangeImpl.status = ExchangeStatus.DONE;
        try {
            messageExchangeImpl.setFault(faultImpl);
            fail();
        } catch (MessagingException e) {
            // do nothing
        }
    }

    public void testSetFaultExceptionBadRole() {
        FaultImpl faultImpl = EasyMock.createMock(FaultImpl.class);
        messageExchangeImpl.role = Role.CONSUMER;
        messageExchangeImpl.fault = null;
        messageExchangeImpl.status = ExchangeStatus.ACTIVE;
        messageExchangeImpl.messages.clear();
        try {
            messageExchangeImpl.setFault(faultImpl);
            fail();
        } catch (MessagingException e) {
            // do nothing
        }
    }

    public void testSetMessageExceptionMessageNull() {
        try {
            messageExchangeImpl.setMessage(null, "test");
            fail();
        } catch (MessagingException e) {
            // do nothing
        }
    }

    public void testSetMessageExceptionNameNull() {
        NormalizedMessage normalizedMessage = EasyMock
            .createMock(NormalizedMessage.class);
        messageExchangeImpl.messages.put("out", normalizedMessage);
        try {
            messageExchangeImpl.setMessage(normalizedMessage, "out");
            fail();
        } catch (MessagingException e) {
            // do nothing
        }
    }

    public void testSetStatusException1() {
        messageExchangeImpl.role = Role.CONSUMER;
        messageExchangeImpl.pattern = MessageExchangeImpl.IN_ONLY_PATTERN;
        try {
            messageExchangeImpl.setStatus(ExchangeStatus.DONE);
            fail();
        } catch (MessagingException e) {
            // do nothing
        }
    }

    public void testSetStatusException2() {
        messageExchangeImpl.role = Role.CONSUMER;
        messageExchangeImpl.pattern = MessageExchangeImpl.IN_OUT_PATTERN;
        messageExchangeImpl.fault = null;
        messageExchangeImpl.messages.put("out", null);
        try {
            messageExchangeImpl.setStatus(ExchangeStatus.DONE);
            fail();
        } catch (MessagingException e) {
            // do nothing
        }
    }

    public void testSetStatusException3() {
        messageExchangeImpl.role = Role.CONSUMER;
        messageExchangeImpl.pattern = MessageExchangeImpl.IN_OPTIONAL_OUT_PATTERN;
        messageExchangeImpl.fault = null;
        messageExchangeImpl.messages.put("out", null);
        try {
            messageExchangeImpl.setStatus(ExchangeStatus.DONE);
            fail();
        } catch (MessagingException e) {
            // do nothing
        }
    }

    public void testSetStatusException4() {
        messageExchangeImpl.role = Role.PROVIDER;
        messageExchangeImpl.pattern = MessageExchangeImpl.ROBUST_IN_ONLY_PATTERN;
        messageExchangeImpl.fault = EasyMock.createMock(FaultImpl.class);
        try {
            messageExchangeImpl.setStatus(ExchangeStatus.DONE);
            fail();
        } catch (MessagingException e) {
            // do nothing
        }
    }

    public void testSetStatusException5() {
        messageExchangeImpl.role = Role.PROVIDER;
        messageExchangeImpl.pattern = MessageExchangeImpl.IN_OUT_PATTERN;
        try {
            messageExchangeImpl.setStatus(ExchangeStatus.DONE);
            fail();
        } catch (MessagingException e) {
            // do nothing
        }
    }

    public void testSetStatusException6() {
        messageExchangeImpl.role = Role.PROVIDER;
        messageExchangeImpl.pattern = MessageExchangeImpl.IN_OPTIONAL_OUT_PATTERN;
        messageExchangeImpl.fault = EasyMock.createMock(FaultImpl.class);
        messageExchangeImpl.messages.put("out", null);
        try {
            messageExchangeImpl.setStatus(ExchangeStatus.DONE);
            fail();
        } catch (MessagingException e) {
            // do nothing
        }
    }

    public void testSetStatusException7() {
        messageExchangeImpl.role = Role.PROVIDER;
        messageExchangeImpl.pattern = MessageExchangeImpl.IN_OPTIONAL_OUT_PATTERN;
        messageExchangeImpl.fault = null;
        messageExchangeImpl.messages.put("out", EasyMock
            .createMock(NormalizedMessage.class));
        try {
            messageExchangeImpl.setStatus(ExchangeStatus.DONE);
            fail();
        } catch (MessagingException e) {
            // do nothing
        }
    }

    public void testToString() {
        messageExchangeImpl.exchangeId = "1";
        assertEquals(messageExchangeImpl.toString(), "MessageExchange@1");
    }

    public void testCheckNotTerminated() throws SecurityException,
        NoSuchMethodException {
        MessageExchangeImpl messageExchangeImpl = EasyMock.createMock(
            MessageExchangeImpl.class, new Method[] {MessageExchangeImpl.class
                .getDeclaredMethod("isTerminated", new Class[0])});

        EasyMock.expect(messageExchangeImpl.isTerminated()).andReturn(true);

        EasyMock.replay(messageExchangeImpl);

        try {
            messageExchangeImpl.checkNotTerminated();
            fail();
        } catch (MessagingException e) {
            // do nothing
        }
    }

    public void testCheckPatternMatchingExceptionPatternNull() {
        messageExchangeImpl.pattern = null;
        try {
            messageExchangeImpl.checkPatternMatching(null);
            fail();
        } catch (MessagingException e) {
            // do nothing
        }
    }

    public void testCheckPatternMatchingNameNull() {
        messageExchangeImpl.pattern = MessageExchangeImpl.IN_ONLY_PATTERN;
        try {
            messageExchangeImpl.checkPatternMatching(null);
            fail();
        } catch (MessagingException e) {
            // do nothing
        }
    }

    public void testCheckPatternMatchingNameTest() {
        messageExchangeImpl.pattern = MessageExchangeImpl.IN_ONLY_PATTERN;
        try {
            messageExchangeImpl.checkPatternMatching("test");
            fail();
        } catch (MessagingException e) {
            // do nothing
        }
    }

    public void testCheckRoleMatchingExceptionRoleNull() {
        messageExchangeImpl.role = null;
        try {
            messageExchangeImpl.checkRoleMatching(null);
            fail();
        } catch (MessagingException e) {
            // do nothing
        }
    }

    public void testCheckRoleMatchingExceptionNameNameNull() {
        messageExchangeImpl.role = Role.CONSUMER;
        try {
            messageExchangeImpl.checkRoleMatching(null);
            fail();
        } catch (MessagingException e) {
            // do nothing
        }
    }

    public void testCheckRoleMatchingExceptionNameNameTest() {
        messageExchangeImpl.role = Role.CONSUMER;
        try {
            messageExchangeImpl.checkRoleMatching("test");
            fail();
        } catch (MessagingException e) {
            // do nothing
        }
    }

    public void testReadObjectDelegate() throws IOException,
        ClassNotFoundException {
        ObjectInputStream objectInputStream = EasyMock
            .createMock(ObjectInputStream.class);

        objectInputStream.defaultReadObject();
        EasyMock.expect(objectInputStream.readShort()).andReturn((short) 0);
        EasyMock.expect(objectInputStream.readShort()).andReturn((short) 0);

        EasyMock.replay(objectInputStream);

        messageExchangeImpl.readObjectDelegate(objectInputStream);
        assertEquals(messageExchangeImpl.role, Role.CONSUMER);
        assertEquals(messageExchangeImpl.status, ExchangeStatus.ACTIVE);
    }

    public void testReadObjectDelegate1() throws IOException,
        ClassNotFoundException {
        ObjectInputStream objectInputStream = EasyMock
            .createMock(ObjectInputStream.class);

        objectInputStream.defaultReadObject();
        EasyMock.expect(objectInputStream.readShort()).andReturn((short) 1);
        EasyMock.expect(objectInputStream.readShort()).andReturn((short) 1);

        EasyMock.replay(objectInputStream);

        messageExchangeImpl.readObjectDelegate(objectInputStream);
        assertEquals(messageExchangeImpl.role, Role.PROVIDER);
        assertEquals(messageExchangeImpl.status, ExchangeStatus.DONE);
    }

    public void testReadObjectDelegate2() throws IOException,
        ClassNotFoundException {
        ObjectInputStream objectInputStream = EasyMock
            .createMock(ObjectInputStream.class);

        objectInputStream.defaultReadObject();
        EasyMock.expect(objectInputStream.readShort()).andReturn((short) 1);
        EasyMock.expect(objectInputStream.readShort()).andReturn((short) 2);

        EasyMock.replay(objectInputStream);

        messageExchangeImpl.readObjectDelegate(objectInputStream);
        assertEquals(messageExchangeImpl.role, Role.PROVIDER);
        assertEquals(messageExchangeImpl.status, ExchangeStatus.ERROR);
    }

    public void testWriteObjectDelegate() throws IOException {
        messageExchangeImpl.role = Role.CONSUMER;
        messageExchangeImpl.status = ExchangeStatus.ACTIVE;
        ObjectOutputStream objectOutputStream = EasyMock
            .createMock(ObjectOutputStream.class);

        objectOutputStream.defaultWriteObject();
        objectOutputStream.writeShort(0);
        objectOutputStream.writeShort(0);

        EasyMock.replay(objectOutputStream);

        messageExchangeImpl.writeObjectDelegate(objectOutputStream);
    }

    public void testWriteObjectDelegate1() throws IOException {
        messageExchangeImpl.role = Role.PROVIDER;
        messageExchangeImpl.status = ExchangeStatus.DONE;
        ObjectOutputStream objectOutputStream = EasyMock
            .createMock(ObjectOutputStream.class);

        objectOutputStream.defaultWriteObject();
        objectOutputStream.writeShort(1);
        objectOutputStream.writeShort(1);

        EasyMock.replay(objectOutputStream);

        messageExchangeImpl.writeObjectDelegate(objectOutputStream);
    }

    public void testWriteObjectDelegate2() throws IOException {
        messageExchangeImpl.role = Role.PROVIDER;
        messageExchangeImpl.status = ExchangeStatus.ERROR;
        ObjectOutputStream objectOutputStream = EasyMock
            .createMock(ObjectOutputStream.class);

        objectOutputStream.defaultWriteObject();
        objectOutputStream.writeShort(1);
        objectOutputStream.writeShort(2);

        EasyMock.replay(objectOutputStream);

        messageExchangeImpl.writeObjectDelegate(objectOutputStream);
    }
}
