/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: InOnlyImplTest.java 17:30:10 ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.jbi.messaging.types;

import java.io.ByteArrayInputStream;

import javax.jbi.messaging.NormalizedMessage;
import javax.jbi.messaging.MessageExchange.Role;
import javax.xml.transform.stream.StreamSource;

import junit.framework.TestCase;

import org.objectweb.petals.jbi.messaging.MessageExchangeImpl;
import org.objectweb.petals.jbi.messaging.NormalizedMessageImpl;

/**
 * Test the InOnlyImpl
 * 
 * @author ddesjardins - eBMWebsourcing
 */
public class InOutImplTest extends TestCase {

    private InOutImpl inOutImpl;

    private MessageExchangeImpl messageExchangeImpl;

    private NormalizedMessageImpl normalizedMessageImpl;

    public void setUp() throws Exception {
        messageExchangeImpl = new MessageExchangeImpl(null);

        normalizedMessageImpl = new NormalizedMessageImpl();

        StreamSource source = new StreamSource();
        byte[] msgByte = "test message".getBytes();
        ByteArrayInputStream in = new ByteArrayInputStream(msgByte);
        source.setInputStream(in);

        normalizedMessageImpl.setContent(source);

    }

    public void testGetInMessage() throws Exception {
        normalizedMessageImpl.setProperty("test", "valuein");
        messageExchangeImpl.setPattern(MessageExchangeImpl.IN_OUT_PATTERN);
        messageExchangeImpl.setMessage(normalizedMessageImpl, "in");
        inOutImpl = new InOutImpl(messageExchangeImpl, Role.CONSUMER);

        NormalizedMessage normalizedMessage = inOutImpl.getInMessage();
        assertEquals(normalizedMessage.getProperty("test"), "valuein");
    }

    public void testGetOutMessage() throws Exception {
        normalizedMessageImpl.setProperty("test", "valueout");
        messageExchangeImpl.setPattern(MessageExchangeImpl.IN_OUT_PATTERN);
        messageExchangeImpl.setRole(Role.PROVIDER);
        messageExchangeImpl.setMessage(normalizedMessageImpl, "out");
        inOutImpl = new InOutImpl(messageExchangeImpl, Role.PROVIDER);

        NormalizedMessage normalizedMessage = inOutImpl.getOutMessage();
        assertEquals(normalizedMessage.getProperty("test"), "valueout");
    }
    
    public void testSetInMessage() throws Exception{
        normalizedMessageImpl.setProperty("test", "valuein");
        messageExchangeImpl.setPattern(MessageExchangeImpl.IN_OUT_PATTERN);
        messageExchangeImpl.setRole(Role.CONSUMER);
        inOutImpl = new InOutImpl(messageExchangeImpl, Role.CONSUMER);
        inOutImpl.setInMessage(normalizedMessageImpl);
        
        NormalizedMessage normalizedMessage = inOutImpl.getInMessage();
        assertEquals(normalizedMessage.getProperty("test"), "valuein");
    }
    
    public void testSetOutMessage() throws Exception{
        normalizedMessageImpl.setProperty("test", "valueout");
        messageExchangeImpl.setPattern(MessageExchangeImpl.IN_OUT_PATTERN);
        messageExchangeImpl.setRole(Role.PROVIDER);
        inOutImpl = new InOutImpl(messageExchangeImpl, Role.PROVIDER);
        inOutImpl.setOutMessage(normalizedMessageImpl);
        
        NormalizedMessage normalizedMessage = inOutImpl.getOutMessage();
        assertEquals(normalizedMessage.getProperty("test"), "valueout");
    }
}
