/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: JoramAgentTest.java 10:37:37 AM ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.jbi.transport;

import java.io.File;
import java.io.IOException;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;

import junit.framework.TestCase;

import org.easymock.classextension.EasyMock;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;

import org.objectweb.petals.PetalsException;
import org.objectweb.petals.kernel.admin.ContainerInformation;
import org.objectweb.petals.util.LoggingUtil;
import org.objectweb.petals.util.SystemUtil;

/**
 * Tests of the JoramAgent
 * 
 * @author ddesjardins - eBMWebsourcing
 */
public class JoramAgentTest extends TestCase {

    protected JoramAgent agentImpl;

    protected String baseDir;

    public void setUp() {
        agentImpl = new JoramAgent(new LoggingUtil(null));
        baseDir = this.getClass().getResource(".").toString();
        baseDir = baseDir.substring(0, baseDir.indexOf("target"));
        baseDir = baseDir.substring(baseDir.indexOf(":") + 1);
        new File(baseDir + "target" + File.separator + "lib").mkdir();
        new File(baseDir + "target" + File.separator + "lib" + File.separator
            + "petals-1.0-SNAPSHOT.jar").mkdir();
        System.setProperty("petals.home", baseDir + "target");
    }

    public void testInitConfiguration() throws IOException {
        agentImpl.initConfiguration();
        assertNotNull(agentImpl.factory);
    }

    /*
     * public void testStart() throws Exception { JoramAgent agentImpl =
     * EasyMock.createMock(JoramAgent.class, new Method[] {
     * JoramAgent.class.getDeclaredMethod("initJoramAgentServer", new Class[]
     * {}), JoramAgent.class.getDeclaredMethod("startJoramAgentServer", new
     * Class[] {})}); agentImpl.log = EasyMock.createMock(LoggingUtil.class);
     * 
     * agentImpl.startJoramAgentServer(); agentImpl.initJoramAgentServer();
     * 
     * EasyMock.replay(agentImpl);
     * 
     * agentImpl.startServer(); }
     */
    public void testStop() throws IllegalLifeCycleException, SecurityException,
        NoSuchMethodException {
        JoramAgent agentImpl = EasyMock.createMock(JoramAgent.class,
            new Method[] {
                JoramAgent.class.getDeclaredMethod("initJoramAgentServer",
                    new Class[] {}),
                JoramAgent.class.getDeclaredMethod("startJoramAgentServer",
                    new Class[] {})});
        agentImpl.log = EasyMock.createMock(LoggingUtil.class);
        agentImpl.stopServer();
    }

    /*
     * public void testStartServerDistributed() throws Exception {
     * SystemUtil.setJoramId("0"); SystemUtil.setHost("127.0.0.1");
     * SystemUtil.setJoramTCPPort("16200");
     * SystemUtil.setJoramDomainPort("16300");
     * 
     * String conf = FileUtils.readFileToString(new File(baseDir + "src" +
     * File.separator + "test-data" + File.separator + "joram" + File.separator +
     * "joram-conf.xml"), "UTF-8"); agentImpl =
     * EasyMock.createMock(JoramAgent.class, new Method[] {
     * JoramAgent.class.getDeclaredMethod( "retrieveDistributedConfiguration",
     * new Class[0]), JoramAgent.class.getDeclaredMethod("updateContainerState",
     * new Class[] {String.class, ContainerState.class}),
     * JoramAgent.class.getDeclaredMethod("initJoramAgentServer", new Class[]
     * {}), JoramAgent.class.getDeclaredMethod("startJoramAgentServer", new
     * Class[] {})}); LoggingUtil log =
     * EasyMock.createNiceMock(LoggingUtil.class);
     * 
     * EasyMock.expect(agentImpl.retrieveDistributedConfiguration())
     * .andReturn(conf);
     * 
     * agentImpl.initJoramAgentServer(); agentImpl.startJoramAgentServer();
     * 
     * EasyMock.replay(log); EasyMock.replay(agentImpl);
     * 
     * agentImpl.log = log;
     * 
     * agentImpl.startServer(); }
     */
    public void testStartServerException() throws SecurityException,
        NoSuchMethodException, IOException {
        SystemUtil.setJoramId("0");
        SystemUtil.setHost("127.0.0.1");
        SystemUtil.setJoramTCPPort("16200");
        SystemUtil.setJoramDomainPort("16300");

        agentImpl = EasyMock.createMock(JoramAgent.class, new Method[] {
            JoramAgent.class.getDeclaredMethod(
                "retrieveDistributedConfiguration", new Class[0]),
            JoramAgent.class.getDeclaredMethod(
                "retrieveAllContainersInformation", new Class[0])});
        LoggingUtil log = EasyMock.createNiceMock(LoggingUtil.class);

        RuntimeException runtimeException = new RuntimeException();

        EasyMock.expect(agentImpl.retrieveDistributedConfiguration()).andThrow(
            runtimeException).anyTimes();
        log.error("Problem while starting the Joram server", runtimeException);
        List<ContainerInformation> conts = new ArrayList<ContainerInformation>();
        conts.add(null);
        conts.add(null);
        EasyMock.expect(agentImpl.retrieveAllContainersInformation())
            .andReturn(conts);

        EasyMock.replay(log);
        EasyMock.replay(agentImpl);

        agentImpl.log = log;

        agentImpl.startServer();
    }

    public void testRetrieveContainerInformation() throws PetalsException {
        SystemUtil.setJmxPort("8081");
        ContainerInformation containerInformation = agentImpl
            .createContainerInformation();
        assertEquals(containerInformation.getJmxPort(), "8081");
    }

}
