/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: ObjectSerializerTest.java 9:41:31 AM ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.jbi.transport;

import javax.jbi.JBIException;
import javax.jms.JMSException;
import javax.jms.ObjectMessage;

import junit.framework.TestCase;

import org.easymock.classextension.EasyMock;
import org.objectweb.joram.client.jms.QueueSession;
import org.objectweb.petals.jbi.messaging.MessageExchangeImpl;
import org.objectweb.petals.jbi.registry.ConsumerEndpoint;

/**
 * Tests of the ObjectSerializer
 * 
 * @author ddesjardins - eBMWebsourcing
 */
public class ObjectSerializerTest extends TestCase {

    public void testJbi2jms() throws JBIException, JMSException {
        ObjectSerializer objectSerializer = new ObjectSerializer();
        MessageExchangeImpl messageExchangeImpl = new MessageExchangeImpl(
            new ConsumerEndpoint("compo", "0"));

        QueueSession queueSession = EasyMock.createMock(QueueSession.class);
        ObjectMessage objectMessage = EasyMock.createMock(ObjectMessage.class);

        EasyMock.expect(queueSession.createObjectMessage(messageExchangeImpl))
            .andReturn(objectMessage).anyTimes();

        EasyMock.replay(queueSession);
        EasyMock.replay(objectMessage);

        assertEquals(objectMessage, objectSerializer.jbi2jms(
            messageExchangeImpl, queueSession));
    }

    public void testJbi2jmsException() throws JBIException, JMSException {
        ObjectSerializer objectSerializer = new ObjectSerializer();
        MessageExchangeImpl messageExchangeImpl = new MessageExchangeImpl(
            new ConsumerEndpoint("compo", "0"));

        QueueSession queueSession = EasyMock.createMock(QueueSession.class);
        ObjectMessage objectMessage = EasyMock.createMock(ObjectMessage.class);

        EasyMock.expect(queueSession.createObjectMessage(messageExchangeImpl))
            .andThrow(new JMSException("error")).anyTimes();

        EasyMock.replay(queueSession);
        EasyMock.replay(objectMessage);

        try {
            objectSerializer.jbi2jms(messageExchangeImpl, queueSession);
            fail();
        } catch (JBIException e) {
            // do nothing
        }
    }

    public void testJms2jbi() throws JBIException, JMSException {
        ObjectSerializer objectSerializer = new ObjectSerializer();

        ObjectMessage objectMessage = EasyMock.createMock(ObjectMessage.class);
        MessageExchangeImpl messageExchangeImpl = new MessageExchangeImpl(
            new ConsumerEndpoint("compo", "0"));

        EasyMock.expect(objectMessage.getObject()).andReturn(
            messageExchangeImpl).anyTimes();

        EasyMock.replay(objectMessage);

        assertEquals(messageExchangeImpl, objectSerializer
            .jms2jbi(objectMessage));
    }

    public void testJms2jbiException() throws JBIException, JMSException {
        ObjectSerializer objectSerializer = new ObjectSerializer();

        ObjectMessage objectMessage = EasyMock.createMock(ObjectMessage.class);

        EasyMock.expect(objectMessage.getObject()).andThrow(
            new JMSException("error")).anyTimes();

        EasyMock.replay(objectMessage);

        try {
            objectSerializer.jms2jbi(objectMessage);
            fail();
        } catch (JBIException e) {
            // do nothing
        }
    }
}
