/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: MockContext.java 10:14:53 ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.jbi.transport.mock;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import javax.naming.Binding;
import javax.naming.Context;
import javax.naming.Name;
import javax.naming.NameClassPair;
import javax.naming.NameParser;
import javax.naming.NamingException;

/**
 * Mock object of the Context
 * 
 * @author ddesjardins - eBMWebsourcing
 */
public class MockContext implements Context {

    private Map<String, Object> bindings = new HashMap<String, Object>();

    public Object lookup(Name name) throws NamingException {
        return lookup(name.toString());
    }

    public Object lookup(String name) throws NamingException {
        return bindings.get(name);
    }

    public void bind(Name name, Object obj) throws NamingException {
    }

    public void bind(String name, Object obj) throws NamingException {
        bindings.put(name, obj);
    }

    public void rebind(Name name, Object obj) throws NamingException {
        bindings.put(name.toString(), obj);
    }

    public void rebind(String name, Object obj) throws NamingException {
        bindings.put(name, obj);
    }

    public void unbind(Name name) throws NamingException {
        unbind(name.toString());
    }

    public void unbind(String name) throws NamingException {
        if (name.indexOf("/") > -1) {
            String sName = name.substring(0, name.indexOf("/"));
            if (bindings.get(sName) instanceof MockContext) {
                ((MockContext) bindings.get(sName)).unbind(name.substring(name
                    .indexOf("/") + 1));
            }
        } else {
            bindings.remove(name);
        }
    }

    public void rename(Name oldName, Name newName) throws NamingException {
    }

    public void rename(String oldName, String newName) throws NamingException {
    }

    public MockNamingEnumeration<NameClassPair> list(Name name)
        throws NamingException {
        return list(name.toString());
    }

    public MockNamingEnumeration<NameClassPair> list(String name)
        throws NamingException {
        Object obj = bindings.get(name);
        MockNamingEnumeration<NameClassPair> namingEnumeration = new MockNamingEnumeration<NameClassPair>();
        if (obj instanceof MockContext) {
            List<NameClassPair> bindings = new ArrayList<NameClassPair>();
            MockContext mockContext = (MockContext) obj;
            for (Entry entry : mockContext.getBindings().entrySet()) {
                Binding binding = new Binding((String) entry.getKey(), entry
                    .getValue());
                bindings.add(binding);
            }
            namingEnumeration.setElems(bindings);
        }
        return namingEnumeration;
    }

    public MockNamingEnumeration<Binding> listBindings(Name name)
        throws NamingException {
        return listBindings(name.toString());
    }

    public MockNamingEnumeration<Binding> listBindings(String name)
        throws NamingException {
        Object obj = bindings.get(name);
        if (obj instanceof MockContext) {
            MockNamingEnumeration<Binding> namingEnumeration = new MockNamingEnumeration<Binding>();
            List<Binding> bindings = new ArrayList<Binding>();
            MockContext mockContext = (MockContext) obj;
            for (Entry entry : mockContext.getBindings().entrySet()) {
                Binding binding = new Binding((String) entry.getKey(), entry
                    .getValue());
                bindings.add(binding);
            }
            namingEnumeration.setElems(bindings);
            return namingEnumeration;
        }
        return null;
    }

    public void destroySubcontext(Name name) throws NamingException {
        bindings.remove(name.toString());
    }

    public void destroySubcontext(String name) throws NamingException {
        bindings.remove(name);
    }

    public Context createSubcontext(Name name) throws NamingException {
        return createSubcontext(name.toString());
    }

    public Context createSubcontext(String name) throws NamingException {
        MockContext context = new MockContext();
        bindings.put(name, context);
        return context;
    }

    public Object lookupLink(Name name) throws NamingException {
        return null;
    }

    public Object lookupLink(String name) throws NamingException {
        return null;
    }

    public NameParser getNameParser(Name name) throws NamingException {
        return null;
    }

    public NameParser getNameParser(String name) throws NamingException {
        return null;
    }

    public Name composeName(Name name, Name prefix) throws NamingException {
        return null;
    }

    public String composeName(String name, String prefix)
        throws NamingException {
        return null;
    }

    public Object addToEnvironment(String propName, Object propVal)
        throws NamingException {
        return null;
    }

    public Object removeFromEnvironment(String propName) throws NamingException {
        return null;
    }

    public Hashtable<?, ?> getEnvironment() throws NamingException {
        return null;
    }

    public void close() throws NamingException {
    }

    public String getNameInNamespace() throws NamingException {
        return null;
    }

    public Map<String, Object> getBindings() {
        return bindings;
    }

    public void setBindings(Map<String, Object> bindings) {
        this.bindings = bindings;
    }

}
