/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: DistributedJMXServerTest.java 9:31:38 AM ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.kernel.admin;

import java.io.IOException;
import java.util.Hashtable;
import java.util.Set;
import java.util.TreeSet;

import javax.management.InstanceNotFoundException;
import javax.management.MBeanException;
import javax.management.MBeanServerConnection;
import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;
import javax.management.ReflectionException;
import javax.management.remote.JMXConnector;

import junit.framework.TestCase;

import org.easymock.classextension.EasyMock;

import static org.easymock.EasyMock.expect;
import static org.easymock.classextension.EasyMock.replay;

/**
 * Test of the DistributedJMXServer
 * 
 * @author ddesjardins - eBMWebsourcing
 */
public class DistributedJMXServerTest extends TestCase {

    public void testGetAdminServiceMBeanName() throws IOException,
        MalformedObjectNameException, NullPointerException {
        JMXConnector connector = EasyMock.createMock(JMXConnector.class);

        MBeanServerConnection connection = EasyMock
                .createMock(MBeanServerConnection.class);

        Set<ObjectName> names = new TreeSet<ObjectName>();
        ObjectName adName = new ObjectName("Petals:type=service");
        names.add(adName);

        Hashtable<String, String> attrs = new Hashtable<String, String>();
        attrs.put("name", "Admin");
        attrs.put("type", "service");
        ObjectName objName = new ObjectName("Petals", attrs);

        expect(connection.queryNames(objName, null)).andReturn(names)
                .anyTimes();
        connector.connect();
        expect(connector.getMBeanServerConnection()).andReturn(connection)
                .anyTimes();

        replay(connection);
        replay(connector);

        DistributedJMXServer distributedJMXServer = new DistributedJMXServer(
                connector);
        ObjectName adminName = distributedJMXServer.getAdminServiceMBeanName();
        assertEquals(adName, adminName);
    }

    public void testInvoke() throws IOException, MalformedObjectNameException,
        NullPointerException, InstanceNotFoundException, MBeanException,
        ReflectionException {
        MBeanServerConnection connection = EasyMock
                .createMock(MBeanServerConnection.class);
        JMXConnector connector = EasyMock.createMock(JMXConnector.class);

        ObjectName adminName = new ObjectName(
                "FC/Petals/jbi/admin-impl@1111:itf=service");

        Object[] objs = new Object[0];
        String[] args = new String[0];
        expect(connection.invoke(adminName, "test", objs, args)).andReturn(
                Boolean.TRUE).anyTimes();
        connector.connect();
        expect(connector.getMBeanServerConnection()).andReturn(connection)
                .anyTimes();

        replay(connection);
        replay(connector);

        DistributedJMXServer distributedJMXServer = new DistributedJMXServer(
                connector);
        assertTrue(((Boolean) distributedJMXServer.invoke(adminName, "test",
                objs, args)).booleanValue());
    }

}
