/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: ConfigurationServiceTest.java 11:28:52 AM ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.kernel.configuration;

import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;

import junit.framework.TestCase;

import org.easymock.classextension.EasyMock;
import org.objectweb.petals.kernel.admin.ContainerInformation;
import org.objectweb.petals.util.LoggingUtil;

/**
 * Tests of the ConfigurationService
 * 
 * @author ddesjardins - eBMWebsourcing
 */
public class ConfigurationServiceTest extends TestCase {

    public void testFindNotTaken() {
        List<String> taken = new ArrayList<String>();
        taken.add("0");
        taken.add("1");
        ConfigurationService configurationService = new ConfigurationService(
            EasyMock.createMock(LoggingUtil.class));
        configurationService.id = "0";
        assertEquals(configurationService.findNotTaken(taken, 0, 255), "2");
    }

    public void testFindNotTakenValues() {
        List<String> taken = new ArrayList<String>();
        taken.add("0");
        taken.add("1");
        ConfigurationService configurationService = new ConfigurationService(
            EasyMock.createMock(LoggingUtil.class));
        configurationService.id = "0";

        List<String> ports = new ArrayList<String>();
        ports.add("16400");
        ports.add("16200");
        ports.add("16300");
        ports.add("8081");
        ports.add("8082");
        configurationService.jndiPort = "16400";
        configurationService.tcpPort = "16200";
        configurationService.htmlPort = "8081";
        configurationService.jmxPort = "8081";
        configurationService.domainPort = "16300";

        configurationService.findNotTakenValues(taken, ports);

        assertEquals(configurationService.id, "2");
        assertEquals(configurationService.jndiPort, "16401");
        assertEquals(configurationService.tcpPort, "16201");
    }

    public void testIsInitialized() {
        ConfigurationService configurationService = new ConfigurationService(
            EasyMock.createMock(LoggingUtil.class));
        assertFalse(configurationService.isInitialized());
    }

    public void testSetUpConfiguration() throws SecurityException,
        NoSuchMethodException {
        ConfigurationService configurationService = EasyMock.createMock(
            ConfigurationService.class,
            new Method[] {ConfigurationService.class.getDeclaredMethod(
                "findNotTakenValues", new Class[] {List.class, List.class})});
        LoggingUtil loggingUtil = EasyMock.createNiceMock(LoggingUtil.class);

        List<ContainerInformation> containers = new ArrayList<ContainerInformation>();
        ContainerInformation containerInformation = new ContainerInformation();
        containerInformation.setHost("127.0.0.1");
        containerInformation.setUid(10);
        containers.add(containerInformation);
        List<String> ids = new ArrayList<String>();
        ids.add(null);
        List<String> takenPorts = new ArrayList<String>();
        takenPorts.add(null);
        takenPorts.add(null);
        takenPorts.add(null);
        takenPorts.add(null);
        takenPorts.add(null);
        configurationService.findNotTakenValues(ids, takenPorts);
        
        EasyMock.replay(configurationService);

        configurationService.host = "127.0.0.1";
        configurationService.log = loggingUtil;
        configurationService.setContainers(containers);

        configurationService.setUpConfiguration();
    }
}
