/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: MX4J_RMIAdaptorTest.java 14:59:06 ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.kernel.jmx;

import java.io.IOException;

import javax.management.remote.JMXConnectorServer;
import javax.management.remote.JMXServiceURL;

import junit.framework.TestCase;

import org.easymock.classextension.EasyMock;
import org.objectweb.fractal.jmx.agent.AdminAttributes;
import org.objectweb.petals.kernel.mx4j.mock.MockMBeanServer;
import org.objectweb.petals.util.LoggingUtil;
import org.objectweb.petals.util.SystemUtil;

/**
 * Test the MX4JRMIAdaptor
 * 
 * @author ddesjardins - eBMWebsourcing
 */
public class RMIConnectorTest extends TestCase {

    protected RMIConnector adaptor;

    public void setUp() {
        adaptor = new RMIConnector();
    }

    public void testStartReturn() throws IOException {
        JMXConnectorServer connectorServer = EasyMock
            .createMock(JMXConnectorServer.class);
        LoggingUtil loggingUtil = EasyMock.createMock(LoggingUtil.class);
        JMXServiceURL jmxUrl = EasyMock.createMock(JMXServiceURL.class);

        loggingUtil.start();
        EasyMock.expect(connectorServer.isActive()).andReturn(Boolean.TRUE);
        EasyMock.expect(connectorServer.getAddress()).andReturn(jmxUrl);
        connectorServer.stop();
        EasyMock.expect(jmxUrl.getPort()).andReturn(8081);

        EasyMock.replay(connectorServer);
        EasyMock.replay(loggingUtil);
        EasyMock.replay(jmxUrl);

        adaptor.log = loggingUtil;
        SystemUtil.setJmxPort("8081");
        adaptor.cs = connectorServer;

        adaptor.start();
    }

    public void testStartStop() {
        adaptor.log = EasyMock.createMock(LoggingUtil.class);
        AdminAttributes adminAttributes = EasyMock
            .createMock(AdminAttributes.class);
        MockMBeanServer mockMBeanServer = new MockMBeanServer();
        EasyMock.expect(adminAttributes.getRawMBeanServer()).andReturn(
            mockMBeanServer).anyTimes();
        EasyMock.replay(adminAttributes);
        adaptor.adminAttributes = adminAttributes;
        adaptor.start();
        adaptor.stop();
    }

    public void testStartStopStart() throws IOException {
        SystemUtil.setHost("127.0.0.1");

        JMXConnectorServer connectorServer = EasyMock
            .createMock(JMXConnectorServer.class);
        LoggingUtil loggingUtil = EasyMock.createMock(LoggingUtil.class);
        JMXServiceURL jmxUrl = EasyMock.createMock(JMXServiceURL.class);
        AdminAttributes adminAttributes = EasyMock
            .createMock(AdminAttributes.class);
        RuntimeException runtimeException = new RuntimeException();

        loggingUtil.start();
        EasyMock.expect(connectorServer.isActive()).andReturn(Boolean.TRUE);
        EasyMock.expect(connectorServer.getAddress()).andReturn(jmxUrl);
        connectorServer.stop();
        EasyMock.expect(jmxUrl.getPort()).andReturn(8080);
        EasyMock.expect(adminAttributes.getRawMBeanServer()).andThrow(
            runtimeException);
        loggingUtil.error(
            "Error during the instanciation of the MX4J RMI adaptor:",
            runtimeException);
        loggingUtil.end();

        EasyMock.replay(connectorServer);
        EasyMock.replay(loggingUtil);
        EasyMock.replay(jmxUrl);
        EasyMock.replay(adminAttributes);

        adaptor.log = loggingUtil;
        adaptor.adminAttributes = adminAttributes;
        SystemUtil.setJmxPort("8081");
        adaptor.cs = connectorServer;

        adaptor.start();
    }
}
