/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: JNDIServerTest.java 9:42:06 AM ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.kernel.registry;

import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.InetSocketAddress;
import java.net.Socket;

import junit.framework.TestCase;

import org.easymock.classextension.EasyMock;
import org.objectweb.petals.kernel.registry.mock.MockConfigurationService;
import org.objectweb.petals.kernel.registry.msg.request.BindRequest;
import org.objectweb.petals.kernel.registry.msg.request.RegistryRequest;
import org.objectweb.petals.kernel.registry.msg.response.RegistryResponse;
import org.objectweb.petals.kernel.registry.msg.response.RegistryResponse.ResponseType;
import org.objectweb.petals.util.LoggingUtil;
import org.objectweb.petals.util.SystemUtil;
import org.objectweb.util.monolog.api.Logger;

/**
 * Test of the JNDIServer
 * 
 * @author ddesjardins - eBMWebsourcing
 */
public class TwoServerTest extends TestCase {

    private RegistryServer server1;

    private RegistryServer server2;

    public void setUp() {
        String groupName = "petals"
            + System.nanoTime();

        server1 = new RegistryServer();
        Logger logger = EasyMock.createMock(Logger.class);
        server1.logger = logger;
        server1.configurationService = new MockConfigurationService();
        server1.log = EasyMock.createMock(LoggingUtil.class);
        server1.registryUtil.setLog(EasyMock.createMock(LoggingUtil.class));
        server1.setGroupName(groupName);

        server2 = new RegistryServer();
        server2.logger = logger;
        server2.configurationService = new MockConfigurationService();
        server2.log = EasyMock.createMock(LoggingUtil.class);
        server2.registryUtil.setLog(EasyMock.createMock(LoggingUtil.class));
        server2.setGroupName(groupName);

        SystemUtil.setJndiPort("16400");
        server2.startServer();
        try {
            Thread.sleep(1000);
        } catch (InterruptedException e) {
            // Do nothing
        }
        SystemUtil.setJndiPort("16401");
        server1.startServer();
    }

    public void testBind() {
        Socket socket = new Socket();
        try {
            // Wait for the servers to start
            Thread.sleep(1000);

            socket.connect(new InetSocketAddress(SystemUtil.getHost(), 16400),
                1000);
            RegistryRequest request = new BindRequest("/", "key", "value", 0, 0);

            ObjectOutputStream objectOutputStream = new ObjectOutputStream(
                socket.getOutputStream());

            objectOutputStream.writeObject(request);
            objectOutputStream.flush();

            ObjectInputStream objectInputStream = new ObjectInputStream(socket
                .getInputStream());
            RegistryResponse jndiResponse = (RegistryResponse) objectInputStream
                .readObject();

            // Wait for the other server to process the message
            Thread.sleep(3000);

            if (!jndiResponse.getType().equals(ResponseType.ack)) {
                fail();
            } else {
                assertEquals(server1.data.size(), 1);
                assertEquals(server2.data.size(), 1);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }

    }

    public void tearDown() {
        server1.stopServer();
        server2.stopServer();
    }

}
