/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: ServerSocketThread.java 10:34:45 AM ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.kernel.registry.jndi;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.InetSocketAddress;
import java.net.ServerSocket;
import java.net.Socket;

import org.objectweb.petals.kernel.registry.msg.request.RegistryRequest;
import org.objectweb.petals.kernel.registry.msg.request.RegistryRequest.RequestType;
import org.objectweb.petals.kernel.registry.msg.response.AckResponse;
import org.objectweb.petals.kernel.registry.msg.response.ExceptionResponse;
import org.objectweb.petals.kernel.registry.msg.response.RegistryResponse;

/**
 * Mock server socket
 * 
 * @author ddesjardins - eBMWebsourcing
 */
public class ServerSocketThread extends Thread {

    protected ServerSocket serverSocket;

    public ServerSocketThread(int port, String host) {
        try {
            serverSocket = new ServerSocket();
            serverSocket.bind(new InetSocketAddress(host, port));
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    protected boolean run;

    public void run() {
        run = true;
        while (!serverSocket.isBound()){
            try {
                Thread.sleep(500);
            } catch (InterruptedException e) {
                // do nothing
            }
        }
        while (run) {
            try {
                Socket socket = serverSocket.accept();
                RegistryRequest request = null;
                RegistryResponse response = null;

                ObjectInputStream objectInputStream = new ObjectInputStream(
                    socket.getInputStream());
                request = (RegistryRequest) objectInputStream.readObject();

                if (request.getType().equals(RequestType.ping)) {
                    response = new AckResponse(0, 0);
                    response.setArg1("test");
                } else {
                    response = new ExceptionResponse(new RuntimeException(), 0,
                        0);
                }
                ObjectOutputStream objectOutputStream = new ObjectOutputStream(
                    socket.getOutputStream());
                objectOutputStream.writeObject(response);
                objectOutputStream.flush();
                objectOutputStream.close();

            } catch (IOException e) {
                e.printStackTrace();
            } catch (ClassNotFoundException e) {
                e.printStackTrace();
            }
        }
    }

    public void terminate() {
        try {
            run = false;
            serverSocket.close();
            this.finalize();
        } catch (Exception e) {
            // do nothing
        } catch (Throwable e) {
            // do nothing
        }
    }

}
