/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: LoadersTest.java 12:04:13 ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.classloader;

import java.io.File;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;

import junit.framework.TestCase;

/**
 * Test the Loaders
 * 
 * @author ddesjardins - eBMWebsourcing
 */
public class LoadersTest extends TestCase {

    private PetalsClassLoader petalsClassLoader;

    public void setUp() throws Exception {
        List<URL> baseUrls = new ArrayList<URL>();
        List<String> classpathLocations = new ArrayList<String>();
        String baseDir = this.getClass().getResource(".").toString();
        baseDir = baseDir.substring(0, baseDir.indexOf("target"));
        baseDir = baseDir.substring(baseDir.indexOf(":")+1);
        baseUrls.add(new URL("file:" + baseDir.replace(File.separator, "/")));
        baseUrls.add(new URL("file:" + baseDir.replace(File.separator, "/") + "target/test-classes/"));
        baseUrls.add(new URL("file:" + baseDir.replace(File.separator, "/") + "target/"));
        baseUrls.add(new URL("file:" + baseDir.replace(File.separator, "/") + "src/test-data/junit-3.8.jar"));
        petalsClassLoader = new PetalsClassLoader(baseUrls.toArray(new URL[0]),
                classpathLocations);
        petalsClassLoader.setParentFirst(false);
    }

    public void testPut() {
        Loaders loaders = Loaders.getInstance();
        loaders.put("test", petalsClassLoader);
    }

    public void testContains() {
        Loaders loaders = Loaders.getInstance();
        loaders.put("test", petalsClassLoader);
        assertTrue(loaders.contains("test"));
    }

    public void testGet() {
        Loaders loaders = Loaders.getInstance();
        loaders.put("test", petalsClassLoader);
        PetalsClassLoader classLoader = loaders.get("test");
        assertEquals(classLoader, petalsClassLoader);
    }

    public void testGetSharedLibrariesClassLoader() {
        Loaders loaders = Loaders.getInstance();
        assertNotNull(loaders.getSharedLibrariesClassLoader());
    }

    public void testRemove() {
        Loaders loaders = Loaders.getInstance();
        loaders.remove("test");
    }
}
