/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: PetalsClassLoaderTest.java 1061 2006-09-13 15:45:20Z ofabre $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.classloader;

import java.io.DataInputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.net.URLStreamHandlerFactory;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;

import junit.framework.TestCase;

import org.easymock.classextension.EasyMock;

/**
 * Test the petals class loader
 * 
 * @author ddesjardins - eBMWebsourcing
 */
public class PetalsClassLoaderTest extends TestCase {

    /**
     * PetalsClassLoader to test
     */
    private PetalsClassLoader petalsClassLoader;

    public void setUp() throws IOException {
        List<URL> baseUrls = new ArrayList<URL>();
        List<String> classpathLocations = new ArrayList<String>();
        String baseDir = this.getClass().getResource(".").toString();
        baseDir = baseDir.substring(0, baseDir.indexOf("target"));
        baseDir = baseDir.substring(baseDir.indexOf(":") + 1);
        baseUrls.add(new URL("file:"
            + baseDir.replace(File.separator, "/")));
        baseUrls.add(new URL("file:"
            + baseDir.replace(File.separator, "/") + "target/test-classes/"));
        baseUrls.add(new URL("file:"
            + baseDir.replace(File.separator, "/") + "target/"));
        baseUrls.add(new URL("file:"
            + baseDir.replace(File.separator, "/")
            + "src/test-data/junit-3.8.jar"));
        petalsClassLoader = new PetalsClassLoader(baseUrls.toArray(new URL[0]),
            classpathLocations);
        petalsClassLoader.setParentFirst(false);
    }

    /**
     * Test to get the classpath
     * 
     */
    public void testGetClasspath() {
        petalsClassLoader.getClasspath();

    }

    /**
     * Test to load a class from a directory
     * 
     */
    public void testLoadClassFromDirectory() {
        try {

            Class class1 = petalsClassLoader.loadClass(
                "org.objectweb.petals.classloader.ComponentClassLoaderTest",
                false);
            class1.getName();

        } catch (ClassNotFoundException e) {

            fail("Class not found : org.objectweb.petals.classloader.ComponentClassLoaderTest");
        }
    }

    /**
     * Test to load a class from a jar
     * 
     */
    public void testLoadClassFromJar() {
        try {

            Class class1 = petalsClassLoader.loadClass(
                "junit.framework.TestCase", false);
            class1.getName();

        } catch (ClassNotFoundException e) {

            fail("Class not found : junit.framework.TestCase");
        }
    }

    /**
     * Test to get a resource from a jar
     * 
     */
    public void testGetResourceFromAJar() {
        URL url = petalsClassLoader.getResource("META-INF/MANIFEST.MF");
        if (url != null) {
        } else {
            fail("testGetResource - resource not found");
        }
    }

    /**
     * Test to get a resource from a directory
     * 
     */
    public void testGetResourceFromADirectory() {
        URL url = petalsClassLoader.getResource("ResourceToLoad");
        if (url != null) {
        } else {
            fail("testGetResource - resource not found");
        }
    }

    /**
     * Test to get a resource that does not exist
     * 
     */
    public void testGetResourceThatDoesNotExist() {
        URL url = petalsClassLoader.getResource("test");
        if (url == null) {
        } else {
            fail("testGetResource - resource found problem");
        }
    }

    /**
     * Test to get a resource as a stream from a directory
     * 
     * @throws IOException
     */
    public void testGetResourceAsStream() throws IOException {
        InputStream inputStream = petalsClassLoader
            .getResourceAsStream("ResourceToLoad");
        if (inputStream != null) {
            DataInputStream dis = new DataInputStream(inputStream);
            byte[] content = new byte[dis.available()];
            dis.readFully(content);
        } else {
            fail("testGetResourceAsStream - stream not found");
        }
    }

    /**
     * Test to get a resource as a stream from a jar
     * 
     * @throws IOException
     */
    public void testGetResourceAsStreamFromAJar() throws IOException {
        InputStream inputStream = petalsClassLoader
            .getResourceAsStream("META-INF/MANIFEST.MF");
        if (inputStream != null) {
            DataInputStream dis = new DataInputStream(inputStream);
            byte[] content = new byte[dis.available()];
            dis.readFully(content);
        } else {
            fail("testGetResourceAsStream - stream not found");
        }
    }

    /**
     * Test to get resources
     * 
     * @throws IOException
     */
    public void testGetResources() throws IOException {
        Enumeration<URL> enumeration = petalsClassLoader
            .getResources("ResourceToLoad");
        if (enumeration.hasMoreElements()) {
        } else {
            fail("testGetResource - resource not found");
        }
    }

    public void testConstructor() throws Throwable {
        URLStreamHandlerFactory urlStreamHandlerFactory = EasyMock
            .createMock(URLStreamHandlerFactory.class);
        PetalsClassLoader petalsClassLoader = new PetalsClassLoader(new URL[0],
            new ArrayList<String>(), this.getClass().getClassLoader(),
            urlStreamHandlerFactory);
        assertEquals(petalsClassLoader.parent, this.getClass().getClassLoader());
        petalsClassLoader.close();
        assertEquals(petalsClassLoader.getBases().length, 0);
    }

}
