/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: DirLocatorTest.java 11:50:06 ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.classloader.locator;

import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;

import junit.framework.TestCase;

/**
 * Test the DirLocator
 * 
 * @author ddesjardins - eBMWebsourcing
 */
public class DirLocatorTest extends TestCase {

    private String baseDir;

    public void setUp() {
        baseDir = this.getClass().getResource(".").toString();
        baseDir = baseDir.substring(0, baseDir.indexOf("target"));
        baseDir = baseDir.substring(baseDir.indexOf(":") + 1);
    }

    public void testHasFile() throws MalformedURLException, IOException {
        DirLocator dirLocator = new DirLocator(new URL("file:"
            + baseDir.replace(File.separator, "/") + "src/test-data/"));
        assertTrue(dirLocator.hasFile("resourceCore.txt"));
    }

    public void testListContent() throws MalformedURLException, IOException {
        DirLocator dirLocator = new DirLocator(new URL("file:"
            + baseDir.replace(File.separator, "/") + "src/test-data/"));
        assertTrue((dirLocator.listContent(".").size() > 0));
    }

    public void testExceptionNotExist() {
        try {
            new DirLocator(new URL("file:\\test%20bad_file"));
            fail();
        } catch (IOException e) {
            // do nothing
        }
    }
    
    public void testExceptionNotDirectory() {
        try {
            new DirLocator(new URL("file:"
                + baseDir.replace(File.separator, "/") + "src/test-data/test.zip"));
            fail();
        } catch (IOException e) {
            // do nothing
        }
    }

}
