/**
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005 EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): EBM WebSourcing
 */
package org.objectweb.petals.tools.ant;

import java.io.FileInputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Properties;

import javax.management.Attribute;
import javax.management.MBeanServerConnection;
import javax.management.ObjectName;
import javax.management.remote.JMXConnector;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.taskdefs.XSLTProcess.Param;
import org.objectweb.petals.tools.ant.managers.InstallerAntTaskAbstract;
import org.objectweb.petals.tools.ant.util.JBIJMXConnectorUtil;

/**
 * Task used to install a jbi component
 * 
 * @author ddesjardins - eBMWebsourcing
 */
public class InstallComponentTask extends InstallerAntTaskAbstract {

    private List<Param> param = new ArrayList<Param>();

    /**
     * Name of the param file
     */
    private String params;

    public InstallComponentTask() {
        super();
    }

    /**
     * Execute the ant task
     * 
     * @throws IOException
     */
    public void execute() throws BuildException {
        try {
            JMXConnector connector = JBIJMXConnectorUtil.getConnection(host,
                    port, username, password);
            MBeanServerConnection connection = connector
                    .getMBeanServerConnection();
            Object result = performInstall(connection,
                    InstallerAntTaskAbstract.LOAD_NEW_INSTALLED);
            if (result instanceof ObjectName) {
                ObjectName installerName = (ObjectName) result;
                // Install the component
                Object[] objects = new Object[0];
                String[] strings = new String[0];
                connection.invoke(installerName, "install", objects, strings);
                try {
                    log("Installed component name : "
                            + installerName.getKeyProperty("name"));
                } catch (NullPointerException e) {
                    // Exception thrown when execute outisde of the ant context
                }
                // Get the installer configuration
                Object result1 = connection.getAttribute(installerName,
                        "InstallerConfigurationMBean");
                if (result1 instanceof ObjectName) {
                    ObjectName configurationMBean = (ObjectName) result1;
                    // Set the properties of the params file
                    Properties props = new Properties();
                    props.load(new FileInputStream(params));
                    for (Map.Entry entry : props.entrySet()) {
                        Attribute attribute = new Attribute((String) entry
                                .getKey(), (String) entry.getValue());
                        connection.setAttribute(configurationMBean, attribute);
                    }
                    // Set the properties of the <param> element
                    for (Param p : param) {
                        Attribute attribute = new Attribute((String) p
                                .getName(), (String) p.getExpression());
                        connection.setAttribute(configurationMBean, attribute);
                    }
                }
            }
            connector.close();
        } catch (Exception e) {
            if (Boolean.parseBoolean(failOnError)) {
                throw new BuildException(e.getMessage(), e.getCause());
            }
        }
    }

    public void setParams(String params) {
        this.params = params;
    }
}
