/**
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005 EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): EBM WebSourcing
 */
package org.objectweb.petals.tools.ant;

import javax.management.MBeanServerConnection;
import javax.management.remote.JMXConnector;

import org.apache.tools.ant.BuildException;
import org.objectweb.petals.tools.ant.managers.InstallerAntTaskAbstract;
import org.objectweb.petals.tools.ant.util.JBIJMXConnectorUtil;

/**
 * Task used to install a shared library
 * 
 * @author ddesjardins - eBMWebsourcing
 */
public class InstallSharedLibTask extends InstallerAntTaskAbstract {

    public InstallSharedLibTask() {
        super();
    }

    /**
     * Execute the ant task
     * 
     * @throws IOException
     */
    public void execute() throws BuildException {
        try {
            JMXConnector connector = JBIJMXConnectorUtil.getConnection(host,
                    port, username, password);
            MBeanServerConnection connection = connector
                    .getMBeanServerConnection();
            Object[] objects = new Object[1];
            objects[0] = file;
            String[] strings = new String[1];
            strings[0] = "java.lang.String";
            Object result = performInstall(connection, INSTALL_SHARED_LIBRARY);
            if (result instanceof String) {
                try {
                    log("Installed shared library name : " + (String) result);
                } catch (NullPointerException e) {
                    // Exception thrown when execute outisde of the ant context
                }
            }
            connector.close();
        } catch (Exception e) {
            if (Boolean.parseBoolean(failOnError)) {
                throw new BuildException(e.getMessage(), e.getCause());
            }
        }
    }
}
