/**
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005 EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): EBM WebSourcing
 */
package org.objectweb.petals.tools.ant;

import java.util.Arrays;
import java.util.Set;
import java.util.TreeSet;

import javax.management.MBeanServerConnection;
import javax.management.remote.JMXConnector;

import org.apache.tools.ant.BuildException;
import org.objectweb.petals.tools.ant.managers.JBIAntTaskAbstract;
import org.objectweb.petals.tools.ant.util.JBIJMXConnectorUtil;

/**
 * Task used to list the service assemblies
 * 
 * @author ddesjardins - eBMWebsourcing
 */
public class ListServiceAssembliesTask extends JBIAntTaskAbstract {

    /**
     * XML namespace
     */
    private static final String XMLNS = "http://java.sun.com/xml/ns/jbi/component-info-list";

    /**
     * List the service assemblies that has service units deployed to this
     * component. Without this parameter, all the service assemblies that are
     * deployed in the JBI Environment will be displayed.
     */
    private String componentName;

    /**
     * List the Bindings that have Service Units deployed to them as part of the
     * Service Assembly deployed in the JBI Environment. Without this parameter,
     * no Service Assembly dependency is verified.
     */
    private String serviceAssemblyName;

    /**
     * Binding component state Valid state are : shutdown, stated, stopped
     */
    private String state;

    /**
     * If supplied, set the given Ant property name value to the XML version of
     * this listing task.
     */
    private String xmlOutput;

    public ListServiceAssembliesTask() {
        super();
    }

    /**
     * Execute the ant task
     * 
     * @throws IOException
     */
    public void execute() throws BuildException {
        try {
            if (state != null
                    && (!state.equalsIgnoreCase("shutdown")
                            || !state.equalsIgnoreCase("started") || !state
                            .equalsIgnoreCase("stopped"))) {
                throw new BuildException(
                        "Valid states are : 'shutdown', 'started', or 'stopped'");
            }
            JMXConnector connector = JBIJMXConnectorUtil.getConnection(host,
                    port, username, password);
            MBeanServerConnection connection = connector
                    .getMBeanServerConnection();
            // Output
            StringBuffer output = new StringBuffer();
            output
                    .append("-------------------------------------------------------\n");
            output.append("------------- Service Assemblies-- ----------\n");
            output
                    .append("-------------------------------------------------------\n");
            StringBuffer outputXML = new StringBuffer();
            outputXML.append("<?xml version='1.0' encoding='utf-8'?>\n");
            outputXML.append("<component-info-list ");
            outputXML.append("xmlns='" + ListServiceAssembliesTask.XMLNS
                    + "' version='1.0'>\n");
            // Get the engines component names
            Object result = connection.getAttribute(JBIJMXConnectorUtil
                    .getDeploymentServiceMBeanName(connection),
                    "DeployedServiceAssemblies");
            Set<String> saForCompo = new TreeSet<String>();
            if (componentName != null) {
                // Get getComponentsForDeployedServiceAssembly
                Object[] objects = new Object[1];
                objects[0] = componentName;
                String[] sig = new String[1];
                sig[0] = "java.lang.String";
                Object resultSA = connection.invoke(JBIJMXConnectorUtil
                        .getDeploymentServiceMBeanName(connection),
                        "getDeployedServiceAssembliesForComponent", objects,
                        sig);
                if (resultSA instanceof String[]) {
                    saForCompo.addAll(Arrays.asList((String[]) result));
                }
            }
            String saName = serviceAssemblyName;
            String saState = state;
            if (result instanceof String[]) {
                String[] assemblies = (String[]) result;
                for (String objectName : assemblies) {
                    Object[] objects = new Object[1];
                    objects[0] = objectName;
                    String[] sig = new String[1];
                    sig[0] = "java.lang.String";
                    String objState = (String) connection.invoke(
                            JBIJMXConnectorUtil
                                    .getDeploymentServiceMBeanName(connection),
                            "getState", objects, sig);
                    String name = objectName;
                    // TODO extract the description
                    String description = "";
                    // Set the variable to have a positive test
                    if (state == null) {
                        saState = objState;
                    }
                    if (serviceAssemblyName == null) {
                        saName = name;
                    }
                    if (componentName == null) {
                        saForCompo.add(name);
                    }
                    if (name.equals(saName) && objState.equals(saState)
                            && saForCompo.contains(name)) {
                        // Get the component for the service assembly
                        objects = new Object[1];
                        objects[0] = objectName;
                        sig = new String[1];
                        sig[0] = "java.lang.String";
                        Set<String> composForSA = new TreeSet<String>(
                                Arrays
                                        .asList((String[]) connection
                                                .invoke(
                                                        JBIJMXConnectorUtil
                                                                .getDeploymentServiceMBeanName(connection),
                                                        "getComponentsForDeployedServiceAssembly",
                                                        objects, sig)));
                        for (String compoName : composForSA) {
                            objects = new Object[1];
                            objects[0] = compoName;
                            sig = new String[1];
                            sig[0] = "java.lang.String";
                            // Set<String> serviceUnits = new TreeSet<String>(
                            // Arrays
                            // .asList((String[]) connection
                            // .invoke(
                            // JBIJMXConnectorUtil
                            // .getDeploymentServiceMBeanName(connection),
                            // "getDeployedServiceUnitList",
                            // objects, sig)));
                            // // TODO get the service unit to the output
                        }
                        // Add the component to the output
                        output.append("Name : " + name + "\n");
                        output.append("State : " + objState + "\n");
                        output.append("Description : " + description + "\n\n");
                        outputXML
                                .append("\t<service-assembly-info type='service-assembly' name='"
                                        + name
                                        + "' state='"
                                        + objState
                                        + "'>\n");
                        outputXML.append("\t\t<description>");
                        outputXML.append(description);
                        outputXML.append("</description>\n");
                        outputXML.append("\t</service-assembly-info>\n");
                    }
                }
                outputXML.append("</service-assembly-info-list>");
                if (xmlOutput == null) {
                    // Display the list on the console
                    try {
                        log(output.toString());
                    } catch (NullPointerException e) {
                        // Exception thrown when execute outisde of the ant
                        // context
                    }
                } else {
                    try {
                        // Fill the property xmlOutput
                        getProject().setNewProperty(xmlOutput,
                                outputXML.toString());
                    } catch (Exception e) {
                        // Exception thrown if not executed into a Ant context
                    }
                }
            }
            connector.close();
        } catch (Exception e) {
            if (Boolean.parseBoolean(failOnError)) {
                throw new BuildException(e.getMessage(), e.getCause());
            }
        }
    }

    public void setComponentName(String componentName) {
        this.componentName = componentName;
    }

    public void setServiceAssemblyName(String serviceAssemblyName) {
        this.serviceAssemblyName = serviceAssemblyName;
    }

    public void setState(String state) {
        this.state = state;
    }

    public void setXmlOutput(String xmlOutput) {
        this.xmlOutput = xmlOutput;
    }
}
