/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: AntTaskComponent.java 09:08:21 ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.tools.ant.managers;

import javax.management.MBeanServerConnection;
import javax.management.ObjectName;
import javax.management.remote.JMXConnector;

import org.apache.tools.ant.BuildException;
import org.objectweb.petals.tools.ant.util.JBIJMXConnectorUtil;

/**
 * This class is used to gathers the operation on the components
 * 
 * @author ddesjardins - eBMWebsourcing
 */
public abstract class ComponentAntTaskAbstract extends JBIAntTaskAbstract {

    /**
     * Action : shutdown the component
     */
    protected static final String SHUTDOWN = "shutDown";

    /**
     * Action : start the component
     */
    protected static final String START = "start";

    /**
     * Action : stop the component
     */
    protected static final String STOP = "stop";

    /**
     * Name of the JBI component in the JBI environment
     */
    protected String name;

    /**
     * Perform an action
     * 
     * @param action
     *            action to perform
     */
    protected void performAction(String action) {
        if (name == null) {
            throw new BuildException("Required attribute : name");
        }
        try {
            JMXConnector connector = JBIJMXConnectorUtil.getConnection(host,
                    port, username, password);
            MBeanServerConnection connection = connector
                    .getMBeanServerConnection();
            Object[] objects = new Object[0];
            String[] strings = new String[0];
            ObjectName objectName = JBIJMXConnectorUtil.getComponentMBeanName(
                    connection, name);
            if (objectName == null) {
                throw new BuildException("The component is not installed");
            }
            connection.invoke(objectName, action, objects, strings);
            try {
                log("Shutdown component name : " + name);
            } catch (NullPointerException e) {
                // Exception thrown when execute outisde of the ant context
            }
            connector.close();
        } catch (Exception e) {
            if (Boolean.parseBoolean(failOnError)) {
                throw new BuildException(e.getMessage(), e.getCause());
            }
        }
    }

    /**
     * Perform an uninstall on the component
     * 
     */
    protected void performUninstall() {
        if (name == null) {
            throw new BuildException("Required attribute : name");
        }
        try {
            JMXConnector connector = JBIJMXConnectorUtil.getConnection(host,
                    port, username, password);
            MBeanServerConnection connection = connector
                    .getMBeanServerConnection();
            Object[] objects = new Object[2];
            objects[0] = name;
            objects[1] = true;
            String[] strings = new String[2];
            strings[0] = "java.lang.String";
            strings[1] = "boolean";
            // Unload the installer
            connection.invoke(JBIJMXConnectorUtil
                    .getInstallationServiceMBeanName(connection),
                    "unloadInstaller", objects, strings);
            try {
                log("Uninstalled component name : " + name);
            } catch (NullPointerException e) {
                // Exception thrown when execute outisde of the ant context
            }
        } catch (Exception e) {
            if (Boolean.parseBoolean(failOnError)) {
                throw new BuildException(e.getMessage(), e.getCause());
            }
        }
    }

    public void setName(String name) {
        this.name = name;
    }

}
