/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: AntTaskInstaller.java 09:02:38 ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.tools.ant.managers;

import java.net.URL;

import javax.management.MBeanServerConnection;

import org.apache.tools.ant.BuildException;
import org.objectweb.petals.tools.ant.util.JBIJMXConnectorUtil;

/**
 * This class is used to install or deploy a JBI component, shared library or
 * service assembly
 * 
 * @author ddesjardins - eBMWebsourcing
 */
public abstract class InstallerAntTaskAbstract extends JBIAntTaskAbstract {

    /**
     * Action : installSharedLibrary
     */
    protected static final String INSTALL_SHARED_LIBRARY = "installSharedLibrary";

    /**
     * Action : loadNewInstaller
     */
    protected static final String LOAD_NEW_INSTALLED = "loadNewInstaller";

    /**
     * Fully qualified installation file path name
     */
    protected String file;

    /**
     * Perform a Install or Deploy action
     * 
     * @param action
     * @return
     * @throws Exception
     */
    protected Object performInstall(MBeanServerConnection connection,
            String action) throws Exception {
        if (file == null) {
            throw new BuildException("Required attribute : file");
        }
        // Test if the provided file name is a correct URL
        new URL(file);
        Object[] objects = new Object[1];
        objects[0] = file;
        String[] strings = new String[1];
        strings[0] = "java.lang.String";
        Object result = connection.invoke(JBIJMXConnectorUtil
                .getInstallationServiceMBeanName(connection), action, objects,
                strings);
        return result;
    }

    public void setFile(String file) {
        this.file = file;
    }

}
