package org.objectweb.petals.tools.ant;

import java.io.File;
import java.io.IOException;

import org.objectweb.petals.tools.ant.InstallSharedLibTask;
import org.objectweb.petals.tools.ant.ListSharedLibraryTask;
import org.objectweb.petals.tools.ant.UninstallSharedLibTask;
import org.objectweb.petals.tools.ant.util.ServerUtil;

import junit.framework.TestCase;

/**
 * Test the shared libraries ant tasks
 * 
 * @author ddesjardins - eBMWebsourcing
 */
public class SharedLibrariesTasksTest extends TestCase {

    /**
     * Port of the JMX server
     */
    private static int port = 9988;

    public void setUp() throws Exception {
        ServerUtil.start(port);
    }

    public void tearDown() throws IOException {
        ServerUtil.stop();
    }

    /**
     * Test exception throwing
     * 
     */
    public void testExceptionFileNull() {
        UninstallSharedLibTask uninstallSharedLibTask = new UninstallSharedLibTask();
        uninstallSharedLibTask.setPort("" + port);
        try {
            uninstallSharedLibTask.execute();
            fail();
        } catch (Exception e) {

        }
    }

    /**
     * Test exception throwing
     * 
     */
    public void testExceptionNameNull() {
        InstallSharedLibTask installSharedLibTask = new InstallSharedLibTask();
        installSharedLibTask.setPort("" + port);
        try {
            installSharedLibTask.execute();
            fail();
        } catch (Exception e) {

        }
    }

    /**
     * Test exception throwing
     * 
     */
    public void testExceptionServerJMX() {
        String baseDir = this.getClass().getResource(".").toString();
        baseDir = baseDir.substring(0, baseDir.indexOf("target"));
        baseDir = baseDir.substring(baseDir.indexOf(":")+1);
        InstallSharedLibTask installSharedLibTask = new InstallSharedLibTask();
        installSharedLibTask.setFile("file:" + baseDir + "src" + File.separator
                + "test-data" + File.separator + "sa.zip");
        installSharedLibTask.setPort("" + port);
        try {
            tearDown();
            installSharedLibTask.execute();
            fail();
        } catch (Exception e) {

        }
    }

    /**
     * Test the shared library uninstallation task
     */
    public void testExceptionServerJMX1() {
        testInstall();
        UninstallSharedLibTask uninstallSharedLibTask = new UninstallSharedLibTask();
        uninstallSharedLibTask.setName("foo003");
        uninstallSharedLibTask.setPort("" + port);
        try {
            tearDown();
            uninstallSharedLibTask.execute();
        } catch (Exception e) {

        }
    }

    /**
     * Test the shared library installation task
     * 
     * @throws Exception
     */
    public void testInstall() {
        String baseDir = this.getClass().getResource(".").toString();
        baseDir = baseDir.substring(0, baseDir.indexOf("target"));
        baseDir = baseDir.substring(baseDir.indexOf(":")+1);
        InstallSharedLibTask installSharedLibTask = new InstallSharedLibTask();
        installSharedLibTask.setFile("file:" + baseDir + "src" + File.separator
                + "test-data" + File.separator + "sa.zip");
        installSharedLibTask.setPort("" + port);
        installSharedLibTask.execute();
    }

    /**
     * Test the shared library list task
     */
    public void testList() {
        testInstall();
        ListSharedLibraryTask listSharedLibraryTask = new ListSharedLibraryTask();
        listSharedLibraryTask.setPort("" + port);
//        listSharedLibraryTask.setComponentName("foo");
//        listSharedLibraryTask.setSharedLibraryName("bar");
//        listSharedLibraryTask.setXmlOutput("test");
        try {
            listSharedLibraryTask.execute();
            fail();
        } catch (Exception e) {
        }
    }

    /**
     * Test the shared library uninstallation task
     */
    public void testUninstall() {
        testInstall();
        UninstallSharedLibTask uninstallSharedLibTask = new UninstallSharedLibTask();
        uninstallSharedLibTask.setName("foo003");
        uninstallSharedLibTask.setPort("" + port);
        uninstallSharedLibTask.execute();
    }

}
