/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: Identification.java 12:14:20 rmarins $
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.tools.jbicommon.descriptor;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;
import org.apache.commons.lang.builder.ToStringBuilder;

/**
 * The identification production is used to provide identifying information for
 * components, shared libraries, service units, and service assemblies.
 * 
 * @version $Rev: 438 $ $Date: 2006-05-19 11:12:26Z $
 * @since Petals 1.0
 * @author <a href="mailto:rmarins@fossilec.com">Rafael Marins</a>
 */
public class Identification extends ExtensibleJbiElement {

    /**
     * Item name.
     */
    private String name;

    /**
     * Item description.
     */
    private String description;

    /**
     * Default constructor.
     */
    public Identification() { // NOPMD by gblondelle
        super();
    }

    @Override
    public boolean equals(final Object other) {
        if (!(other instanceof Identification)) {
            return false; // NOPMD by gblondelle
        }
        Identification castOther = (Identification) other;
        return new EqualsBuilder().append(name, castOther.name).append(
                description, castOther.description).isEquals();
    }

    /**
     * A longer (one sentence) description of the item.
     * 
     * @return <code>String</code> the item description.
     */
    public String getDescription() {
        return description;
    }

    /**
     * The name of the item. This is a human-friendly identifier for the item.
     * 
     * @return <code>String</code> the item name.
     */
    public String getName() {
        return name;
    }

    //
    // Getters and Setters
    //

    @Override
    public int hashCode() {
        return new HashCodeBuilder().append(name).append(description)
                .toHashCode();
    }

    @Override
    public String toString() {
        return new ToStringBuilder(this).append("name", name).append(
                "description", description).toString();
    }

    /**
     * A longer (one sentence) description of the item.
     * 
     * @param description
     *            <code>String</code> item description to set.
     */
    protected void setDescription(final String description) {
        this.description = description;
    }

    /**
     * The name of the item. This is a human-friendly identifier for the item.
     * 
     * @param name
     *            <code>String</code> item name to set.
     */
    protected void setName(final String name) {
        this.name = name;
    }

}
