/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005-2006 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.tools.jbicommon.descriptor;

import javax.xml.namespace.QName;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;
import org.apache.commons.lang.builder.ToStringBuilder;

/**
 * The provides elements is used to declare a service provided by a service
 * unit. A provided service is declared with the following attributes:
 * <ul>
 * <li><b>interface-name.</b> This qualified name indicates the type of
 * service to be provided.</li>
 * <li><b>service-name and endpoint-name.</b>This pair of attributes declares
 * the name of the endpoint that the service unit will, after deployment,
 * activate with JBI.</li>
 * </ul>
 * 
 * @version $Rev: 438 $ $Date: 2006-05-19 11:12:26Z $
 * @since Petals 1.0
 * @author <a href="mailto:rmarins@fossilec.com">Rafael Marins</a>
 */
public class Provides extends ExtensibleJbiElement {

    /**
     * Endpoint name to be provided.
     */
    private String endpointName;

    /**
     * Qualified interface name.
     */
    private QName interfaceName;

    /**
     * Service name to be provided.
     */
    private QName serviceName;

    /**
     * Default constructor.
     */
    public Provides() { // NOPMD by gblondelle
        super();
    }

    @Override
    public boolean equals(final Object other) {
        if (!(other instanceof Provides)) {
            return false; // NOPMD by gblondelle
        }
        Provides castOther = (Provides) other;
        return new EqualsBuilder().append(endpointName, castOther.endpointName)
                .append(interfaceName, castOther.interfaceName).append(
                        serviceName, castOther.serviceName).isEquals();
    }

    /**
     * Returns the endpoint name that the service unit will, after deployment,
     * activate with JBI.
     * 
     * @return <code>String</code> the endpoint name.
     */
    public String getEndpointName() {
        return endpointName;
    }

    /**
     * Return the qualified name indicating the type of service to be provided.
     * 
     * @return <code>QName</code> the interface name.
     */
    public QName getInterfaceName() {
        return interfaceName;
    }

    /**
     * Returns the service name that the service unit will, after deployment,
     * activate with JBI.
     * 
     * @return <code>QName</code> the service name.
     */
    public QName getServiceName() {
        return serviceName;
    }

    //
    // Getters and Setters
    //

    @Override
    public int hashCode() {
        return new HashCodeBuilder().append(endpointName).append(interfaceName)
                .append(serviceName).toHashCode();
    }

    @Override
    public String toString() {
        return new ToStringBuilder(this).append("endpointName", endpointName)
                .append("interfaceName", interfaceName).append("serviceName",
                        serviceName).toString();
    }

    /**
     * Set the endpoint name which will be activated in JBI.
     * 
     * @param endpointName
     *            the endpoint name.
     */
    void setEndpointName(final String endpointName) {
        this.endpointName = endpointName;
    }

    /**
     * Set the interface name indicating the type of service to be provided.
     * 
     * @param interfaceName
     *            the interface name.
     */
    void setInterfaceName(final QName interfaceName) {
        this.interfaceName = interfaceName;
    }

    /**
     * Set the service name which will be activated in JBI.
     * 
     * @param serviceName
     *            the service name.
     */
    void setServiceName(final QName serviceName) {
        this.serviceName = serviceName;
    }
}
