/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: ServiceUnit.java 12:14:20 rmarins $
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.tools.jbicommon.descriptor;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;
import org.apache.commons.lang.builder.ToStringBuilder;

/**
 * The Service Unit production is used to describe a service component within a
 * service assembly deployment package.
 * 
 * @version $Rev: 438 $ $Date: 2006-05-19 11:12:26Z $
 * @since Petals 1.0
 * @author <a href="mailto:rmarins@fossilec.com">Rafael Marins</a>
 */
public class ServiceUnit extends ExtensibleJbiElement {

    /**
     * Service Unit identification data.
     */
    private Identification identification;

    /**
     * Name of the artifact archive file.
     */
    private String targetArtifactsZip; // NOPMD by gblondelle

    /**
     * Component name.
     */
    private String targetComponentName; // NOPMD by gblondelle

    /**
     * Default constructor.
     */
    public ServiceUnit() { // NOPMD by gblondelle
        super();
    }

    @Override
    public boolean equals(final Object other) {
        if (!(other instanceof ServiceUnit)) {
            return false; // NOPMD by gblondelle
        }
        ServiceUnit castOther = (ServiceUnit) other;
        return new EqualsBuilder().append(identification,
                castOther.identification).append(targetArtifactsZip,
                castOther.targetArtifactsZip).append(targetComponentName,
                castOther.targetComponentName).isEquals();
    }

    /**
     * Returns service unit's identification data.
     * 
     * @return <code>Identification</code> the SU's identification.
     */
    public Identification getIdentification() {
        return identification;
    }

    /**
     * Returns the name of the artifact archive file within the deployment
     * package that will be deployed.
     * 
     * @return <code>String</code> the artifact archive file.
     */
    public String getTargetArtifactsZip() {
        return targetArtifactsZip;
    }

    //
    // Getters and Setters
    //

    /**
     * Returns the name of the component that will have the <i>artifacts zip</i>
     * deployed to it.
     * 
     * @return <code>String</code> the component name.
     */
    public String getTargetComponentName() {
        return targetComponentName;
    }

    @Override
    public int hashCode() {
        return new HashCodeBuilder().append(identification).append(
                targetArtifactsZip).append(targetComponentName).toHashCode();
    }

    @Override
    public String toString() {
        return new ToStringBuilder(this).append("identification",
                identification)
                .append("targetArtifactsZip", targetArtifactsZip).append(
                        "targetComponentName", targetComponentName).toString();
    }

    /**
     * Set the service unit identification data.
     * 
     * @param identification
     *            The identification to set.
     */
    protected void setIdentification(final Identification identification) {
        this.identification = identification;
    }

    /**
     * Set the target artifacts archive file within the deployment package.
     * 
     * @param targetArtifactsZip
     *            <code>String</code> the archive file.
     */
    protected void setTargetArtifactsZip(final String targetZip) {
        this.targetArtifactsZip = targetZip;
    }

    /**
     * Set the target component name to which the artifacts archive file is to
     * be deployed.
     * 
     * @param targetComponentName
     *            <code>String</code> the component name.
     */
    protected void setTargetComponentName(final String targetCompName) {
        this.targetComponentName = targetCompName;
    }

}
