/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: SharedLibraryList.java 477 2006-05-29 15:18:07Z ddesjardins $
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.tools.jbicommon.descriptor;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;
import org.apache.commons.lang.builder.ToStringBuilder;

/**
 * Shared library list production is used together with a parametrized
 * {@link List} to provide the pair info of shared library listing used by the
 * component.
 * 
 * @version $Rev: 477 $ $Date: 2006-05-29 15:18:07Z $
 * @since Petals 1.0
 * @author <a href="mailto:rmarins@fossilec.com">Rafael Marins</a>
 */
public class SharedLibraryList {

    /**
     * Shared library name.
     */
    private String name;

    /**
     * Optional shared library version.
     */
    private String version;

    /**
     * Default constructor.
     */
    SharedLibraryList() {
        super();
    }

    @Override
    public boolean equals(final Object other) {
        if (!(other instanceof SharedLibraryList)) {
            return false; // NOPMD by gblondelle
        }
        SharedLibraryList castOther = (SharedLibraryList) other;
        return new EqualsBuilder().append(name, castOther.name).append(version,
                castOther.version).isEquals();
    }

    /**
     * Returns the shared library name.
     * 
     * @return <code>String</code> the shared library name.
     */
    public String getName() {
        return name;
    }

    /**
     * Returns the shared library version. May be <code>null</code> since it's
     * optional.
     * 
     * @return <code>String</code> optional shared library version.
     */
    public String getVersion() {
        return version;
    }

    //
    // Getters and Setters
    //

    @Override
    public int hashCode() {
        return new HashCodeBuilder().append(name).append(version).toHashCode();
    }

    @Override
    public String toString() {
        return new ToStringBuilder(this).append("name", name).append("version",
                version).toString();
    }

    /**
     * Set shared library name.
     * 
     * @param name
     *            <code>String</code> shared library name.
     */
    void setName(final String name) {
        this.name = name;
    }

    /**
     * Set the optional shared library version.
     * 
     * @param version
     *            <code>String</code> shared library version.
     */
    void setVersion(final String version) {
        this.version = version;
    }

}
