/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005-2006 Fossil E-Commerce, http://www.fossilec.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: FileUtil.java 508 2006-06-02 08:05:25Z ddesjardins $
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.tools.jbicommon.util;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * This util class is used to manage file
 * 
 * @version $Rev: 508 $ $Date: 2006-06-02 08:05:25Z $
 * @since Petals 1.0
 * @author <a href="mailto:rmarins@fossilec.com">Rafael Marins</a>
 * @author ddesjardins - eBMWebsourcing
 */
public final class FileUtil {

    private static final Logger LOGGER = Logger.getLogger(XMLUtil.class
            .getName());

    /**
     * Buffer size used when copying the content of an input stream to an output
     * stream.
     */
    private static final int DEFAULT_BUFFER_SIZE = 4096; // NOPMD by gblondelle

    /**
     * Do not construct.
     */
    private FileUtil() {
        super();
    }

    /**
     * Copy an in stream to an out stream
     * 
     * @param in
     * @param out
     * @throws IOException
     */
    public static void copyInputStream(final InputStream in,
            final OutputStream out) throws IOException {
        byte[] buffer = new byte[DEFAULT_BUFFER_SIZE];
        int len;
        // while ((len = in.read(buffer)) >= 0) {
        // out.write(buffer, 0, len);
        // }
        try {
            while (true) {
                len = in.read(buffer);
                if (len == -1) {
                    break;
                }
                out.write(buffer);
            }
        } finally {
            if (in != null) {
                in.close();
            }
            if (out != null) {
                out.close();
            }
            in.close();
            out.close();
        }
    }

    /**
     * @param dir
     *            the directory to delete
     * @return true if the directory is completely removed
     */
    public static boolean removeDirectory(final File dir) {

        boolean completeRemoval = true;

        if (!dir.isDirectory() && !dir.exists()) {
            return false; // NOPMD by gblondelle
        }

        // first remove directory contents
        for (File aFile : dir.listFiles()) {

            if (aFile.isDirectory()) {
                completeRemoval = removeDirectory(aFile);
            } else {
                completeRemoval = aFile.delete();
            }
        }

        if (completeRemoval) {
            // delete main directory (now empty)
            completeRemoval = dir.delete();
        } else {
            // do nothing simply log
            LOGGER.log(Level.WARNING,
                    "Directory not deleted because not empty :"
                            + dir.getAbsolutePath());
        }

        return completeRemoval;
    }

}
