/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2006 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.tools.jbicommon.util;

import java.util.Timer;
import java.util.TimerTask;

/**
 * A polling util used to schedule recursive task to be executed. Extend this
 * abstract class and implement the run method that will be launched each N ms.
 * 
 * @author Christophe HAMERLING - eBMWebSourcing
 * 
 */
public abstract class PeriodicTask extends TimerTask {

    /**
     * The timer used for task scheduling
     */
    protected Timer timer;

    /**
     * The polling period
     */
    protected long period;

    /**
     * The first task execution delay
     */
    protected long delay;

    /**
     * The default polling period
     */
    private static final long DEFAULT_PERIOD = 1000L;

    /**
     * Creates a new instance of {@link PeriodicTask}
     * 
     */
    public PeriodicTask() {
        this(DEFAULT_PERIOD, DEFAULT_PERIOD);
    }

    /**
     * Creates a new instance of {@link PeriodicTask}
     * 
     * @param period
     *            in ms
     */
    public PeriodicTask(long delay, long period) {
        this.delay = delay;
        this.period = period;
        timer = new Timer();
    }

    /**
     * Start the task processing
     * 
     */
    public void startProcessing() {
        timer.schedule(this, delay, period);
    }

    /**
     * Stop the task processing
     * 
     */
    public void stopProcessing() {
        timer.cancel();
    }
}
