/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2006 EBM Websourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: StringHelperTest.java 98 2006-02-24 16:18:48Z alouis $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.tools.util;

import java.util.ArrayList;
import java.util.List;

import org.objectweb.petals.tools.jbicommon.util.StringHelper;

import junit.framework.TestCase;

/**
 * 
 * @version $Rev: 98 $ $Date: 2006-02-24 17:18:48 +0100 (ven., 24 févr. 2006) $
 * @since Petals 1.0
 * @author Adrien LOUIS
 */
public class StringHelperTest extends TestCase {

    /*
     * Test method for 'org.objectweb.petals.util.StringHelper.isEmpty(String)'
     */
    public void testIsEmpty1() {
        assertTrue(StringHelper.isNullOrEmpty(null));
    }

    public void testIsEmpty2() {
        assertTrue(StringHelper.isNullOrEmpty(""));
    }

    public void testIsEmpty3() {
        assertFalse(StringHelper.isNullOrEmpty("a"));
    }

    /*
     * Test method for 'org.objectweb.petals.util.StringHelper.equal(String,
     * String)'
     */
    public void testEqual1() {
        assertTrue(StringHelper.equal(null, null));
    }

    public void testEqual2() {
        assertTrue(StringHelper.equal("a", "a"));
    }

    public void testEqual3() {
        assertFalse(StringHelper.equal(null, ""));
    }

    public void testEqual4() {
        assertFalse(StringHelper.equal("", null));
    }

    public void testEqual5() {
        assertFalse(StringHelper.equal("a", "A"));
    }

    public void testEqual6() {
        assertFalse(StringHelper.equal("a", "b"));
    }

    /*
     * Test method for
     * 'org.objectweb.petals.util.StringHelper.equalIgnoreCase(String, String)'
     */
    public void testequalIgnoreCaseIgnoreCase1() {
        assertTrue(StringHelper.equalIgnoreCase(null, null));
    }

    public void testequalIgnoreCaseIgnoreCase2() {
        assertTrue(StringHelper.equalIgnoreCase("a", "a"));
    }

    public void testequalIgnoreCaseIgnoreCase5() {
        assertTrue(StringHelper.equalIgnoreCase("a", "A"));
    }

    public void testequalIgnoreCaseIgnoreCase3() {
        assertFalse(StringHelper.equalIgnoreCase(null, ""));
    }

    public void testequalIgnoreCaseIgnoreCase4() {
        assertFalse(StringHelper.equalIgnoreCase("", null));
    }

    public void testequalIgnoreCaseIgnoreCase6() {
        assertFalse(StringHelper.equalIgnoreCase("a", "b"));
    }

    /*
     * Test method for
     * 'org.objectweb.petals.util.StringHelper.extractValueForAttribute(String
     * string, String attribute,String separator)'
     */

    // null parameters
    public void testextractValueForAttribute1() {
        assertNull(StringHelper.extractValueForAttribute(null, "", ""));
    }

    public void testextractValueForAttribute2() {
        assertNull(StringHelper.extractValueForAttribute("", null, ""));
    }

    public void testextractValueForAttribute3() {
        assertNull(StringHelper.extractValueForAttribute("", "", null));
    }

    // att found
    public void testextractValueForAttribute4() {
        assertEquals("value", StringHelper.extractValueForAttribute(
                "att=value", "att", null));
    }

    public void testextractValueForAttribute5() {
        assertEquals("value", StringHelper.extractValueForAttribute(
                "yo att=value", "att", null));
    }

    public void testextractValueForAttribute6() {
        assertEquals("value", StringHelper.extractValueForAttribute(
                "yo att=value&yo", "att", "&"));
    }

    public void testextractValueForAttribute7() {
        assertEquals("value etc", StringHelper.extractValueForAttribute(
                "yo att=value etc", "att", null));
    }

    /*
     * Test method for StringHelper#splitPathElements(String)
     */
    public void testSplitPathElements1() {
        assertTrue(StringHelper.splitPathElements(null).isEmpty());
    }

    public void testSplitPathElements2() {
        List<String> result = new ArrayList<String>();
        result.add("foo");
        result.add("bar");
        assertEquals(result, StringHelper.splitPathElements("/foo/bar"));
    }

    public void testSplitPathElements3() {
        List<String> result = new ArrayList<String>();
        result.add("foo");
        result.add("bar");
        assertEquals(result, StringHelper.splitPathElements("foo/bar"));
    }

    public void testSplitPathElements4() {
        List<String> result = new ArrayList<String>();
        result.add("foo");
        result.add("bar");
        assertEquals(result, StringHelper.splitPathElements("/foo/bar/"));
    }

    public void testSplitPathElements5() {
        List<String> result = new ArrayList<String>();
        result.add("foo");
        result.add("bar");
        assertEquals(result, StringHelper.splitPathElements("//foo///bar//"));
    }

}
