/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: PackageAssemblyMojo.java 16:56:43 ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.tools.jbiplugin;

import javax.management.MBeanServerConnection;
import javax.management.ObjectName;
import javax.management.remote.JMXConnector;

import org.objectweb.petals.tools.jbiplugin.util.JBIJMXConnectorUtil;

/**
 * This class gathers the action on a service assembly
 * 
 * @author cdeneux - Capgemini Sud
 */
public abstract class JMXComponentAbstractMojo extends JMXTaskAbstractMojo {

	/**
	 * Action : installSharedLibrary
	 */
	protected static final String INSTALL_SHARED_LIBRARY = "installSharedLibrary";

	/**
	 * Action : loadNewInstaller
	 */
	protected static final String LOAD_NEW_INSTALLED = "loadNewInstaller";

	/**
	 * Action : loadNewInstaller
	 */
	protected static final String UNLOAD_INSTALLER = "unloadInstaller";

	/**
	 * Action : install
	 */
	protected static final String INSTALL = "install";

	/**
	 * Action : uninstall
	 */
	protected static final String UNINSTALL = "uninstall";

	/**
	 * Action : shutdown a component
	 */
	protected static final String SHUTDOWN = "shutDown";

	/**
	 * Action : start a component
	 */
	protected static final String START = "start";

	/**
	 * Action : stop a component
	 */
	protected static final String STOP = "stop";

	/**
	 * Get state of a service assembly
	 * 
	 * @param componentName
	 *            component
	 */
	protected String getState(String componentName) throws Exception {
		JMXConnector connector = null;
		String state = null;
		try {
			
			connector = JBIJMXConnectorUtil.getConnection(host, port, username,
					password);
			MBeanServerConnection connection = connector
					.getMBeanServerConnection();
			ObjectName objectName = JBIJMXConnectorUtil.getComponentMBeanName(
					connection, componentName);
			if (objectName == null) {
				// The component is not installed, we look for if an installer exists
				objectName = JBIJMXConnectorUtil.getInstallerComponentMBeanName(
						connection, componentName);
				if (objectName == null) {
					// No installer of the component exists
					state = "Unknown";
				}
				else {
					state = "Loaded";
				}
			}
			else {
				state = (String) connection.getAttribute(
                        objectName, "CurrentState");
			}
			
		} catch (Exception e) {
			if (failOnError) {
				throw e;
			}
		} finally {
			connector.close();
		}
		
		System.out.println("Component: "
				+ componentName + " is in state: " + state);
		return state;
		
	}
	
	/**
	 * Perform an action on the component
	 * 
	 * @param action
	 *            action to perform
	 */
	protected void performComponentAction(String action, String componentName)
			throws Exception {
		JMXConnector connector = null;
		try {
			connector = JBIJMXConnectorUtil.getConnection(host, port, username,
					password);
			MBeanServerConnection connection = connector
					.getMBeanServerConnection();
			Object[] objects = new Object[0];
			String[] strings = new String[0];
			ObjectName objectName = JBIJMXConnectorUtil.getComponentMBeanName(
					connection, componentName);
			if (objectName == null) {
				throw new Exception("The component is not installed");
			}
			connection.invoke(objectName, action, objects, strings);
			System.out.println(action + " component name : " + componentName);

		} catch (Exception e) {
			if (failOnError) {
				throw e;
			}
		} finally {
			connector.close();
		}
	}

	/**
	 * Perform an action on the component installer
	 * 
	 * @param action
	 *            action to perform (install, uninstall)
	 */
	protected void performInstallerComponentAction(String action, String componentName)
			throws Exception {
		JMXConnector connector = null;
		try {
			connector = JBIJMXConnectorUtil.getConnection(host, port, username,
					password);
			MBeanServerConnection connection = connector
					.getMBeanServerConnection();
			Object[] objects = new Object[0];
			String[] strings = new String[0];
			ObjectName objectName = JBIJMXConnectorUtil.getInstallerComponentMBeanName(
					connection, componentName);
			if (objectName == null) {
				throw new Exception("The component has no installer");
			}
			connection.invoke(objectName, action, objects, strings);
			System.out.println(action + " component name : " + componentName);

		} catch (Exception e) {
			if (failOnError) {
				throw e;
			}
		} finally {
			connector.close();
		}
	}

	/**
	 * Perform an action on the Installation service
	 * 
	 * @param action
	 * @return
	 * @throws Exception
	 */
	protected void performInstallationAction(String action, String componentName) throws Exception {

		JMXConnector connector = null;
		try {
			connector = JBIJMXConnectorUtil.getConnection(host, port, username,
					password);
			MBeanServerConnection connection = connector
					.getMBeanServerConnection();
			Object[] objectsLoad = new Object[1];
			objectsLoad[0] = componentName;
			String[] stringsLoad = new String[1];
			stringsLoad[0] = "java.lang.String";
			connection.invoke(JBIJMXConnectorUtil
					.getInstallationServiceMBeanName(connection),
					action, objectsLoad,
					stringsLoad);
			System.out.println(action + " component name : " + componentName);
		
		} catch (Exception e) {
			if (failOnError) {
				throw e;
			}
		} finally {
			connector.close();
		}
	}

	/**
	 * Perform an uninstall on the component
	 * 
	 */
	protected void performUnload(String componentName) throws Exception {
		JMXConnector connector = null;
		try {
			connector = JBIJMXConnectorUtil.getConnection(host, port, username,
					password);
			MBeanServerConnection connection = connector
					.getMBeanServerConnection();
			Object[] objects = new Object[2];
			objects[0] = componentName;
			objects[1] = true;
			String[] strings = new String[2];
			strings[0] = "java.lang.String";
			strings[1] = "boolean";
			// Unload the installer
			connection.invoke(JBIJMXConnectorUtil
					.getInstallationServiceMBeanName(connection),
					JMXComponentAbstractMojo.UNLOAD_INSTALLER, objects, strings);
			System.out.println("Unload component name : " + componentName);
		} catch (Exception e) {
			if (failOnError) {
				throw e;
			}
		} finally {
			connector.close();
		}
	}
}
