/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: PackageAssemblyMojo.java 16:56:43 ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.tools.jbiplugin;

import java.io.DataInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

/**
 * Make a JBI Archive
 * 
 * @author cdeneux - Capgemini Sud
 */
public abstract class PackageAbstractMojo extends JBIAbstractMojo {

	/**
     * JBI sources directory
     * 
     * @parameter expression="${basedir}/src/main/jbi/"
     * @required
     * @description Path of the JBI sources directory
     */
    protected File jbiDirectory;

    /**
     * Recurse a directory to add its content to a zip file
     * 
     * @param zipOutputStream
     *            zip file
     * @param directory
     *            directory to recurse
     * @param entryDirectoryName
     *            directory to put the content in the zip
     * @throws Exception
     */
    protected void recurseDirectory(ZipOutputStream zipOutputStream,
        File directory, String entryDirectoryName) throws Exception {
        for (File file : directory.listFiles()) {
            if (file.isFile()) {
            	if (verbose) {
                    System.out.println("     "
                        + entryDirectoryName + File.separator + file.getName() + " added to JBI Service Assembly archive");
                }
                ZipEntry zipEntry = new ZipEntry(("".equals(entryDirectoryName)) ? file.getName() : (entryDirectoryName
                    + File.separator + file.getName()));
                DataInputStream dis = new DataInputStream(
                    new FileInputStream(file));
                byte[] content = new byte[dis.available()];
                dis.readFully(content);
                zipOutputStream.putNextEntry(zipEntry);
                zipOutputStream.write(content);
                zipOutputStream.closeEntry();
            } else {
                if (!file.getName().startsWith(".")) {
                    // Avoid to add the .svn directory
                    recurseDirectory(zipOutputStream, file, ("".equals(entryDirectoryName)) ? file.getName() : (entryDirectoryName
                            + File.separator + file.getName()));
                }
            }
        }
    }
}
