/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: PackageAssemblyMojo.java 16:56:43 ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.tools.jbiplugin;

import java.io.DataInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

import org.apache.maven.model.Dependency;
import org.apache.maven.plugin.MojoExecutionException;

/**
 * Make a JBI Archive of a service assembly project
 * 
 * @goal jbi-service-assembly
 * @description Make a JBI Service Assembly Archive from a service assembly project
 * @requiresDependencyResolution runtime
 * 
 * @author cdeneux - Capgemini Sud
 */
public class PackageServiceAssemblyMojo extends PackageAbstractMojo {

    /**
     * Execute the plugin
     */
    public void execute() throws MojoExecutionException {
        try {
            project.getArtifact().setFile(new File(outputDirectory
                + File.separator + jbiName + ".zip"));
            if (jbiDirectory.exists()) {
                if (verbose) {
                    System.out.println("Start building JBI Service Assembly archive "
                        + new File(outputDirectory
                            + File.separator + jbiName + ".zip")
                            .getAbsolutePath());
                }
                
                new File(outputDirectory
                    + File.separator + jbiName + ".zip").delete();
                
                outputDirectory.mkdirs();

                // Add the jbi files
                ZipOutputStream zipOutputStream = new ZipOutputStream(
                    new FileOutputStream(new File(outputDirectory
                        + File.separator + jbiName + ".zip")));
                recurseDirectory(zipOutputStream, jbiDirectory, "");
                zipOutputStream.flush();
                
                // Add the service unit dependencies files
                addServiceUnitDepencies(zipOutputStream);
                zipOutputStream.flush();

                zipOutputStream.close();
                if (verbose) {
                    System.out.println("JBI Service Assembly archive building done.\n");
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
            throw new MojoExecutionException(e.getLocalizedMessage(), e);
        }
    }
    
    /**
     * Add all JBI service unit declared as dependencies
     * 
     */
    private void addServiceUnitDepencies(ZipOutputStream zipOutputStream) throws Exception {
    	
    	for (Object object : project.getDependencies()) {
            if (object instanceof Dependency) {
                Dependency dep = (Dependency) object;
                // TODO: Change the condition using <packaging> tag of the dependency
                if ("zip".equals(dep.getType())) {
                	String zipDependencyFileName = dep.getArtifactId() + "-" + dep.getVersion() + ".zip";
                	
                	ZipEntry zipEntry = new ZipEntry(zipDependencyFileName);
                    DataInputStream dis = new DataInputStream(new FileInputStream(
                        new File(localRepository.getBasedir()
                            + File.separator
                            + dep.getGroupId().replace(".", File.separator)
                            + File.separator + dep.getArtifactId()
                            + File.separator + dep.getVersion()
                            + File.separator + zipDependencyFileName)));
                    byte[] content = new byte[dis.available()];
                    dis.readFully(content);
                    zipOutputStream.putNextEntry(zipEntry);
                    zipOutputStream.write(content);
                    zipOutputStream.closeEntry();
                    if (verbose) {
                        System.out.println("     "
                            + zipDependencyFileName + " added to JBI Service Assembly archive");
                    }
                }
            }
        }
    }
}
