/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: PackageAssemblyMojo.java 16:56:43 ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.tools.jbiplugin;

import java.io.File;
import java.io.FileOutputStream;
import java.util.zip.ZipOutputStream;

import org.apache.maven.plugin.MojoExecutionException;

/**
 * Make a JBI Archive of a service unit project
 * 
 * @goal jbi-service-unit
 * @description Make a JBI Service Unit Archive from a service unit project
 * @requiresDependencyResolution runtime
 * 
 * @author cdeneux - Capgemini Sud
 */
public class PackageServiceUnitMojo extends PackageAbstractMojo {

    /**
     * Execute the plugin
     */
    public void execute() throws MojoExecutionException {
        try {
            project.getArtifact().setFile(new File(outputDirectory
                + File.separator + jbiName + ".zip"));
            if (jbiDirectory.exists()) {
                if (verbose) {
                    System.out.println("Start building JBI Service unit archive "
                        + new File(outputDirectory
                            + File.separator + jbiName + ".zip")
                            .getAbsolutePath());
                }
                
                new File(outputDirectory
                    + File.separator + jbiName + ".zip").delete();
                
                outputDirectory.mkdirs();

                // Add the jbi files
                ZipOutputStream zipOutputStream = new ZipOutputStream(
                    new FileOutputStream(new File(outputDirectory
                        + File.separator + jbiName + ".zip")));
                recurseDirectory(zipOutputStream, jbiDirectory, "");
                zipOutputStream.flush();

                zipOutputStream.close();
                if (verbose) {
                    System.out.println("JBI Service unit archive building done.\n");
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
            throw new MojoExecutionException(e.getLocalizedMessage(), e);
        }
    }
    
}
