/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 Fossil E-Commerce, http://www.fossilec.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: ZipUtil.java 410 2006-05-17 13:25:42Z ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.tools.jbiplugin.util;

import java.io.DataInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

/**
 * Contains utilities methods in relation to Zip operations.
 * 
 * @version $Rev: 410 $ $Date: 2006-05-17 15:25:42 +0200 (mer, 17 mai 2006) $
 * @since Petals 1.0
 * @author <a href="mailto:rmarins@fossilec.com">Rafael Marins</a>
 * @author ddesjardins - eBMWebsourcing
 */
public final class ZipUtil {

    /**
     * Do not construct.
     */
    private ZipUtil() {
        super();
    }

    /**
     * Add all the content of directory to a zip file to a specific entry
     * 
     * @param archive
     *            zip file
     * @param directory
     *            directory to add
     * @param entryDirectory
     *            entry directory
     * @throws Exception
     * @throws PetalsException
     */
    public static void zipDirectory(File archive, File directory,
            String entryDirectoryName) throws Exception {
        if (directory.exists()) {
            ZipOutputStream zipOutputStream = new ZipOutputStream(
                    new FileOutputStream(archive, true));
            recurseDirectory(zipOutputStream, directory, entryDirectoryName);
            zipOutputStream.flush();
            zipOutputStream.close();
        }
    }

    /**
     * Recurse a directory to add its content to a zip file
     * 
     * @param zipOutputStream
     *            zip file
     * @param directory
     *            directory to recurse
     * @param entryDirectoryName
     *            directory to put the content in the zip
     * @throws Exception
     */
    private static void recurseDirectory(ZipOutputStream zipOutputStream,
            File directory, String entryDirectoryName) throws Exception {
        for (File file : directory.listFiles()) {
            if (file.isFile()) {
                ZipEntry zipEntry = new ZipEntry(entryDirectoryName
                        + File.separator + file.getName());
                DataInputStream dis = new DataInputStream(new FileInputStream(
                        file));
                byte[] content = new byte[dis.available()];
                dis.readFully(content);
                zipOutputStream.putNextEntry(zipEntry);
                zipOutputStream.write(content);
                zipOutputStream.closeEntry();
            } else {
                if (!file.getName().startsWith(".")) {
                    // Avoid to add the .svn directory
                    recurseDirectory(zipOutputStream, file, entryDirectoryName
                            + File.separator + file.getName());
                }
            }
        }
    }
}
