/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: JBIPluginMojoTest.java 11:10:18 ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.tools.jbiplugin;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import junit.framework.TestCase;

import org.apache.maven.model.Dependency;
import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.project.MavenProject;
import org.objectweb.petals.tools.jbiplugin.mock.ArtifactMock;
import org.objectweb.petals.tools.jbiplugin.mock.ArtifactRepositoryMock;

/**
 * Test the jbi maven plugin
 * 
 * @author ddesjardins - eBMWebsourcing
 */
public class JBIPluginMojoTest extends TestCase {

    public void testExecute() throws MojoExecutionException {
        String baseDir = this.getClass().getResource(".").toString();
        baseDir = baseDir.substring(0, baseDir.indexOf("target"));
        baseDir = baseDir.substring(baseDir.indexOf(":") + 1);
        JBIPluginMojo plugin = new JBIPluginMojo();
        plugin.jbiDirectory = new File(baseDir + "src" + File.separator
            + "test-data" + File.separator + "conf");

        ArtifactRepositoryMock artifactRepositoryMock = new ArtifactRepositoryMock();
        artifactRepositoryMock.setBaseDir(baseDir + "src" + File.separator
            + "test-data" + File.separator + "repository");
        plugin.localRepository = artifactRepositoryMock;

        MavenProject project = new MavenProject();
        project.setArtifact(new ArtifactMock());
        List<Dependency> dependencies = new ArrayList<Dependency>();
        Dependency dependency = new Dependency();
        dependency.setArtifactId("test");
        dependency.setGroupId("test");
        dependency.setVersion("1.0");
        dependencies.add(dependency);

        project.setDependencies(dependencies);
        plugin.toExcludes = "";
        plugin.project = project;
        plugin.outputDirectory = new File(baseDir + "target");
        plugin.jbiName = "test";

        plugin.componentJar = new File(baseDir + "src" + File.separator
            + "test-data" + File.separator + "component.jar");

        plugin.execute();
    }

    public void testExecuteWithSubDep() throws MojoExecutionException {
        String baseDir = new File(".").getAbsolutePath();
        baseDir = baseDir.substring(0, baseDir.length() - 1);
        if (!baseDir.endsWith("jbi-plugin" + File.separator)) {
            baseDir += "tools" + File.separator + "jbi-plugin" + File.separator;
        }
        JBIPluginMojo plugin = new JBIPluginMojo();
        plugin.jbiDirectory = new File(baseDir + "src" + File.separator
            + "test-data" + File.separator + "conf");

        ArtifactRepositoryMock artifactRepositoryMock = new ArtifactRepositoryMock();
        artifactRepositoryMock.setBaseDir(baseDir + "src" + File.separator
            + "test-data" + File.separator + "repository");
        plugin.localRepository = artifactRepositoryMock;

        MavenProject project = new MavenProject();
        project.setArtifact(new ArtifactMock());
        List<Dependency> dependencies = new ArrayList<Dependency>();
        Dependency dependency = new Dependency();
        dependency.setArtifactId("test");
        dependency.setGroupId("test");
        dependency.setVersion("1.0");
        dependencies.add(dependency);

        project.setDependencies(dependencies);
        plugin.project = project;
        plugin.toExcludes = "";
        plugin.outputDirectory = new File(baseDir + "target");
        plugin.jbiName = "test";

        plugin.componentJar = new File(baseDir + "src" + File.separator
            + "test-data" + File.separator + "component.jar");
        plugin.includeSubDependencies = true;

        plugin.execute();
    }

    public void testExecuteUpdateJBI() throws MojoExecutionException {
        String baseDir = new File(".").getAbsolutePath();
        baseDir = baseDir.substring(0, baseDir.length() - 1);
        if (!baseDir.endsWith("jbi-plugin" + File.separator)) {
            baseDir += "tools" + File.separator + "jbi-plugin" + File.separator;
        }
        JBIPluginMojo plugin = new JBIPluginMojo();
        plugin.jbiDirectory = new File(baseDir + "src" + File.separator
            + "test-data" + File.separator + "conf");

        ArtifactRepositoryMock artifactRepositoryMock = new ArtifactRepositoryMock();
        artifactRepositoryMock.setBaseDir(baseDir + "src" + File.separator
            + "test-data" + File.separator + "repository");
        plugin.localRepository = artifactRepositoryMock;

        MavenProject project = new MavenProject();
        project.setArtifact(new ArtifactMock());
        List<Dependency> dependencies = new ArrayList<Dependency>();
        Dependency dependency = new Dependency();
        dependency.setArtifactId("test");
        dependency.setGroupId("test");
        dependency.setVersion("1.0");
        dependencies.add(dependency);

        project.setDependencies(dependencies);
        plugin.project = project;
        plugin.toExcludes = "";
        plugin.updateJBIXml = true;
        plugin.outputDirectory = new File(baseDir + "target");
        plugin.jbiName = "test";

        plugin.componentJar = new File(baseDir + "src" + File.separator
            + "test-data" + File.separator + "component.jar");
        plugin.includeSubDependencies = true;

        plugin.execute();
    }

}
