/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: CopyFileSet.java 15:05:18 ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.tools.packageplugin.utils;

import java.io.File;
import java.io.IOException;

import org.apache.commons.io.FileUtils;

/**
 * Copy a fileset to a directory
 * 
 * @author ddesjardins - eBMWebsourcing
 */
public final class CopyFileSetUtil {

    private CopyFileSetUtil() {
        // Do nothing
    }

    /**
     * Copy a fileset
     * 
     * @param baseDir
     *            base directory
     * @param toDir
     *            directory to copy to
     * @param includePattern
     *            file pattern
     * @throws IOException
     */
    public static void copyFileSetToDirectory(File baseDir, File toDir,
        String includePattern, boolean keepDirectory) throws IOException {
        copyFileSetToDirectory(baseDir, toDir, includePattern, "",
            keepDirectory);
    }

    /**
     * Copy a fileset
     * 
     * @param baseDir
     *            base directory
     * @param toDir
     *            directory to copy to
     * @param includePattern
     *            file pattern to include
     * @param excludePattern
     *            file pattern to exclude
     * @throws IOException
     */
    public static void copyFileSetToDirectory(File baseDir, File toDir,
        String includePattern, String excludePattern, boolean keepDirectory)
        throws IOException {
        if (baseDir.exists()) {
            String dirName = "**";
            String excludeDirName = "";
            String filePattern = includePattern;
            if (includePattern.startsWith("**")) {
                if (includePattern.indexOf(File.separator) > -1) {
                    dirName = includePattern.substring(0, includePattern
                        .indexOf(File.separator));
                } else {
                    dirName = "";
                }
                filePattern = includePattern.substring(3);
            } else {
                dirName = "";
            }
            if (!"".equals(excludePattern)
                && excludePattern.indexOf(File.separator) > -1) {
                excludeDirName = excludePattern.substring(0, excludePattern
                    .indexOf(File.separator));
            } else {
                excludeDirName = excludePattern;
            }
            for (File file : baseDir.listFiles()) {
                if (file.isDirectory()
                    && ("**".equals(dirName) || file.getName().equals(dirName))
                    && (!file.getName().equals(excludeDirName))) {
                    recurseDirectory(baseDir, file, toDir, includePattern,
                        excludePattern, keepDirectory);
                } else if (file.isFile()
                    && file.getName().contains(filePattern.replace("*", ""))) {
                    FileUtils.copyFileToDirectory(file, toDir);
                }
            }
        }
    }

    /**
     * Recurse a directory
     * 
     * @param dir
     *            directory to recurse
     * @param toDir
     *            directory to copy to
     * @param includePattern
     *            include pattern
     * @param excludePattern
     *            exclude pattern
     * @throws IOException
     */
    private static void recurseDirectory(File baseDir, File dir, File toDir,
        String includePattern, String excludePattern, boolean keepDirectory)
        throws IOException {
        String dirName = "**";
        String filePattern = includePattern;
        String lastDirName = includePattern;
        String excludeDirName = "";
        if (includePattern.startsWith("**")) {
            dirName = includePattern.substring(0, includePattern
                .indexOf(File.separator));
            filePattern = includePattern.substring(3);
            if (filePattern.indexOf(File.separator) > -1) {
                lastDirName = filePattern.substring(0, filePattern
                    .indexOf(File.separator));
            }
        } else {
            if (includePattern.indexOf(File.separator) > -1) {
                dirName = includePattern.substring(0, includePattern
                    .indexOf(File.separator));
                filePattern = includePattern.substring(dirName.length() + 1);
            }
        }
        if (!"".equals(excludePattern)
            && excludePattern.indexOf(File.separator) > -1) {
            excludeDirName = excludePattern.substring(0, excludePattern
                .indexOf(File.separator));
        } else {
            excludeDirName = excludePattern;
        }
        if (dir.getName().equals(lastDirName)) {
            filePattern = filePattern.substring(lastDirName.length() + 1);
            includePattern = filePattern;
            if (includePattern.indexOf(File.separator) > -1) {
                dirName = includePattern.substring(0, includePattern
                    .indexOf(File.separator));
                filePattern = includePattern.substring(dirName.length() + 1);
            }
        }
        for (File file : dir.listFiles()) {
            if (file.isDirectory()
                && ("**".equals(dirName) || file.getName().equals(dirName))
                && (!file.getName().equals(excludeDirName))) {
                recurseDirectory(baseDir, file, toDir, includePattern,
                    excludePattern, keepDirectory);
            } else if (file.isFile()
                && file.getName().contains(filePattern.replace("*", ""))) {
                String fileDir = file.getAbsolutePath().substring(
                    baseDir.getAbsolutePath().length());
                fileDir = fileDir.substring(0, fileDir.length()
                    - file.getName().length());
                if (keepDirectory) {
                    new File(toDir
                        + fileDir).mkdir();
                    FileUtils.copyFileToDirectory(file, new File(toDir
                        + fileDir));
                } else {
                    FileUtils.copyFileToDirectory(file, toDir);
                }
            }
        }
    }

}
