/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: PackageAssemblyMojoTest.java 1:09:38 PM ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.tools.packageplugin;

import java.io.File;
import java.io.IOException;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;

import org.apache.maven.artifact.repository.ArtifactRepository;
import org.apache.maven.model.Dependency;
import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.project.MavenProject;
import org.easymock.classextension.EasyMock;

import junit.framework.TestCase;

/**
 * Tests of PackageAssemblyMojo
 * 
 * @author ddesjardins - eBMWebsourcing
 */
public class PackageAssemblyMojoTest extends TestCase {

    private String baseDir;

    public void setUp() {
        baseDir = this.getClass().getResource(".").toString();
        baseDir = baseDir.substring(0, baseDir.indexOf("target"));
        baseDir = baseDir.substring(baseDir.indexOf(":") + 1);
    }

    public void testExecute() throws MojoExecutionException, SecurityException,
        NoSuchMethodException, IOException {
        PackageAssemblyMojo assemblyMojo = EasyMock.createMock(
                PackageAssemblyMojo.class, new Method[] {
                    PackageAssemblyMojo.class.getDeclaredMethod("prepare",
                            new Class[] {}),
                    PackageAssemblyMojo.class.getDeclaredMethod(
                            "copyRessources", new Class[] {}),
                    PackageAssemblyMojo.class.getDeclaredMethod("copyLib",
                            new Class[] {}),
                    PackageAssemblyMojo.class.getDeclaredMethod(
                            "copyMonitoringRessources", new Class[] {}),
                    PackageAssemblyMojo.class.getDeclaredMethod(
                            "makeMonitoringServerJar", new Class[] {}),
                    PackageAssemblyMojo.class.getDeclaredMethod(
                            "makeServerJar", new Class[] {})});

        assemblyMojo.prepare();
        assemblyMojo.copyRessources();
        assemblyMojo.copyLib();
        assemblyMojo.makeServerJar();
        
        EasyMock.replay(assemblyMojo);

        assemblyMojo.execute();

        EasyMock.verify(assemblyMojo);
    }

    public void testPrepare() throws IOException {
        PackageAssemblyMojo assemblyMojo = new PackageAssemblyMojo();

        File as = new File(baseDir + "target" + File.separator
                + "test-assembly");
        assemblyMojo.petalsAssemblyDirectory = as;

        assemblyMojo.prepare();

        assertTrue(new File(as + File.separator + "ant").exists());
        assertTrue(new File(as + File.separator + "bin").exists());
        assertTrue(new File(as + File.separator + "components").exists());
        assertTrue(new File(as + File.separator + "conf").exists());
        assertTrue(new File(as + File.separator + "demos").exists());
        assertTrue(new File(as + File.separator + "lib").exists());
        assertTrue(new File(as + File.separator + "logs").exists());
        assertTrue(new File(as + File.separator + "schema").exists());
        assertTrue(new File(as + File.separator + "install").exists());
        assertTrue(new File(as + File.separator + "installed").exists());
        assertTrue(new File(as + File.separator + "uninstalled").exists());
    }

    public void testCopyRessources() throws IOException {
        PackageAssemblyMojo assemblyMojo = new PackageAssemblyMojo();

        String assemblyDir = baseDir.replace("tools", "platform");
        assemblyDir = assemblyDir.replace("package-plugin", "assembly");

        assemblyMojo.baseDir = new File(assemblyDir);
        File as = new File(baseDir + "target" + File.separator
                + "test-assembly");
        assemblyMojo.petalsAssemblyDirectory = as;

        assemblyMojo.copyRessources();

        assertTrue(new File(as + File.separator + "bin" + File.separator
                + "startup.bat").exists());
        assertTrue(new File(as + File.separator + "conf" + File.separator
                + "julia.cfg").exists());
        assertTrue(new File(as + File.separator + "schema" + File.separator
                + "jbi.xsd").exists());
    }

    public void testCopyLib() throws IOException {
        PackageAssemblyMojo assemblyMojo = new PackageAssemblyMojo();

        assemblyMojo.baseDir = new File(baseDir.replace("tools"
                + File.separator + "package-plugin", "platform"
                + File.separator + "assembly"));
        File as = new File(baseDir + "target" + File.separator
                + "test-assembly");
        assemblyMojo.petalsAssemblyDirectory = as;

        MavenProject project = EasyMock.createMock(MavenProject.class);
        ArtifactRepository artifactRepository = EasyMock
                .createMock(ArtifactRepository.class);

        List<Dependency> deps = new ArrayList<Dependency>();
        Dependency dependency = new Dependency();
        dependency.setArtifactId("jbi");
        dependency.setGroupId("javax.jbi");
        dependency.setVersion("1.0");
        dependency.setScope(null);
        deps.add(dependency);

        EasyMock.expect(project.getDependencies()).andReturn(deps).anyTimes();

        String bd = System.getProperty("user.home");
        EasyMock.expect(artifactRepository.getBasedir()).andReturn(
                bd + File.separator + ".m2" + File.separator + "repository");

        EasyMock.replay(project);
        EasyMock.replay(artifactRepository);

        assemblyMojo.project = project;
        assemblyMojo.localRepository = artifactRepository;
        assemblyMojo.copyLib();

        assertTrue(new File(as + File.separator + "lib" + File.separator
                + "jbi-1.0.jar").exists());
    }

    public void testMakeServerJar() throws IOException {
        PackageAssemblyMojo assemblyMojo = new PackageAssemblyMojo();

        assemblyMojo.baseDir = new File(baseDir.replace("tools"
                + File.separator + "package-plugin", "platform"
                + File.separator + "assembly"));
        File as = new File(baseDir + "target" + File.separator
                + "test-assembly");
        assemblyMojo.petalsAssemblyDirectory = as;

        assemblyMojo.makeServerJar();

        assertTrue(new File(as + File.separator + "bin" + File.separator
                + "server.jar").exists());
    }

}
