package org.objectweb.petals.tools.petalsadmin.bean;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.ListIterator;
import java.util.Map;
import java.util.Random;

import org.objectweb.petals.tools.petalsadmin.utils.JBIEntity;
import org.objectweb.petals.tools.petalsadmin.utils.JMXUtils;

public class MonitorBean {
	
	private String serverHost;
	private String jmxPort;
	private String jmxUser;
	private String jmxPasswd;
	
	public MonitorBean() {
	}
	
	public List<List<Map<String,Object>>> bulkData(Map<String, String> serverConf, long sinceTime) 
		throws Exception
	{
		try {
			JMXUtils utilsJmx = new JMXUtils(serverHost,jmxPort,jmxUser,jmxPasswd);
			return utilsJmx.bulkData(serverConf,sinceTime);
		}
		catch (Exception e) {
			throw new Exception(e);
		}
	}
	
	/*
	 * List<Map<String,Object>> : liste des captures d'un echange
	 * List<List<Map<String,Object>>> : liste des listes de captures
	 */
	public List<List<Map<String,Object>>>  getExchanges(Map<String, String> serverConf, long sinceTime, String component)
		throws Exception
	{
		//recuperation des echanges
		List<List<Map<String, Object>>> bulkDatas;
		try {
			bulkDatas = this.bulkData(serverConf, sinceTime);
			//creation de la liste des echanges a retourner
			List<List<Map<String,Object>>> componentExchanges = new ArrayList<List<Map<String,Object>>>();
		
			//parcours des echanges et tri
			ListIterator<List<Map<String,Object>>> parcExchanges = bulkDatas.listIterator();
			while(parcExchanges.hasNext())
			{
				//booleen pour indiquer que l'echange correspond
				boolean concernComponent=false;
				//recuperation de la liste des captures de l'echange
				List<Map<String,Object>> currentExchange = parcExchanges.next();
				//parcours des captures pour trouver celles qui correspondent au composant demande
				ListIterator<Map<String,Object>> parcCaptures = currentExchange.listIterator();
				while (parcCaptures.hasNext())
				{
					//recuperation de la capture en cours
					Map<String,Object> capture = parcCaptures.next();
					if (capture.get("component").equals(component) && !concernComponent)
					{
						//la capture correspond au composant demande
						//->on ajoute l'echange a la liste a retourner
						concernComponent=true;
					}
				}
				//une capture contenue par le composant concerne le composant demande
				//-> ajout du composant dans la liste a retourner
				if (concernComponent)
				{
					componentExchanges.add(currentExchange);
				}
			}
			return componentExchanges;
		} catch (Exception e) {
			throw new Exception(e);
		}
		
	}

	public List<Map<String, String>> getStartedServers() throws Exception
	{
		try {
			JMXUtils utilsJmx = new JMXUtils(serverHost,jmxPort,jmxUser,jmxPasswd);
			return utilsJmx.getStartedServers();
		}
		catch (Exception e) {
			throw new Exception(e);
		}
	}
	
	public List<Map<String, String>> getAllServers() throws Exception
	{
		try {
			JMXUtils utilsJmx = new JMXUtils(serverHost,jmxPort,jmxUser,jmxPasswd);
			return utilsJmx.getAllServers();
		}
		catch (Exception e) {
			throw new Exception(e);
		}
	}
	
	public Map<String, JBIEntity> listComponents(String phost, String pport, String pusername, String ppassword) 
	throws Exception {
		try {
			JMXUtils utilsJmx = new JMXUtils(phost,pport,pusername,ppassword);
			return utilsJmx.listComponents(phost, pport, pusername, ppassword);
		}
		catch (Exception e) {
			throw new Exception(e);
		}
	}
	
	public boolean monitor(String phost, String pport, String pusername, String ppassword,  String action) 
	throws Exception {
		try {
			JMXUtils utilsJmx = new JMXUtils(phost,pport,pusername,ppassword);
			boolean boolAction;
			if (action.equals("start"))
				boolAction=true;
			else
				boolAction=false;
			return utilsJmx.monitor(phost,pport,pusername,ppassword,boolAction);
		}
		catch (Exception e) {
			throw new Exception(e);
		}
	}
	
	public boolean monitorAllServers(List<Map<String, String>> startedServers, String action) 
	throws Exception {
		try {
			JMXUtils utilsJmx = new JMXUtils(serverHost,jmxPort,jmxUser,jmxPasswd);
			boolean boolAction;
			boolean result=true;
			if (action.equals("start"))
				boolAction=true;
			else
				boolAction=false;
			if (startedServers!=null && startedServers.size()>0)
			{
				ListIterator<Map<String, String>> parcServers = startedServers.listIterator();
				while (parcServers.hasNext())
				{
					Map<String, String> currentServer = parcServers.next();
					String serverHost = currentServer.get("host");
					String serverPort = currentServer.get("jmxPort");
					String serverLogin = currentServer.get("jmxLogin");
					String serverPasswd = currentServer.get("jmxPassword");
					utilsJmx.monitor(serverHost,serverPort,serverLogin,serverPasswd,boolAction);
				}
			}
			return result;
		}
		catch (Exception e) {
			return false;
		}
	}
	
	public String getAttribute(Map<String, String> serverConf,  String attributeName) 
	throws Exception {
		try {
			String serverHost = serverConf.get("host");
			String serverPort = serverConf.get("jmxPort");
			String serverLogin = serverConf.get("jmxLogin");
			String serverPasswd = serverConf.get("jmxPassword");
			JMXUtils utilsJmx = new JMXUtils(serverHost,serverPort,serverLogin,serverPasswd);
			return utilsJmx.getAttribute(serverHost,serverPort,serverLogin,serverPasswd,attributeName);
		}
		catch (Exception e) {
			throw new Exception(e);
		}
	}
	
	
	/**************************************************************************/
	/**                          FAUSSES FONCTIONS                           **/
	/**************************************************************************/
	
	/**
	 * return a set of exchanges
	 * @param sinceTime : time of first exchange to return
	 * @return ArrayList<HashMap> set of HashMap
	 * each HashMap is a <id,datas> representation of an exchange
	 * datas are <id, value> HashMap too : time, compo, role , type, status
	 */
	public List<Map.Entry<String, List<Map<String,Object>>>> oldbulkData(long sinceTime)
	{
		List<Map.Entry<String, List<Map<String,Object>>>> list_exchanges = new ArrayList<Map.Entry<String, List<Map<String,Object>>>>();
		Random rd = new Random();

		//recuperation de la liste des composants
		ArrayList<String> compo_list = this.getComponents("all");

		//creation d'un echange 
		for (int i=1;i<=rd.nextInt(50);i++)
		{
			Map<String, List<Map<String,Object>>> exchange = new HashMap<String, List<Map<String,Object>>>();

			String id = new String("ex_"+Math.abs(rd.nextLong())); //id
			
			//ensemble des captures d'un echange
			List<Map<String,Object>> captureList = new ArrayList<Map<String,Object>>();
			//objet capture reutilisable
			Map<String,Object> capture = new HashMap<String,Object>();
			//creation de 4 captures par echange
			for (int j=1;j<=4;j++)
			{
				capture = new HashMap<String,Object>();
				
				Long t1 = new Date().getTime();
				
				//selection d'un composant au hasard
				String component = new String(compo_list.get(rd.nextInt(compo_list.size())));

				//selection d'un role au hasard
				String[] roles_list = new String[2];
				roles_list[0]="consumer";
				roles_list[1]="provider";
				String role = roles_list[rd.nextInt(2)];
				
				//selection d'un type au hasard
				String[] types_list = new String[4];
				types_list[0]="in";
				types_list[1]="out";
				types_list[2]="fault";
				types_list[3]="ANY";
				String type=types_list[rd.nextInt(4)];
				
				//selection d'un statut au hasard
				String[] status_list= new String[3];
				status_list[0]="active";
				status_list[1]="done";
				status_list[2]="error";
				String status=status_list[rd.nextInt(3)];
				
				//ajout des donnees dans la capture
				capture.put("time", t1);
				capture.put("component",component);
				capture.put("role",role);
				capture.put("type", type);
				capture.put("status", status);
				
				//ajout de la capture dans la liste des captures
				captureList.add(capture);
			}
			
			//ajout de l'id et de la liste des captures a l'echange
			exchange.put(id, captureList);
			
			//ajout de l'echange dans la liste d'echanges
			list_exchanges.add(exchange.entrySet().iterator().next());
		}
		return list_exchanges;
	}

	/**
	 * 
	 * @param group
	 * @return
	 */
	public ArrayList<String> getComponents(String group) {
		ArrayList<String> components = new ArrayList<String>();
		if (group=="BC")
		{
			components.add("FileTransfer");
			components.add("JMS");
			components.add("XQuare");
		}
		else
		{
			if (group=="SE")
			{
				components.add("Clock");
				components.add("CSV");
				components.add("HelloWorld");
				components.add("SampleClient");
				components.add("XSLT");
			}
			else
			{
				if (group=="all")
				{
					components.add("FileTransfer");
					components.add("JMS");
					components.add("XQuare");
					components.add("Clock");
					components.add("CSV");
					components.add("HelloWorld");
					components.add("SampleClient");
					components.add("XSLT");
				}
				else
				{
					components.add("none");
				}
			}
		}
		return components;
	}

	public String getJmxPasswd() {
		return jmxPasswd;
	}

	public void setJmxPasswd(String jmxPasswd) {
		this.jmxPasswd = jmxPasswd;
	}

	public String getJmxPort() {
		return jmxPort;
	}

	public void setJmxPort(String jmxPort) {
		this.jmxPort = jmxPort;
	}

	public String getJmxUser() {
		return jmxUser;
	}

	public void setJmxUser(String jmxUser) {
		this.jmxUser = jmxUser;
	}

	public String getServerHost() {
		return serverHost;
	}

	public void setServerHost(String serverHost) {
		this.serverHost = serverHost;
	}
	
}
