package org.objectweb.petals.tools.petalsadmin.utils;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.management.Attribute;
import javax.management.AttributeNotFoundException;
import javax.management.MBeanServerConnection;
import javax.management.ObjectName;
import javax.management.remote.JMXConnector;

public class JMXUtils {

	/**
	 * Host for the JMX connection
	 */
	private String host = "";

	/**
	 * Password the JMX connection
	 */
	private String password = "";

	/**
	 * Port for the JMX connection
	 */
	private String port = "";

	/**
	 * Username for the JMX connection
	 */
	private String username = "";

	public JMXUtils(String serverHost,String jmxPort, String jmxUser, String jmxPasswd) {
		this.host = serverHost;
		this.port=jmxPort;
		this.username=jmxUser;
		this.password=jmxPasswd;
	}

	/**
	 * Get a JMX connector
	 * 
	 * @param host
	 *            host
	 * @param port
	 *            port
	 * @param username
	 *            username
	 * @param password
	 *            password
	 * @return jmx connector
	 * @throws IOException
	 */
	/*public static JMXConnector getConnection(String host, String port,
			String username, String password) throws IOException {
		JMXConnector connector = null;
		Properties env = System.getProperties();
		env.put("java.naming.factory.initial",
				"com.sun.jndi.rmi.registry.RegistryContextFactory");
		Map<String, Object> args = new HashMap<String, Object>();
		if (!"".equals(username)) {
			if (password == null) {
				password = "";
			}
			String[] credentials = new String[] { username, password };
			args.put("jmx.remote.credentials", credentials);
		}
		String urlStr = "service:jmx:rmi:///jndi/rmi://" + host + ":" + port
				+ "/management/rmi-jmx-connector";
		JMXServiceURL url = new JMXServiceURL(urlStr);
		connector = JMXConnectorFactory.connect(url, args);
		return connector;
	}*/

	/**
	 * List the deployed JBI components
	 * 
	 * @return map of components
	 * @throws Exception
	 */
	public Map<String, JBIEntity> listComponents(String phost, String pport, String pusername, String ppassword) 
		throws Exception {
		Map<String, JBIEntity> components = new HashMap<String, JBIEntity>();
		JMXConnector connector = null;
		try {
			connector = retrieveConnector(phost, pport, pusername, ppassword);
			MBeanServerConnection connection = connector
					.getMBeanServerConnection();
			Object result = connection.getAttribute(JBIJMXConnectorUtil
					.getAdminServiceMBeanName(connection), "BindingComponents");
			if (result instanceof ObjectName[]) {
				ObjectName[] engines = (ObjectName[]) result;
				for (ObjectName objectName : engines) {
					JBIEntity component = new JBIEntity();
					String objState = (String) connection.getAttribute(
							objectName, "CurrentState");
					String name = objectName.getKeyProperty("name");
					component.setName(name);
					component.setState(objState);
					component.setObjectName(objectName);
					// TODO find a way to get the description of a component
					component.setDescription("");
					component.setType(JBIEntity.BINDING_COMPONENT);
					components.put(component.getName(), component);
				}
			}
			result = connection.getAttribute(JBIJMXConnectorUtil
					.getAdminServiceMBeanName(connection), "EngineComponents");
			if (result instanceof ObjectName[]) {
				ObjectName[] engines = (ObjectName[]) result;
				for (ObjectName objectName : engines) {
					JBIEntity component = new JBIEntity();
					String objState = (String) connection.getAttribute(
							objectName, "CurrentState");
					String name = objectName.getKeyProperty("name");
					component.setName(name);
					component.setState(objState);
					component.setObjectName(objectName);
					// TODO find a way to get the description of a component
					component.setDescription("");
					component.setType(JBIEntity.SERVICE_ENGINE);
					components.put(component.getName(), component);
				}
			}
			connector.close();
		} catch (Exception e) {
			try {
				if (connector != null) {
					connector.close();
				}
			} catch (IOException e1) {
				// Do nothing
			}
			throw new Exception(e);
		}
		return components;
	}

	/**
	 * Retrieve a JMX connector
	 * 
	 * @return a JMXConnector
	 * @throws IOException
	 */
	public JMXConnector retrieveConnector() throws IOException {
		return JBIJMXConnectorUtil
				.getConnection(host, port, username, password);
	}

	/**
	 * Retrieve a JMX connector
	 * 
	 * @return a JMXConnector
	 * @throws IOException
	 */
	public JMXConnector retrieveConnector(String phost, String pport, String pusername, String ppassword) 
		throws IOException {
		return JBIJMXConnectorUtil
				.getConnection(phost, pport, pusername, ppassword);
	}

	@SuppressWarnings("unchecked")
	public List<Map<String, String>> getStartedServers() throws Exception {
		//list of the servers name to return
		List<Map<String, String>> serversList= new ArrayList<Map<String, String>>();
		JMXConnector connector = null;
		try {
			connector = retrieveConnector();
			MBeanServerConnection connection = connector
					.getMBeanServerConnection();
			ObjectName petalsAdminName = new ObjectName(
					"Petals:name=PetalsAdmin,type=service");
			@SuppressWarnings("unused")
			Collection<Map<String, String>> serversConfigurations = (Collection<Map<String, String>>) connection
					.getAttribute(petalsAdminName,"AllStartedServersConfiguration");
			/* donnees des Map de serversConfigurations
    		String CONF_HOST="host";
    		String CONF_HTML_PORT="htmlPort";
    		String CONF_JMX_LOGIN="jmxLogin";
    		String CONF_JMX_PASSWORD="jmxPassword";
    		String CONF_JMX_PORT="jmxPort";
    		String CONF_NAME="name";
			 */
			Iterator<Map<String, String>> parcServers = serversConfigurations.iterator();
			while(parcServers.hasNext())
			{
				Map<String, String> currentServer = parcServers.next();
				serversList.add(currentServer);
			}
			
		} catch (Exception e) {
			throw new Exception(e);
		}
		return serversList;
	}
	
	@SuppressWarnings("unchecked")
	public List<Map<String, String>> getAllServers() throws Exception {
		//list of the servers name to return
		List<Map<String, String>> serversList= new ArrayList<Map<String, String>>();
		JMXConnector connector = null;
		try {
			connector = retrieveConnector();
			MBeanServerConnection connection = connector
					.getMBeanServerConnection();
			ObjectName petalsAdminName = new ObjectName(
					"Petals:name=PetalsAdmin,type=service");
			Collection<Map<String, String>> serversConfigurations = (Collection<Map<String, String>>) connection
					.getAttribute(petalsAdminName,"AllServersConfiguration");
			/* donnees des Map de serversConfigurations
    		String CONF_HOST="host";
    		String CONF_HTML_PORT="htmlPort";
    		String CONF_JMX_LOGIN="jmxLogin";
    		String CONF_JMX_PASSWORD="jmxPassword";
    		String CONF_JMX_PORT="jmxPort";
    		String CONF_NAME="name";
			 */
			Iterator<Map<String, String>> parcServers = serversConfigurations.iterator();
			while(parcServers.hasNext())
			{
				Map<String, String> currentServer = parcServers.next();
				serversList.add(currentServer);
			}
			
		} catch (Exception e) {
			throw new Exception(e);
		}
		return serversList;
	}
	
	@SuppressWarnings("unchecked")
	public List<List<Map<String,Object>>> bulkData(Map<String, String> serverConf, long sinceTime)
		throws Exception
	{
		List<List<Map<String,Object>>> list_exchanges = new ArrayList<List<Map<String,Object>>>();
		JMXConnector connector = null;
		try {
			String serverHost = serverConf.get("host");
			String serverPort = serverConf.get("jmxPort");
			String serverLogin = serverConf.get("jmxLogin");
			String serverPasswd = serverConf.get("jmxPassword");

			connector = retrieveConnector(serverHost,serverPort,serverLogin,serverPasswd);
			MBeanServerConnection connection = connector
					.getMBeanServerConnection();
			ObjectName petalsAdminName = new ObjectName(
			"Petals:name=PetalsAdmin,type=service");
			list_exchanges = (List<List<Map<String,Object>>>) connection.invoke(petalsAdminName,"bulkData", new Object[]{sinceTime}, new String[]{Long.TYPE.getCanonicalName()});
		}
		catch (Exception e)
		{
			throw new Exception(e);
		}
		return list_exchanges;
	}
	
	/**
	 * start or stop monitoring on a specific server
	 * @param serverHost ip address of the server to manage
	 * @param action : "start" to start monitoring, "stop" to stop monitoring
	 * @return
	 */
	public boolean monitor(String serverHost, String serverPort, String serverLogin, String serverPasswd, boolean action)
		throws Exception, AttributeNotFoundException
	{
		JMXConnector connector = null;
		try {
			connector = retrieveConnector(serverHost,serverPort,serverLogin,serverPasswd);
			MBeanServerConnection connection = connector
					.getMBeanServerConnection();
			ObjectName petalsAdminName = new ObjectName(
					"Petals:name=PetalsAdmin,type=service");
			Attribute attAction = new Attribute("Monitoring",action);
			connection.setAttribute(petalsAdminName, attAction);
			return true;
		}
		catch (AttributeNotFoundException anf) {
			return true;
		}
		catch (Exception e) {
			throw new Exception(e);
		}
	}

	/**
	 * start or stop monitoring on a specific server
	 * @param serverHost ip address of the server to manage
	 * @param action : "start" to start monitoring, "stop" to stop monitoring
	 * @return
	 */
	public String getAttribute(String serverHost, String serverPort, String serverLogin, String serverPasswd, String attributeName)
		throws Exception, AttributeNotFoundException
	{
		JMXConnector connector = null;
		try {
			connector = retrieveConnector(serverHost,serverPort,serverLogin,serverPasswd);
			MBeanServerConnection connection = connector
					.getMBeanServerConnection();
			ObjectName petalsAdminName = new ObjectName(
					"Petals:name=PetalsAdmin,type=service");
			return connection.getAttribute(petalsAdminName,"Monitoring").toString();
		}
		catch (AttributeNotFoundException anf) {
			return "not found";
		}
		catch (Exception e) {
			throw new Exception(e);
		}
	}

	public String getHost() {
		return host;
	}

	public void setHost(String host) {
		this.host = host;
	}

	public String getPassword() {
		return password;
	}

	public void setPassword(String password) {
		this.password = password;
	}

	public String getPort() {
		return port;
	}

	public void setPort(String port) {
		this.port = port;
	}

	public String getUsername() {
		return username;
	}

	public void setUsername(String username) {
		this.username = username;
	}

}
